<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:param name="PAGE_MODE_PUBLIC">true</xsl:param>

<xsl:template name="repeatString">
  <xsl:param name="str"/>
  <xsl:param name="count">1</xsl:param>
  <xsl:if test="$count &gt; 0">
    <xsl:value-of select="$str" disable-output-escaping="yes"/>
    <xsl:if test="$count &gt; 1">
      <xsl:call-template name="repeatString">
        <xsl:with-param name="str" select="$str" />
        <xsl:with-param name="count" select="$count - 1" />
      </xsl:call-template>
    </xsl:if>
  </xsl:if>
</xsl:template>

<xsl:template name="max">
  <xsl:param name="values"/>
  <xsl:choose>
    <xsl:when test="function-available('max')"><xsl:value-of select="max($values)"/></xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="emulateMinMax">
        <xsl:with-param name="values" select="$values"/>
        <xsl:with-param name="mode">max</xsl:with-param>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="min">
  <xsl:param name="values"/>
  <xsl:choose>
    <xsl:when test="function-available('min')"><xsl:value-of select="min($values)"/></xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="emulateMinMax">
        <xsl:with-param name="values" select="$values"/>
        <xsl:with-param name="mode">min</xsl:with-param>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="emulateMinMax">
  <xsl:param name="values"/>
  <xsl:param name="current">1</xsl:param>
  <xsl:param name="saved">0</xsl:param>
  <xsl:param name="mode">max</xsl:param>
  <xsl:variable name="result">
    <xsl:choose>
      <xsl:when test="$current = 1"><xsl:value-of select="$values[position() = 1]"/></xsl:when>
      <xsl:when test="$mode = 'max' and $saved &gt;= $values[position() = $current]"><xsl:value-of select="$saved"/></xsl:when>
      <xsl:when test="$mode = 'min' and $saved &lt;= $values[position() = $current]"><xsl:value-of select="$saved"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$values[position() = $current]"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="next" select="$current + 1"/>
  <xsl:choose>
    <xsl:when test="$next &gt; count($values)"><xsl:value-of select="$result"/></xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="emulateMinMax">
        <xsl:with-param name="values" select="$values"/>
        <xsl:with-param name="current" select="$next"/>
        <xsl:with-param name="saved" select="$result"/>        
        <xsl:with-param name="mode" select="$mode"/>        
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="space">
  <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
</xsl:template>
<xsl:template name="copyright">
  <xsl:text disable-output-escaping="yes">&amp;copy;</xsl:text>
</xsl:template>

<xsl:template name="forminputerror">
  <xsl:param name="error"/>
  <xsl:if test="$error">
    <span style="color: #FF0000; font-weight: bold;">*</span>
  </xsl:if>
</xsl:template>

<xsl:template match="metatags">
  <xsl:element name="meta">
    <xsl:attribute name="name">date</xsl:attribute>
    <xsl:attribute name="content"><xsl:value-of select="metatag[@type='date']" /></xsl:attribute>
  </xsl:element>
  <xsl:element name="meta">
    <xsl:attribute name="name">keywords</xsl:attribute>
    <xsl:attribute name="content"><xsl:value-of select="metatag[@type='keywords']" /></xsl:attribute>
  </xsl:element>
  <xsl:element name="meta">
    <xsl:attribute name="name">description</xsl:attribute>
    <xsl:attribute name="content"><xsl:value-of select="metatag[@type='description']" /></xsl:attribute>
  </xsl:element>
</xsl:template>

<xsl:template name="content_default">
  <xsl:param name="content" select="/page/content/topic"/>
  <xsl:if test="not($PAGE_MODE_PUBLIC = 1)">
    <div class="warning" style="text-align: center; color: red; margin: 20px;"><b>WARNING: Using default template</b></div>
  </xsl:if>
  <xsl:apply-templates select="$content/text" />
</xsl:template>

<xsl:template name="content_xhtml">
  <xsl:param name="content" select="/page/content/topic"/>
  <xsl:apply-templates select="$content/text" />
</xsl:template>


<xsl:template match="text">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="noescape|oldhtml">
  <xsl:value-of select="." disable-output-escaping="yes"/>
</xsl:template>

<xsl:template match="*">
  <xsl:element name="{local-name()}">
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates select="node()" />
  </xsl:element>
</xsl:template>

</xsl:stylesheet>