<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="de_DE_formatNumber">
  <xsl:param name="float"/>
  <xsl:param name="pattern">###,##0.00</xsl:param>
  <xsl:choose>
    <xsl:when test="$float + 1 &gt; 1"><xsl:value-of select="translate(format-number($float, $pattern), '.,', ',.')"/></xsl:when>
    <xsl:when test="$float - 1 &lt; -1"><xsl:value-of select="translate(format-number($float, $pattern), '.,', ',.')"/></xsl:when>
    <xsl:otherwise><xsl:value-of select="translate(format-number(0, $pattern), '.,', ',.')"/></xsl:otherwise>
  </xsl:choose>  
</xsl:template>

<xsl:template name="de_DE_formatShortDate">
  <xsl:param name="isodate"/>
  <xsl:variable name="year" select="substring($isodate, 1, 4)"/>
  <xsl:variable name="month" select="substring($isodate, 6, 2)"/>
  <xsl:variable name="day" select="substring($isodate, 9, 2)"/>
  <xsl:value-of select="$day"/>.<xsl:value-of select="$month"/>.<xsl:value-of select="$year"/>
</xsl:template>

<xsl:template name="de_DE_formatLongDate">
  <xsl:param name="isodate"/>
  <xsl:variable name="year" select="substring($isodate, 1, 4)"/>
  <xsl:variable name="month" select="substring($isodate, 6, 2)"/>
  <xsl:variable name="day" select="substring($isodate, 9, 2)"/>
  <xsl:value-of select="$day"/>. 
  <xsl:call-template name="languageMonth">
    <xsl:with-param name="month" select="$month"/>
  </xsl:call-template>
  <xsl:value-of select="$year"/>
</xsl:template>

<xsl:template name="de_DE_formatLongDateTime">
  <xsl:param name="isodatetime"/>
  <xsl:variable name="isodate" select="substring($isodatetime, 1, 10)"/>
  <xsl:variable name="isotime" select="substring($isodatetime, 12, 5)"/>
  <xsl:call-template name="de_DE_formatLongDate">
    <xsl:with-param name="isodate" select="$isodate"/>  
  </xsl:call-template>
  <xsl:text> </xsl:text>
  <xsl:value-of select="$isotime"/>
</xsl:template>

<xsl:template name="de_DE_languageMonth">
  <xsl:param name="month"/>
  <xsl:variable name="i_month">
    <xsl:choose>
      <xsl:when test="substring($month, 1, 1) = '0'"><xsl:value-of select="substring($month, 2)"/></xsl:when>
      <xsl:otherwise><xsl:value-of select="$month"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:choose>
    <xsl:when test="$i_month=1">Januar </xsl:when>
    <xsl:when test="$i_month=2">Februar </xsl:when>
    <xsl:when test="$i_month=3">M&#xE4;rz </xsl:when>
    <xsl:when test="$i_month=4">April </xsl:when>
    <xsl:when test="$i_month=5">Mai </xsl:when>
    <xsl:when test="$i_month=6">Juni </xsl:when>
    <xsl:when test="$i_month=7">Juli </xsl:when>
    <xsl:when test="$i_month=8">August </xsl:when>
    <xsl:when test="$i_month=9">September </xsl:when>
    <xsl:when test="$i_month=10">Oktober </xsl:when>
    <xsl:when test="$i_month=11">November </xsl:when>
    <xsl:when test="$i_month=12">Dezember </xsl:when>
  </xsl:choose>  
</xsl:template>

<xsl:template name="de_DE_languageWeekday">
  <xsl:param name="wday"/>
  <xsl:choose>
    <xsl:when test="$wday=1">Montag</xsl:when>
    <xsl:when test="$wday=2">Dienstag</xsl:when>
    <xsl:when test="$wday=3">Mittwoch</xsl:when>
    <xsl:when test="$wday=4">Donnerstag</xsl:when>
    <xsl:when test="$wday=5">Freitag</xsl:when>
    <xsl:when test="$wday=6">Samstag</xsl:when>
    <xsl:when test="$wday=7">Sonntag</xsl:when>
  </xsl:choose>  
</xsl:template>

<xsl:template name="de_DE_languageText">
  <xsl:param name="text"/>
  <xsl:choose>
    <xsl:when test="$text='more'">mehr</xsl:when>
    <xsl:when test="$text='Search'">Suche</xsl:when>
    <xsl:when test="$text='DoSearch'">Suchen</xsl:when>
    <xsl:otherwise><xsl:value-of select="$text"/></xsl:otherwise>
  </xsl:choose>  
</xsl:template>

</xsl:stylesheet>