<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:variable name="PAGE_LANGUAGE">
  <xsl:choose>
    <xsl:when test="/page/meta/metatags/language/text()"><xsl:value-of select="/page/meta/metatags/language/text()"/></xsl:when>  
    <xsl:otherwise>en_US</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xsl:template name="formatNumber">
  <xsl:param name="float"/>
  <xsl:param name="pattern">###,##0.00</xsl:param>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_formatNumber">
        <xsl:with-param name="float" select="$float"/>
        <xsl:with-param name="pattern" select="$pattern"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_formatNumber">
        <xsl:with-param name="float" select="$float"/>
        <xsl:with-param name="pattern" select="$pattern"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="formatShortDate">
  <xsl:param name="isodate"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_formatShortDate">
        <xsl:with-param name="isodate" select="$isodate"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_formatShortDate">
        <xsl:with-param name="isodate" select="$isodate"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="formatLongDate">
  <xsl:param name="isodate"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_formatLongDate">
        <xsl:with-param name="isodate" select="$isodate"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_formatLongDate">
        <xsl:with-param name="isodate" select="$isodate"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="formatLongDateTime">
  <xsl:param name="isodatetime"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_formatLongDateTime">
        <xsl:with-param name="isodatetime" select="$isodatetime"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_formatLongDateTime">
        <xsl:with-param name="isodatetime" select="$isodatetime"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="languageMonth">
  <xsl:param name="month"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_languageMonth">
        <xsl:with-param name="month" select="$month"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_languageMonth">
        <xsl:with-param name="month" select="$month"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="languageWeekday">
  <xsl:param name="wday"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_languageWeekday">
        <xsl:with-param name="wday" select="$wday"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_languageWeekday">
        <xsl:with-param name="wday" select="$wday"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="languageText">
  <xsl:param name="text"/>
  <xsl:choose>
    <xsl:when test="$PAGE_LANGUAGE = 'de_DE'">
      <xsl:call-template name="de_DE_languageText">
        <xsl:with-param name="text" select="$text"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="en_US_languageText">
        <xsl:with-param name="text" select="$text"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>