<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">

<xsl:template name="content_faq">
  <xsl:param name="content" select="/page/content/topic"/>
  <div id="faqcontent">
    <xsl:if test="$content/faq">
      <xsl:choose>
        <xsl:when test="$content/faq/searchdlg/dialog">                  
         <xsl:call-template name="faq_search">
           <xsl:with-param name="results" select="$content/faq/results"/>
           <xsl:with-param name="dialog" select="$content/faq/searchdlg/dialog"/>
           <xsl:with-param name="faqtitle" select="$content/faq/@title"/> 
           <xsl:with-param name="links" select="$content/faq/links" />  
           <xsl:with-param name="message" select="$content/faq/message" />    
         </xsl:call-template>
        </xsl:when>    
        <xsl:when test="$content/faq/groups/group/entries/entry[@selected]">
          <xsl:call-template name="faq_entrypage">
             <xsl:with-param name="content" select="$content"/>            
          </xsl:call-template>
        </xsl:when> 
        <xsl:when test="$content/faq/groups/group[@selected]"> 
          <xsl:call-template name="faq_grouppage">
             <xsl:with-param name="content" select="$content"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="$content/faq">
          <xsl:variable name="searchself" select="$content/faq/@shref"/>    
           <a href="{$searchself}" class="search">Suche</a>
           <h1 class="contentTitle">
             <xsl:value-of select="/page/content/topic/title" disable-output-escaping="yes"/> - 
             <xsl:value-of select="/page/content/topic/faq/@title" disable-output-escaping="yes"/>
           </h1>
          <xsl:call-template name="faq_mainpage">
             <xsl:with-param name="content" select="$content"/>
             <xsl:with-param name="faqtitle" select="$content/faq/@title"/>
          </xsl:call-template>
        </xsl:when>        
        <xsl:otherwise></xsl:otherwise>
      </xsl:choose>
    </xsl:if>
  </div> 
</xsl:template>

<xsl:template name="faq_search">
  <xsl:param name="results"/>
  <xsl:param name="dialog"/>
  <xsl:param name="faqtitle"/>
  <xsl:param name="links"/>
  <xsl:param name="message"/>
  <xsl:variable name="class">
    <xsl:choose>
      <xsl:when test="not(position() mod 2)">low</xsl:when>
        <xsl:otherwise>high</xsl:otherwise> 
    </xsl:choose>
  </xsl:variable>  
  <a href="{/page/content/topic/faq/@href}" class="back" >Zur&#xFC;ck</a>
  <h1 class="contentTitle">
    Suche
    <span class="contentSubtitle">
      <xsl:value-of select="/page/content/topic/title" disable-output-escaping="yes"/> -
      <xsl:value-of select="/page/content/topic/faq/@title" disable-output-escaping="yes"/>
    </span> 
  </h1>
  <form action="{$dialog/@action}" method="post">
  <xsl:copy-of select="$dialog/input[@type='hidden']"/>
  <table class="faqSearch" border="0">
    <tr>   
      <td class="title">Suchen:</td>
      <td class="inputField">
       <xsl:variable name="field" select="$dialog/lines/line[@fid='searchfor']/input"/>
       <input name="{$field/@name}" value="{$field/@value}" class="searchform"/>
       </td>   
      <td class="searchButton"> 
        <input type="submit" name="threadarr_{position()}" value="Suchen" />  
      </td>
    </tr> 
  </table>
  </form>  
  <xsl:if test="count($results/result) &gt; 0">
    <xsl:for-each select="$results/result">
      <div class="faqSearchResults">
        <div class="question"><xsl:value-of select="./text()" disable-output-escaping="yes"/></div>
        <a href="{@href}" class="more">mehr</a>
      </div>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xsl:template name="faq_entrypage"> 
  <xsl:param name="content"/>
  <xsl:variable name="entry" select="$content/faq/groups/group/entries/entry[@selected]"/>
  <xsl:variable name="searchself" select="$content/faq/@shref"/>
  <a href="{$content/faq/groups/group[@selected]/@href}" class="back">Zur&#xFC;ck</a>
  <a href="{$searchself}" class="search">Suche</a>
  <h1 class="contentTitle">
    <xsl:value-of select="$content/title" disable-output-escaping="yes"/> - 
    <xsl:value-of select="$content/faq/@title" disable-output-escaping="yes"/>
    <span class="contentSubtitle"><xsl:value-of select="$content/faq/groups/group[@selected]/@title" disable-output-escaping="yes"/></span>
  </h1>
  <div class="faqEntryPage">
    <div class="faqEntry">
      <div class="question">
        <xsl:value-of select="$entry/question" disable-output-escaping="yes"/>
      </div>
      <div class="answer">
        <xsl:value-of select="$entry/answer" disable-output-escaping="yes"/>       
      </div>  
    </div>
  </div>   
</xsl:template> 

<xsl:template name="faq_grouppage"> 
  <xsl:param name="content"/>  
  <xsl:variable name="faqgroup" select="$content/faq/groups/group[@selected]"/>
  <xsl:variable name="searchself" select="$content/faq/@shref"/>
  <a href="{$content/faq/@href}" class="back">Zur&#xFC;ck</a>
  <a href="{$searchself}" class="search">Suche</a>
  <h1 class="contentTitle">
    <xsl:value-of select="$content/title" disable-output-escaping="yes"/> - 
    <xsl:value-of select="$content/faq/@title" disable-output-escaping="yes"/>
    <span class="contentSubtitle"><xsl:value-of select="$faqgroup/@title" disable-output-escaping="yes"/></span>
  </h1>  
  <xsl:choose>
    <xsl:when test="count($faqgroup/entries/entry) &gt; 0">
      <xsl:call-template name="faqentrytable">
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise><div class="message">Keine Eintr&#228;ge in dieser Gruppe.</div></xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="faq_mainpage"> 
  <xsl:param name="content1" select="/page/content/topic/faq" />
  <xsl:param name="groupscont" select="$content1/groups/group"/>
  <xsl:variable name="topiccount" select="count($groupscont)" />
  <xsl:variable name="linecount">2</xsl:variable>                 
  <xsl:choose>
    <xsl:when test="$topiccount &gt; 0" >     
      <table class="faqGroupTable" border="0">
        <xsl:for-each select="$groupscont[(position() mod $linecount) = 1]">
          <xsl:variable name="indexmin" select="position() * $linecount - $linecount" />
          <xsl:variable name="indexmax" select="position() * $linecount" />
          <xsl:call-template name="faq_group">
            <xsl:with-param name="groupentry" 
            select="$groupscont[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
            <xsl:with-param name="topiccount" select="$topiccount" />
            <xsl:with-param name="linecount" select="$linecount"/>
          </xsl:call-template>
        </xsl:for-each>
      </table>  
    </xsl:when>
    <xsl:otherwise><div class="message">Keine Gruppen in dieser FAQ.</div></xsl:otherwise>
  </xsl:choose><br />  
</xsl:template>

<xsl:template name="faq_group">
    <xsl:param name="groupentry"/>    
    <xsl:param name="topiccount"/>
    <xsl:param name="linecount"/>
    <xsl:param name="count" select="count($groupentry)" />
    <xsl:param name="width" select="round(100 div $linecount)"/>
    <tr valign="top">
      <xsl:for-each select="$groupentry">   
        <xsl:variable name="swidth">
          <xsl:choose>
            <xsl:when test="($linecount &gt; $count) and (position() = $count)">
              <xsl:value-of select="0" />
            </xsl:when>
            <xsl:when test="position() != $linecount">
              <xsl:value-of select="$width" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="( 100 - ( $width * ( $linecount - 1 ) ) )" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>    
        <xsl:call-template name="faqgroup">
          <xsl:with-param name="groupentry" select="."/>
          <xsl:with-param name="width" select="$swidth" />
        </xsl:call-template>
      </xsl:for-each>
      <xsl:variable name="colspan" select="$linecount - $count"/>
      <xsl:if test="($colspan &gt; 0) and (position() > 1)">
        <td class="spacer">
          <xsl:if test="$colspan &gt; 1">
            <xsl:attribute name="colspan"><xsl:value-of select="$colspan"/></xsl:attribute>
          </xsl:if>
          <xsl:call-template name="space"/>
        </td> 
      </xsl:if>  
    </tr> 
</xsl:template>

<xsl:template name="faqgroup">
  <xsl:param name="groupentry" />
  <xsl:param name="width" />
    <td> 
      <xsl:if test="$width &gt; 0">
        <xsl:attribute name="width"><xsl:value-of select="$width"/>%</xsl:attribute>
      </xsl:if>
      <xsl:value-of select="$groupentry/@title" />
      <div class="more">      
        <a href="{$groupentry/@href}" class="more">mehr</a>
      </div>
    </td>
</xsl:template>

<xsl:template name="faqentrytable">
  <xsl:variable name="content" select="/page/content/topic/faq"/>
  <xsl:variable name="entrycont"  select="$content/groups/group/entries/entry[./question/text()]" />
  <xsl:for-each select="$entrycont">
    <xsl:call-template name="faqentry">  
      <xsl:with-param name="entryentry" select="."/>
    </xsl:call-template>  
  </xsl:for-each>
</xsl:template>

<xsl:template name="faqentry">
  <xsl:param name="entryentry"/>
    <div class="faqEntry">      
      <div class="question">
        <xsl:value-of select="$entryentry/question" disable-output-escaping="yes"/>
      </div>
      <div class="answer">
        <xsl:value-of select="$entryentry/answer" disable-output-escaping="yes"/>
      </div>
      <xsl:call-template name="space"/>
      <div class="more">      
        <a href="{$entryentry/@href}" class="more">mehr</a>
      </div>
    </div>  
</xsl:template>

</xsl:stylesheet>
