<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">

<xsl:template name="content_quiz">
  <xsl:variable name="content" select="/page/content/topic" />
  <xsl:variable name="language" select="/page/meta/metatags/language" />
  <h1 class="quizTitle"><xsl:value-of select="$content/title" disable-output-escaping="yes"/></h1>
  <xsl:if test="$content/teaser != ''">
    <div class="quizDescription">
      <xsl:value-of select="$content/teaser" disable-output-escaping="yes"/>
    </div>
  </xsl:if>
  <xsl:if test="$content/quiz/question">
    <xsl:call-template name="content_quiz_form">
      <xsl:with-param name="quiz" select="$content/quiz" />
      <xsl:with-param name="language" select="$language" />
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="$content/quiz/summary">
    <xsl:call-template name="content_quiz_summary">
      <xsl:with-param name="quiz" select="$content/quiz" />
      <xsl:with-param name="language" select="$language" />
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="content_quiz_summary">
  <xsl:param name="quiz" />
  <xsl:param name="language" />

  <div class="quizSummary">
    <h2>
      <xsl:choose>
        <xsl:when test="$language = 'de_DE'">Auswertung</xsl:when>
        <xsl:otherwise>Summary</xsl:otherwise>
      </xsl:choose>
    </h2>
    <ol>
    <xsl:for-each select="$quiz/summary/question">
      <li class="quizQuestion">
        <xsl:if test="position() = 1">
          <xsl:attribute name="class">quizFirstQuestion</xsl:attribute>
        </xsl:if>
          <div>
            <xsl:if test="reply/@correct = 1">
              <xsl:attribute name="class">correctAnswer</xsl:attribute>
            </xsl:if>
            <xsl:if test="reply/@correct != 1">
              <xsl:attribute name="class">wrongAnswer</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="text/text()" />
          </div>
        <div class="quizAnswer">
          <xsl:choose>
            <xsl:when test="$language = 'de_DE'">Ihre Antwort war:<xsl:text> </xsl:text></xsl:when>
            <xsl:otherwise>Your answer was:<xsl:text> </xsl:text></xsl:otherwise>
          </xsl:choose>
          <span><xsl:text>"</xsl:text><xsl:value-of select="given_answer/text()" /><xsl:text>"</xsl:text></span>
        </div>
        <div class="quizReply"><xsl:value-of select="reply/text()" /></div>
      </li>
    </xsl:for-each>
    </ol>
  </div>
</xsl:template>

<xsl:template name="content_quiz_form">
  <xsl:param name="quiz" />
  <xsl:param name="language" />
  <h2 class="quizQuestion">
    <xsl:value-of select="$quiz/question/@number" /><xsl:text>. </xsl:text>
    <xsl:value-of select="$quiz/question/text()" />
  </h2>
  <form class="quizForm" action="{$quiz/@action}" method="post">
    <xsl:for-each select="$quiz/answer">
      <div class="answer">
        <input type="radio" name="{@fieldname}" value="{@id}" id="quiz_{@id}" /><xsl:text> </xsl:text>
         <label for="quiz_{@id}"><xsl:value-of select="."/></label>
      </div>
    </xsl:for-each>
    <div class="button">
      <input class="submitButton" type="submit">
        <xsl:attribute name="value">
          <xsl:choose>
            <xsl:when test="$quiz/question/@last = 1">
              <xsl:choose>
                <xsl:when test="$language = 'de_DE'">fertig</xsl:when>
                <xsl:otherwise>finish</xsl:otherwise>
              </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
              <xsl:choose>
                <xsl:when test="$language = 'de_DE'">weiter</xsl:when>
                <xsl:otherwise>continue</xsl:otherwise>
              </xsl:choose>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </input>
    </div>
    <br clear="all" class="clear" />
  </form>
</xsl:template>

</xsl:stylesheet>