<?php
/**
* XSLT output filter
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: filter_xslt.php,v 1.15 2006/05/19 12:24:43 rekowski Exp $
*/

/**
* Basic class output filters
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_outputfilter.php');

define('PAPAYA_PATH_XSLTFILES', PAPAYA_PATH_TEMPLATES.'xslt/');

/**
* XSLT output filter
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class filter_xslt extends base_outputfilter {

  var $editFields = array(
    'xslfile'=> array ('XSL stylesheet', 'isFile', TRUE, 'filecombo', array(PAPAYA_PATH_XSLTFILES, '/^\w+\.xsl$/i'), ''),
    'fullpage' => array('Full page', 'isNum', TRUE, 'yesno', '', '', 0)
  );

  var $errorMessage = '';

  function parsePage(&$topic, &$layout) {
    $layout->setXSL(PAPAYA_PATH_XSLTFILES.$this->data['xslfile']);
    return $layout->xhtml();
  }

  function parseBox(&$topic, &$box, &$xmlString) {
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_xsl.php');
    $xsl = &new papaya_xsl(PAPAYA_PATH_XSLTFILES.$this->data['xslfile']);
    $xsl->setParam('PAGE_TODAY', date('Y-m-d H:i:s'));
    $xsl->setParam('PAGE_THEME_PATH', PAPAYA_PATH_WEB.'papaya-themes/'.PAPAYA_LAYOUT_THEME.'/');
    $xsl->setParam('PAGE_WEB_PATH', PAPAYA_PATH_WEB);
    $xsl->setParam('PAGE_LANGUAGE', $topic->currentLanguage['lng_short']);

    if (FALSE === strpos($xmlString, '<?xml')) {
      if (defined('PAPAYA_LATIN1_COMPATIBILITY') && PAPAYA_LATIN1_COMPATIBILITY) {
        include_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');
        $xmlString = papaya_strings::ensureUTF8($xmlString);
      }
      $xsl->xmlData = papaya_strings::entityToXML('<'.'?xml version="1.0" encoding="utf-8" ?'.'>'.$xmlString);
    } else {
      $xsl->xmlData = papaya_strings::entityToXML($xmlString);
    }
    if (!($result = $xsl->parse())) {
      $this->errorMessage = $xsl->getLastErrorHTML();
      return FALSE;
    } else {
      return $result;
    }
  }

  function checkConfiguration($page = TRUE) {
    if (isset($this->data['xslfile']) && trim($this->data['xslfile']) != '') {
      if (file_exists(PAPAYA_PATH_XSLTFILES.$this->data['xslfile'])) {
        return TRUE;
      } else {
        $this->errorMessage = 'XSL file "'.$this->data['xslfile'].'" not found.';
      }
    } else {
        $this->errorMessage = 'No XSL file specified.';
      return FALSE;
    }
  }

}
?>
