<?php
/**
* Date calendar basic functionality
*
* @package module_calendar
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_calendar.php,v 1.4 2006/04/26 10:28:59 weinert Exp $
*/

/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic cass check conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
  * Date calendar basic functionality
  *
  * @author Thomas Weinert <weinert@papaya-cms.com>
  * @package module_calendar
  */
class base_calendar extends base_db {

  var $tableTopics = "";
  var $tableRegulars = "";
  var $tableRegignore = "";
  var $tableModules = "";

  var $fieldsBase = ' topic_id, topic_start, topic_end, author_id, topic_state';
  var $fieldsDetail = ' topic_id, topic_start, topic_end, topic_state, group_id, topic_title, topic_data, topic_module_guid, topic_date, author_id, regular_id ';

  var $secOfDay = 86400;

  var $paramName;
  var $params;
  var $baseLink;
  var $authorId = '';

  var $dates;
  var $topics;
  var $selectedDays;
  var $selectedTopic;

  var $mode = '';
  var $selectedDate;

  var $dateFrom = NULL;
  var $dateTo = NULL;

  var $startMonday = TRUE;
  var $showEdit = FALSE;

  var $wdaysShort = array('Mo','Tu','We','Th','Fr','Sa','So');
  var $wdaysLong = array('Monday','Thuesday','Wednesday','Thursday','Friday','Saturday','Sunday');
  var $monthsLong = array('January', 'February', 'March', 'April', 'May', 'June', 'July',
                          'August', 'September', 'October', 'November', 'December');

  var $tplQuickinfo = '%s date(s)';
  var $tplDatestr = 'm/d/Y';

  var $tplMonthprev = 'Previous month';
  var $tplMonthnext = 'Next month';
  var $tplMonthactual = 'Actual month';

  var $tplChangelink = 'Change';
  var $tplAddlink = 'New date';
  var $tplCopylink = 'Copy date';
  var $tplAddlinkReg = 'New regulary date';
  var $tplDellink = 'Delete';

  var $tplCapTitle = 'Title';
  var $tplCapDate = 'Date (text)';
  var $tplCapFrom = 'From (Y-m-d)';
  var $tplCapTo = 'To (Y-m-d)';
  var $tplCapState = 'State';
  var $tplCapData = 'Text';
  var $tplSavecaption = 'Save';
  var $tplDelcaption = 'Delete';

  var $tplDefaulttitle = 'New date';

  var $tplMsgsaved = 'Changes saved.';
  var $tplDelconfirm = 'Delete this date?';

  var $tplInputsize = 30;
  var $tplInputcols = 35;
  var $tplInputrows = 10;

  /**
  * Base event
  *
  * @access public
  */
  function __construct() {
    $this->tableTopics = PAPAYA_DB_TABLEPREFIX.'_calendar';
    $this->tableRegulars = PAPAYA_DB_TABLEPREFIX.'_calreg';
    $this->tableRegignore = PAPAYA_DB_TABLEPREFIX.'_calreg_ignore';
    $this->tableModules = PAPAYA_DB_TBL_MODULES;
  }

  /**
  * Initialize
  *
  * @param string $paramName optional, default value 'cal'
  * @param string $userId optional, default value ''
  * @access public
  */
  function initialize($paramName = 'cal',$userId='') {
    $this->paramName = $paramName;
    $this->authorId = $userId;
    $this->initializeParams();
    if ($this->params['date'] > 0) {
      $this->selectedDate = $this->parseDateToArray($this->params['date']);
    } else {
      $this->selectedDate = $this->parseDateToArray(time());
    }
  }

  /**
  * Execute - basic funtion for handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'day' :
        $this->mcalClickDay($this->params['newdate']);
        break;
      case 'wday' :
        $this->mcalClickWeekDay($this->params['wday']);
        break;
      case 'week' :
        $this->mcalClickWeek($this->params['week']);
        break;
      case 'month' :
        $this->mcalClickMonth($this->params['month']);
        break;
      case 'del' :
        if ($this->showEdit) {
          $this->mcalClickDelete($this->params['topic']);
        }
        break;
      case 'add' :
        if ($this->showEdit) {
          $this->params['topic'] = $this->add();
        }
      case 'copy' :
        if ($this->showEdit) {
          $this->params['topic'] = $this->copy();
        }
      case 'topic' :
        if ($this->showEdit) {
          $this->mcalClickEdit($this->params['topic']);
          $row = $this->prepareInput($this->loadTopic($this->selectedTopic));
          if ($this->params['save']) {
            if ($this->checkInput($row)) {
              if ($this->save($row)) {
                $this->addMsg(MSG_INFO, "Daten gespeichert.");
              }
            }
          }
        }
        break;
    }
  }

  /**
  * Load modules list
  *
  * @access public
  */
  function loadModulesList() {
    unset($this->cronModules);
    unset($this->contentModules);
    $sql = "SELECT module_guid, module_type, module_class, module_path, module_file, module_title
            FROM %s
            WHERE ((module_type = 'time') OR (module_type = 'event')) AND module_active = 1";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModules)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['module_type'] == 'time') {
          $this->cronModules[$row['module_guid']] = $row;
        } else {
          $this->contentModules[$row['module_guid']] = $row;
        }
      }
    }
  }

  /**
  * Load
  *
  * @param integer $min
  * @param mixed $max optional, default value NULL
  * @access public
  * @return boolean
  */
  function load($min,$max=NULL) {
    unset($this->dates);
    if (!isset($max)) {
      $max = $min;
    }
    $this->dateFrom = $this->parseDateToArray($min);
    $this->dateTo   = $this->parseDateToArray($max);
    if (!$this->showEdit) {
      $addFilter = 'AND topic_state = 2';
    } else {
      $addFilter = '';
    }
    $sql = "SELECT {$this->fieldsBase}
            FROM {$this->tableTopics}
            WHERE ((topic_start >= '$min' AND topic_start <= '$max')
              OR (topic_end >= '$min' AND topic_end <= '$max')
              OR (topic_start <= '$min' AND topic_end >= '$max'))
          $addFilter";
  if ($res = $this->databaseQuery($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->dates[$row['topic_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load details
  *
  * @param integer $min
  * @param mixed $max optional, default value NULL
  * @access public
  * @return boolean
  */
  function loadDetails($min,$max=NULL) {
    unset($this->topics);
    if (!isset($max)) {
      $max = $min;
    }
    $this->dateFrom = $this->parseDateToArray($min);
    $this->dateTo   = $this->parseDateToArray($max);
    if (!$this->showEdit) {
      $addFilter = 'AND topic_state = 2';
    } else {
      $addFilter = '';
    }
    $sql = "SELECT {$this->fieldsDetail}
            FROM {$this->tableTopics}
            WHERE ((topic_start >= '$min' AND topic_start <= '$max')
              OR (topic_end >= '$min' AND topic_end <= '$max')
              OR (topic_start <= '$min' AND topic_end >= '$max'))
          $addFilter
            ORDER BY topic_start, topic_title, topic_id";
    if ($res = $this->databaseQuery($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row['topic_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load topic
  *
  * @param integer $topic
  * @access public
  * @return array
  */
  function loadTopic($topic) {
    $result = FALSE;
    $this->dateFrom = array();
    $this->dateTo   = array();
    if ($topic > 0) {
      $sql = "SELECT {$this->fieldsDetail}
              FROM {$this->tableTopics}
              WHERE topic_id = $topic";
      if ($res = $this->databaseQuery($sql)) {
        if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $row['topic_startf'] = date('Y-m-d', $row['topic_start']);
          $row['topic_endf'] = date('Y-m-d', $row['topic_end']);
          $result = $row;
          $this->dateId = $row['topic_id'];
        }
      }

    }
    $this->loadedTopic = $result;
    return $result;
  }

  /**
  * Load details by days
  *
  * @param array $days
  * @access public
  * @return boolean
  */
  function loadDetailsByDays($days) {
    unset($this->topics);
    if (isset($days) && is_array($days)) {
      $filter = '';
      foreach ($days as $day) {
        $filter .= sprintf("OR (topic_start <= '$day' AND topic_end >= '$day') \r\n");
      }
      $filter = preg_replace('/^OR/','',$filter);
    if (!$this->showEdit) {
      $addFilter = 'AND topic_state = 2';
    } else {
      $addFilter = '';
    }
    if ($filter != '') {
      $filter = sprintf("WHERE (%s) %s", $filter, $addFilter);
    }
      $sql = "SELECT {$this->fieldsDetail}
              FROM {$this->tableTopics}
              $filter
              ORDER BY topic_start, topic_title, topic_id";
      if ($res = $this->databaseQuery($sql)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->topics[$row['topic_id']] = $row;
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Save
  *
  * @param array $row
  * @access public
  * @return boolean
  */
  function save($row) {
    $result = FALSE;
    if ($row['topic_id'] > 0) {
      $data = array(
        'topic_title' => $row['topic_title'],
        'topic_start' => $row['topic_start'],
        'topic_end' => $row['topic_end'],
        'topic_state' => $row['topic_state'],
        'topic_date' => $row['topic_date'],
        'topic_module_guid' => $row['topic_module_guid'],
        'author_id' => $this->authorId,
      );
      if ($this->databaseUpdateRecord($this->tableTopics, $data, 'topic_id', (int)$row['topic_id'])) {
        $this->msgType = MSG_INFO;
        $this->msg = $this->_gt($this->tplMsgsaved);
        $result = TRUE;
      $this->logMsg(0,4, $this->_gt('Calendar date modified.'),
        sprintf($this->_gt('Calendar date "%s" (%s) modified.'), htmlspecialchars($row['topic_title']),
          htmlspecialchars($row['topic_date'])));
      }
    }
    return $result;
  }

  /**
  * Add
  *
  * @access public
  * @return mixed
  */
  function add() {
    $data = array(
      'topic_title'=>$this->_gt($this->tplDefaulttitle),
      'topic_start'=>$this->selectedDate['date'],
      'topic_end'  =>$this->selectedDate['date']);
    return $this->databaseInsertRecord($this->tableTopics, 'topic_id', $data);
  }

  /**
  * Copy
  *
  * @access public
  * @return mixed
  */
  function copy() {
    $topicId = (int)$this->params['topic'];
    if ($topicId > 0) {
      $sql = "SELECT {$this->fieldsDetail}
              FROM {$this->tableTopics}
              WHERE topic_id = $topicId";
      if ($res = $this->databaseQuery($sql)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $data = array(
            'topic_title'=>$row['topic_title'],
            'topic_start'=>$row['topic_start'],
            'topic_end'=>$row['topic_end'],
            'topic_state'=>1,
            'topic_date'=>$row['topic_date'],
            'topic_data'=>$row['topic_data'],
            'author_id'=>$this->authorId);
          if ($new = $this->databaseInsertRecord($this->tableTopics, 'topic_id', $data)) {
            $this->selectedTopic=$new;
            return $new;
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Delete
  *
  * @param integer $topic
  * @access public
  * @return mixed
  */
  function delete($topic) {
    if ($topic > 0) return $this->databaseDeleteRecord($this->tableTopics, 'topic_id', (int)$topic);
    return FALSE;
  }

  /**
  * Get border days
  *
  * @param integer $date
  * @access public
  * @return array
  */
  function getBorderDays($date) {
    $firstDay = mktime(0,0,0,$date['month'],1,$date['year']);
    $lastDay = mktime(0,0,0,$date['month'],date('t',$date['date']),$date['year']);
    return array($firstDay,$lastDay);
  }

  /**
  * Get days of month
  *
  * @access public
  */
  function getDaysOfMonth() {
    $this->selectedDays = array();
    list($firstDay,$lastDay) = $this->getBorderDays($this->selectedDate);
    if ($this->load($firstDay,$lastDay)) {
      if (isset($this->dates) && is_array($this->dates)) {
        foreach($this->dates as $row) {
          for ($day = (int)@$row['topic_start'];
                $day <= (int)@$row['topic_end'];) {
            $dayArray = $this->parseDateToArray($day);
            @$this->selectedDays[$dayArray['year'].$dayArray['month'].$dayArray['day']]++;
            $day = mktime(0,0,0,$dayArray['month'], $dayArray['day']+1, $dayArray['year']);
          }
        }
      }
    }
  }

  /**
  * Date to array
  *
  * @param integer $date
  * @access public
  * @return array
  */
  function parseDateToArray($date) {
    $parsed = getdate($date);
    $result['date'] = $date;
    $result['day'] = str_pad($parsed['mday'], 2, '0', STR_PAD_LEFT);
    $result['month'] = str_pad($parsed['mon'], 2, '0', STR_PAD_LEFT);
    $result['year'] = $parsed['year'];
    $result['wday'] = $parsed['wday'];
    return $result;
  }

  /**
  * Get week day
  *
  * @param integer $date
  * @access public
  * @return string
  */
  function getWDay($date) {
    $result = strftime('%u',$date);
    return $result;
  }

  /**
  * Week day to string
  *
  * @param integer $wday
  * @param boolean $shortStr optional, default value FALSE
  * @access public
  * @return string
  */
  function wDay2Str($wday,$shortStr=FALSE) {
    $result = $this->wDay2Idx($wday);
    if ($shortStr) {
      return $this->wdaysShort[$result-1];
    } else {
      return $this->wdaysLong[$result-1];
    }
  }

  /**
  * Week day to index
  *
  * @param integer $wday
  * @access public
  * @return integer
  */
  function wDay2Idx($wday) {
    if (!$this->startMonday) {
      $result = --$wday;
      if ($result == 0) {
        $result = 7;
      }
      return $result;
    } else {
      return $wday;
    }
  }

  /**
  * Prepare magic quotes
  *
  * @param string $str
  * @access public
  * @return string
  */
  function prepareMagicQuotes($str) {
    if (get_magic_quotes_gpc()==1) {
      return $str;
    } else {
      return AddSlashes($str);
    }
  }

  /**
  * Build link
  *
  * @param string $caption
  * @param string $url optional, default value ''
  * @param mixed $params optional, default value NULL
  * @param string $target optional, default value ''
  * @param string $class optional, default value ''
  * @param string $title optional, default value ''
  * @param boolean $htmlTag optional, default value TRUE
  * @access public
  * @return string
  */
  function buildLink($caption,$url='',$params=NULL,$target='',$class='',$title='',$htmlTag=TRUE) {
    if ($url == '') {
      $url = $this->baseLink;
    }
    if (strpos($url,'?')) {
      $queryString = sprintf('&amp;%s[%s]=%s',$this->paramName, 'date', $this->selectedDate['date']);
    } else {
      $queryString = sprintf('?%s[%s]=%s',$this->paramName, 'date', $this->selectedDate['date']);
    }
    if (isset($params) && is_array($params)) {
      foreach($params as $key=>$val) {
        $queryString .= sprintf('&amp;%s[%s]=%s',$this->paramName,urlencode($key),urlencode($val));
      }
    }
    if ($target == '') {
      $target = '_self';
    }
    if ($htmlTag) {
      if ($class == '') {
        $result = sprintf('<a href="%s%s" target="%s" title="%s"><![CDATA[%s]]></a>',
          $url, $queryString, $target, $title, $caption);
      } else {
        $result = sprintf('<a href="%s%s" target="%s" class="%s" title="%s"><![CDATA[%s]]></a>',
          $url, $queryString, $target, $class, $title, $caption);
      }
    } else {
      $result = $url.$queryString;
    }
    return $result;
  }


  /**
  * Click week day
  *
  * @param string $day
  * @access public
  */
  function mcalClickWeekday($day) {
    if ($day > 0) {
      $this->mode = 'wday';
      $this->selectedDate['wday'] = $day;
    }
  }

  /**
  * Click Day
  *
  * @param integer $newDate
  * @access public
  */
  function mcalClickDay($newDate) {
    if ($newDate > 0) {
      $this->mode = 'day';
      $this->selectedDate = $this->parseDateToArray($newDate);
    }
  }

  /**
  * Click week
  *
  * @param integer $week
  * @access public
  */
  function mcalClickWeek($week) {
    if ($week > 0) {
      $this->mode = 'week';
      $this->selectedDate['week'] = $week;
    }
  }

  /**
  * Click month
  *
  * @param string $month
  * @access public
  */
  function mcalClickMonth($month) {
    $this->mode = 'month';
    $today = $this->selectedDate;
    if ($today['month'] == 0) {
      $today['month'] = 12;
      $today['year']--;
    } else if ($today['month'] == 13) {
      $today['month'] = 1;
      $today['year']++;
    } else {
      $today['month'] = $month;
    }
    if ($today['day'] <= 28) {
      $this->selectedDate = $this->parseDateToArray(mktime(0,0,0));
    } else {
      $dayCount = date('t',mktime(0,0,0,$today['month'],1,$today['year']));
      if ($dayCount < $today['day']) {
        $today['day'] = $dayCount;
      }
    }
    $this->selectedDate = $this->parseDateToArray(mktime(0,0,0,$today['month'],$today['day'],$today['year']));
  }

  /**
  * Click topic
  *
  * @param integer $topic
  * @access public
  */
  function mcalClickTopic($topic) {
    $this->selectedTopic = $topic;
    $this->mode = 'topic';
  }

  /**
  * Click edit
  *
  * @param integer $topic
  * @access public
  */
  function mcalClickEdit($topic) {
    $this->selectedTopic = $topic;
    $this->mode = 'edit';
  }

  /**
  * Click delete
  *
  * @param integer $topic
  * @access public
  */
  function mcalClickDelete($topic) {
    $this->selectedTopic = $topic;
    if (isset($this->params['delete']) && $this->params['delete']) {
      $this->delete($topic);
      $this->mode = 'day';
    } else {
      $this->mode = 'del';
    }
  }

  /**
  * Show month table
  *
  * @access public
  * @return string
  */
  function showMonthTable() {
    $this->getDaysOfMonth();
    $today = $this->parseDateToArray($this->selectedDate['date']);
    $first = $this->parseDateToArray(mktime(0,0,0,$today['month'],1,$today['year']));
    $monthDay = date('t',$this->selectedDate['date']);

    $result = '<monthcalendar>'."\r\n";

    $result .=  '<monthnav>'."\r\n";

    $m = $today['month'];
    if ($m < 0) $m = 12;
    if ($m == 1) {
      $mPrev = 12;
    } else {
      $mPrev = $m - 1;
    }
    if ($m == 12) {
      $mNext = 1;
    } else {
      $mNext = $m + 1;
    }
    $href = $this->buildLink($this->_gt($this->tplMonthprev),'',
      array('cmd'=>'month','month'=>$today['month']-1),'','mcalmonth', '', FALSE);
    $result .= sprintf('<month position="%s" title="%s" hint="%s" href="%s" month="%d" />'.LF,
      'prior', $this->_gt($this->monthsLong[$mPrev-1]),
      $this->_gt($this->monthsLong[$mPrev-1]).' - '.$this->_gt($this->tplMonthprev),
      $href, $mPrev);

    $href = $this->buildLink($this->monthsLong[$today['month']-1].' '.$today['year'],'',
      array('cmd'=>'month','month'=>$today['month']),'','mcalmonth', '', FALSE);
    $result .= sprintf('<month position="%s" title="%s" hint="%s" href="%s" month="%d" year="%d"/>'.LF,
      'actual', $this->_gt($this->monthsLong[$today['month']-1]),
        $this->_gt($this->monthsLong[$today['month']-1]).' - '.$this->_gt($this->tplMonthactual),
        $href, $today['month'], $today['year']);

    $href = $this->buildLink($this->_gt($this->tplMonthnext),'',
      array('cmd'=>'month','month'=>$today['month']+1),'','mcalmonth', '', FALSE);
    $result .= sprintf('<month position="%s" title="%s" hint="%s" href="%s" month="%d" />'.LF,
      'next', $this->_gt($this->monthsLong[$mNext-1]),
      $this->_gt($this->monthsLong[$mNext-1]).' - '.$this->_gt($this->tplMonthnext),
      $href, $mNext);

    $result .= '</monthnav>';


    $result .= '<weekdays>'."\r\n";
    for ($i=1;$i<=7;$i++) {
      $href = $this->buildLink($this->wDay2Str($i,TRUE),'',
        array('cmd'=>'wday','wday'=>$this->wDay2Idx($i)),'','mcalwday', '', FALSE);
      $result .= sprintf('<wday href="%s" title="%s" hint="%s" wday="%d" />'."\r\n",
        $href, $this->_gt($this->wDay2Str($i,TRUE)), $this->_gt($this->wDay2Str($i,FALSE)), $i);
    }
    $result .= '</weekdays>'."\r\n";

    $result .= '<weeks>'."\r\n";
    $y = strftime('%V',mktime(0,0,0,$today['month'],1,$today['year']));
    $href = $this->buildLink($y,'', array('cmd'=>'week','week'=>$y),'','mcalwday', '', FALSE);
    $result .= sprintf('<week no="%s" href="%s">'.LF, $y, $href);
    if ($this->startMonday) {
      $offset = $first['wday'];
    } else {
      $offset = $first['wday']+1;
      if ($offset > 7) {
        $offset = 1;
      }
    }
    if ($offset > 1) {
      $result .= sprintf('<day type="spacer" dayspan="%s" />'."\r\n",$offset-1);
    } elseif ($offset == 0) {
      $result .= sprintf('<day type="spacer" dayspan="%s" />'.LF, 6);
    }
    for ($x=1;$x<=$monthDay;$x++) {
      if (($x == $today['day']) && ($this->mode == 'day')) {
        $isSelected = ' selected="selected"';
      } else {
        $isSelected = '';
      }
      $thisDay =  mktime(0,0,0,$today['month'],$x,$today['year']);
      $href = $this->buildLink($x, '', array('cmd'=>'day','newdate'=>$thisDay),'','mcalday', '', FALSE);
      $thisDate = $today['year'].$today['month'].(($x < 10) ? '0'.$x : $x);
      if (isset($this->selectedDays[$thisDate])) {
        $result .= sprintf('<day type="filled" href="%s" hint="%s" dates="%s" %s>%s</day>'.LF,
          $href, sprintf($this->_gt($this->tplQuickinfo), $this->selectedDays[$thisDate]),
          $this->selectedDays[$thisDate], $isSelected, $x);
      } elseif ($this->showEdit) {
        $result .= sprintf('<day type="empty" href="%s" %s>%s</day>'.LF,
          $href, $isSelected, $x);
      } else {
        $result .= sprintf('<day type="empty" %s>%s</day>'.LF,
          $isSelected, $x);
      }
      if (!(($x+$offset-1) % 7) && ($x != $monthDay)) {
        $result .= '</week>'.LF;
        if ($this->startMonday) {
          $y = strftime('%V',mktime(0,0,0,$today['month'],$x+1,$today['year']));
        } else {
          $y = strftime('%V',mktime(0,0,0,$today['month'],$x+2,$today['year']));
        }
        $href = $this->buildLink($y,'', array('cmd'=>'week','week'=>$y),'','mcalwday', '', FALSE);
        $result .= sprintf('<week no="%s" href="%s">'.LF, $y, $href);
      }
    }
    $end = 7 - (($x+$offset-1) % 7) + 1;
    while ($end > 7) {
      $end -= 7;
    }
    if (($end > 0) && ($end < 7)) {
      $result .= sprintf('<day type="spacer" dayspan="%s" />'."\r\n",$end);
    }
    $result .= '</week>'."\r\n";
    $result .= '</weeks>'."\r\n";

    $result .= '</monthcalendar>'."\r\n";
    return $result;
  }

  /**
  * Show detail
  *
  * @access public
  * @return string
  */
  function showDetail() {
    switch ($this->mode) {
      case 'day' :
        $result = $this->showDay();
        break;
      case 'wday' :
        $result = $this->showWeekDay();
        break;
      case 'week' :
        $result = $this->showWeek();
        break;
      case 'topic' :
        $result = $this->showTopic();
        break;
      case 'del'  :
        if (!$this->params['delete']) {
          $result = $this->showDelConfirm();
        }
        break;
      case 'edit' :
        $result = $this->showEdit($this->prepareInput($this->loadTopic($this->selectedTopic)));
        break;
      default :
        $result = $this->showMonth();
    }
    return $result;
  }

  /**
  * Show day
  *
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showDay($detail = TRUE) {
    $this->loadDetails($this->selectedDate['date']);
    return $this->showTopics(date($this->_gt($this->tplDatestr),$this->selectedDate['date']), $detail);
  }

  /**
  * Show week days
  *
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showWeekDay($detail = TRUE) {
    list($firstDay,$lastDay) = $this->getBorderDays($this->selectedDate);
    unset($weekDays);
    for ($day = $firstDay;
          $day <= $lastDay;
          $day += $this->secOfDay) {
      if ($this->getWDay($day) == $this->selectedDate['wday']) {
        $weekDays[] = $day;
      }
    }
    $this->loadDetailsByDays($weekDays);

    $headString = sprintf('%s - %s %s',
      $this->_gt($this->wDay2Str($this->selectedDate['wday'])),
      $this->_gt($this->monthsLong[$this->selectedDate['month']-1]), $this->selectedDate['year']);
    return $this->showTopics($headString, $detail);
  }

  /**
  * Show week
  *
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showWeek($detail = TRUE) {
    $selectedWeek = strftime('%V',$this->selectedDate['date']);
    $diff = ($selectedWeek - $this->selectedDate['week']) * 7 * $this->secOfDay;
    $today = $this->parseDateToArray($this->selectedDate['date']-$diff);
    $firstDay = mktime(0,0,0, $today['month'], $today['day']-$today['wday']+1,$today['year']);
    $lastDay = mktime(0,0,0,$today['month'], $today['day']-$today['wday']+7,$today['year']);
    $this->loadDetails($firstDay,$lastDay);

    $headString = sprintf('%s - %s',
                  date($this->_gt($this->tplDatestr),$firstDay),
                  date($this->_gt($this->tplDatestr),$lastDay));
    return $this->showTopics($headString, $detail);
  }

  /**
  * Show month
  *
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showMonth($detail = TRUE) {
    list($firstDay,$lastDay) = $this->getBorderDays($this->selectedDate);
    $this->loadDetails($firstDay,$lastDay);
    return $this->showTopics($this->_gt($this->monthsLong[$this->selectedDate['month']-1]).' '.$this->selectedDate['year'], $detail);
  }


  /**
  * Show topics
  *
  * @param string $headString
  * @param boolean $detail optional, default value TRUE
  * @access public
  * @return string
  */
  function showTopics($headString, $detail = TRUE) {
    $this->loadModulesList();
    $result = sprintf('<dategroup mode="%s">'."\r\n",
      $this->mode);
    $result .= @sprintf('<date-selected year="%d" month="%d" day="%d" wday="%d" />'."\r\n",
      $this->selectedDate['year'], $this->selectedDate['month'], $this->selectedDate['day'],
      $this->selectedDate['wday']);
    $result .= @sprintf('<date-from year="%d" month="%d" day="%d" wday="%d" />'."\r\n",
      $this->dateFrom['year'], $this->dateFrom['month'], $this->dateFrom['day'],
      $this->dateFrom['wday']);
    $result .= @sprintf('<date-to year="%d" month="%d" day="%d" wday="%d" />'."\r\n",
      $this->dateTo['year'], $this->dateTo['month'], $this->dateTo['day'],
      $this->dateTo['wday']);
    $result .= sprintf('<title>%s</title>'."\r\n", htmlspecialchars($headString));
    if (isset($this->topics) && is_array($this->topics)) {
      foreach($this->topics as $row) {
        $result .= '<date>'."\r\n";
        $result .= sprintf('<datetitle>%s</datetitle>'."\r\n",
          $this->buildLink($row['topic_title'], '', array('cmd'=>'topic','topic'=>$row['topic_id']), '', 'mcaldtitle'));
        $result .= sprintf('<datestr><![CDATA[%s]]></datestr>'."\r\n", $row['topic_date']);

        $content = (strpos($row['topic_data'], '<data>') === 0) ? $row['topic_data'] :
          sprintf('<data><data-element name="text"><![CDATA[%s]]></data-element></data>',
            $row['topic_data']);

        $str = '';
        if (isset($this->contentModules[$row['topic_module_guid']])) {
          $module = $this->contentModules[$row['topic_module_guid']];
          $moduleObject = &$this->createPluginObject($module['module_path'].$module['module_file'], $module['module_class'],
              $module['module_guid'], $this, $content);

          if (isset($moduleObject) && is_object($moduleObject)) {
            if (method_exists($moduleObject, 'getParsedTeaser')) {
              $str = $moduleObject->getParsedTeaser();
            } elseif (method_exists($moduleObject, 'getParsedData')) {
              $str = $moduleObject->getParsedData();
            }
          }
        }

        $result .= sprintf('<datedetail>%s</datedetail>'."\r\n", $str);
        $result .= '</date>'."\r\n";
      }
    }
    $result .= '</dategroup>'."\r\n";
    return $result;
  }

  /**
  * Show topic
  *
  * @access public
  * @return string
  */
  function showTopic() {
    $row = $this->loadTopic($this->selectedTopic);
    $this->loadModulesList();
    if (isset($row) && is_array($row)) {
      if ($this->showEdit) {
        $result = $this->showEdit($row);
      } else {
        $result = '<date>';
        $result .= sprintf('<datetitle><![CDATA[%s]]></datetitle>'."\r\n", $row['topic_title']);
        $result .= sprintf('<datestr><![CDATA[%s]]></datestr>'."\r\n", $row['topic_date']);

        $content = (strpos($row['topic_data'], '<data>') === 0) ? $row['topic_data'] :
          sprintf('<data><data-element name="text"><![CDATA[%s]]></data-element></data>',
            $row['topic_data']);

        if (isset($this->contentModules[$row['topic_module_guid']])) {
          $module = $this->contentModules[$row['topic_module_guid']];
          $moduleObject = &$this->createPluginObject($module['module_path'].$module['module_file'], $module['module_class'],
              $module['module_guid'], $this, $content);
        }
        if (isset($moduleObject) && is_object($moduleObject) && method_exists($moduleObject, 'getParsedData')) {
          $str = $moduleObject->getParsedData();
        } else {
          $str = '';
        }

        $result .= sprintf('<datedetail module="%s" guid="%s">%s</datedetail>'."\r\n",
          htmlspecialchars($module['module_class']), htmlspecialchars($module['module_guid']), $str);
        $result .= '</date>'."\r\n";
      }
    }
    return $result;
  }

  /**
  * Show delete confirm dialog
  *
  * @access public
  * @return string
  */
  function showDelConfirm() {
    $row = $this->loadTopic($this->selectedTopic);
    $result = sprintf('<msgdialog action="%s" width="100%%" type="question">'."\r\n",
      $this->buildLink($this->_gt($this->tplChangelink),'',array('cmd'=>'del','topic'=>$row['topic_id']),'','','',FALSE));
    if (isset($row) && is_array($row)) {
      $result .= '<message>'."\r\n";
      $result .= sprintf('Soll der Termin "%s" gelscht werden?'."\r\n", htmlspecialchars($row['topic_title']));
      $result .= '</message>'."\r\n";
      $result .= sprintf('<dlgbutton name="%s[delete]" value="%s" />'."\r\n", $this->paramName, $this->_gt($this->tplDelcaption));
    }
    $result .= '</msgdialog>'."\r\n";
    return $result;
  }

  /**
  * Prepare Input
  *
  * @param array $row
  * @access public
  * @return array
  */
  function prepareInput($row) {
    $data = $row;
    if (isset($data) && is_array($data)) {
      if (isset($this->params['topic_title'])) {

        $data['topic_title'] = $this->prepareMagicQuotes($this->params['topic_title']);
      }
      if (isset($this->params['topic_date'])) {
        $data['topic_date'] = $this->prepareMagicQuotes($this->params['topic_date']);
      }
      if (isset($this->params['topic_data'])) {
        $data['topic_data'] = $this->prepareMagicQuotes($this->params['topic_data']);
      }
      if (isset($this->params['topic_state'])) {
        $data['topic_state'] = (int)$this->params['topic_state'];
      }
      if (isset($this->params['topic_startf'])) {
        if ($this->params['topic_startf'] == '') {
          $data['topic_start'] = time();
        } else {
          $date = explode('-',$this->params['topic_startf']);
          $data['topic_start'] = mktime(0,0,0,$date[1],$date[2],$date[0]);
        }
      }
      $data['topic_startf'] = date('Y-m-d',$row['topic_start']);
      if (isset($this->params['topic_endf'])) {
        $date = explode('-',$this->params['topic_endf']);
        $data['topic_end'] = mktime(0,0,0,$date[1],$date[2],$date[0]);
        if ($data['topic_end'] < $data['topic_start']) {
          $data['topic_end'] = $data['topic_start'];
        }
      }
      $data['topic_endf'] = date('Y-m-d',$data['topic_end']);
      return $data;
    } else {
      return $row;
    }
  }

  /**
  * Show edit
  *
  * @param array $row
  * @access public
  * @return string html
  */
  function showEdit($row) {
    $result = sprintf('<form action="%s" method="POST">',
      $this->buildLink($this->_gt($this->tplChangelink),'',array('cmd'=>'edit','topic'=>$row['topic_id']),'','','',FALSE));
    $result .= '<table border="0" cellpadding="0" cellspacing="0"><tr><td class="mcalborder">';
    $result .= '<table border="0" cellpadding="2" cellspacing="1" class="mcaldetail">';
    if (isset($row) && is_array($row)) {
      if (isset($this->msg)) {
        $result .= sprintf('<tr><th class="mcaldetailhead" colspan="2">%s</th></tr>',$this->msg);
      }
      $result .= sprintf('<tr><td class="mcaldetail">%s</td><td class="mcaldetail">',$this->_gt($this->tplCapTitle));
      $result .= sprintf('<input type="text" name="%s[%s]" size="%s" value="%s" /></td></tr>',
        $this->paramName, 'topic_title', $this->_gt($this->tplInputsize), htmlentities(stripslashes($row['topic_title'])));
      $result .= sprintf('<tr><td class="mcaldetail">%s</td><td class="mcaldetail">',$this->_gt($this->tplCapDate));
      $result .= sprintf('<input type="text" name="%s[%s]" size="%s" value="%s" /></td></tr>',
        $this->paramName, 'topic_date', $this->_gt($this->tplInputsize),  htmlentities(stripslashes($row['topic_date'])));
      $result .= sprintf('<tr><td class="mcaldetail">%s</td><td class="mcaldetail">',$this->_gt($this->tplCapFrom));
      $result .= sprintf('<input type="text" name="%s[%s]" size="%s" value="%s" /></td></tr>',
        $this->paramName, 'topic_startf', $this->_gt($this->tplInputsize), $row['topic_startf']);
      $result .= sprintf('<tr><td class="mcaldetail">%s</td><td class="mcaldetail">',$this->_gt($this->tplCapTo));
      $result .= sprintf('<input type="text" name="%s[%s]" size="%s" value="%s" /></td></tr>',
        $this->paramName, 'topic_endf', $this->_gt($this->tplInputsize), $row['topic_endf']);
      $result .= sprintf('<tr><td class="mcaldetail">%s</td><td class="mcaldetail">',$this->_gt($this->tplCapState));
    if ($row['topic_state'] != 2) {
        $result .= sprintf('<select name="%s[%s]">
        <option value="1" selected="selected">Angelegt</option><option value="2">verffentlicht</option></input>'.
          '</td></tr>',
          $this->paramName, 'topic_state');
    } else {
        $result .= sprintf('<select name="%s[%s]">
          <option value="1">Angelegt</option><option value="2" selected="selected">verffentlicht</option></input>'.
          '</td></tr>',
          $this->paramName, 'topic_state');
    }
      $result .= sprintf('<tr><td class="mcaldetail" colspan="2">%s</td></tr><td class="mcaldetail" colspan="2">',$this->_gt($this->tplCapData));
      $result .= sprintf('<textarea name="%s[%s]" cols="%s" rows="%s">%s</textarea></td></tr>',
        $this->paramName, 'topic_data', $this->_gt($this->tplInputcols), $this->_gt($this->tplInputrows),
        htmlentities(stripslashes($row['topic_data'])));
      $result .= sprintf('<tr><td colspan="2" align="right" class="mcaldetail"><input type="submit" class="btn" name="%s[save]" value="%s"></td></tr>', $this->paramName, $this->_gt($this->tplSavecaption));
      $result .= sprintf('<tr><td colspan="2" align="right" class="mcaldetail">%s',
        $this->showDelLink($row['topic_id']));
    } else {
      $result .= '<tr><td class="mcaldetail">Not Found!</td></tr>';
    }
    $result .= '</table>';
    $result .= '</td></tr></table>';
    $result .= '</form>';
    return $result;
  }

  /**
  * Check input
  *
  * @param array $data
  * @access public
  * @return boolean
  */
  function checkInput($data) {
    $result = TRUE;
    unset($this->errors);
    if (!checkit::isNoHTML($data['topic_title'],TRUE)) {
      $this->errors['topic_title'] = TRUE;
    }
    if (!checkit::isNoHTML($data['topic_date'],TRUE)) {
      $this->errors['topic_date'] = TRUE;
    }
    if (!checkit::isNoHTML($data['topic_data'],TRUE)) {
      $this->errors['topic_title'] = TRUE;
    }
    if (!checkit::check($data['topic_startf'],'/^\d{4}\-\d{2}\-\d{2}$/',TRUE)) {
      $this->errors['topic_startf'] = TRUE;
    }
  $data['topic_state'] = (int)$data['topic_state'];
    return $result;
  }

  /**
  * Decode ISO datetime String
  *
  * @param string $isoDateString
  * @access public
  * @return mixed integer or NULL
  */
  function decodeISODatetimeStr($isoDateString) {
    if (preg_match('#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01]))\s+((1\d)|(2[0-3])|(0?\d)):([0-6]\d)$#',
      $isoDateString, $regs)) {
      return mktime($regs[8], $regs[12], 0, $regs[2], $regs[5], $regs[1]);
    } elseif (preg_match('#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01]))\s*$#',
      $isoDateString, $regs)) {
      return mktime(0, 0, 0, $regs[2], $regs[5], $regs[1]);
    }
    return NULL;
  }

  /**
  * Show edit link
  *
  * @param integer $topic
  * @access public
  * @return string
  */
  function showEditLink($topic) {
    if ($topic > 0) {
      $result = $this->buildLink($this->_gt($this->tplChangelink),'',array('cmd'=>'edit','topic'=>$topic));
    }
    return $result;
  }

  /**
  * Show add link
  *
  * @access public
  * @return string
  */
  function showAddLink() {
    $result = $this->buildLink($this->_gt($this->tplAddlink),'',array('cmd'=>'add'));
    return $result;
  }

  /**
  * Show delete link
  *
  * @param integer $topic
  * @access public
  * @return string
  */
  function showDelLink($topic) {
    if ($topic > 0) {
      $result = $this->buildLink($this->_gt($this->tplDellink),'',array('cmd'=>'del','topic'=>$topic));
    }
    return $result;
  }
}

?>