<?php
/**
* Function class discussion
*
* @package module_forum
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_forum.php,v 1.9 2006/05/30 13:18:20 weinert Exp $
*/

/**
* database class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Function class discussion
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package module_forum
*/
class base_forum extends base_db {
  /**
  * Categories table
  * @var string $tableCategs
  */
  var $tableCategs = '';
  /**
  * Forum / boards table
  * @var string $tableBoards
  */
  var $tableBoards = '';
  /**
  * Entry table
  * @var string $tableEntries
  */
  var $tableEntries = '';
  /**
  * Surfer table
  * @var string $tableSurfers
  */
  var $tableSurfers = '';

  /**
  * Css input class
  * @var string
  */
  var $inputFieldSize = 'x-large';

  /**
  * Parameter prefix name
  * @var string
  */
  var $paramName = '';
  /**
  * Seassion parameter prefix name
  * @var string
  */
  var $sessionParamName = '';
  /**
  * Parameters
  * @var array
  */
  var $params = NULL;

  /**
  * Categories
  * @var array
  */
  var $categs = NULL;
  var $categ = NULL;

  /**
  * Category tree
  * @var array
  */
  var $categTree = NULL;
  /**
  * Boards
  * @var array
  */
  var $boards = NULL;
  /**
  * Board
  * @var array
  */
  var $board = NULL;

  /**
  * Entries
  * @var array
  */
  var $entries = NULL;
  /**
  * Entry
  * @var array
  */
  var $entry = NULL;
  /**
  * Entry tree
  * @var array
  */
  var $entryTree = NULL;

  /**
  * Threads
  * @var array
  */
  var $threads = NULL;
  /**
  * Thread id list
  * @var array
  */
  var $entryIdList = NULL;

  /**
  * Full text search ?
  * @var array
  */
  var $fullTextSearch = PAPAYA_SEARCH_BOOLEAN;
  /**
  * Cache search results
  * @var boolean
  */
  var $cacheSearchResults = FALSE;


  var $cuttedCategs = NULL;
  var $cuttedList = NULL;
  var $cuttedThreads = NULL;
  var $categsById = NULL;

  var $allowedTags = array('b', 'i', 'tt');

  /**
  * Constructor
  * @param string $paramName Name des Parameterarrays
  */
  function __construct($paramName = 'ff') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_'.$paramName;
    $this->databaseURI=PAPAYA_DB_URI;
    $this->tableCategs=PAPAYA_DB_TABLEPREFIX.'_forumcategs';
    $this->tableBoards=PAPAYA_DB_TABLEPREFIX.'_forum';
    $this->tableEntries=PAPAYA_DB_TABLEPREFIX.'_forumentries';
    $this->tableSurfers=PAPAYA_DB_TABLEPREFIX.'_surfer';
  }

  /**
  * Does category exist?
  *
  * @param integer $id category id
  * @access public
  * @return boolean
  */
  function categExists($id) {
    $sql = "SELECT count(*) FROM %s WHERE forumcat_id = '%d'";
    $params = array($this->tableCategs, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Is category empty ?
  *
  * @param integer $id category id
  * @access public
  * @return boolean
  */
  function categIsEmpty($id) {
    $sql = "SELECT count(*) FROM %s WHERE forumcat_prev = '%d'";
    $params = array($this->tableCategs, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ($row[0] == 0) {
          $sql = "SELECT count(*) FROM %s WHERE forumcat_id = '%d'";
          $params = array($this->tableBoards, $id);
          if ($res = $this->databaseQueryFmt($sql, $params)) {
            if ($row = $res->fetchRow()) {
              return ((bool)$row[0] == 0);
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Load categories
  * @return boolean
  */
  function loadCategs($id = NULL, $repair = NULL) {
    unset($this->categs);
    unset($this->categTree);
    if (isset($id)) {
      $sql = "SELECT forumcat_id, forumcat_prev, forumcat_title, forumcat_desc, forumcat_path
                FROM %s
                WHERE forumcat_prev = '%d'
                ORDER BY forumcat_title ASC, forumcat_id ASC";
    } elseif (isset($repair) && $repair == TRUE ) {
      $sql = "SELECT forumcat_id, forumcat_prev, forumcat_title, forumcat_path
                FROM %s
                WHERE forumcat_prev > -1
                ORDER BY forumcat_title ASC, forumcat_id ASC";
    } else {
      $sql = "SELECT forumcat_id, forumcat_prev, forumcat_title, forumcat_path
                FROM %s
                ORDER BY forumcat_title ASC, forumcat_id ASC";
    }
    if ($res = $this->databaseQueryFmt($sql, array($this->tableCategs, (int)$id))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->categs[(int)$row['forumcat_id']] = $row;
        $this->categTree[(int)$row['forumcat_prev']][] = (int)$row['forumcat_id'];
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load category
  *
  * @access public
  * @return boolean
  */
  function loadCateg($catId = NULL) {

    $callCateg = (isset($catId)) ? $catId : $this->params['categ_id'];

    $sql = "SELECT forumcat_id, forumcat_desc, forumcat_path
              FROM %s
              WHERE forumcat_id = '%d'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableCategs,$callCateg ))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->categs[(int)$row['forumcat_id']]['forumcat_desc'] = $row['forumcat_desc'];
      }
      return TRUE;
    }
    return FALSE;
  }

    /**
  * Load category
  *
  * @access public
  * @return boolean
  */
  function loadOneCateg($catId) {

    $sql = "SELECT forumcat_id, forumcat_path, forumcat_prev
              FROM %s
              WHERE forumcat_id = '%d'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableCategs,$catId ))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->categ = $row;
      }
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * Load forums
  *
  * @param boolean $all optional, default value FALSE
  * @param boolean $details optional, default value FALSE
  * @access public
  * @return boolean
  */
  function loadBoards($all = FALSE, $details = FALSE) {
    unset($this->boards);
    if ($all) {
      $sql = "SELECT forum_id, forumcat_id, forum_title
                FROM %s
                ORDER BY forum_title ASC, forum_id ASC";
    } elseif ($details) {
      $sql = "SELECT forum_id, forumcat_id, forum_title, forum_desc
                FROM %s
                WHERE forumcat_id = %d
                ORDER BY forum_title ASC, forum_id ASC";
    } else {
      $sql = "SELECT forum_id, forumcat_id, forum_title
                FROM %s
                WHERE forumcat_id = %d
                ORDER BY forum_title ASC, forum_id ASC";
    }
    $params = array($this->tableBoards, (int)$this->params['categ_id']);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->boards[(int)$row['forum_id']] = $row;
        if (isset($this->categs) && is_array($this->categs)) {
          $this->categs[(int)$row['forumcat_id']]['boards'][] = (int)$row['forum_id'];
        }
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load forums of category
  *
  * @param integer $forumId forum id
  * @access public
  * @return boolean
  */
  function loadBoard($forumId) {
    unset($this->board);
    $sql = "SELECT forum_id, forumcat_id, forum_title, forum_desc
              FROM %s
              WHERE forum_id = %d
              ORDER BY forum_title ASC, forum_id ASC";
    $params = array($this->tableBoards, (int)$forumId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->board = $row;
        return TRUE;
      }
      $res->free();
    }
    return FALSE;
  }

  /**
  * Does forum exist?
  *
  * @param integer $id forum id
  * @access public
  * @return boolean
  */
  function forumExists($id) {
    $sql = "SELECT count(*) FROM %s WHERE forum_id = '%d'";
    $params = array($this->tableBoards, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Get forum by page id
  *
  * @param integer $categId category id
  * @param integer $pageId page id
  * @access public
  * @return boolean
  */
  function getForumByPageId($categId, $pageId) {
    $sql = "SELECT forum_id FROM %s WHERE forumcat_id = '%d' AND page_id = '%d'";
    $params = array($this->tableBoards, $categId, $pageId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return (int)$row[0];
      }
    }
    return FALSE;
  }

  /**
  * Add fourm
  *
  * @param integer $parent parent id
  * @param string $title optional, default value ''
  * @param string $description optional, default value ''
  * @param integer $pageId optional, default value 0
  * @access public
  * @return mixed FALSE or Id of new record
  */
  function addForum($parent, $title = '', $description = '', $pageId = 0) {
    if ($this->categExists($parent)) {
      $data = array(
        'forumcat_id'=>$parent,
        'forum_title'=>((trim($title) != '') ? $title : $this->_gt('New forum')),
        'forum_desc'=>$description,
        'page_id'=>$pageId
      );
      return $this->databaseInsertRecord($this->tableBoards, 'forum_id', $data);
    }
    return FALSE;
  }

  /**
  * Update thread
  *
  * @param integer $id thread id
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function updateThread($id) {
    $sql = "SELECT COUNT(*) FROM %s WHERE entry_tid = %d";
    $counted = 0;
    if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries, $id))) {
      if ($row = $res->fetchRow()) {
        $counted = $row[0]-1;
        if ($counted < 2) {
          $sql = "SELECT COUNT(*) FROM %s WHERE entry_tid = 0 and entry_id = %d";
          if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries, $id))) {
            if ($row = $res->fetchRow()) {
              $counted += $row[0];
            }
          }
        }
      }
    }
    $data = array(
      'entry_thread_modified' => time(),
      'entry_thread_count' => $counted,
      'entry_tid' => "entry_id"
    );
    return $this->databaseUpdateRecord($this->tableEntries, $data, "entry_id", (int)$id);
  }

  /**
  * Search a thread
  *
  * @param integer $forumId
  * @param array $searchFor
  * @param array &$threadIds
  * @access public
  * @return boolean
  */
  function searchThreads($categId, $forumId, $searchFor, &$threadIds) {
    $threadIds = NULL;
    include_once(PAPAYA_INCLUDE_PATH.'system/base_searchstringparser.php');
    $parser = &new searchstringparser;
    $filter = $parser->getSQL($searchFor,
      array('entry_subject', 'entry_text', 'entry_username'), $this->fullTextSearch);

    $forumIds = array();
    if ($categId > 0) {
      $sql = "SELECT forumcat_id, forumcat_path
                FROM %s
                WHERE forumcat_id = %d";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableCategs, $categId))) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $prevPath = $row['forumcat_path'].$row['forumcat_id'].';';
          $sql = "SELECT f.forum_id
                    FROM %s AS c, %s AS f
                    WHERE (f.forumcat_id = c.forumcat_id AND c.forumcat_path LIKE '%s%%')
                      OR f.forumcat_id = %d";
          if ($res = $this->databaseQueryFmt($sql, array($this->tableCategs, $this->tableBoards, $categId, $prevPath))) {
            while($row = $res->fetchRow()) {
              $forumIds[] = (int)$row[0];
            }
          }
        }
      }
    } elseif ($forumId > 0) {
      $forumIds = array($forumIds);
    }


    if ($filter && isset($forumIds) && is_array($forumIds) && count($forumIds) > 0) {
      $forumIds = array_unique($forumIds);
      if (count($forumIds) > 1) {
        $forumFilter = ' IN ('.implode(',', $forumIds).') ';
      } else {
        $forumFilter = ' = '.(int)$forumIds[0];
      }
      $sql = "SELECT entry_tid, entry_id
              FROM $this->tableEntries
              WHERE forum_id $forumFilter AND
                ($filter)";
      if ($res = $this->databaseQuery($sql)) {
        while($row = $res->fetchRow()) {
          if ($row[0] > 0) {
            $threadIds[$row[0]] = 1;
          } else {
            $threadIds[$row[1]] = 1;
          }
        }
        if (isset($threadIds) && is_array($threadIds)) $threadIds = array_keys($threadIds);
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Search entries
  *
  * @param integer $forumId
  * @param integer $searchFor
  * @param integer $offset
  * @param integer $perPage
  * @access public
  * @return boolean
  */
  function searchEntries($categId, $forumId, $searchFor, $offset, $perPage) {
    unset($this->threads);
    unset($threadIds);
    $searched = FALSE;
    if ($this->cacheSearchResults) {
      $searchResult = $this->getSessionValue($this->sessionParamName.'_searchresult');
      if ((int)$categId == (int)$searchResult['categ_id'] && (int)$forumId == (int)$searchResult['forum_id'] &&
        $searchResult['searchfor'] == $searchFor && $searchResult['time'] > time()) {
        $threadIds = $searchResult['thread_ids'];
        $searched = TRUE;
      } else {
        $searched = $this->searchThreads($categId, $forumId, $searchFor, $threadIds);
        if ($searched) {
          $searchResult = array(
            'categ_id' => (int)$categId,
            'forum_id' => (int)$forumId,
            'searchfor' => $searchFor,
            'time' => time()+1800,
            'thread_ids' => $threadIds);
          $this->setSessionValue($this->sessionParamName.'_searchresult', $searchResult);
        }
      }
    } else {
      $searched = $this->searchThreads($categId, $forumId, $searchFor, $threadIds);
    }
    if (isset($threadIds) && is_array($threadIds)) {
      $filter = (count($threadIds) > 1)
        ? "IN ('".implode("','", $threadIds)."')"
        : " = '".(int)$threadIds[0]."'";
      $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
                entry_created, entry_modified,
                entry_thread_modified, entry_thread_count,
                entry_subject, entry_username
                FROM %s
              WHERE entry_id $filter
              ORDER BY entry_thread_modified DESC, entry_tid DESC";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries), $perPage, $offset)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->threads[$row['entry_id']] = $row;
        }
        $this->threadCount = $res->absCount();
        return TRUE;
      }
    } else {
      return $searched;
    }
    return FALSE;
  }

  /**
  * Does entry exist ?
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function entryExists($id) {
    $sql = "SELECT count(*) FROM %s WHERE entry_id = '%d'";
    $params = array($this->tableEntries, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
      $res->free();
    }
    return FALSE;
  }


  /**
  * Load entry
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadEntry($id) {
    unset($this->entry);
    $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
              entry_created, entry_modified, entry_thread_modified, entry_thread_count,
              entry_subject, entry_text, entry_username, entry_userid, entry_useremail, entry_userregistered,
              entry_sendanswers
              FROM %s
              WHERE entry_id = '%d'";
    $params = array($this->tableEntries, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->entry = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Save entry
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function saveEntry() {
    $data = array(
      'entry_modified' => time(),
      'entry_subject' => $this->params['entry_subject'],
      'entry_text' => $this->params['entry_text'],
      'entry_username' => $this->params['entry_username'],
      'entry_useremail' => $this->params['entry_useremail'],
      'entry_userregistered' => $this->params['entry_userregistered'],
      'entry_sendanswers' => $this->params['entry_sendanswers']
    );
    return $this->databaseUpdateRecord($this->tableEntries, $data, 'entry_id', (int)$this->params['entry_id']);
  }

  /**
  * Get child ids
  *
  * @param array $ids
  * @access public
  * @return array
  */
  function getChildIds($ids) {
    $result = array();
    if (isset($ids) && is_array($ids)) {
      foreach ($ids as $id) {
        if (isset($this->entryTree[$id]) && is_array($this->entryTree[$id])) {
          $result = array_merge($result, $this->entryTree[$id]);
          $result = array_merge($result, $this->getChildIds($this->entryTree[$id]));
        }
      }
    }
    return $result;
  }

  /**
  * Load thread
  *
  * @param integer $id
  * @param boolean $allData optional, default value FALSE
  * @param mixed $offset optional, default value NULL
  * @param mixed $perPage optional, default value NULL
  * @access public
  * @return boolean
  */
  function loadThread($id, $allData = FALSE, $offset=NULL, $perPage=NULL) {
    unset($this->entries);
    unset($this->entryTree);
    $addFields = ($allData) ? ', entry_text' : '';
    $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
              entry_created, entry_modified,
              entry_subject, entry_username,
              entry_userid, entry_userregistered $addFields
              FROM %s
              WHERE entry_id = '%d'";
    $params = array($this->tableEntries, $id, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->entries[(int)$row['entry_id']] = $row;
        $this->entryTree[(int)$row['entry_pid']][] = (int)$row['entry_id'];
      }
      $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
                entry_created, entry_modified,
                entry_subject, entry_username,
                entry_userid, entry_userregistered $addFields
                FROM %s
                WHERE entry_tid = '%d'
                ORDER BY entry_created ASC, entry_id ASC";
      $params = array($this->tableEntries, $id, $id);
      if (isset($perPage) && isset($offset)) {
        if ($offset < 1) {
          $max = $perPage;
          $from = 0;
        } else {
          unset($this->entries);
          unset($this->entryTree);
          $max = $perPage;
          $from = $offset-1;
        }
      } else {
        $max = NULL;
        $from = NULL;
      }
      if ($res = $this->databaseQueryFmt($sql, $params, $max, $from)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->entries[(int)$row['entry_id']] = $row;
          $this->entryTree[(int)$row['entry_pid']][] = (int)$row['entry_id'];
        }
        if ($offset < 1) {
          $this->entryCount = $res->absCount()+1;
        } else {
          $this->entryCount = $res->absCount();
        }
        return TRUE;
      }
    }
    return FALSE;
  }


  /**
  * Get thread by page id
  *
  * @param integer $forumId
  * @param integer $pageId
  * @access public
  * @return boolean
  */
  function getThreadByPageId($forumId, $pageId) {
    $sql = "SELECT entry_id FROM %s WHERE forum_id = '%d' AND page_id = '%d'";
    $params = array($this->tableEntries, $forumId, $pageId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return (int)$row[0];
      }
    }
    return FALSE;
  }

  /**
  * Load thread list
  *
  * @param integer $fid
  * @param integer $offset
  * @param integer $perPage
  * @access public
  * @return boolean
  */
  function loadThreads($fid, $offset, $perPage) {
    unset($this->threads);
    $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
              entry_created, entry_modified, entry_thread_modified, entry_thread_count,
              entry_subject, entry_username, entry_userid, entry_userregistered
              FROM %s
              WHERE forum_id='%d' AND entry_pid = 0
              ORDER BY entry_thread_modified DESC, entry_created DESC, entry_id DESC";
    $params = array($this->tableEntries, $fid);
    if ($res = $this->databaseQueryFmt($sql, $params, $perPage, $offset)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->threads[(int)$row['entry_id']] = $row;
      }
      $this->threadCount = $res->absCount();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load last entries
  *
  * @param integer $forumId
  * @param integer $threadId
  * @param integer $maxCount
  * @access public
  */
  function loadLastEntries($forumId, $threadId, $maxCount) {
    unset($this->entries);
    unset($this->entryTree);
    if ($threadId > 0) {
        $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
                  entry_created, entry_modified,
                  entry_subject, entry_username, entry_userid, entry_userregistered
                FROM %s
                WHERE forum_id = '%d' and entry_tid = '%d' and entry_pid > 0
                ORDER BY entry_created DESC, entry_id DESC";
    } else {
        $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
                  entry_created, entry_modified,
                  entry_subject, entry_username
                FROM %s
                WHERE forum_id = '%d'
                ORDER BY entry_created DESC, entry_id DESC";
    }
    $params = array($this->tableEntries, $forumId, $threadId);
    if ($res = $this->databaseQueryFmt($sql, $params, $maxCount)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->entries[(int)$row['entry_id']] = $row;
      }
    }
  }

  function loadLastEntriesInCateg($categId, $maxCount) {
    if ((int)$categId == 0 || $this->loadOneCateg((int)$categId)) {
      $sql = "SELECT f.forum_id
                FROM %s AS c, %s AS f
                WHERE forumcat_path LIKE '%s%%' AND f.forumcat_id = c.forumcat_id";
      $params = array($this->tableCategs, $this->tableBoards, $this->categ['forumcat_path']);
      $forumIds = array();
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow()) {
          $forumIds[] = (int)$row[0];
        }
      }
      if (count($forumIds) < 1) {
        return FALSE;
      } elseif (count($forumIds) == 1) {
        $filter = " = ".(int)$forumIds[0];
      } else {
        $filter = " IN ('".implode("','", $forumIds)."') ";
      }
      $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
                    entry_created, entry_modified,
                    entry_subject, entry_username
                  FROM %s
                  WHERE forum_id $filter
                  ORDER BY entry_created DESC, entry_id DESC";
      $params = array($this->tableEntries);
      if ($res = $this->databaseQueryFmt($sql, $params, $maxCount)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->entries[(int)$row['entry_id']] = $row;
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Load entries by id
  *
  * @param integer $ids
  * @access public
  */
  function loadEntriesById($ids) {
    unset($this->entries);
    unset($this->entryTree);
    if (isset($ids) && is_array($ids) && count($ids) > 1) {
      $filter = " IN ('".implode("','", $ids)."')";
    } else {
      $filter = " = '".(int)$ids[0]."'";
    }
    $keys = array_flip($ids);

    $sql = "SELECT entry_id, entry_pid, entry_tid, forum_id,
              entry_created, entry_modified,
              entry_subject, entry_username, entry_userid, entry_userregistered,
              entry_text
              FROM %s
              WHERE entry_id $filter";
    $params = array($this->tableEntries);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['index'] = $keys[(int)$row['entry_id']];
        $this->entries[(int)$row['entry_id']] = $row;
      }
      if (isset($this->entries) && is_array($this->entries)) {
        uasort($this->entries, array($this, 'cmpEntriesByIndex'));
      }
    }
  }

  /**
  * Compare entries by index
  *
  * @param array $a
  * @param array $b
  * @access public
  * @return integer -1;0;1
  */
  function cmpEntriesByIndex($a,$b) {
    if ($a['index'] != $b['index']) {
      return ($a['index'] > $b['index']) ? 1 : -1;
    } else {
      return 0;
    }
  }
  
  /**
  * Get output
  *
  * @param array $data
  * @param array $conf
  * @access public
  * @return string
  */
  function getOutput($data, $conf) {
    $this->initializeParams();

    if (!isset($this->params['offset'])) {
      $this->params['offset'] = 0;
    }

    $this->backLink = '';
    $this->searchLink = '';
    $this->addLink = '';

    if (isset($_REQUEST[$this->paramName.'_id'])) {
      $gotoId = $_REQUEST[$this->paramName.'_id'];
      if ($this->loadEntry($gotoId)) {
        $this->params['categ_id'] = $this->entry['forumcat_id'];
        $this->params['forum_id'] = $this->entry['forum_id'];
        $this->params['entry_id'] = $this->entry['entry_id'];
        $this->params['thread_id'] = ($this->entry['entry_tid'] > 0) ?
          $this->entry['entry_tid'] : $this->entry['entry_id'];
        if (isset($_REQUEST[$this->paramName.'_nomail']) &&
            (strlen(trim($_REQUEST[$this->paramName.'_nomail'])) == 8)) {
          if ($this->checkNotify($this->entry['entry_tid'], trim($_REQUEST[$this->paramName.'_nomail']))) {
            return $this->getDeactivateNotifyForm(trim($_REQUEST[$this->paramName.'_nomail']),
              htmlspecialchars($conf['notifyquestion'])
            );
          }
        }
      }
    }

    if (isset($this->params['cmd']) && ($this->params['cmd'] == 'notifychg') &&
      ($this->params['confirm_change'])) {

      if ($this->loadEntry($this->params['entry_id'])) {
        $this->deactivateNotify($this->entry['entry_tid'], trim($this->params['nomail']));
      }
    }

    $result = '';
    if (isset($this->params['search']) && $this->params['search']) {
      if ($data['mode'] == 'categ') {
        $categId = (int)$data['id'];
        $forumId = 0;
      } else {
        $categId = 0;
        $forumId = (int)$data['id'];
      }
      $this->initializeSearchForm($conf);


      $result .= '<forum>';
      if (trim(@$this->params['searchfor']) != '') {
        $searched = $this->searchEntries($categId, $forumId,
          $this->params['searchfor'],
          (int)@$this->params['offset'],
          (int)$conf['perpage']
        );
        if ($searched) {
          if (isset($this->threads) && is_array($this->threads) && ($this->threadCount > 0)) {
            $result .= $this->getOutputThreadList((int)$forumId, (int)$conf['perpage']);
          } else {
            $result .= sprintf('<message no="2"><![CDATA[%s]]></message>',
              htmlspecialchars($conf['msg_noresult'])
            );
          }
        } else {
          $result .= sprintf('<message no="1"><![CDATA[%s]]></message>',
            htmlspecialchars($conf['msg_invalid'])
          );
        }
      }
      $result .= $this->getSearchForm(@$conf['caption_search'], $conf);
      $result .= '</forum>';
    } else {
      $this->searchLink = $this->buildLink(array(
        'entry_id'=>0,
        'offset'=>0,
        'search'=>TRUE
      ));
      if ($data['mode'] == 'categ') {
        if (isset($this->params['categ_id']) && $this->params['categ_id'] > 0) {
          $this->loadOneCateg($this->params['categ_id']);
          if ($this->params['categ_id'] == $data['id']) {
            $categOk = TRUE;
          } elseif (strpos($this->categ['forumcat_path'], ';'.(int)$data['id'].';')) {
            $categOk = TRUE;
          } else {
            unset($this->categ);
            $categOk = FALSE;
          }
        } else {
          $this->params['categ_id'] = $data['id'];
          $this->loadOneCateg($data['id']);
          $categOk = TRUE;
        }


        if (isset($this->params['forum_id']) && ($this->params['forum_id'] > 0) &&
          $this->loadBoard($this->params['forum_id'])) {
          $categId = (int)@$this->board['forumcat_id'];
        } else {
          $categId = (isset($this->params['categ_id']) &&
            $this->params['categ_id'] > 0) ? (int)$this->params['categ_id'] : $data['id'];
        }

        if (isset($this->categ) && is_array($this->categ)){
          preg_match_all('/\d+/',$this->categ['forumcat_path'].$categId.';',
            $regs, PREG_PATTERN_ORDER
          );
          $catIds = $regs[0];
          $this->loadCategsByIds($catIds);
        }

        if ((isset($catIds) && is_array($catIds) &&
          in_array($categId,$catIds))) {

          $this->backLink = $this->buildLink(array('categ_id'=> $this->categ['forumcat_prev']));
          $this->pathNavigation = $this->getXMLNavigation();

          $result .= $this->getOutputCateg($categId, $conf);

        }
      } else {
        $result .= $this->getOutputForum($data['id'], $conf);
      }
    }
    return $result;
  }

  /**
  * Get output category
  *
  * @param integer $categId
  * @param array $conf
  * @access public
  * @return string
  */
  function getOutputCateg($categId, $conf) {
    //Kategorie
    if (isset($this->board) && is_array($this->board)) {
      $this->backLink = $this->buildLink(array('categ_id'=>(int)$this->board['forumcat_id']));
      $result = $this->getOutputForum($this->board['forum_id'], $conf, TRUE);
    } else {
      $this->params['categ_id'] = (int)$categId;
      $this->loadCategs($categId);
      $result = '<forum>'.LF;
      if (isset($this->categs) && is_array($this->categs)) {
        $result .= '<categs>'.LF;
        foreach($this->categs as $categ) {
          $result .= sprintf('<categ title="%s" href="%s"><![CDATA[%s]]></categ>',
            htmlspecialchars($categ['forumcat_title']),
            $this->buildLink(array('categ_id'=>(int)$categ['forumcat_id'])),
            $this->module->getXHTMLString(isset($categ['forumcat_desc']) ? $categ['forumcat_desc'] : '', TRUE));
        }
        $result .= '</categs>'.LF;
      }
      $this->loadBoards(FALSE, TRUE);
      if (isset($this->boards) && is_array($this->boards)) {
        $result .= '<forums>'.LF;
        foreach($this->boards as $board) {
          $result .= sprintf('<forum title="%s" href="%s"><![CDATA[%s]]></forum>',
            htmlspecialchars($board['forum_title']),
            $this->buildLink(array('forum_id'=>(int)$board['forum_id'])),
            $this->module->getXHTMLString($board['forum_desc'], TRUE));
        }
        $result .= '</forums>'.LF;
      }
      $result .= '<links>';
      if (isset($this->backLink) && $this->backLink != '') {
        $result .= sprintf('<back href="%s"/>'.LF, $this->backLink);
      }
      if (isset($this->searchLink) && $this->searchLink != '') {
        $result .= sprintf('<search href="%s"/>'.LF, $this->searchLink);
      }
      $result .= '</links>';
      if (isset($this->pathNavigation) && $this->pathNavigation != '') {
        $result .= $this->pathNavigation;
      }
      $result .= '</forum>'.LF;
    }
    return $result;
  }

  /**
  * Get output formular
  *
  * @param integer $forumId
  * @param array $conf
  * @access public
  * @return string
  */
  function getOutputForum($forumId, $conf, $categMode = FALSE) {
    //einzelnes Forum
    $result="";
    $this->loadBoard($forumId);
    $this->params['forum_id'] = $forumId;
    if (isset($this->params['entry_sendanswers']) && $this->params['entry_sendanswers']) {
      $this->params['entry_sendanswers'] = 1;
    } else {
      $this->params['entry_sendanswers'] = 0;
    }

    if (isset($this->params['cmd']) && $this->params['cmd'] == 'add_entry') {
      $this->loadEntry((int)$this->params['entry_id']);
      $this->initializeOutputForm($conf);
      if (isset($this->params['save']) && $this->outputDialog->checkDialogInput()) {
        if (isset($this->entry) && is_array($this->entry)) {
          $pid = (int)$this->entry['entry_id'];
          $tid = ($this->entry['entry_tid'] > 0) ? (int)$this->entry['entry_tid'] : $pid;
        } else {
          $pid = 0;
          $tid = 0;
        }

        if (($pid > 0 && $conf['post_anwers']) || ($pid == 0 && $conf['post_questions'])) {
          echo 'Mal gucken ob die Postsperre geht';
          exit;
          if ($this->checkDoublePost($pid)) {
            if ($newId = $this->addEntry($pid, $tid, $conf)) {
              $this->params['cmd'] = '';
              $this->params['entry_id'] = $newId;
              $this->params['thread_id'] = $tid;
              $linkParams = array(
                'forum_id'=>(int)$this->board['forum_id'],
                'offset'=>0,
                'bbsoffset'=> '-1',
                'entry_id'=>(int)$newId,
                'thread_id'=> ((isset($tid) && $tid > 0) ? (int)$tid : $newId)
              );
              $this->module->deleteCache();
              $url = $this->getAbsoluteURL($this->buildLink($linkParams, NULL, '&'));
              header('Location: '.$url);
              exit;
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Double Post.'));
          }
        }
      }
    }

    $result = sprintf('<forum title="%s" mode="%d">',
      htmlspecialchars($this->board['forum_title']),
      $conf['mode']
    );
    if (isset($this->params['thread_id']) && $this->params['thread_id'] > 0) {
      $this->backLink = $this->buildLink(array(
        'forum_id'=>(int)$this->board['forum_id'],
        'offset'=>(int)@$this->params['offset'])
      );
      if (@$this->params['cmd'] == 'add_entry' || @$this->params['cmd'] == 'add_entry_form') {
        if ($this->params['entry_id'] > 0) {
          if (!isset($this->entry)) {
            $this->loadEntry((int)$this->params['entry_id']);
          }
          $this->backLink = $this->buildLink(array(
            'forum_id'=>(int)$this->board['forum_id'],
            'entry_id'=>(int)@$this->entry['entry_id'],
            'thread_id'=>(int)@$this->entry['entry_tid'],
            'offset'=>(int)@$this->params['offset'],
            'bbsoffset'=>(int)$this->params['bbsoffset']
          ));
        }
        $result .= $this->getOutputForm(@$conf['caption_newanswer'], $conf);
      } else {
        switch ($conf['mode']) {
          case 0 :
            $this->loadThread((int)@$this->params['thread_id']);
            $result .= $this->getOutputThread();
            if (isset($this->entries[(int)@$this->params['entry_id']])) {
              $this->loadEntry((int)@$this->params['entry_id']);
            } else {
              $this->loadEntry((int)@$this->params['thread_id']);
            }
            $result .= $this->getOutputEntry($this->entry, 0);
            $result .= $this->getOutputForm((isset($conf['caption_newanswer'])) ? $conf['caption_newanswer'] : '', $conf);
            break;
          case 2 :
            //threadedbbs
            $this->loadThread((int)$this->params['thread_id']);
            $this->loadEntry((int)$this->params['thread_id']);
            unset($this->entryIdList);
            $result .= $this->getOutputThread($conf['bbsperpage']);
            if (isset($this->params['entry_id']) &&
                isset($this->entries[(int)$this->params['entry_id']]) &&
                is_array($this->entries[(int)$this->params['entry_id']])) {
              $this->params['bbsoffset'] = $this->entries[(int)$this->params['entry_id']]['bbsoffset'];
            }
            $ids = array_slice($this->entryIdList,
              (int)@$this->params['bbsoffset'],
              (int)@$conf['bbsperpage']
            );
            $this->loadEntriesById($ids);
            $result .= $this->getOutputThreadedBBS((int)@$conf['bbsperpage']);
            $result .= $this->getOutputForm(@$conf['caption_newanswer'], $conf);
            break;
          default :
            $this->loadEntry((int)$this->params['thread_id']);
            if (@$this->params['bbsoffset'] == -1) {
              $this->params['bbsoffset'] =                 floor($this->entry['entry_thread_count']/$conf['bbsperpage'])*$conf['bbsperpage'];
            }
            $this->loadThread((int)$this->params['thread_id'], TRUE,
              (int)@$this->params['bbsoffset'], (int)@$conf['bbsperpage']
            );
            $result .= $this->getOutputBBS((int)$conf['bbsperpage']);
            $result .= $this->getOutputForm(@$conf['caption_newanswer'], $conf);
        }
      }
    } else {
      if (isset($this->params['cmd']) && $this->params['cmd'] == 'add_entry') {
        if ($this->params['entry_id'] > 0) {
          if (!isset($this->entry)) {
            $this->loadEntry((int)$this->params['entry_id']);
          }
          $this->backLink = $this->buildLink(array(
            'forum_id'=>(int)$this->board['forum_id'],
            'entry_id'=>(int)@$this->entry['entry_id'],
            'thread_id'=>(int)@$this->entry['entry_tid'],
            'offset'=>(int)@$this->params['offset']
          ));
        }
        $result .= $this->getOutputForm(@$conf['caption_newquestion'], $conf);
      } else {
        $this->loadThreads((int)$forumId, (int)@$this->params['offset'], (int)$conf['perpage']);
        $result .= $this->getOutputThreadList((int)$forumId, (int)$conf['perpage']);
        $result .= $this->getOutputForm(@$conf['caption_newquestion'], $conf);
      }
    }
    $result .= '<links>';
    if ($categMode ||
        isset($this->categ) || (@$this->params['cmd'] == 'add_entry') || isset($this->params['search']) ||
        isset($this->entry) || (isset($this->params['entry_id']) && $this->params['entry_id'] > 0)) {
      $result .= sprintf('<back href="%s"/>'.LF, $this->backLink);
    }
    $result .= sprintf('<search href="%s"/>'.LF, $this->searchLink);
    if ((isset($this->entry) && is_array($this->entry) && $conf['post_anwers']) ||
        ((!(isset($this->entry) && is_array($this->entry))) && $conf['post_questions'])) {
      $result .= sprintf('<add href="%s"/>'.LF, $this->addLink);
    }
    $result .= '</links>';
    $result .= '</forum>';
    return $result;
  }

  /**
  * Get output of thread list
  *
  * @param integer $forumId
  * @param integer $pageMax
  * @access public
  * @return string
  */
  function getOutputThreadList($forumId, $pageMax) {
    $result = '';
    if (isset($this->threads) && is_array($this->threads)) {
      if ($this->params['offset']-$pageMax >= 0) {
        if (@$this->params['search']) {
          $newer = $this->buildLink(array('forum_id'=>$forumId,
            'offset'=>$this->params['offset']-$pageMax,
            'search' => 1, 'searchfor' => $this->params['searchfor']));
        } else {
          $newer = $this->buildLink(array('forum_id'=>$forumId,
            'offset'=>$this->params['offset']-$pageMax));
        }
      } elseif ($this->params['offset'] > 0) {
        if ($this->params['search']) {
          $newer = $this->buildLink(array('forum_id'=>$forumId,'offset'=>0,
            'search' => 1, 'searchfor' => $this->params['searchfor']));
        } else {
          $newer = $this->buildLink(array('forum_id'=>$forumId,'offset'=>0));
        }
      } else {
        $newer = '';
      }
      if (($this->params['offset']+$pageMax) < $this->threadCount) {
        if (isset($this->params['search']) && $this->params['search']) {
          $older = $this->buildLink(array('forum_id'=>$forumId,
            'offset'=>$this->params['offset']+$pageMax,
            'search' => 1, 'searchfor' => $this->params['searchfor']));
        } else {
          $older = $this->buildLink(array('forum_id'=>$forumId,
            'offset'=>$this->params['offset']+$pageMax));
        }
      } else {
        $older = '';;
      }
      $result = sprintf('<threads nlink="%s" olink="%s">'.LF, $newer, $older);
      foreach($this->threads as $thread) {
        $selected = (isset($this->params['thread_id']) && $thread['entry_id'] == $this->params['thread_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<thread href="%s" title="%s" created="%s" modified="%s" answers="%d" %s/>',
          $this->buildLink(
            array('forum_id'=>(int)$thread['forum_id'],
                  'thread_id'=>(int)$thread['entry_id'],
                  'offset'=>(int)@$this->params['offset']
            )
          ),
          htmlspecialchars($thread['entry_subject']),
          htmlspecialchars(date('Y-m-d H:i:s', $thread['entry_created'])),
          htmlspecialchars(date('Y-m-d H:i:s', $thread['entry_thread_modified'])),
          (int)$thread['entry_thread_count'],
          $selected);
      }
      $result .= '</threads>';
    }
    return $result;
  }

  /**
  * Get output thread
  *
  * @param integer $pageMax optional, default value 0
  * @access public
  * @return string
  */
  function getOutputThread($pageMax = 0) {
    $entry = &$this->entries[(int)$this->params['thread_id']];
    if (isset($entry) && is_array($entry)) {
      $result = sprintf('<thread href="%s" title="%s" created="%s" modified="%s" answers="%d">',
        $this->buildLink(
          array('forum_id'=>(int)$entry['forum_id'],
                'thread_id'=>(int)$entry['entry_id'],
                'offset'=>(int)@$this->params['offset']
          )
        ),
        htmlspecialchars($entry['entry_subject']),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_created'])),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_modified'])),
        (int)@$entry['entry_thread_count']);
      $result .= '<entries>';
      $result .= $this->getOutputThreadElement($entry['entry_id'], 0, $pageMax);
      $result .= '</entries>';
      $result .= '</thread>';
    }
    return $result;
  }

  /**
  * Get output for thread element
  *
  * @param integer $id
  * @param integer $indent
  * @param integer $maxPerPage optional, default value 0
  * @access public
  * @return string
  */
  function getOutputThreadElement($id, $indent, $maxPerPage = 0) {
    $entry = $this->entries[(int)$id];
    if (isset($entry) && is_array($entry)) {
      $result = $this->getOutputEntry($entry, $indent, $maxPerPage);
      $result .= $this->getOutputSubThread($entry['entry_id'], $indent, $maxPerPage);
      return $result;
    }
    return '';
  }

  /**
  * Get output for sub thread
  *
  * @param integer $tid
  * @param integer $indent
  * @param integer $maxPerPage
  * @access public
  * @return string
  */
  function getOutputSubThread($tid, $indent, $maxPerPage) {
    $result = '';
    if (isset($this->entryTree[$tid]) && is_array($this->entryTree[$tid])) {
      foreach($this->entryTree[$tid] as $id) {
        $result .= $this->getOutputThreadElement($id, $indent+1, $maxPerPage);
      }
    }
    return $result;
  }

  /**
  * Get output for BBS view
  *
  * @param mixed $pageMax optional, default value NULL
  * @access public
  * @return string
  */
  function getOutputBBS($pageMax = NULL) {
    $entry = &$this->entry;
    if (isset($entry) && is_array($entry)) {
      $result = sprintf('<thread href="%s" title="%s" created="%s" modified="%s" answers="%d">',
        $this->buildLink(
          array('forum_id'=>(int)$entry['forum_id'],
                'thread_id'=>(int)$entry['entry_id'],
                'offset'=>(int)@$this->params['offset']
          )
        ),
        htmlspecialchars($entry['entry_subject']),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_created'])),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_modified'])),
        (int)$entry['entry_thread_count']);
      $pageLinks = NULL;
      $pageCount = floor($entry['entry_thread_count'] / $pageMax);
      for ($i = 0; $i <= $pageCount; $i++) {
        $pageLinks[$i+1] = $i * $pageMax;
      }
      if (isset($pageMax)) {
        if (@$this->params['bbsoffset']-$pageMax >= 0) {
          $older = $this->buildLink(array(
            'forum_id'=>(int)$entry['forum_id'],
            'thread_id'=>(int)$entry['entry_id'],
            'offset'=>(int)@$this->params['offset'],
            'bbsoffset'=>$this->params['bbsoffset']-$pageMax));
        } elseif (@$this->params['bbsoffset'] > 0) {
          $older = $this->buildLink(array('forum_id'=>$forumId,'bbsoffset'=>0));
        } else {
          $older = '';
        }
        if ((@$this->params['bbsoffset']+$pageMax) < $this->entryCount) {
          $newer = $this->buildLink(array(
            'forum_id'=>(int)$entry['forum_id'],
            'thread_id'=>(int)$entry['entry_id'],
            'offset'=>(int)@$this->params['offset'],
            'bbsoffset'=>@$this->params['bbsoffset']+$pageMax));
        } else {
          $newer = '';
        }
        $result .= sprintf('<entries nlink="%s" olink="%s">'.LF, $newer, $older);
      } else {
        $result .= '<entries>';
      }
      foreach($this->entries as $id=>$item) {
        $result .= $this->getOutputEntry($item);
      }
      if (isset($pageLinks) && is_array($pageLinks) && $pageLinks > 2) {
        foreach($pageLinks as $page=>$offset) {
          $selected = ($offset == @$this->params['bbsoffset']) ? ' selected="selected"' : '';
          $result .= sprintf('<pagelink href="%s" no="%d" %s/>'.LF,
            $this->buildLink(
              array('forum_id'=>(int)$entry['forum_id'],
                    'thread_id'=>(int)$entry['entry_id'],
                    'offset'=>(int)@$this->params['offset'],
                    'bbsoffset'=>(int)$offset)),
            $page, $selected);
        }
      }
      $result .= '</entries>';
      $result .= '</thread>';
    }
    return $result;
  }

  /**
  * Get output for threaded BBS view
  *
  * @param integer $pageMax
  * @access public
  * @return string
  */
  function getOutputThreadedBBS($pageMax) {
    $result = '';
    if (isset($this->entries) && is_array($this->entries)) {
      $result .= sprintf('<entrydata new_suffix="&amp;%s[cmd]=%s">', $this->paramName, 'add_entry_form');
      $pageLinks = NULL;

      $pageCount = floor($this->entry['entry_thread_count'] / $pageMax);
      for ($i = 0; $i <= $pageCount; $i++) {
        $pageLinks[$i+1] = $i * $pageMax;
      }
      if (isset($pageMax)) {
        if (@$this->params['bbsoffset']-$pageMax >= 0) {
          $older = $this->buildLink(array(
            'forum_id'=>(int)$this->entry['forum_id'],
            'thread_id'=>(int)$this->entry['entry_id'],
            'offset'=>(int)@$this->params['offset'],
            'bbsoffset'=>$this->params['bbsoffset']-$pageMax));
        } elseif ((int)@$this->params['bbsoffset'] > 0) {
          $older = $this->buildLink(array('forum_id'=>$forumId,'bbsoffset'=>0));
        } else {
          $older = '';
        }
        if ((@$this->params['bbsoffset']+$pageMax) < $this->entryCount) {
          $newer = $this->buildLink(array(
            'forum_id'=>(int)$this->entry['forum_id'],
            'thread_id'=>(int)$this->entry['entry_id'],
            'offset'=>(int)@$this->params['offset'],
            'bbsoffset'=>@$this->params['bbsoffset']+$pageMax));
        } else {
          $newer = '';
        }
        $result .= sprintf('<entries nlink="%s" olink="%s">'.LF, $newer, $older);
      } else {
        $result .= '<entries>';
      }
      foreach($this->entries as $id=>$item) {
        $result .= $this->getOutputEntry($item);
      }
      if (isset($pageLinks) && is_array($pageLinks) && $pageLinks > 2) {
        foreach($pageLinks as $page=>$offset) {
          $selected = ($offset == @$this->params['bbsoffset']) ? ' selected="selected"' : '';
          $result .= sprintf('<pagelink href="%s" no="%d" %s/>'.LF,
            $this->buildLink(
              array('forum_id'=>(int)$this->entry['forum_id'],
                    'thread_id'=>(int)$this->entry['entry_id'],
                    'offset'=>(int)@$this->params['offset'],
                    'bbsoffset'=>(int)$offset)),
            $page, $selected);
        }
      }
      $result .= '</entries>';
      $result .= '</entrydata>';
    }
    return $result;
  }

  /**
  * Get output for entry
  *
  * @param array &$entry
  * @param integer $indent optional, default value 0
  * @param integer $maxPerPage optional, default value 0
  * @access public
  * @return string
  */
  function getOutputEntry(&$entry, $indent=0, $maxPerPage = 0) {
    if (isset($entry) && is_array($entry)) {
      $selected = (isset($this->params['entry_id']) && $entry['entry_id'] == $this->params['entry_id']) ? ' selected="selected"' : '';
      $threadId = ($entry['entry_tid'] > 0) ? $entry['entry_tid'] : $entry['entry_id'];
      $this->entryIdList[] = (int)$entry['entry_id'];
      $index = count($this->entryIdList)-1;
      if ($maxPerPage > 0) {
        $iOffset = ((int)($index / $maxPerPage)) * $maxPerPage;
      } else {
        $iOffset = (int)@$this->params['bbsoffset'];
      }
      $this->entries[(int)$entry['entry_id']]['bbsoffset'] = $iOffset;

      if (isset($entry['entry_userregistered']) && $entry['entry_userregistered']) {
        $uHref = ' userhref="'.$this->getUserDataLink($entry['entry_userid']).'" ';
      } else {
        $uHref = '';
      }

      $result = sprintf('<entry href="%s" index="%d" title="%s" username="%s" created="%s" modified="%s" indent="%d" %s%s>',
        $this->buildLink(
          array('forum_id'=>(int)$entry['forum_id'],
                'thread_id'=>(int)$threadId,
                'entry_id'=>(int)$entry['entry_id'],
                'offset'=>(int)@$this->params['offset'],
                'bbsoffset'=>$iOffset
          )
        ),
        $index,
        htmlspecialchars($entry['entry_subject']),
        htmlspecialchars($entry['entry_username']),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_created'])),
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_modified'])),
        $indent, $selected, $uHref);
      if (isset($entry['entry_text']) && trim($entry['entry_text']) != '') {
        $result .= '<![CDATA['.$this->module->getXHTMLString($this->removeEvilTags($entry['entry_text'], '<b></b><i></i><a>'), TRUE).']]>';
      }
      $result .= '</entry>';
      return $result;
    }
    return '';
  }

  /**
  * Get output for last entries
  *
  * @param integer $forumId
  * @param integer $threadId
  * @param integer $maxCount
  * @access public
  * @return string
  */
  function getOutputLastEntries($forumId, $threadId, $maxCount) {
    $result = '';
    $this->loadLastEntries($forumId, $threadId, $maxCount);
    if (isset($this->entries) && is_array($this->entries)) {
      foreach($this->entries as $entry) {
        $result .= $this->getOutputEntry($entry);
      }
    }
    return $result;
  }

  /**
  * Get output for last entries in a category
  *
  * @param integer $forumId
  * @param integer $threadId
  * @param integer $maxCount
  * @access public
  * @return string
  */
  function getOutputLastEntriesInCateg($categId, $maxCount) {
    $result = '';
    $this->loadLastEntriesInCateg($categId, $maxCount);
    if (isset($this->entries) && is_array($this->entries)) {
      foreach($this->entries as $entry) {
        $result .= $this->getOutputEntry($entry);
      }
    }
    return $result;
  }

  /**
  * escape unknown tags
  *
  * @param string $tagSource
  * @access public
  * @return string
  */
  function escapeHTMLTags($match) {
    if (isset($match[2]) && $match[2] != '' && in_array($match[2], $this->allowedTags)) {
      $result = '<'.$match[1].'>';
    } else {
      $result = htmlspecialchars($match[0]);
    }
    return $result;
  }

  /**
  * Remove evil tags
  *
  * @param string $source
  * @access public
  * @return string
  */
  function removeEvilTags($source) {
    return preg_replace_callback('~<(/?(.*?))>|[<>]~i', array(&$this, 'escapeHTMLTags'), $source);
  }

  /**
  * Initialize search formular
  *
  * @param array $conf
  * @access public
  */
  function initializeSearchForm($conf) {
    if (!(isset($this->searchDialog) && is_object($this->searchDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'cmd'=>'search',
        'save'=>1,
        'offset'=>0,
        'entry_id'=>0,
        'search'=>TRUE
      );
      $fields = array(
        'searchfor' => array('Search', 'isNoHTML', TRUE, 'input', 200),
      );

      $this->searchDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->searchDialog->msgs = &$this->msgs;
      $this->searchDialog->loadParams();
    }
  }

  /**
  * Get search formular
  *
  * @param string $caption
  * @param array $conf
  * @access public
  * @return string
  */
  function getSearchForm($caption, $conf) {
    $this->initializeOutputForm($conf);
    $this->searchDialog->baseLink = $this->baseLink;
    $this->searchDialog->dialogTitle = htmlspecialchars($caption);
    $this->searchDialog->dialogDoubleButtons = FALSE;
    return '<searchdlg>'.$this->searchDialog->getDialogXML().'</searchdlg>';
  }

  /**
  * Inititalize output formular
  *
  * @param array $conf
  * @access public
  */
  function initializeOutputForm($conf) {
    if (!(isset($this->outputDialog) && is_object($this->outputDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');

      if (is_object($GLOBALS['PAPAYA_SURFER']) && $GLOBALS['PAPAYA_SURFER']->isValid) {
        $data['entry_username'] = $GLOBALS['PAPAYA_SURFER']->surferGivenname.' '.$GLOBALS['PAPAYA_SURFER']->surferSurname;
        $data['entry_useremail'] = $GLOBALS['PAPAYA_SURFER']->surferEmail;
        $usernameInput = FALSE;
      } else {
        $usernameInput = TRUE;
      }

      if (isset($this->entry) && isset($this->entry['entry_subject'])) {
        if (0 === strpos(strtolower($this->entry['entry_subject']), 're:')) {
          $data['entry_subject'] = $this->entry['entry_subject'];
        } else {
          $data['entry_subject'] = 'Re: '.$this->entry['entry_subject'];
        }
      } else {
        $data['entry_subject'] = '';
      }

      $hidden = array('cmd'=>'add_entry', 'save'=>1,
        'entry_id'=>(int)@$this->entry['entry_id'],
        'thread_id'=>  (((int)@$this->entry['entry_tid'] > 0) ? (int)@$this->entry['entry_tid'] : (int)@$this->entry['entry_id']),
        'forum_id'=>(int)@$this->params['forum_id'],
        'offset'=>(int)@$this->params['offset'],
        'bbsoffset'=>(int)@$this->params['bbsoffset']);
      $fields = array(
        'entry_subject' => array('Subject', 'isNoHTML', TRUE, 'input', 200),
        'entry_text' => array('Message', 'isSomeText', TRUE, 'textarea', 8),
        'entry_username' => array('Username', 'isNoHTML', TRUE, (($usernameInput) ? 'input' : 'disabled_input'), 100),
        'entry_useremail' => array('EMail', 'isEMail', FALSE, (($usernameInput) ? 'input' : 'disabled_input'), 150)
      );
      if ($conf['sendmails'] && (is_object($GLOBALS['PAPAYA_SURFER']) && $GLOBALS['PAPAYA_SURFER']->isValid)) {
        $fields['entry_sendanswers'] = array('Send answers', 'isNum', FALSE, 'yesno', 1, '', 0, 'left');
      }

      $this->outputDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->outputDialog->dialogTitle = ((int)@$this->entry['entry_tid'] > 0) ?
        $conf['caption_newanswer'] : $conf['caption_newquestion'];
      $this->outputDialog->msgs = &$this->msgs;
      $this->outputDialog->loadParams();
      if (!$usernameInput) {
        $this->outputDialog->params['entry_username'] =
          $GLOBALS['PAPAYA_SURFER']->surferGivenname.' '.$GLOBALS['PAPAYA_SURFER']->surferSurname;
        $this->outputDialog->params['entry_useremail'] =  $GLOBALS['PAPAYA_SURFER']->surferEmail;
      }
    }
  }

  /**
  * Get output formular
  *
  * @param string $caption
  * @param array $conf
  * @access public
  * @return string
  */
  function getOutputForm($caption, $conf) {
    if ((isset($this->entry) && is_array($this->entry) && $conf['post_anwers']) ||
        ((!(isset($this->entry) && is_array($this->entry))) && $conf['post_questions'])) {
      $this->initializeOutputForm($conf);
      $this->outputDialog->baseLink = $this->baseLink;
      $this->outputDialog->dialogTitle = htmlspecialchars($caption);
      $this->outputDialog->dialogDoubleButtons = FALSE;
      $this->addLink = $this->buildLink(array(
        'cmd'=>'add_entry',
        'forum_id'=>(int)$this->board['forum_id'],
        'offset'=>(int)@$this->params['offset'],
        'entry_id'=> (isset($this->entry) ? $this->entry['entry_id'] : 0)
      ));
      if (!(isset($this->backLink) && $this->backLink != '')) {
        $this->backLink = $this->buildLink(array(
          'forum_id'=>(int)$this->board['forum_id'],
          'entry_id'=>(int)@$this->params['entry_id'],
          'thread_id'=>(int)@$this->params['thread_id'],
          'offset'=>(int)@$this->params['offset'],
          'bbsoffset'=>(int)@$this->params['bbsoffset']
        ));
      }

      $error = (isset($this->outputDialog->inputErrors)) ? ' error="error"' : '';

      return '<newdlg'.$error.'>'.$this->outputDialog->getDialogXML().'</newdlg>';
    }
    return '';
  }

  /**
  * Add entry
  *
  * @param integer $pid
  * @param integer $tid
  * @param array &$conf
  * @access public
  * @return mixed id or FALSE
  */
  function addEntry($pid, $tid, &$conf) {
    $data = array(
      'forum_id'          => (int)$this->params['forum_id'],
      'entry_pid'         => (int)$pid,
      'entry_tid'         => (int)$tid,
      'entry_subject'     => @(string)$this->params['entry_subject'],
      'entry_text'        => @(string)$this->params['entry_text'],
      'entry_username'    => @(string)$this->params['entry_username'],
      'entry_useremail'   => @(string)$this->params['entry_useremail'],
      'entry_sendanswers' => (int)$this->params['entry_sendanswers'],
      'entry_created'     => time(),
      'entry_modified'     => time(),
      'entry_thread_modified' => time()
    );

    srand((double)microtime()*1000000);
    $rand = uniqid(rand());
    $data['entry_sendmarker'] = substr(crypt($rand, 'PA'), 2, 8);

    if (is_object($GLOBALS['PAPAYA_SURFER']) && $GLOBALS['PAPAYA_SURFER']->isValid) {
      $data['entry_username'] = $GLOBALS['PAPAYA_SURFER']->surferGivenname.' '.$GLOBALS['PAPAYA_SURFER']->surferSurname;
      $data['entry_userid'] = $GLOBALS['PAPAYA_SURFER']->surferId;
      $data['entry_useremail'] = $GLOBALS['PAPAYA_SURFER']->surferEmail;
      $data['entry_userregistered'] = TRUE;
    }
   
    if ($newId = $this->databaseInsertRecord($this->tableEntries, 'entry_id', $data)) {
      $data['entry_id'] = $newId;
      if ($tid > 0) {
        $this->updateThread($tid);
        $this->sendNotifies($data, $conf);
      }
      $this->sendAdminNotify($data, $conf);
      return $newId;
    }
    return FALSE;
  }

  function checkDoublePost($pid) {
    $sql = "SELECT COUNT(*) 
              FROM %s
             WHERE forum_id = '%d'
               AND entry_pid = '%d'
               AND entry_subject = '%s'
               AND entry_text = '%s'
               AND entry_username = '%s'
               AND entry_useremail = '%s'";
    $params = array(
      $this->tableEntries,
      (int)$this->params['forum_id'],
      (int)$pid,
      @(string)$this->params['entry_subject'],
      @(string)$this->params['entry_text'],
      @(string)$this->params['entry_username'],
      @(string)$this->params['entry_useremail']      
    );
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      list($counted) = $res->fetchRow();
      return $counted == 0;
    }    
    return TRUE;
  }
  
  /**
  * Add thread
  *
  * @param integer $forumId
  * @param string $title
  * @param string $teaser
  * @param integer $topicId optional, default value 0
  * @access public
  * @return mixed id or FALSE
  */
  function addThread($forumId, $title, $teaser, $topicId = 0) {
    $data = array(
      'forum_id'          => (int)$forumId,
      'entry_pid'         => 0,
      'entry_tid'         => 0,
      'entry_subject'     => $title,
      'entry_text'        => $teaser,
      'entry_username'    => PAPAYA_PROJECT_TITLE,
      'entry_useremail'   => '',
      'entry_sendanswers' => 0,
      'entry_created'     => time(),
      'entry_modified'     => time(),
      'entry_thread_modified' => time(),
      'page_id'           => (int)$topicId,
      'entry_sendmarker'  => '',
      'entry_userregistered' => FALSE
    );
    if ($newId = $this->databaseInsertRecord($this->tableEntries, 'entry_id', $data)) {
      return $newId;
    }
    return FALSE;
  }

  /**
  * build link
  *
  * @param mixed $params optional, default value NULL
  * @param mixed $file optional, default value NULL
  * @param string $seperator optional, default value '&amp;'
  * @access public
  * @return string
  */
  function buildLink($params = NULL, $file = NULL, $seperator = '&amp;') {
    $link = (isset($file)) ? $file : $this->baseLink;
    $sql = '';
    if (isset($params) && is_array($params)) {
      foreach($params as $key=>$val) {
        $sql .= $seperator.$this->paramName.'['.urlencode($key).']='.urlencode($val);
      }
    }
    if ($sql != '') {
      return $link.'?'.substr($sql, strlen($seperator));
    } else {
      return $link;
    }
  }

  /**
  * send notifies
  *
  * @param array &$entry
  * @param array &$conf
  * @access public
  */
  function sendNotifies(&$entry, &$conf) {
    if (isset($entry) && is_array($entry) && $conf['sendmails']) {
      if ($entry['entry_tid'] > 0 && $entry['entry_tid'] != $entry['entry_id']) {
        $sql = "SELECT
                    s.surfer_id, e.entry_useremail, s.surfer_email,
                    e.entry_username, s.surfer_givenname, s.surfer_surname,
                    e.entry_sendmarker, e.entry_id
                  FROM %s AS e
                  LEFT OUTER JOIN %s AS s ON ((entry_userregistered) AND s.surfer_id = e.entry_userid)
                  WHERE (e.entry_tid = %d) AND (NOT(entry_useremail = '')) AND entry_sendanswers = 1";
        $params = array($this->tableEntries, $this->tableSurfers, (int)$entry['entry_tid'], (int)$entry['entry_tid']);
        if ($res = $this->databaseQueryFmt($sql, $params)) {
          while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            if ($row['surfer_id'] > 0) {
              $row['email'] = $row['surfer_email'];
              $row['user'] = $row['surfer_givenname'].' '.$row['surfer_surname'];
            } else {
              $row['email'] = $row['entry_useremail'];
              $row['user'] = $row['entry_username'];
            }
            if (!($row['email'] == $entry['entry_useremail'])) {
              $addresses[$row['email']] = $row;
            }
          }
          if (isset($addresses) && is_array($addresses) && count($addresses) > 0) {
            if (preg_match_all('/%([\w]+)%/', $conf['mailtext'], $regs)) {
              $placeHolder = $regs[0];
            }
            if (isset($placeHolder) && is_array($placeHolder)) {
              foreach($placeHolder as $key=>$val) {
                $tmp[$val] = '';
              }
            }
            $placeHolder = &$tmp;
            foreach($addresses as $email=>$val) {
              $repl = $placeHolder;
              $repl['%NAME%'] = $val['user'];
              $repl['%EMAIL%'] = $email;
              $repl['%TITLE%'] = $entry['entry_subject'];
              $repl['%PROJECT%'] = PAPAYA_PROJECT_TITLE;
              $repl['%LINK%'] = 'http://'.$_SERVER['HTTP_HOST'].$this->getBasePath().$this->baseLink.
                '?'.$this->paramName.'_id='.$entry['entry_id'];
              $repl['%NOMAILURL%'] = 'http://'.$_SERVER['HTTP_HOST'].$this->getBasePath().$this->baseLink.
                '?'.$this->paramName.'_id='.$val['entry_id'].
                '&'.$this->paramName.'_nomail='.urlencode($val['entry_sendmarker']);
              $repl['%TEXT%'] = $entry['entry_text'];
              $msg = strtr($conf['mailtext'], $repl);
              unset($repl);
              $to = $name.' <'.$email.'>';
              $addHeader = 'From:'.$conf['mailfrom_name'].' <'.$conf['mailfrom_email'].">\n";
              $addHeader .= 'Content-type: text/plain; charset="ISO-8859-1"';
              $subject = $this->escapeEmailHeader($conf['mailsubject']);
              if (!mail($to, $subject, $msg, $addHeader)) {
                $this->logMsg(MSG_WARNING, 6,
                  $this->_gtf('Can not send mail to %s.', array($to)),
                  $this->_gtf('Can not send mail to %s.', array($to)));
              }
            }
          }
        }
      }
    }
  }

  function sendAdminNotify(&$entry, &$conf) {
    if (isset($entry) && is_array($entry) && $conf['admin_sendmails'] && checkit::isEmail($conf['admin_email'])) {
      if (preg_match_all('/%([\w]+)%/', $conf['mailtext'], $regs)) {
        $placeHolder = $regs[0];
      }
      if (isset($placeHolder) && is_array($placeHolder)) {
        foreach($placeHolder as $key=>$val) {
          $tmp[$val] = '';
        }
      }
      $placeHolder = &$tmp;
      $email = $conf['admin_email'];
      $repl = $placeHolder;
      $repl['%NAME%'] = $conf['admin_name'];
      $repl['%EMAIL%'] = $email;
      $repl['%TITLE%'] = $entry['entry_subject'];
      $repl['%PROJECT%'] = PAPAYA_PROJECT_TITLE;
      $repl['%LINK%'] = 'http://'.$_SERVER['HTTP_HOST'].$this->getBasePath().$this->baseLink.
        '?'.$this->paramName.'_id='.$entry['entry_id'];
      $repl['%NOMAILURL%'] = '';
      $repl['%TEXT%'] = $entry['entry_text'];
      $msg = strtr($conf['mailtext'], $repl);
      unset($repl);
      $to = $conf['admin_name'].' <'.$email.'>';
      if (checkit::isEmail($entry['entry_useremail'], TRUE)) {
        $addHeader = 'From: ('.$conf['mailfrom_name'].') '.strip_tags($entry['entry_username']).' <'.$entry['entry_useremail'].">\n";
      } else {
        $addHeader = 'From:'.$conf['mailfrom_name'].' <'.$conf['mailfrom_email'].">\n";
      }
      $addHeader .= 'Content-type: text/plain; charset="ISO-8859-1"';
      $subject = $this->escapeEmailHeader($conf['mailsubject']);
      if (!mail($to, $subject, $msg, $addHeader)) {
        $this->logMsg(MSG_WARNING, 6,
          $this->_gtf('Can not send mail to %s.', array($to)),
          $this->_gtf('Can not send mail to %s.', array($to)));
      }
    }
  }

  function escapeEmailHeader($subject) {
    $result = rawurlencode($subject);
    $result = strtr($result, '%', '=');
    return '=?iso-8859-1?Q?'.$result.'?=';
  }

  /**
  * Get deactivate notify formular
  *
  * @param string $marker
  * @param string $msg
  * @access public
  * @return string
  */
  function getDeactivateNotifyForm($marker, $msg) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'notifychg',
      'entry_id' => $this->entry['entry_id'],
      'entry_tid' => $this->entry['entry_tid'],
      'forum_id' => $this->entry['forum_id'],
      'nomail' => $marker,
      'confirm_change' =>1,
    );
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->baseLink = $this->baseLink;
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'OK';
    return $dialog->getMsgDialog();
  }

  /**
  * Check notify
  *
  * @param integer $threadId
  * @param string $marker
  * @access public
  * @return boolean
  */
  function checkNotify($threadId, $marker) {
    $sql = "SELECT COUNT(*) FROM %s WHERE entry_tid = %d AND entry_sendmarker = '%s' AND entry_sendanswers = 1";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries, $threadId, $marker))) {
      if ($row = $res->fetchRow()) {
        return ($row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Deactivate notify
  *
  * @param integer $threadId
  * @param string $marker
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deactivateNotify($threadId, $marker) {
    $sql = "SELECT entry_userid, entry_useremail FROM %s WHERE entry_tid = %d AND entry_sendmarker = '%s' AND entry_sendanswers = 1";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries, $threadId, $marker))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['entry_userid'] > 0) {
          return $this->databaseUpdateRecord($this->tableEntries, array('entry_sendanswers' => 0),
            sprintf('entry_tid = %d AND (entry_userid = %d OR entry_useremail = "%s")',
              $threadId, $row['entry_userid'], $row['entry_useremail']));
        } else {
          return $this->databaseUpdateRecord($this->tableEntries, array('entry_sendanswers' => 0),
            sprintf('entry_tid = %d AND entry_useremail = "%s"',
              $threadId, $row['entry_useremail']));
        }
      }
    }
    return FALSE;
  }

  /**
  * Get user data link
  *
  * @param integer $userId
  * @access public
  * @return string
  */
  function getUserDataLink($userId) {
    if (defined('PAPAYA_PAGEID_USERDATA') && PAPAYA_PAGEID_USERDATA > 0) {
      return $this->getBaseLink(PAPAYA_PAGEID_USERDATA).'?user_id='.$userId;
    }
    return '';
  }

  /**
  * Cut Categ to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutCateg () {
    $data = array('forumcat_prev' => -1);
    return (FALSE !== $this->databaseUpdateRecord($this->tableCategs,
      $data, 'forumcat_id', (int)$this->params['categ_id'])
    );
  }

  /**
  * Cut Forum to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutForum () {
    $data = array('forumcat_id' => -1);
    return (FALSE !== $this->databaseUpdateRecord($this->tableBoards,
      $data, 'forum_id', (int)$this->params['forum_id'])
    );
  }

  /**
  * Cut Thread to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutThread () {
    $data = array('forum_id'=>'-1');
    $cond = array('entry_id' => $this->params['thread_id'],
      'or', 'entry_tid' => $this->params['thread_id']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableEntries, $data, $cond));
  }

  /**
  * Check if exists cuted Elements
  *
  * @access public
  * @return boolean
  */
  function cutOutExists () {
    $exists = FALSE;
    $sql = "SELECT count(*) FROM %s WHERE forumcat_prev = '%d'";
    $params = array($this->tableCategs, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }

    $sql = "SELECT count(*) FROM %s WHERE forumcat_id = '%d'";
    $params = array($this->tableBoards, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }

    $sql = "SELECT count(*) FROM %s WHERE forum_id = '%d'";
    $params = array($this->tableEntries, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }
    return $exists;
  }

  /**
  * Get XML for Clipboard
  *
  * @access public
  */
  function getXMLCutedList () {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('Clipboard'));
      $result .= '<items>'.LF;
      $result .= $this->getXMLCutedElements();
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
  }

  /**
  * Get XML of elements for clipboard
  *
  * @access public
  * @return string $result XML
  */
  function getXMLCutedElements () {
    $result = '';
    if (isset($this->cuttedCategs) && is_array($this->cuttedCategs)){
      foreach($this->cuttedCategs as $cuttedCateg) {
        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedCateg['forumcat_title'], $this->images[56]
        );
        $result .= sprintf('<subitem align="right">'.LF);
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_categ',
            'cut_categ_id'=>(int)$cuttedCateg['forumcat_id'],
            'categ_id'=>$this->params['categ_id']
          ))
        );
        $result .= sprintf('<glyph src="paste.gif"/>'.LF);
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }
    if (isset($this->cuttedList) && is_array($this->cuttedList)){
      foreach($this->cuttedList as $cuttedForum) {

        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedForum['forum_title'],
          $this->images[56]
        );
        $result .= sprintf('<subitem align="%s">'.LF, "right");
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_forum',
            'cut_forum_id'=>(int)$cuttedForum['forum_id'])
          )
        );
        $result .= sprintf('<glyph src="%s"/>'.LF, 'paste.gif');
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }

    if (isset($this->cuttedThreads) && is_array($this->cuttedThreads)){
      foreach($this->cuttedThreads as $cuttedThread) {

        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedThread['entry_subject'],
          $this->images[94]
        );
        $result .= sprintf('<subitem align="%s">'.LF, "right");
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_thread',
            'cut_thread_id'=>(int)$cuttedThread['entry_id'])
          )
        );
        $result .= sprintf('<glyph src="%s"/>'.LF, 'paste.gif');
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }
    return $result;
  }

  /**
  * Load Elements from clipboard
  *
  * @access public
  */
  function loadCutedElement () {
    $sql = 'SELECT forumcat_id,
                    forumcat_prev,
                    forumcat_title
              FROM %s
              WHERE forumcat_prev = %d';
    $params = array($this->tableCategs, -1);

    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedCategs[(int)$row['forumcat_id']] = $row;
      }
    }

    $sql = 'SELECT forum_id,
                    forumcat_id,
                    forum_title
              FROM %s
              WHERE forumcat_id = %d';
    $params = array($this->tableBoards, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedList[(int)$row['forum_id']] = $row;
      }
    }

    $sql = 'SELECT entry_id,
                    entry_subject
              FROM %s
              WHERE forum_id = %d
              AND entry_pid = %d';
    $params = array($this->tableEntries, -1, 0);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedThreads[(int)$row['entry_id']] = $row;
      }
    }
  }

  /**
  * Load Categs by Ids from Array
  *
  * @param array $categIds
  * @access public
  * @return boolean
  */
  function loadCategsByIds($categIds) {
    if (count($categIds) > 1) {
      $filter = ' IN ('.implode(',', $categIds).')';
    } else {
      $filter = ' = '.@(int)$categIds[0];
    }
    $sql ='SELECT  forumcat_id,
                    forumcat_title
              FROM %s
            WHERE forumcat_id '.$filter.'
          ORDER BY forumcat_path';
    $params = array($this->tableCategs);

    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->categsById[(int)$row['forumcat_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Get XML Categorynavigation
  *
  * @access public
  * @return string $string XML
  */
  function getXMLNavigation() {
    $string = '';
    //get Navigation
    if (isset($this->categsById) && is_array($this->categsById)) {
      $string .= sprintf('<pathnavi>'.LF);
      foreach ($this->categsById as $categ) {
        $string .= sprintf('<categ title="%s" href="%s" />'.LF,
          $categ['forumcat_title'],
          $this->getLink(array('categ_id' => $categ['forumcat_id']))
        );
      }
      $string .= sprintf('</pathnavi>'.LF);
    }
    return $string;
  }
}
?>