<?php
/**
* Generate an progressbar using an background and a bar image
*
* image plugins must be inherited from this superclass
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: image_progressbar.php,v 1.8 2006/04/05 11:29:40 weinert Exp $
*/

/**
* Base class for image plugins
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_dynamicimage.php');

/**
* Generate an progressbar using an background and a bar image
*
* image  plugins must be inherited from this superclass
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class image_progressbar extends base_dynamicimage {
  
  var $editFields = array(
    'limit_min' => array ('Minimum (0%)', 'isNum', TRUE, 'input', 4, '', 0),
    'limit_max' => array ('Maximum (100%)', 'isNum', TRUE, 'input', 4, '', 10),
    'Images',
    'image_fg' => array ('Foreground', 'isSomeText', TRUE, 'imagefixed', 400),
    'image_bg' => array ('Background', 'isSomeText', FALSE, 'imagefixed', 400), 
    'Background',
    'padding'  => array ('Padding / Space', 'isNum', TRUE, 'input', 4, '', 1),
    'color_bg' => array ('Color', 'isHTMLColor', TRUE, 'color', 7, '', '#FFFFFF'),
  );
  
  var $attributeFields = array(
    'position' => array ('Current position', 'isNum', TRUE, 'input', 4, '', 0),
  );

  /**
  * generate the image
  *
  * @param object base_imagegenerator &$controller controller object
  * @access public
  * @return resource image
  */
  function &generateImage(&$controller) {
    if ($imageFG = &$controller->getMediaFileImage(@$this->data['image_fg'])) {
      $steps = @(int)$this->data['limit_max'] - @(int)$this->data['limit_min'];
      if ($steps < 1) {
        $steps = 1;
      }
      //calc size and create image
      $height = imagesy($imageFG) + (@(int)$this->data['padding'] * 2);
      $width = ((imagesx($imageFG) + @(int)$this->data['padding']) * $steps) + @(int)$this->data['padding'];
      $result = imagecreate($width, $height);
      
      // background color
      $bgColor = $controller->colorToRGB(@$this->data['color_bg']);
      $bgColorIdx = imagecolorallocate($result, $bgColor['red'], $bgColor['green'], $bgColor['blue']);
      imagefilledrectangle($result, 0, 0, $width, $height, $bgColorIdx);
      
      // background image
      if ($imageBG = &$controller->getMediaFileImage(@$this->data['image_bg'])) {
        for ($x = @(int)$this->data['padding']; ($x <= $width); $x += imagesx($imageBG) + @(int)$this->data['padding']) {
          imagecopy($result, $imageBG, $x, @(int)$this->data['padding'], 0, 0, imagesx($imageBG), imagesy($imageBG));
        }
      }
      
      
      $imgCount = (int)$this->attributes['position'] - @(int)$this->data['limit_min'];
      
      // progress
      for ($i = 0; ($i < $imgCount && $i < $steps); $i++) {
        $x = ($i * (imagesx($imageFG) + @(int)$this->data['padding'])) +  @(int)$this->data['padding'];
        imagecopy($result, $imageFG, $x, @(int)$this->data['padding'], 0, 0, imagesx($imageFG), imagesy($imageFG));
      }         
      
      return $result;
    } else {
      $this->lastError = 'Can not load foreground image';
    }
    $result = FALSE;
    return $result;
  }
}
?>