<?php
/**
* Page module - Site to send page link
*
* Configured by xml-file
*
* @package module_mail
* @author Bodo Grassmann <grassmann@papaya-cms.com>
* @version $Id: content_pagesend.php,v 1.25 2006/04/25 09:53:56 weinert Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');

/**
* Library for sting validation
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Send pages via email
*
* Configured by xml-file
*
* @package module_mail
* @author Bodo Grassmann <grassmann@papaya-cms.com>
*/
class content_pagesend extends base_content {

  /**
  * Linked topic
  * @var
  */
  var $linkedTopic = NULL;

  /**
  * cacheable ?
  * @var boolean
  */
  var $cacheable = FALSE;

  /**
  * Parameter name
  * @var string
  */
  var $paramName = 'mail';

  /**
  * Module actions
  * @var array
  */
  var $moduleActions = array(
    1 => 'article sent',
  );

  /**
  * Content edit fields
  * @var array
  */
  var $editFields = array(
    'title' => array('Page Title', 'isSomeText', FALSE, 'input', 200, '', ''),
    'Email',
    'msg_subject' => array('Subject template of email', 'isNoHTML', false, 'input', 200, '', ''),
    'msg_body' => array('Body template of email', 'isNoHTML', false, 'textarea', 20, '', "{%TITLE%}\n\n{%COMMENT%}\n\n{%FROM%}\n\n{%TEXT%}\n\n{%LINK%}"),
    'Captions',
    'cap_recipient' => array('Recipient', 'isNoHTML', false, 'input', 200, '', 'Recipient'),
    'cap_sender' => array('Sender', 'isNoHTML', false, 'input', 200, '', 'Your Email Adress'),
    'cap_comments' => array('Comments', 'isNoHTML', false, 'input', 200, '', 'Comment'),
    'Messages',
    'msg_hello'=> array('Hello', 'isSomeText', FALSE, 'textarea', 6, '', ''),
    'msg_send'=> array('Confirmation', 'isSomeText', FALSE, 'textarea', 6, '',
      'Message sent. Thank You.'),
    'msg_error'=> array('Input Error', 'isSomeText', FALSE, 'textarea', 6, '',
      'Please check your input.'),
    'msg_notsend'=> array('Send Error', 'isSomeText', FALSE, 'textarea', 6, '',
      'Sent Error. Please try again.'),
    'msg_notopic'=> array('Topic Error', 'isSomeText', FALSE, 'textarea', 6, '',
      'No page selected'),
    'msg_privacy' => array('Privacy', 'isSomeText', FALSE, 'textarea', 10, '', ''),
  );

  /**
  * Get mail data
  *
  * @param integer $topicId
  * @access public
  * @return array
  */
  function getMailData($topicId){
    $content = NULL;
    if (isset($GLOBALS['PAPAYA_PAGE'])) {
      $className = get_class($this->topicObj);
      $this->linkedTopic = &new $className();
      if ($this->linkedTopic->topicExists($topicId) &&
          $this->linkedTopic->loadOutput($topicId, $this->topicObj->getContentLanguageId())) {
        if ($GLOBALS['PAPAYA_PAGE']->validateAccess($topicId)) {
          if ($str = $this->linkedTopic->parseContent(FALSE)) {
            $content['xml'] = $str;
            $xmlTree = simple_xmltree::create();
            $xmlTree->loadFromString($str);
            if (is_object($xmlTree->root)) {
              foreach ($xmlTree->root->children as $node) {
                if (is_object($node)) {
                  switch ($node->name) {
                    case 'text' :
                    case 'title' :
                      $s = '';
                      $node->printText($s);
                      $content[strtoupper($node->name)] = $s;
                    break;
                  }
                }
              }
            }
          }
        }
      }
    }
    return $content;
  }


  /**
  * Get parsed data
  *
  * @access public
  * @return string xml
  */
  function getParsedData() {
    $this->initializeParams();
    include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
    $this->emailObj = new email();

    if (isset($_GET['refpage']) && $_GET['refpage'] > 0){
      $this->params['refpage'] = $_GET['refpage'];
      $refererPage = $this->params['refpage'];
    } elseif (isset($this->params['refpage']) && $this->params['refpage'] > 0) {
      $refererPage = $this->params['refpage'];
    } else {
      $refererPage = 0;
    }

    if (isset($_GET['urlparams']) && trim($_GET['urlparams']) != '') {
      $this->params['urlparams'] = $_GET['urlparams'];
      $urlParams = $this->params['urlparams'];
    } elseif (isset($this->params['urlparams'])) {
      $urlParams = $this->params['urlparams'];
    } else {
      $urlParams = '';
    }

    $result  = '';

    if (isset($this->data['title'])) {
      $result .= sprintf('<title>%s</title>', @papaya_strings::escapeHTMLChars($this->data['title']));
    }
    $result .= '<mail>';

    $content = $this->getMailData($refererPage);
    if (isset($content) && is_array($content)) {
      $result .= $content['xml'];

      if (isset($this->params['submit']) &&  $this->params['submit']) {
        $errors = $this->checkMailDialogInput();
        if (!$errors) {
          $content['LINK'] = $this->getAbsoluteURL($this->getWebLink($refererPage)).$this->recodeQueryString($urlParams);
          $content['FROM'] = $this->params['mail_from'];
          $content['COMMENT']= $this->params['mail_comments'];
          if (isset($this->params['mail_from'])) {
            $this->emailObj->setHeader('From',$this->params['mail_from']);
            $this->emailObj->setHeader('ReplyTo',$this->params['mail_from']);
          }
          $this->emailObj->setTemplate('subject',$this->data['msg_subject'], $content);
          $this->emailObj->setTemplate('body',$this->data['msg_body'], $content);
          if($this->emailObj->send($this->params['mail_to'])){
            if (defined('PAPAYA_PAGE_STATISTIC') && PAPAYA_PAGE_STATISTIC) {
              include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic_actions.php');
              $statisticActionObj = &new base_statistic_actions();
              $statisticActionObj->initializeStatisticObj();
              $statisticActionObj->logAction($this->guid, 1, $this->moduleActions[1], $refererPage, $this->recodeQueryString($urlParams));
            }
            $result .= sprintf('<message type="normal">%s</message>',@$this->getXHTMLString($this->data['msg_send']));
          }else{
            $result .= sprintf('<message type="warning">%s</message>',$this->getXHTMLString($this->data['msg_notsend']));
          }
        } else {
          $result .= '<message type="error">';
          $result .= @$this->getXHTMLString($this->data['msg_error']);
          if (is_array($errors)) {
            $result .= '<ul>';
            foreach ($errors as $fieldCaption) {
              $result .= sprintf('<li>%s</li>', papaya_strings::escapeHTMLChars($fieldCaption));
            }
            $result .= '</ul>';
          }
          $result .= '</message>';
          $result .= $this->getMailDialog($refererPage, $urlParams);
        }
      } else {
        $result .= sprintf('<message type="normal">%s</message>', @$this->getXHTMLString($this->data['msg_hello']));
        $result .= $this->getMailDialog($refererPage, $urlParams);
      }
    } else {
      $result .= sprintf('<message type="warning">%s</message>', @$this->getXHTMLString($this->data['msg_notopic']));
    }
    $result .= sprintf('<privacy>%s</privacy>', @$this->getXHTMLString($this->data['msg_privacy']));
    $result .= '</mail>';
    return $result;
  }

  /**
  * Get mail dialog
  *
  * @param string $refererPage
  * @param string $urlParams
  * @access public
  * @return string xml
  */
  function getMailDialog($refererPage, $urlParams) {
    $result = sprintf('<form action="%s" method="post">', $this->baseLink);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
      $this->paramName, 'submit', 1);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
      $this->paramName, 'refpage', (int)$refererPage);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />',
      $this->paramName, 'urlparams', papaya_strings::escapeHTMLChars($urlParams));
    $result .= sprintf('<label for="%s[%s]">%s</label>',
      $this->paramName, 'mail_to', papaya_strings::escapeHTMLChars(@$this->data['cap_recipient']));
    $result .= sprintf('<input type="text" name="%s[%s]" value="%s" class="text" />',
      $this->paramName, 'mail_to', papaya_strings::escapeHTMLChars(@$this->params['mail_to']));
    $result .= sprintf('<label for="%s[%s]">%s</label>',
      $this->paramName, 'mail_from', papaya_strings::escapeHTMLChars(@$this->data['cap_sender']));
    $result .= sprintf('<input type="text" name="%s[%s]" value="%s" class="text" />',
      $this->paramName, 'mail_from', papaya_strings::escapeHTMLChars(@$this->params['mail_from']));
    $result .= sprintf('<label for="%s[%s]">%s</label>',
      $this->paramName, 'mail_comments', papaya_strings::escapeHTMLChars(@$this->data['cap_comments']));
    $result .= sprintf('<textarea name="%s[%s]" class="text">%s</textarea>',
      $this->paramName, 'mail_comments', papaya_strings::escapeHTMLChars(@$this->params['mail_comments']));
    $result .= '</form>';
    return $result;
  }

  /**
  * Check mail dialog input
  *
  * @access public
  * @return array
  */
  function checkMailDialogInput() {
    $result = FALSE;
    if (!checkit::isEmail(@$this->params['mail_to'], TRUE)) {
      $result['mail_to'] = @$this->data['cap_recipient'];
    }
    if (!checkit::isEmail(@$this->params['mail_from'], FALSE)) {
      $result['mail_from'] = @$this->data['cap_sender'];
    }
    if (!checkit::isSomeText(@$this->params['mail_comments'], TRUE)) {
      $result['mail_comments'] = @$this->data['cap_comments'];
    }
    return $result;
  }
}

?>