<?php
/**
* Page ranking administration
*
* @package module_pageranking
* @author Bodo Grassmann <grassmann@dimensional.de>
* @version $Id: admin_pagerating.php,v 1.18 2006/04/28 13:36:26 rekowski Exp $
*/

/**
* Basicclass for database access
*/
require_once(dirname(__FILE__)."/base_pagerating.php");
/**
* Page ranking administration
*
* @author Bodo Grassmann <grassmann@dimensional.de>
* @package module_pageranking
*/
class admin_pagerating extends base_pagerating {

  /**
  * topic parameter name
  * @var string
  */
  var $topicParamName = 'rate';

  /**
  * ???
  * @var array ?
  */
  var $voteSort = NULL;

  /**
  * Initial function for module
  *
  * @access public
  */
  function initialize() {
    $this->initializeParams();
    $this->sessionPageParams = $this->getSessionValue('PAPAYA_SESS_'.$this->topicParamName);
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('topic_id');
  }

  /**
  * Execute - basic function for handling parameters
  */
  /**
  * Parameter check
  *
  * @access public
  */
  function execute() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    switch (@$this->params['cmd']) {
      case 'add_categ' :
        if ($newId = $this->addCateg()) {
          $this->addMsg(MSG_INFO, $this->_gt('Category added.'));
          $this->params['categ_id'] = $newId;
          $this->initializeSessionParam('categ_id', array('cmd'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
        break;
      case 'load_statistic' :
        $this->loadStatisticData($this->lngSelect->currentLanguageId);
        break;
    }
    //ablegen der neuen session daten
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    if ($this->loadPageList($this->lngSelect->currentLanguageId)){
      $this->loadTopicTitle($this->lngSelect->currentLanguageId);
      $this->calculateRating();
      $this->getTopVoted($this->lngSelect->currentLanguageId);
    } else {
      $this->addMsg(MSG_WARNING, $this->_gt('Please refresh Statistic Data.'));
      $this->params['cmd'] = '';
    }
  }


  /**
  * Get xml for page output
  *
  * @access public
  */
  function getXML() {
    if (is_object($this->layout)) {
      $this->getXMLButtons();
      switch (@$this->params['cmd']) {
        case 'edit_poll' :
          $this->getXMLPollForm();
          break;
        default :
          $this->getPageInfo();
          $this->getXMLResultTree();
      }
    }
  }

  /**
  * Get List with topics
  *
  * @access public
  */
  function getPageInfo(){
    $tmp = 0;
    $this->layout->add($this->lngSelect->getComboFormXML(340));
    if (isset($this->params['topic_id']) && isset($this->pageList)
      && is_array($this->pageList) && isset($this->pageList[$this->params['topic_id']])){
      $result = sprintf('<listview title="%s">'.LF, $this->_gt('Pages Info'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Id'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Value'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $result .= sprintf('<listitem title="%s">'.LF, 'Page Title');
      $result .= sprintf('<subitem>%s</subitem>'.LF,
        $this->pageList[$this->params['topic_id']]['detail']['topic_title']);
      $result .= '</listitem>';
      $result .= sprintf('<listitem title="%s">'.LF, 'Page Id');
      $result .= sprintf('<subitem>%s</subitem>'.LF,
        $this->pageList[$this->params['topic_id']]['detail']['topic_id']);
      $result .= '</listitem>';
      $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>'.LF, $this->_gt('Total Votes'),
        $this->pageList[$this->params['topic_id']]['votes'], $countList);
      foreach($this->pageList[$this->params['topic_id']]['answer'] as $id => $answer){
        $title = ($id < 0) ? $this->_gt('Negative') : $this->_gt('Positive');
        $result .= sprintf('<listitem indent="1" title="%s (%d)"><subitem>%s</subitem></listitem>'.LF, $title, $id, $answer);
        if ($id == -1) {
          $result .= sprintf('<listitem indent="1" title="%s (%d)"><subitem>%s</subitem></listitem>'.LF, $this->_gt('Neutral'), 0,
            ($this->pageList[$this->params['topic_id']]['votes'] - array_sum($this->pageList[$this->params['topic_id']]['answer'])));
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->add($result);
    }
  }

  /**
  * Get top voted topic
  *
  * @param integer $lngId
  * @param integer $offset optional, default value 0
  * @param integer $rowCount optional, default value 100
  * @access public
  */
  function getTopVoted($lngId, $offset = 0, $rowCount = 100){
    $sql = "SELECT topic_id, count
              FROM %s
             WHERE lng_id = '%d'
               AND answer_id = 0
             ORDER BY count DESC, topic_id DESC
             LIMIT $offset, $rowCount";
    $params = array($this->tableAbstract, $lngId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->voteSort[$row['topic_id']] = TRUE;
      }
    }
  }

  /**
  * Page xml list
  *
  * @access public
  */
  function getXMLResultTree() {
    $tmp = 0;
    $result = sprintf('<listview title="%s" width="650">'.LF, $this->_gt('Pages'));
    if (isset($this->pageList) && is_array($this->pageList)) {
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Page'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Votes'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Rating'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      if (isset($this->params['cmd']) && $this->params['cmd'] == 'top_voted'){
        $parm = $this->voteSort;
      } else {
        $parm = $this->pageList;
      }
      foreach($parm as $id => $dummy){
        if (isset($this->params) && isset($this->params['topic_id'])){
          $selected = ($this->params['topic_id'] == $this->pageList[$id]['detail']['topic_id']) ? ' selected="selected"' : '';
        } else {
          $selected = '';
        }
        $result .= sprintf('<listitem href="%s" title="%s" %s>'.LF,
          $this->getLink(array('topic_id' => $this->pageList[$id]['detail']['topic_id'])),
          $this->pageList[$id]['detail']['topic_title'].'  #'.$this->pageList[$id]['detail']['topic_id'],
          $selected
        );
        $result .= sprintf('<subitem>%s</subitem>'.LF,
          $this->pageList[$id]['votes']
        );
        $result .= sprintf('<subitem>%s</subitem>'.LF,
          $this->pageList[$id]['rating'].'%'
        );
        $result .= '</listitem>';
      }
      $result .= '</items>'.LF;
    }
    $result .= '</listview>'.LF;
    $this->layout->addLeft($result);
  }

  /**
  * Get XML buttons
  *
  * @access public
  */
  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    $toolbar->addSeperator();
    if (isset($this->params['cmd']) && $this->params['cmd'] == 'top_voted'){
      $toolbar->addButton('Normal mode',
        $this->getLink(), 93
      );
    } else {
      $toolbar->addButton('Top voted',
        $this->getLink(array('cmd' => 'top_voted')), 93
      );
    }

    $toolbar->addButton('Refresh Statistic Data',
      $this->getLink(array('cmd' => 'load_statistic')), 93
    );

    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit', $str));
    }
  }
}
?>
