<?php
/**
* Generate an rating bar using an background and colors
*
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: image_pagerating.php,v 1.4 2006/04/05 11:55:10 weinert Exp $
*/

/**
* Base class for image plugins
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_dynamicimage.php');

/**
* Generate an rating bar using an background and colors
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class image_pagerating extends base_dynamicimage {
  
  var $editFields = array(
    'Colors',
    'color_left' => array ('Left', 'isHTMLColor', TRUE, 'color', 7, '', '#FF0000'),
    'color_right' => array ('Right', 'isHTMLColor', TRUE, 'color', 7, '', '#008000'),
    'color_grid' => array ('Grid', 'isHTMLColor', TRUE, 'color', 7, '', '#C0C0C0'),
    'Background',
    'image_bg' => array ('Image', 'isSomeText', TRUE, 'imagefixed', 400), 
    'Padding',    
    'padding_h'  => array ('Horizontal', 'isNum', TRUE, 'input', 4, '', 10),
    'padding_v'  => array ('Vertical', 'isNum', TRUE, 'input', 4, '', 10),
  );
  
  var $attributeFields = array(
    'position' => array ('Current position', 'isNum', TRUE, 'input', 3, '', 0),
  );

  /**
  * generate the image
  *
  * @param object base_imagegenerator &$controller controller object
  * @access public
  * @return resource image
  */
  function &generateImage(&$controller) {
    if ($imageBG = &$controller->getMediaFileImage(@$this->data['image_bg'])) {
      $width = imagesx($imageBG);
      $height = imagesy($imageBG);
      
      $left = @(int)$this->data['padding_h'];
      $top = @(int)$this->data['padding_v'];
      
      $center = floor($width / 2);
      $barWidth = $width - $left - $left;
      $barHeight = $height - $top - $top;
      
      $left--;
      $top--;
    
      $result = imagecreate($width, $height);
      imagecopy($result, $imageBG, 0, 0, 0, 0, $width, $height);
      
      //bar
      $barPosition = (int)$this->attributes['position'];
      if ($barPosition > 100) {
        $barPosition = 100;
      } elseif ($barPosition < -100) {
        $barPosition = -100;      
      }
      
      $barSize = floor(abs($barPosition / 2) * $barWidth / 100);
      
      if ($barPosition > 0) {
        $barColor = $controller->colorToRGB(@$this->data['color_right']);
        $barColorIdx = imagecolorallocate($result, $barColor['red'], $barColor['green'], $barColor['blue']);
        imagefilledrectangle($result, $center, $top, $center + $barSize, $top + $barHeight, $barColorIdx);
      } elseif ($barPosition < 0) {
        $barColor = $controller->colorToRGB(@$this->data['color_left']);
        $barColorIdx = imagecolorallocate($result, $barColor['red'], $barColor['green'], $barColor['blue']);
        imagefilledrectangle($result, $center - $barSize, $top, $center, $top + $barHeight, $barColorIdx);      
      }     
      
      $gridColor = $controller->colorToRGB(@$this->data['color_grid']);
      $gridColorIdx = imagecolorallocate($result, $gridColor['red'], $gridColor['green'], $gridColor['blue']);
      imagerectangle($result, $left, $top, $left + $barWidth, $top + $barHeight, $gridColorIdx);  
      
      //grid
      $gridColor = $controller->colorToRGB(@$this->data['color_grid']);
      $gridColorIdx = imagecolorallocate($result, $gridColor['red'], $gridColor['green'], $gridColor['blue']);
      imagerectangle($result, $left, $top, $left + $barWidth, $top + $barHeight, $gridColorIdx);   
      
      $gridTop = $top - 3;
      $gridBottom = $top + $barHeight + 3;
      imagefilledrectangle($result, $left, $gridTop, $left + 3, $gridBottom, $gridColorIdx);
      imagefilledrectangle($result, $center -1, $gridTop, $center + 1, $gridBottom, $gridColorIdx);
      imagefilledrectangle($result, $left + $barWidth - 3, $gridTop, $left + $barWidth, $gridBottom, $gridColorIdx);
      
      $gridTop = $top - 2;
      $gridBottom = $top + $barHeight + 2;
      $step = round($barWidth / 10);
      for ($i = $step; $i < ($barWidth / 2); $i += $step) {
        imageline($result, $center - $i, $gridTop, $center - $i, $gridBottom, $gridColorIdx);
        imageline($result, $center + $i, $gridTop, $center + $i, $gridBottom, $gridColorIdx);
      }
      
      return $result;
    } else {
      $this->lastError = 'Can not load background image';
    }
    $result = FALSE;
    return $result;
  }
}
?>