<?php
/**
* Cronjob-administration
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_cron.php,v 1.8 2006/04/07 14:43:26 weinert Exp $
*/

/**
* database access base class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Cronjob-administration
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_cronjobs extends base_db {

  var $params = NULL;
  var $paramName = 'cron';

  var $tableCronjobs = PAPAYA_DB_TBL_CRONJOBS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;

  var $cronjobs = NULL;

  /**
  * Initialisize function
  *
  * @param mixed $paramName optional, default value NULL
  * @access public
  */
  function initialize($paramName = NULL) {
    if (isset($paramName)) $this->paramName = $paramName;

    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('id');
    $this->initializeSessionParam('mode');
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Load function
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function load($id) {
    unset($this->cronjob);
    $sql = "SELECT cj.cronjob_id,  cj.cronjob_active, cj.cronjob_title, cj.cronjob_description,
                    cj.cronjob_lastexec, cj.cronjob_nextexec, cj.cronjob_start, cj.cronjob_end,
                    cj.cron_module_guid, cj.cron_data,
                    cj.job_module_guid, cj.job_data,
                    cm.module_class AS cron_module_class,
                    cm.module_title AS cron_module_title,
                    cm.module_path AS cron_module_path,
                    cm.module_file AS cron_module_file,
                    jm.module_class AS job_module_class,
                    jm.module_title AS job_module_title,
                    jm.module_path AS job_module_path,
                    jm.module_file AS job_module_file
            FROM %s AS cj
              LEFT OUTER JOIN %s AS cm
                ON (cj.cron_module_guid = cm.module_guid AND cm.module_active = 1)
              LEFT OUTER JOIN %s AS jm
                ON (cj.job_module_guid = jm.module_guid AND cm.module_active = 1)
            WHERE cronjob_id = '%d'";
    $params = array($this->tableCronjobs, $this->tableModules, $this->tableModules, $id);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['cron_module_file'] = $row['cron_module_path'].$row['cron_module_file'];
        $row['job_module_file'] = $row['job_module_path'].$row['job_module_file'];
        $row['cronjob_start_str'] = date('Y-m-d H:i', $row['cronjob_start']);
        $row['cronjob_end_str'] = date('Y-m-d H:i', $row['cronjob_end']);
        $this->cronjob = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Get next cronjob
  *
  * @access public
  * @return mixed
  */
  function getNext() {
    $sql = "SELECT cronjob_id, cronjob_nextexec
            FROM %s
            WHERE cronjob_active = 1 AND cronjob_nextexec <= '%d'
            ORDER BY cronjob_nextexec ASC";
    $params = array($this->tableCronjobs,time());
    if ($res = $this->databaseQueryFmt($sql, $params, 1)) {
      if($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return FALSE;
  }

  /**
  * Save changes of cronjob
  *
  * @access public
  * @return mixed
  */
  function save() {
    $values = array(  'cronjob_title' => $this->params['cronjob_title'],
                      'cronjob_description' => $this->params['cronjob_description'],
                      'cronjob_start' => $this->params['cronjob_start'],
                      'cronjob_end' => $this->params['cronjob_end'],
                      'cron_module_guid' => $this->params['cron_module_guid'],
                      'job_module_guid' => $this->params['job_module_guid'],
                      'cronjob_active' => '0'
              );
    return (FALSE !== $this->databaseUpdateRecord($this->tableCronjobs, $values, 'cronjob_id', (int)$this->params['id']));
  }

  /**
  * Save cronjob data
  *
  * @param array $data
  * @access public
  * @return mixed
  */
  function saveCronData($data) {
    $values = array (
                      'cron_data' => $data,
                      'cronjob_active' => '0'
              );
    return (FALSE !== $this->databaseUpdateRecord($this->tableCronjobs, $values, 'cronjob_id', (int)$this->params['id']));
  }

  /**
  * Save cronjob job data
  *
  * @param array $data
  * @access public
  * @return mixed
  */
  function saveJobData($data) {
    $values = array (
                      'job_data' => $data,
                      'cronjob_active' => '0'
              );
    return (FALSE !== $this->databaseUpdateRecord($this->tableCronjobs, $values, 'cronjob_id', (int)$this->params['id']));
  }

  /**
  * Activate cronjob
  *
  * @param integer $nextExecutionTime
  * @param integer $lastExecutionTime optional, default value 0
  * @param mixed $id optional, default value NULL
  * @access public
  * @return mixed
  */
  function activate($nextExecutionTime, $lastExecutionTime = 0, $id = NULL) {
    $cronjobId = (isset($id)) ? (int)$id : (int)$this->params['id'];
    $values = array(
      'cronjob_nextexec' => (int)$nextExecutionTime,
      'cronjob_active' => '1'
    );
    if (isset($lastExecutionTime) && $lastExecutionTime > 0) {
      $values['cronjob_lastexec'] = $lastExecutionTime;
    }
    return (FALSE !== $this->databaseUpdateRecord($this->tableCronjobs, $values, 'cronjob_id', $cronjobId));
  }

  /**
  * Deactivate Cronjob
  *
  * @param mixed $id optional, default value NULL
  * @param integer $lastExecutionTime optional, default value 0
  * @access public
  * @return mixed
  */
  function deactivate($id = NULL, $lastExecutionTime = 0) {
    $cronjobId = (isset($id)) ? (int)$id : (int)$this->params['id'];
    $values['cronjob_active'] = '0';
    if (isset($lastExecutionTime) && $lastExecutionTime > 0) {
      $values['cronjob_lastexec'] = $lastExecutionTime;
    }
    return (FALSE !== $this->databaseUpdateRecord($this->tableCronjobs, $values, 'cronjob_id', $cronjobId));
  }

  /**
  * Create cronjob
  *
  * @access public
  * @return mixed
  */
  function create() {
    $start = time();
    $end = $start+31536000;
    $values = array(  'cronjob_title' => $this->_gt('New cronjob'),
                      'cronjob_start' => $start,
                      'cronjob_end' => $end,
                      'cronjob_active' => '0'
              );
    if ($this->databaseInsertRecord($this->tableCronjobs, 'cronjob_id', $values)) {
      $sql = "SELECT cronjob_id FROM %s ORDER BY cronjob_id DESC";
      $params = array($this->tableCronjobs);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow()) {
          return $row[0];
        }
      }
    }
    return FALSE;
  }

  /**
  * Delete cronjob
  *
  * @access public
  * @return mixed
  */
  function delete() {
    return $this->databaseDeleteRecord($this->tableCronjobs, 'cronjob_id', (int)$this->params['id']);
  }

  /**
  * Check cronjob data
  *
  * @access public
  * @return boolean
  */
  function check() {
    $result = TRUE;
    if (@!checkit::isNoHTML($this->params['cronjob_title'], TRUE)) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR,
          sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Title')));
    }
    if (@!checkit::isNoHTML($this->params['cronjob_description'], TRUE)) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR,
          sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Description')));
    }
    if (@!checkit::isGUID($this->params['cron_module_guid'], TRUE)) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR,
          sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Time module')));
    }
    if (@!checkit::isGUID($this->params['job_module_guid'], TRUE)) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR,
          sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt('Job module')));
    }
    $this->params['cronjob_start'] = $this->decodeISODatetimeStr($this->params['cronjob_start_str']);
    $this->params['cronjob_end'] = $this->decodeISODatetimeStr($this->params['cronjob_end_str']);
    if ($this->params['cronjob_end'] <= $this->params['cronjob_start']) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR, $this->_gt('Invalid time frame.'));
    } elseif ($this->params['cronjob_end'] <= time()) {
      $result = FALSE;
      $this->addMsg(MSG_ERROR, $this->_gt('Invalid time frame.'));
    }
    return $result;
  }

  /**
  * Decode ISO Datetime String
  *
  * @param string $isoDateString
  * @access public
  * @return mixed
  */
  function decodeISODatetimeStr($isoDateString) {
    if (preg_match('#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01]))\s+((1\d)|(2[0-3])|(0?\d)):([0-6]\d)$#',
      $isoDateString, $regs)) {
      return mktime($regs[8], $regs[12], 0, $regs[2], $regs[5], $regs[1]);
    } elseif (preg_match('#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01]))\s*$#',
      $isoDateString, $regs)) {
      return mktime(0, 0, 0, $regs[2], $regs[5], $regs[1]);
    }
    return NULL;
  }

  /**
  * Load list
  *
  * @access public
  */
  function loadList() {
    unset($this->cronjobs);
    $sql = "SELECT cronjob_id,  cronjob_active, cronjob_title, cronjob_nextexec
            FROM %s
            ORDER BY cronjob_title, cronjob_id DESC";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableCronjobs))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cronjobs[$row['cronjob_id']] = $row;
      }
    }
  }

  /**
  * Load modules list
  *
  * @access public
  */
  function loadModulesList() {
    unset($this->cronModules);
    unset($this->jobModules);
    $sql = "SELECT module_guid, module_type, module_class, module_file, module_path, module_title
            FROM %s
            WHERE ((module_type = 'cronjob') OR (module_type = 'time')) AND module_active = 1";
    $params = array($this->tableModules);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        switch ($row['module_type']) {
          case 'cronjob'  : $this->jobModules[$row['module_guid']] = $row; break;
          case 'time' : $this->cronModules[$row['module_guid']] = $row; break;
        }
      }
    }
  }

  /**
  * Basic method for handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'save' :
        if (isset($this->params['id']) && ($this->params['id'] > 0)) {
          if ($this->check()) {
            if ($this->save()) {
              $this->addMsg(MSG_INFO, $this->_gt('Changes saved.').' - '.$this->_gt('Cronjob is deactivated.'));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database Error').' - '.$this->_gt('Changes not saved.'));
            }
          }
        }
        break;
      case 'add' :
        if ($newId = $this->create()) {
          $this->addMsg(MSG_INFO, 'Cronjob created.');
          $this->params['id'] = $newId;
          $this->params['mode'] = 0;
          $this->initializeSessionParam('id');
          $this->initializeSessionParam('mode');
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database Error').' - '.$this->_gt('Could not create cronjob.'));
        }
        break;
      case 'delete' :
        if (isset($this->params['id']) && ($this->params['id'] > 0) && isset($this->params['delconfirm'])) {
          if ($this->delete((int)$this->params['id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Cronjob deleted.'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database Error').' - '.$this->_gt('Could not delete cronjob.'));
          }
        }
        break;
      case 'activate' :
        if (isset($this->params['id'])) {
          $this->load((int)$this->params['id']);
          if ($this->setActive()) {
              $this->addMsg(MSG_INFO, $this->_gt('Cronjob activated'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Could not activate cronjob.'));
          }
        }
        break;
      case 'deactivate' :
        if (isset($this->params['id'])) {
          $this->load((int)$this->params['id']);
          if ($this->deactivate((int)$this->params['id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Cronjob deactivated'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Could not deactivate cronjob.'));
          }
        }
        break;
    }
    $this->loadList();
    $this->loadModulesList();
    if (isset($this->params['id']) && isset($this->cronjobs[(int)$this->params['id']])) {
      $this->load((int)$this->params['id']);
    }
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  function executeJob($id) {
    echo "Loading: ID $id \n";
    if ($this->load($id)) {
      echo "Loaded: ".$this->cronjob['cronjob_title'].LF;
      $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_CRONJOBS,
        sprintf($this->_gt('Cronjob "%s" (%s) loaded.'),
          papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']));
      if ($this->cronjob['cronjob_nextexec'] > 0) {
        $jobModule = &$this->createPluginObject($this->cronjob['job_module_file'],
          $this->cronjob['job_module_class'], $this->cronjob['job_module_guid'], $parent,
          $this->cronjob['job_data']);
        $time = time();
        if (isset($jobModule) && is_object($jobModule)) {
          echo "Module initialized: ".$this->cronjob['job_module_class'].LF;
          $jobModule->cron = &$this;
          if (0 === ($msg = $jobModule->execute())) {
            echo "Job successfully executed.\n";
            $this->logMsg(MSG_INFO, PAPAYA_LOGTYPE_CRONJOBS,
              sprintf($this->_gt('Cronjob "%s" (%s) executed.'),
                papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']));
          } else {
            $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_CRONJOBS,
              sprintf($this->_gt('Cronjob "%s" (%s) failed.'),
                papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']),
              sprintf($this->_gt('Cronjob "%s" (%s) failed.')."\n\n%s",
                papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id'], $msg));
            echo "Job failed.\n";
          }
          if ($this->updateActive($id, $time)) {
            echo "Time updated.\n";
          } else {
            $this->deactivate($id, $time);
            echo "No Time/Data - Job deaktivated.\n";
          }
        } else {
          $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_CRONJOBS,
            sprintf($this->_gt('Cronjob "%s" (%s) failed.'),
              papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']),
            sprintf(
              $this->_gt('Cronjob "%s" (%s) failed.').' '.
              $this->_gt('Could not initialize job module "%s" (%s).').' '.
              $this->_gt('Cronjob deaktivated.'),
              papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id'],
              papaya_strings::escapeHTMLChars($this->cronjob['job_module_class']),
              papaya_strings::escapeHTMLChars($this->cronjob['job_module_file'])));
          $this->deactivate($id, $time);
        }
      } else {
        $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_CRONJOBS,
          sprintf($this->_gt('Cronjob "%s" (%s) failed.'),
            papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']),
          sprintf(
            $this->_gt('Cronjob "%s" (%s) failed.').' '.
            $this->_gt('Invalid time.').' '.
            $this->_gt('Cronjob deaktivated.'),
            papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']));
        $this->deactivate($id, $time);
      }
    } else {
      $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_CRONJOBS,
        sprintf($this->_gt('Could not load cronjob "%s" (%s).'),
          papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']), $this->cronjob['cronjob_id']));
        $this->deactivate($id, $time);
    }
  }

  /**
  * Set active
  *
  * @access public
  * @return mixed
  */
  function setActive() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      if ($calculatedData = $this->checkCronJobData()) {
        if (($calculatedData['nextexec'] > time()) && ($calculatedData['jobdatamsg'])) {
          return $this->activate($calculatedData['nextexec']);
        }
      }
    }
    return FALSE;
  }

  /**
  * Update active
  *
  * @param mixed $id optional, default value NULL
  * @param integer $lastExecutionTime optional, default value 0
  * @access public
  * @return mixed
  */
  function updateActive($id = NULL, $lastExecutionTime = 0) {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      if ($calculatedData = $this->checkCronJobData()) {
        if (($calculatedData['nextexec'] > time()) && ($calculatedData['jobdatamsg'])) {
          return $this->activate($calculatedData['nextexec'], $lastExecutionTime, $id);
        }
      }
    }
    return FALSE;
  }

  /**
  * Get XML for output
  *
  * @access public
  */
  function getXML() {
    if (isset($this->layout) && is_object($this->layout)) {
      $this->getListXML();
      if (isset($this->params['id']) && isset($this->cronjobs[(int)$this->params['id']])) {
        $this->getToolbarXML();
        if (isset($this->params['cmd']) && $this->params['cmd'] == 'delete' && (!isset($this->params['delconfirm']))) {
          $this->getDelConfirmXML();
        }
        switch($this->params['mode']) {
          case 1 :
            $this->getCronModuleEdit();
            break;
          case 2 :
            $this->getJobModuleEdit();
            break;
          case 3 :
            $this->getActivateFormXML();
            break;
          default :
            $this->getPropertyFormXML();
        }
        $this->getPropertyListXML();
      }
      $this->getButtonsXML();
    }
  }

  /**
  * Get list XML
  *
  * @access public
  */
  function getListXML() {
    if (isset($this->cronjobs) && is_array($this->cronjobs)) {
      $result = sprintf('<listview title="%s" width="250">', $this->_gt('Cronjobs'));
      $result .= '<items>';
      foreach($this->cronjobs as $cronjobId=>$cronjob) {
        $nextexecStr = ($cronjob['cronjob_nextexec'] > time()) ?
          date('Y-m-d H:i:s', $cronjob['cronjob_nextexec']) : $this->_gt('now');
        $idx = ($cronjob['cronjob_active']) ? 68 : 69;
        $href = $this->baseLink.'?'.$this->paramName.'[id]='.(int)$cronjobId;
        $selected = ($cronjobId == $this->params['id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem title="%s" image="%s" href="%s" %s>',
          papaya_strings::escapeHTMLChars($cronjob['cronjob_title']), $this->images[$idx], $href, $selected);
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  function initializePropertyDialog() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      if (!(isset($this->propertyDialog) && is_array($this->propertyDialog))) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
        
        $hidden = array(
          'cmd' => 'save',
          'id' => (int)$this->cronjob['cronjob_id']
        );
        
        $cronModules = array();
        if (isset($this->cronModules) && is_array($this->cronModules)) {
          foreach($this->cronModules as $moduleGuid=>$module) {
            $cronModules[$moduleGuid] = $module['module_title'];
          }
        }
        $jobModules = array();
        if (isset($this->jobModules) && is_array($this->jobModules)) {
          foreach($this->jobModules as $moduleGuid=>$module) {
            $jobModules[$moduleGuid] = $module['module_title'];
          }
        }
        
        $fields = array(
          'cronjob_title' => array('Title', 'isSomeText', TRUE, 'input', 200),
          'cronjob_description' => array('Description', 'isSomeText', FALSE, 'textarea', 5),
          'Time',
          'cronjob_start_str' => array('Start' , 'isISODateTime', TRUE, 'input', 50),
          'cronjob_end_str' => array('End' , 'isISODateTime', TRUE, 'input', 50),
          'Modules',
          'cron_module_guid' => array('Time' , 'isGUID', TRUE, 'combo', $cronModules),
          'job_module_guid' => array('Job' , 'isGUID', TRUE, 'combo', $jobModules),
        );
        
        $this->propertyDialog = &new base_dialog($this, $this->paramName, $fields, 
          $this->cronjob, $hidden);        
        $this->propertyDialog->msgs = &$this->msgs;
        $this->propertyDialog->loadParams();
        //$dialog->inputFieldSize = $this->inputFieldSize;
        $this->propertyDialog->baseLink = $this->baseLink;
        $this->propertyDialog->dialogTitle = $this->_gt('Properties');
        $this->propertyDialog->buttonTitle = $this->_gt('Save');
        $this->propertyDialog->dialogDoubleButtons = FALSE;
      }
    }
  }
  
  /**
  * Get property formular XML
  *
  * @access public
  */
  function getPropertyFormXML() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      $this->initializePropertyDialog();
      $this->layout->add($this->propertyDialog->getDialogXML());
    }
  }

  /**
  * Get property list XML
  *
  * @access public
  */
  function getPropertyListXML() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      $result = '<listview title="Informationen" width="100%">';
      $result .= '<items>';
      $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Title'), papaya_strings::escapeHTMLChars($this->cronjob['cronjob_title']));
      $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Active'), $this->_gt( ($this->cronjob['cronjob_active']) ? 'Yes' : 'No'));
      $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Last execution'), ($this->cronjob['cronjob_lastexec'] > 1) ?
          date('Y-m-d H:i:s', $this->cronjob['cronjob_lastexec']) : 'noch nie');
      $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>',
        $this->_gt('Next execution'), ($this->cronjob['cronjob_nextexec'] > time()) ?
          date('Y-m-d H:i:s', $this->cronjob['cronjob_nextexec']) : 'sofort');
      $result .= sprintf('<listitem title="%s"><subitem/></listitem>',
        $this->_gt('Time calculation'));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
        $this->_gt('Module'), papaya_strings::escapeHTMLChars($this->cronjob['cron_module_title']));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
        $this->_gt('Class'), papaya_strings::escapeHTMLChars($this->cronjob['cron_module_class']));
      $result .= sprintf('<listitem title="%s"><subitem/></listitem>',
        $this->_gt('Job module'));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
        $this->_gt('Module'), papaya_strings::escapeHTMLChars($this->cronjob['job_module_title']));
      $result .= sprintf('<listitem title="%s" indent="1"><subitem>%s</subitem></listitem>',
        $this->_gt('Class'), papaya_strings::escapeHTMLChars($this->cronjob['job_module_class']));
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addRight($result);
    }
  }

  /**
  * Get delete confirm XML
  *
  * @access public
  */
  function getDelConfirmXML() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'delete',
        'id' => $this->cronjob['cronjob_id'],
        'delconfirm'=>1,
      );
      $msg = sprintf($this->_gt('Delete cronjob "%s" (%s)?'),
        $this->cronjob['cronjob_title'], $this->cronjob['cronjob_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get Buttons XML
  *
  * @access public
  */
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $menubar = &new base_btnbuilder;
    $menubar->images = &$this->images;
      $menubar->addButton('New cronjob', $this->getLink(array('cmd'=>'add')), 70, 'New cronjob');
    if (isset($this->params['id']) && isset($this->cronjobs[(int)$this->params['id']])) {
      $menubar->addButton('Delete cronjob', 
        $this->getLink(array('cmd'=>'delete', 'id' => (int)$this->params['id'])),
        10, 'Delete cronjob');
      if ($this->cronjobs[(int)$this->params['id']]['cronjob_active']) {
        $menubar->addSeperator();
        $menubar->addButton('Deactivate cronjob', 
          $this->getLink(array('cmd'=>'deactivate', 'id' => (int)$this->params['id'])),
          69, 'Deactivate cronjob');      
      }
    }
    if ($result = $menubar->getXML()) {
      $this->layout->add('<menu>'.$result.'</menu>', 'menus');
    }
  }

  /**
  * Get toolbar XML
  *
  * @access public
  */
  function getToolbarXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

      $toolbar->addButton('Properties',
      $this->baseLink.'?'.$this->paramName.'[mode]=0',
      44, 'General properties', ($this->params['mode'] == 0));
      $toolbar->addButton('Time',
      $this->baseLink.'?'.$this->paramName.'[mode]=1',
      72, 'Time configuration',($this->params['mode'] == 1));
      $toolbar->addButton('Job',
      $this->baseLink.'?'.$this->paramName.'[mode]=2',
      45, 'Job', ($this->params['mode'] == 2));
    $toolbar->addSeperator();
      $toolbar->addButton('Execution',
      $this->baseLink.'?'.$this->paramName.'[mode]=3',
      68, 'Execution', ($this->params['mode'] == 3));
    if ($result = $toolbar->getXML()) {
      $this->layout->add('<toolbar>'.$result.'</toolbar>', 'toolbars');
    }
  }

  /**
  * edit field for time module
  *
  * @access public
  */
  function getCronModuleEdit() {
    $result = FALSE;
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      $this->getModuleEdit($this->cronjob['cron_module_file'], $this->cronjob['cron_module_class'],
        $this->cronjob['cron_module_guid'], $this->cronjob['cron_data'], 'saveCronData');
    }
  }

  /**
  * Get job module edit
  *
  * @access public
  */
  function getJobModuleEdit() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      $this->getModuleEdit($this->cronjob['job_module_file'], $this->cronjob['job_module_class'],
      $this->cronjob['job_module_guid'], $this->cronjob['job_data'], 'saveJobData');
    }
  }

  /**
  * Get module edit
  *
  * @param string $fileName
  * @param string $className
  * @param array $data
  * @param adr $saveFunc
  * @access public
  */
  function getModuleEdit($fileName, $className, $guid, $data, $saveFunc) {
    $parent = &$this;
    $moduleObj = &$this->createPluginObject($fileName, $className, $guid, &$parent, $data);
    if (isset($moduleObj) && is_object($moduleObj)) {
      $moduleObj->initializeDialog();
      if ($moduleObj->modified()) {
        if ($moduleObj->checkData()) {
          if ($this->$saveFunc($moduleObj->getData())) {
            $this->addMsg(MSG_INFO, 'Changes saved.');
          } else {
            $this->addMsg(MSG_ERROR, 'Database Error - Changes not saved!');
          }
        }
      }
      $this->layout->add($moduleObj->getForm());
    }
  }

  /**
  * Check conjob data
  *
  * @access public
  * @return mixed FALSE or array
  */
  function checkCronJobData() {
    $parent = NULL;
    $cronModule = &$this->createPluginObject($this->cronjob['cron_module_file'],
      $this->cronjob['cron_module_class'], $this->cronjob['cron_module_guid'], $parent,
      $this->cronjob['cron_data']);
    $jobModule = &$this->createPluginObject($this->cronjob['job_module_file'],
      $this->cronjob['job_module_class'], $this->cronjob['job_module_guid'], $parent,
      $this->cronjob['job_data']);
    if (isset($cronModule) && isset($jobModule) && is_object($cronModule) && is_object($jobModule)) {
      $calculatedData['nextexec'] = $this->getNextExecute($cronModule);
      $calculatedData['jobdatamsg'] = $jobModule->checkExecParams();
      return $calculatedData;
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Can not activate module(s).'));
    }
    return FALSE;
  }

  /**
  * Get next execution
  *
  * @param object &$cronModule
  * @access public
  * @return integer
  */
  function getNextExecute(&$cronModule) {
    if (@is_object($cronModule)) {
      $nextExecutionTime = $cronModule->getNextDateTime(time());
      if (-1 == $this->checkTimeFrame($nextExecutionTime) && 0 < $nextExecutionTime) {
        if ($nextExecutionTime == ($i = $cronModule->getNextDateTime($this->cronjob['cronjob_start']))) {
          break;
        } else {
          $nextExecutionTime = $i;
        }
      }
      if ($nextExecutionTime > 0) {
        switch ($this->checkTimeFrame($nextExecutionTime)) {
          case -1 : return -1;
          case  0 : return $nextExecutionTime;
        }
      }
    }
    return 0;
  }

  /**
  * Check time frame
  *
  * @param string $dateTime
  * @access public
  * @return integer
  */
  function checkTimeFrame($dateTime) {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      if ($dateTime >= $this->cronjob['cronjob_start'] && $dateTime <= $this->cronjob['cronjob_end']) {
        return 0;
      } elseif ($dateTime <= $this->cronjob['cronjob_start']) {
        return -1;
      }
    }
    return 1;
  }

  /**
  * Get activate formular XML
  *
  * @access public
  */
  function getActivateFormXML() {
    if (isset($this->cronjob) && is_array($this->cronjob)) {
      $canActivate = TRUE;
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->cronjob;
      $fields = array(
        'cronjob_title' => array('Title', 'isSomeText', TRUE, 'info', '', '', '', 'left'),
        'cronjob_description' => array('Description', 'isSomeText', FALSE, 'info', '', '', '', 'left'),
        'Time',
        'cronjob_start_str' => array('Start' , 'isSomeText', TRUE, 'info', '', '', '', 'left'),
        'cronjob_end_str' => array('End' , 'isSomeText', TRUE, 'info', '', '', '', 'left'),
        'Execution',
        'nextexec' => array('Next execution' , 'isSomeText', TRUE, 'info', '', '', '', 'left'),
        'jobdatamsg' => array('Job' , 'isSomeText', TRUE, 'info', '', '', '', 'left'),
      );
      if ($calculatedData = $this->checkCronJobData()) {
        if ($calculatedData['nextexec'] > time()) {
          $data['nextexec'] = date('Y-m-d H:i:s', $calculatedData['nextexec']);
        } else {
          $data['nextexec'] = papaya_strings::escapeHTMLChars($this->_gt('never'));
          $canActivate = FALSE;
        }
        if ($calculatedData['jobdatamsg']) {
          $data['jobdatamsg'] = papaya_strings::escapeHTMLChars($calculatedData['jobdatamsg']);
        } else {
          $data['jobdatamsg'] = papaya_strings::escapeHTMLChars($this->_gt('none'));
          $canActivate = FALSE;
        }
      }      
      if ($canActivate) {
        $hidden = array(
          'cmd' => 'activate',
          'id' => (int)$this->cronjob['cronjob_id']
        );
      } else {
        $hidden = array(
          'cmd' => 'deactivate',
          'id' => (int)$this->cronjob['cronjob_id']
        );      
      }      
      $dialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);        
      $dialog->msgs = &$this->msgs;
      $dialog->baseLink = $this->baseLink;
      $dialog->dialogTitle = $this->_gt('Activate cronjob');
      if ($canActivate) {
        $dialog->buttonTitle = 'Activate';
      } else {
        $dialog->dialogHideButtons = TRUE;
      }
      $dialog->dialogDoubleButtons = FALSE;
        
      $this->layout->add($dialog->getDialogXML());
    }
  }
}

?>