<?php
/**
* Delegation-class for dialogs
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_dialog.php,v 1.18 2006/05/11 11:50:29 rekowski Exp $
*/

/**
* basic class object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* Delegation-class for dialogs
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_dialog extends base_object {

  var $baseLink = '';
  /**
  * Input field width class
  * x-small, small, medium, large, x-large, xx-large
  * @var string
  */
  var $inputFieldSize = 'medium';

  var $owner = NULL;

  var $paramName = '';
  /**
  * Name => (
      0 => Aufschrift,
      1 => Prffunction,
      2 => Pflichtfeld (TRUE/FALSE),
      3 => Typ (input, combo, textarea)
      4 => Paramter zum Typ (Maximale Lnge, Werte-Hash, Zeilen)
      5 => Quickinfo
      6 => Defaultwert
      7 => Ausrichtung)

  * @var array $fields
  */
  var $fields = NULL;
  var $data = NULL;
  var $hidden = NULL;
  var $buttons = array();

  var $dialogTitle = 'Edit';
  var $buttonTitle = 'Save';
  var $dialogHint  = '';

  var $textYes = 'Yes';
  var $textNo = 'No';

  var $dialogId = '';

  var $dialogDoubleButtons = FALSE;
  var $dialogHideButtons = FALSE;

  var $parser = NULL;
  var $expandPapayaTags = FALSE;

  var $inputErrors = NULL;

  /**
  * use richtext editor
  * @var boolean
  */
  var $useRichtext = TRUE;

  /**
  * dialog tokens
  * @var array
  */
  var $dialogTokens = array();

  /**
  * token found in request and session
  * @var boolean
  */
  var $validDialogToken = FALSE;

  /**
  * token key suffix can be changed to resolve conflicts
  * @var string
  */
  var $tokenKeySuffix = 'dialog';

  /**
  * Constructor
  *
  * @param object base_object &$aOwner
  * @param string $paramName
  * @param array &$fields
  * @param array &$data
  * @param array &$hidden
  * @access public
  */
  function __construct(&$aOwner, $paramName, &$fields, &$data, &$hidden) {
    $this->owner = &$aOwner;
    $this->paramName = $paramName;
    $this->fields = &$fields;
    $this->data = &$data;
    $this->hidden = &$hidden;
    $this->baseLink = $this->getBaseLink();

    $useRichtext = $this->getSessionValue('PAPAYA_SESS_USE_RICHTEXT');
    $this->useRichtext = (isset($useRichtext)) ? (bool)$useRichtext : TRUE;

    $this->dialogTokens = $this->getSessionValue('PAPAYA_SESS_DIALOG_TOKENS');
  }
    /**
  * PHP 4 constuctor redirect
  *
  * @param object &$aOwner
  * @param string $paramName
  * @param array &$fields
  * @param array &$data
  * @param array &$hidden
  * @access public
  */
  function base_dialog(&$aOwner, $paramName, &$fields, &$data, &$hidden) {
    $this->__construct($aOwner, $paramName, $fields, $data, $hidden);
  }

  /**
  * Load parameters
  *
  * @access public
  */
  function loadParams() {
    $this->initializeParams();
  }

  function addButton($buttonName, $buttonTitle) {
    $this->buttons[$buttonName] = $this->_gt($buttonTitle);
  }

  /**
  * Get dialog XML
  *
  * @access public
  * @return string '' or XML
  */
  function getDialogXML() {
    $result = '';
    if (isset($this->fields) && is_array($this->fields)) {
      $dlgHint = (trim($this->dialogHint) != '') ? sprintf(' hint="%s"', papaya_strings::escapeHTMLChars($this->dialogHint)) : '';
      $result = sprintf('<dialog title="%s" action="%s" width="100%%" %s%s>'.LF,
        papaya_strings::escapeHTMLChars($this->dialogTitle), $this->baseLink,
        ($this->dialogId) ? sprintf(' name="%s"', $this->dialogId) : '' ,
        $dlgHint);
      $result .= $this->getHidden();
      switch($this->inputFieldSize) {
        case 'x-small' : $fieldSizeClass = 'XSmall'; break;
        case 'small' : $fieldSizeClass = 'Small'; break;
        case 'large' : $fieldSizeClass = 'Large'; break;
        case 'x-large' : $fieldSizeClass = 'XLarge'; break;
        case 'xx-large' : $fieldSizeClass = 'XXLarge'; break;
        default : $fieldSizeClass = 'Medium'; break;
      }
      $result .= sprintf('<lines class="dialog%s">'.LF, $fieldSizeClass);

      foreach($this->fields as $key=>$val) {
        if (isset($this->params[$key]) && $val[3] != 'info') {
          $data = $this->params[$key];
        } elseif (isset($this->data[$key])) {
          $data = $this->data[$key];
        } else {
          $data = @(isset($val[6])) ? $val[6] : '';
        }
        if (isset($val) && is_array($val)) {
          if ($val[3] == 'yesno') {
            $align = ' align="left"';
          } else {
            $align = (isset($val[7])) ? sprintf(' align="%s"', papaya_strings::escapeHTMLChars($val[7])) : '';
          }
          $hint = (isset($val[5]) && trim($val[5]) != '') ? sprintf(' hint="%s"', papaya_strings::escapeHTMLChars($this->_gt($val[5]))) : '';
          $error = (@$this->inputErrors[$key] == 1) ? ' error="error"' : '';
          if ($str = $this->getDlgElement($key, $val, $data)) {
            $result .= sprintf('<line caption="%s" fid="%s" %s%s%s>%s</line>'.LF,
              $this->_gt($val[0]), $key, $hint, $error, $align, $str);
          }
        } elseif (is_string($val)) {
          $result .= sprintf('<subtitle caption="%s" />', papaya_strings::escapeHTMLChars($this->_gt($val)));
        }
      }
      $result .= '</lines>'.LF;
      if (!$this->dialogHideButtons) {
        $result .= sprintf('<dlgbutton value="%s" %s/>'.LF,
          papaya_strings::escapeHTMLChars($this->_gt($this->buttonTitle)), (($this->dialogDoubleButtons) ? ' sandwich="sandwich"' : ''));
        if (isset($this->buttons) && is_array($this->buttons) && count($this->buttons) > 0) {
          foreach($this->buttons as $buttonName=>$buttonTitle) {
            $result .= sprintf('<dlgbutton name ="%s" value="%s" %s/>'.LF,
              papaya_strings::escapeHTMLChars($buttonName),
              papaya_strings::escapeHTMLChars($buttonTitle),
              (($this->dialogDoubleButtons) ? ' sandwich="sandwich"' : ''));
          }
        }
      }
      $result .= '</dialog>'.LF;
    }
    return $result;
  }

  /**
  * Get hidden
  *
  * @access public
  * @return string '' or HTML
  */
  function getHidden() {
    $result = '';
    $result .= sprintf('<input type="hidden" name="%s[token]" value="%s"/>'.LF,
       $this->paramName,
       papaya_strings::escapeHTMLChars($this->getDialogToken()));

    if (@is_array($this->hidden)) {
      foreach($this->hidden as $name=>$value) {
        if (is_array($value)) {
          foreach($value as $idx=>$subValue) {
            $result .= sprintf('<input type="hidden" name="%s[%s][%s]" value="%s"/>'.LF,
              $this->paramName, papaya_strings::escapeHTMLChars($name), papaya_strings::escapeHTMLChars($idx), papaya_strings::escapeHTMLChars($subValue));
          }
        } else {
          $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>'.LF,
            $this->paramName, papaya_strings::escapeHTMLChars($name), papaya_strings::escapeHTMLChars($value));
        }
      }
    }
    return $result;
  }

  /**
  *
  *
  * @param string $name
  * @param array &$element
  * @param string $data
  * @access public
  * @return string
  */
  function getDlgElement($name, &$element, $data) {
    $result = '';
    if (isset($element) && is_array($element)) {
      $elementId = (isset($this->dialogId) && $this->dialogId) ? sprintf(' id="%s_%s"', $this->dialogId, $name) : '';
      if (strpos($element[3], 'disabled_') === 0) {
        $elementType = substr($element[3], strlen('disabled_'));
        $disabled = ' disabled="disabled"';
      } else {
        $elementType = $element[3];
        $disabled = '';
      }
      switch ($elementType) {
        case 'info' :
          $result .= '<div class="dialogInfo dialogScale"><![CDATA['.$data.']]></div>';
          break;
        case 'input' :
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="%s" class="dialogInput dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4],$name,
            $elementId, $disabled);
          break;
        case 'pageid' :
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="%s" class="dialogPageId dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4],$name,
            $elementId, $disabled);
          break;
        case 'date' :
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="%s" class="dialogInputDate dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4],$name,
            $elementId, $disabled);
          break;
        case 'color' :
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="%s" class="dialogInputColor dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4],$name,
            $elementId, $disabled);
          break;
        case 'file' :
          $result .= sprintf('<input type="file" name="%s[%s]" value="%s" maxlength="%s" class="dialogFileSelect dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4], $name,
            $elementId, $disabled);
          break;
        case 'checkbox' :
          $selected = ($data != '' && $data) ? ' checked="checked"' : '';
          $value = (isset($element[4]) && $element[4]) ? papaya_strings::escapeHTMLChars($element[4]) : 'Yes';
          $result .= sprintf('<input type="checkbox" name="%s[%s]" value="%s" class="dialogCheckbox dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, $value, $name, $selected,
            $elementId, $disabled);
          break;
        case 'password' :
          $result .= sprintf('<input type="password" name="%s[%s]" value="%s" maxlength="%s" class="dialogPassword dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $element[4], $name,
            $elementId, $disabled);
          break;
        case 'textarea' :
          $result .= sprintf('<textarea name="%s[%s]" rows="%s" cols="30" class="dialogTextarea dialogScale" wrap="virtual" fid="%s" %s %s><![CDATA[%s]]></textarea>'.LF,
            $this->paramName, $name, $element[4], $name, $elementId, $disabled,
            papaya_strings::escapeHTMLChars($data));
          break;
        case 'image' :
          if (FALSE !== strpos($data, '<')) {
            $dataArr = array(
              'src' => '',
              'width' => 0,
              'height' => 0
            );
            if (preg_match_all('~(src|width|height)=("|\')([^"]+)(\\2)~i', $data, $regs, PREG_SET_ORDER)) {
              foreach($regs as $reg) {
                $dataArr[strtolower($reg[1])] = $reg[3];
              }
            }
            $data = implode('/', $dataArr);
          }
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="200" class="dialogImage dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $name,
            $elementId, $disabled);
          break;
        case 'mediafile' :
          if (FALSE !== strpos($data, '<')) {
            $dataArr = array(
              'src' => ''
            );
            if (preg_match_all('~(src)=("|\')([^"]+)(\\2)~i', $data, $regs, PREG_SET_ORDER)) {
              foreach($regs as $reg) {
                $dataArr[strtolower($reg[1])] = $reg[3];
              }
            }
            $data = implode('/', $dataArr);
          }
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="200" class="dialogMediaFile dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $name,
            $elementId, $disabled);
          break;
        case 'imagefixed' :
          if (FALSE !== strpos($data, '<')) {
            if (preg_match('~(src)=("|\')([^"]+)(\\2)~i', $data, $regs)) {
              $data = $regs[3];
            } else {
              $data = '';
            }
          }
          $result .= sprintf('<input type="text" name="%s[%s]" value="%s" maxlength="200" class="dialogFixedImage dialogScale" fid="%s" %s %s/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $name,
            $elementId, $disabled);
          break;
        case 'richtext' :
          $result .= sprintf('<textarea name="%s[%s]" rows="%s" cols="30" class="dialogRichtext dialogScale" wrap="virtual" fid="%s" %s><![CDATA[%s]]></textarea>'.LF,
            $this->paramName, $name, $element[4], $name, $elementId,
            $this->prepareRichTextData($data));
          break;
        case 'simplerichtext' :
          $result .= sprintf('<textarea name="%s[%s]" rows="%s" cols="30" class="dialogSimpleRichtext dialogScale" wrap="virtual" fid="%s" %s><![CDATA[%s]]></textarea>'.LF,
            $this->paramName, $name, $element[4], $name, $elementId,
            $this->prepareRichTextData($data));
          break;
        case 'combo' :
          $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale" fid="%s" %s %s>'.LF,
            $this->paramName, $name, $name, $elementId, $disabled);
          if (isset($element[4]) && is_array($element[4])) {
            foreach($element[4] as $skey=>$sval) {
              if ($skey == $data) {
                $selected = ' selected="selected"';
              } else {
                $selected = '';
              }
              $result .= sprintf('<option value="%s"%s>%s</option>'.LF,
                papaya_strings::escapeHTMLChars($skey), $selected, papaya_strings::escapeHTMLChars($sval));
            }
          }
          $result .= '</select>'.LF;
          break;
        case 'radio' :
          if (isset($element[4]) && is_array($element[4])) {
            foreach($element[4] as $skey=>$sval) {
              if ($skey == $data) {
                $selected = ' checked="checked"';
              } else {
                $selected = '';
              }
              $result .= sprintf('<input class="dialogRadio dialogScale" type="radio" name="%s[%s]" value="%s"%s>%s</input>'.LF, $this->paramName,$name,
                papaya_strings::escapeHTMLChars($skey), $selected, papaya_strings::escapeHTMLChars($sval));
            }
          }
          break;
        case 'filecombo' :
          $result .= base_dialog::getFileCombo($name, $element, $data, $element[4][0], $element[4][1], @(bool)$element[4][2]);
          break;
        case 'dircombo' :
          $result .= base_dialog::getDirectoryCombo($name, $element, $data, $element[4][0], $element[4][1]);
          break;
        case 'mediaimage' :
          $result .= base_dialog::getMediaImage($name, $element, $data, $elementId);
          break;
        case 'function' :
          if (method_exists($this->owner,$element[4])) {
            if ($str = $this->owner->$element[4]($name, $element, $data)) {
              $result .= $str;
            }
          }
          break;
        case 'yesno' :
          if ($data == '0') {
            $result .= sprintf('<input type="radio" value="1" name="%s[%s]" class="dialogRadio dialogScale" fid="%s">%s</input>'.LF,
              $this->paramName, $name, $name, $this->_gt($this->textYes));
            $result .= sprintf('<input type="radio" value="0" checked="checked" name="%s[%s]" class="dialogRadio dialogScale" fid="%s">%s</input>'.LF,
              $this->paramName, $name, $name, $this->_gt($this->textNo));
          } else {
            $result .= sprintf('<input type="radio" value="1" checked="checked" name="%s[%s]" class="dialogRadio dialogScale" fid="%s">%s</input>'.LF,
              $this->paramName, $name, $name, $this->_gt($this->textYes));
            $result .= sprintf('<input type="radio" value="0" name="%s[%s]" class="dialogRadio dialogScale" fid="%s">%s</input>'.LF,
              $this->paramName, $name, $name, $this->_gt($this->textNo));
          }
          break;
        case 'lookup_combo' :
          if ($str = $this->getFieldCombo($name, $element, $data)) {
            $result .= $str;
          }
          break;
        case 'lookup_radiogroup' :
          if ($str = $this->getFieldRadioGroup($name, $element, $data)) {
            $result .= $str;
          }
          break;
        case 'lookup_checkgroup' :
          if ($str = $this->getFieldCheckGroup($name, $element, $data)) {
            $result .= $str;
          }
          break;
      }
    }
    return $result;
  }

  /**
  * Prepare HTML area data
  *
  * @param string $str
  * @access public
  * @return string
  */
  function prepareRichTextData($str) {
    if ($this->expandPapayaTags) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
      $lngSelect = &base_language_select::getInstance();
      $lngId = $lngSelect->currentLanguageId;
      if (!is_object($this->parser)) {
        include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
        $this->parser = &new papaya_parser;
        if (isset($this->tableTopicsTrans) && trim($this->tableTopicsTrans) != '') {
          $this->parser->tableTopicsTrans = $this->tableTopicsTrans;
        }
      }
      if (isset($GLOBALS['PAPAYA_USER'])) {
        if ($GLOBALS['PAPAYA_USER']->options['PAPAYA_USE_RICHTEXT'] && $this->useRichtext) {
          return papaya_strings::escapeHTMLChars($this->parser->parse($str, $lngId, TRUE));
        } else {
          return papaya_strings::escapeHTMLChars($this->parser->parse($str, $lngId, TRUE, TRUE));
        }
      } elseif (PAPAYA_USE_RICHTEXT && $this->useRichtext) {
        return papaya_strings::escapeHTMLChars($this->parser->parse($str, $lngId, TRUE));
      } else {
        return papaya_strings::escapeHTMLChars($this->parser->parse($str, $lngId, TRUE, TRUE));
      }
    }
    return papaya_strings::escapeHTMLChars($str);
  }

  /**
  * Get file combo
  *
  * @param string $name
  * @param array $element
  * @param array $data
  * @param string $path
  * @param string $regEx
  * @access public
  * @return string
  */
  function getFileCombo($name, $element, $data, $path, $regEx, $emptyAllowed = FALSE) {
    $result = sprintf('<select name="%s[%s]" class="dialogSelect dialogScale" fid="%s">'.LF,
    $this->paramName, $name, $name);
    if ($dh = opendir($path)) {
      if ($emptyAllowed) {
        $result .= sprintf('<option value=""><![CDATA[%s]]></option>'.LF,
          papaya_strings::escapeHTMLChars($this->_gt('None')));
      }
      while($file = readdir($dh)) {
        if (preg_match($regEx, $file)) {
          $files[] = $file;
        }
      }
      closedir($dh);
      if (isset($files) && is_array($files)) {
        sort($files);
        foreach($files as $file) {
          $selected = ($file == $data) ? ' selected="selected"' : '';
          $result .= sprintf('<option value="%s"%s><![CDATA[%s]]></option>'.LF,
            papaya_strings::escapeHTMLChars($file), $selected, $file);
        }
      }
    }
    $result .= '</select>'.LF;
    return $result;
  }

  /**
  * Get directory combo
  *
  * @param string $name
  * @param array $element
  * @param array $data
  * @param string $path
  * @param string $baseDirectory optional, default value ''
  * @access public
  * @return string
  */
  function getDirectoryCombo($name, $element, $data, $path, $baseDirectory = '') {
    $result = '';
    switch ($baseDirectory) {
      case 'page' :
        $dir = $_SERVER['DOCUMENT_ROOT'].PAPAYA_PATH_WEB.$path;
        break;
      case 'admin' :
        $dir = $_SERVER['DOCUMENT_ROOT'].PAPAYA_PATH_WEB;
        if (substr($dir, -1) == '/') $dir = substr($dir, 0, -1);
        $dir .= PAPAYA_PATH_ADMIN.'/'.$path;
        break;
      case 'upload' :
        $dir = PAPAYA_PATH_DATA.$path;
        break;
    }
    if (substr($dir, -1) != '/') $dir .= '/';

    if (file_exists($dir) && $dh = @opendir($dir)) {
      $result = sprintf('<select name="%s[%s]" class="dialogSelect dialogScale" fid="%s">'.LF,
        $this->paramName, $name, $name);
      while($file = readdir($dh)) {
        if (is_dir($dir.$file) && substr($file, 0, 1) != '.') {
          $files[] = $file;
        }
      }
      closedir($dh);
      if (isset($files) && is_array($files)) {
        sort($files);
        foreach($files as $file) {
          $selected = ($file == $data) ? ' selected="selected"' : '';
          $result .= sprintf('<option value="%s"%s><![CDATA[%s]]></option>'.LF,
            papaya_strings::escapeHTMLChars($file), $selected, $file);
        }
      }
      $result .= '</select>'.LF;
    } else {
      $result .= '<div class="dialogInfo dialogScale"><![CDATA['.$this->_gt('Can not open directory.').']]></div>';
    }
    return $result;
  }

  function getMediaImage($name, $element, $data, $elementId) {
    $result = sprintf('<input type="text" name="%s[%s]" value="%s" class="dialogInput dialogScale" fid="%s"/>'.LF,
            $this->paramName, $name, papaya_strings::escapeHTMLChars($data), $name);
    $result .= '<div class="infos">';
    if ($data == '') {
      //nix
    } elseif (file_exists(PAPAYA_PATH_MEDIAFILES.$data)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
      $thumbnail = &new base_thumbnail;
      $thumbnail->baseDirectory = PAPAYA_PATH_MEDIAFILES;
      $thumbnail->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
      $thumbnail->width = (isset($element[4]['width']) && $element[4]['width'] > 0) ? (int)$element[4]['width'] : 0;
      $thumbnail->height = (isset($element[4]['height']) && $element[4]['height'] > 0) ? (int)$element[4]['height'] : 0;
      $thumbFileName = $thumbnail->getThumb($data);
      $result .= sprintf('<img src="%s"/>', $this->getWebMediaLink($thumbFileName, 'thumb'));
    } else {
      $result .= $this->_gt('Image not found');
    }
    $result .= '</div>';
    return $result;
  }

  /**
  * Get field combo
  *
  * @param string $name
  * @param array $element
  * @param array $data
  * @access public
  * @return string
  */
  function getFieldCombo($name, $element, $data) {
    $items = split("(\r\n)|(\n\r)|([\r\n])", $this->owner->data['lookup_'.$element[4]]);
    if (isset($items) && is_array($items)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale" fid="%s">'.LF,
      $this->paramName, $name, $name);
      foreach ($items as  $idx=>$item) {
        $selected = ($item == $data) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s"%s><![CDATA[%s]]></option>'.LF,
            papaya_strings::escapeHTMLChars($item), $selected, $item);
      }
      $result .= '</select>'.LF;
    }
    return $result;
  }

  /**
  * Get field radio group
  *
  * @param string $name
  * @param array $element
  * @param array $data
  * @access public
  * @return string
  */
  function getFieldRadioGroup($name, $element, $data) {
    $result ='';
    $items = split("(\r\n)|(\n\r)|([\r\n])", $this->owner->data['lookup_'.$element[4]]);
    if (isset($items) && is_array($items)) {
      foreach ($items as $idx=>$item) {
        $selected = (($item == $data) || ($idx == 0 && $data == '')) ? ' checked="checked"' : '';
        $result .= sprintf('<input type="dialogRadio" value="%s" name="%s[%s]" class="%s" fid="%s" %s><![CDATA[%s]]></input>'.LF,
          papaya_strings::escapeHTMLChars($item),
          $this->paramName, $name, $name,
          $selected, $item);
      }
    }
    return $result;
  }

  /**
  * Get field check group
  *
  * @param string $name
  * @param array $element
  * @param array $data
  * @access public
  * @return string
  */
  function getFieldCheckGroup($name, $element, $data) {
    $result = '';
    $items = split("(\r\n)|(\n\r)|([\r\n])", $this->owner->data['lookup_'.$element[4]]);
    if (isset($items) && is_array($items)) {
      foreach ($items as  $idx=>$item) {
        $selected =  (@in_array($item, $data)) ? ' checked="checked"' : '';
        $result .= sprintf('<input type="checkbox" value="%s" name="%s[%s][]" class="dialogCheckbox" fid="%s" %s><![CDATA[%s]]></input>'.LF,
          papaya_strings::escapeHTMLChars($item),
          $this->paramName, $name, $name,
          $selected, $item);
      }
    }
    return $result;
  }

  /**
  * Check dialog input
  *
  * @access public
  * @return boolean
  */
  function checkDialogInput() {
    $result = TRUE;
    unset($this->inputErrors);

    if (!$this->checkDialogToken()) {
      return FALSE;
    }

    $checkFunctions = get_class_methods('checkit');
    foreach($checkFunctions as $idx=>$functionName) {
      $checkFunctions[$idx] = strtolower($functionName);
    }
    $checkObj = &new checkit;
    if (isset($checkFunctions) && is_array($checkFunctions)) {
      foreach($this->fields as $key=>$val) {
        if (isset($val) && is_array($val)) {
          if (strpos($val[3], 'disabled_') === 0) {
            $elementType = substr($val[3], strlen('disabled_'));
            //field does not allow user inputs - use default data
            if (isset($this->data[$key])) {
              $this->params[$key] = $this->cleanInputString($elementType, $this->data[$key]);
            } else {
              $this->params[$key] = NULL;
            }
          } else {
            $elementType = $val[3];
            //active field
            $checkFunction = strtolower($val[1]);
            if (in_array($checkFunction, $checkFunctions)) {
              if (isset($this->params[$key]) && is_array($this->params[$key])) {
                foreach($this->params[$key] as $subKey => $subValue) {
                  if ($checkObj->$checkFunction($subValue)) {
                    $this->inputErrors[$key] = 0;
                  } else {
                    $this->inputErrors[$key] = 1;
                    $this->addMsg(MSG_ERROR,
                      sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt($val[0])));
                    $result = FALSE;
                    break;
                  }
                }
              } elseif ($checkObj->$checkFunction(isset($this->params[$key]) ? $this->params[$key] : '', $val[2])) {
                $this->inputErrors[$key] = 0;
                $this->data[$key] = isset($this->params[$key]) ? $this->cleanInputString($elementType, $this->params[$key]) : '';
              } else  {
                $this->addMsg(MSG_ERROR,
                  sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt($val[0])));
                $this->inputErrors[$key] = 1;
                $result = FALSE;
              }
            } elseif ($checkFunction == '') {
              $this->data[$key] = isset($this->params[$key]) ? $this->params[$key] : NULL;
            } else {
              if (@preg_match($checkFunction, isset($this->params[$key]) ? $this->params[$key] : '')) {
                $this->inputErrors[$key] = 0;
                $this->data[$key] = isset($this->params[$key]) ? $this->cleanInputString($elementType, $this->params[$key]) : '';
              } elseif ($val[2] == FALSE && ((!isset($this->params[$key])) || $this->params[$key] == ''))  {
                $this->inputErrors[$key] = 0;
                $this->data[$key] = isset($this->params[$key]) ? $this->cleanInputString($elementType, $this->params[$key]) : '';
              } else {
                $this->addMsg(MSG_ERROR,
                  sprintf($this->_gt('The input in field "%s" is not correct.'), $this->_gt($val[0])));
                $this->inputErrors[$key] = 1;
                $result = FALSE;
              }
            }
          }
        }
      }
    }
    return $result;
  }

  function cleanInputString($elementType, $value) {
    if ($elementType == 'richtext' || $elementType == 'simplerichtext') {
      return papaya_strings::cleanInputString($value);
    }
    return $value;
  }


  /**
  * Did data in formular change ? If TRUE save data
  *
  * @param string $marker optional, default value 'save'
  *
  * @return boolean
  */
  function modified($marker = 'save') {
    if (isset($this->params[$marker]) && is_array($this->fields)) {
      foreach($this->fields as $key=>$val) {
        if ($val[3] == 'checkbox') {
          if (@$this->params[$key] != @$this->data[$key]) {
            return TRUE;
          }
        } elseif (isset($this->params[$key]) && $this->params[$key] != @$this->data[$key]) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  function checkDialogToken() {
    if (isset($GLOBALS['PAPAYA_SESSION']) && $GLOBALS['PAPAYA_SESSION']->started) {
      $this->validDialogToken = FALSE;
      if (isset($this->params['token']) && !empty($this->params['token'])) {
        if (isset($this->dialogTokens[$this->getDialogTokenKey()])) {
          $this->validDialogToken =
            ($this->dialogTokens[$this->getDialogTokenKey()] == $this->params['token']);
        }
      }
      return $this->validDialogToken;
    }
    return TRUE;
  }

  function getDialogTokenKey() {
    return get_class($this->owner).'_'.$this->paramName.'_'.$this->tokenKeySuffix;
  }

  function getDialogToken() {
    if (isset($this->params['token']) && !empty($this->params['token'])) {
      if (isset($this->dialogTokens[$this->getDialogTokenKey()])) {
        unset($this->dialogTokens[$this->getDialogTokenKey()]);
      }
    }
    srand((double)microtime()*1000000);
    $newToken = md5(uniqid(rand()));
    $this->dialogTokens[$this->getDialogTokenKey()] = $newToken;
    $this->setSessionValue('PAPAYA_SESS_DIALOG_TOKENS', $this->dialogTokens);
    return $newToken;
  }
}

?>