<?php
/**
* basic class for file access
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_fileman.php,v 1.28 2006/05/03 08:38:33 weinert Exp $
*/


/**
* basic class for database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* basic class for file access
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_filemanager extends base_db {

  /**
  * Language Selector
  * @var objeckt
  */
  var $lngSelect = NULL;
  
  /**
  * Files
  * @var array
  */
  var $files;
  
  /**
  * Data dictionary
  * @var string
  */
  var $dataDirectory = "";
  
  /**
  * Thumbnail directory
  * @var string
  */
  var $thumbnailDirectory = "";
  
  /**
  * Thumbnail width
  * @var integer
  */
  var $thumbWidth = 140;
  
  /**
  * Thumbnail height
  * @var integer
  */
  var $thumbHeight = 140;
  
  /**
  * Mini thumbnail width
  * @var integer
  */
  var $minithumbWidth = 80;
  
  /**
  * Mini thumbnail height
  * @var integer
  */
  var $minithumbHeight = 80;
  
  /**
  * Web data directory
  * @var string
  */
  var $webDataDir = "";
  
  /**
  * cols
  * @var 5
  */
  var $cols = 5;
  
  /**
  * max per page
  * @var 20
  */
  var $maxPerPage = 20;

  /**
  * Maximum size
  * @var number
  */
  var $maximumSize = 6291456; // 6MB
  
  /**
  * upload fields
  * @var string
  */
  var $uploadFiels = 'addfile';

  /**
  * Table files
  * @var string
  */
  var $tableFiles = PAPAYA_DB_TBL_MEDIA_FILES;

  /**
  * Table file translation
  * @var string
  */
  var $tableFilesTrans = PAPAYA_DB_TBL_MEDIA_FILES_TRANS;

  /**
  * Table folder
  * @var string
  */
  var $tableFolders = PAPAYA_DB_TBL_MEDIA_FOLDERS;

  /**
  * Table folder Translation
  * @var string
  */
  var $tableFoldersTrans = PAPAYA_DB_TBL_MEDIA_FOLDERS_TRANS;


  /**
  * Forbidden extensions 
  * @var string
  */
  var $forbiddenExt = array('.cgi','.php','.phtml','.php3','.php4','.php5','.pl','.htaccess','.htpasswd','.bat');

  /**
  * Constructor
  *
  * @param string $paramName optional, default value 'fm'
  * @access public
  */
  function __construct($paramName='fm') {
    $this->configureUploadSize();
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_fileman_'.$paramName;
  }

  /**
  * Configure upload size
  *
  * @access public
  */
  function configureUploadSize() {
    if (PAPAYA_MAX_UPLOAD_SIZE > 0) {
      $confMax = PAPAYA_MAX_UPLOAD_SIZE;
    } else {
      $confMax = 6;
    }
    if ($iniMax = ini_get('upload_max_filesize')) {
      if (preg_match('#(\d+)M#', $iniMax, $regs)) {
        $iniMax = $regs[1];
      } else {
        $iniMax = (int)$iniMax;
      }
    } else {
      $iniMax = 0;
    }
    if ($iniMax > 0 && $iniMax < $confMax) {
      $this->maximumSize = ($iniMax * 1048576);
    } else {
      $this->maximumSize = ($confMax * 1048576);
    }
  }

  /**
  * Synchronize file data
  *
  * @param integer $fileId
  * @param integer $size
  * @param string $mimeType
  * @access public
  * @return mixed
  */
  function syncronizeFileData($fileId, $size, $mimeType) {
    $tempFileName = $this->dataDirectory.$fileId;
    if (file_exists($tempFileName)) {
      $tempFileSize = @filesize($tempFileName);
      list(,,$tempFileType) = @getimagesize($tempFileName);
      if (($tempFileSize != $size) || ($tempFileType != $mimeType)) {
        return $this->dbUpdateFileData($fileId, $tempFileSize, $tempFileType, time());
      }
    }
    return FALSE;
  }

    /**
  * Check upload file
  *
  * @param mixed $fileData upload file data or temporary filename
  * @param string $fileName
  * @access public
  * @return mixed
  */
  function checkUploadFile($fileData, $fileName) {
    if (isset($fileData)) {
      if (is_array($fileData)) {
        switch($fileData['error']) {
          case 1 :
            $this->addMsg(MSG_ERROR, sprintf($this->_gt('File "%s" is to large.'), $fileName).' (php.ini)');
            return FALSE;
          case 2 :
            $this->addMsg(MSG_ERROR, sprintf($this->_gt('File "%s" is to large.'), $fileName).' (form)');
            return FALSE;
          case 3 :
            $this->addMsg(MSG_ERROR, sprintf($this->_gt('File "%s" is not complete.'), $fileName));
            return FALSE;
          case 6 :
            $this->addMsg(MSG_ERROR, $this->_gt('No temporary upload path.'));
            return FALSE;
          case 4 :
            return FALSE;
          case 0 :
          default :
            $tempFileName = (string)$fileData['tmp_name'];
            break;
        }
      } else {
        $tempFileName = (string)$fileData;
      }
      if (@file_exists($tempFileName) && is_uploaded_file($tempFileName)) {
        $tempFileSize = @filesize($tempFileName);
        list(,,$tempFileType) = @getimagesize($tempFileName);
        if ($tempFileSize <= 0) {
          $this->addMsg(MSG_ERROR, $this->_gt('No temporary file found.'));
          @unlink($tempFileName);
        } elseif ($tempFileSize < $this->maximumSize) {
          $ext = $this->getExtension($fileName);
          if (!(@in_array($ext,$this->forbiddenExt))) {
            $fileId = $this->getRandFilename($ext);
            return array('id'=>$fileId, 'format'=> $tempFileType, 'size'=>$tempFileSize);
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Invalid file type!'));
            @unlink($tempFileName);
          }
        } else {
          $this->addMsg(MSG_ERROR, sprintf($this->_gt('File "%s" is to large.'), $fileName));
          @unlink($tempFileName);
        }
      }
    }
    return FALSE;
  }

  /**
  * Add file to media database
  *
  * @param string $tempFileName
  * @param integer $fileId
  * @param string $fileName
  * @param integer $fileSize
  * @param string $fileFormat
  * @param integer $folderId
  * @access public
  * @return mixed id or FALSE
  */
  function addFileToMediaDatabase($tempFileName, $fileId, $fileName, $fileSize, $fileFormat, $folderId) {
    if ($this->dataInsertFile($fileId,$fileName,$fileSize,$fileFormat,time(),$folderId)) {
      if (move_uploaded_file($tempFileName, $this->dataDirectory.$fileId)) {        
        chmod($this->dataDirectory.$fileId, 0644);
        //eventuelle Thumbnails eines Vorgngers entfernen
          include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
          $thumbnail = &new base_thumbnail;
        $thumbnail->baseDirectory = $this->dataDirectory;        
        $thumbnail->thumbnailDirectory = $this->thumbnailDirectory;
        $thumbnail->deleteThumbs($fileId);
        return $fileId;
      } else {
        $this->addMsg(MSG_ERROR, sprintf($this->_gt('Can not move file "%s".'), $fileName));
        $this->dbDeleteFile($fileId);
        @unlink($tempFileName);
      }
    } else {
      $this->addMsg(MSG_ERROR, sprintf($this->_gt('Can not add file "%s" to database.'), $fileName));
      @unlink($tempFileName);
    }
    return FALSE;
  }

  /**
  * Insert file into database
  *
  * @param integer $fileId
  * @param string $fileName
  * @param integer $size
  * @param string $mimeType
  * @param integer $date
  * @param mixed $folderId optional, default value NULL
  * @access public
  * @return boolean
  */
  function dataInsertFile($fileId, $fileName, $size, $mimeType, $date, $folderId = NULL) {
    $iFolder = (isset($folderId)) ? $folderId : (int)@$this->folderId;
    $sql = "SELECT COUNT(*) FROM %s WHERE file_id = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableFiles, $fileId))) {
      list($count) = $res->fetchRow();
      $data = array(
        'folder_id' => $iFolder,
        'file_name' => $fileName,
        'file_size' => $size,
        'file_type' => isset($mimeType) ? $mimeType : '',
        'file_date' => $date
      );
      if ($count > 0) {
        return (FALSE !== $this->databaseUpdateRecord($this->tableFiles, $data, 'file_id', $fileId));
      } else {
        $data['file_id'] = $fileId;
        return (FALSE !== $this->databaseInsertRecord($this->tableFiles, NULL, $data));
      }
    }
    return FALSE;
  }

  /**
  * Update file data in database
  *
  * @param integer $fileId
  * @param integer$size
  * @param string $mimeType
  * @param integer $date
  * @access public
  * @return boolean
  */
  function dbUpdateFileData($fileId, $size, $mimeType, $date) {
    $data = array(
      'file_size' => $size,
      'file_type' => $mimeType,
      'file_date' => $date);
    return (FALSE !== $this->databaseUpdateRecord($this->tableFiles, $data, 'file_id', $fileId));
  }

  /**
  * Get random created filename
  *
  * @param string $ext file extension
  * @access public
  * @return string filename
  */
  function getRandFilename($ext) {
    srand((double)microtime());
    do
      do
        $fileName = md5(uniqid(rand(),1));
      while (@file_exists($this->dataDirectory.$fileName.$ext));
    while ($this->dbFileExists($fileName));
    return $fileName.$ext;
  }

  /**
  * Get file extension
  *
  * @param string $fileName
  * @access public
  * @return string result or ''
  */
  function getExtension($fileName) {
    if (preg_match('/\.[^\.]+$/', $fileName, $match)) {
      return $match[0];
    } else {
      return '';
    }
  }
  
  /**
  *
  *
  * @param integer $imageType
  * @param string $fileName
  * @access public
  * @return mixed NULL or mimetype string
  */
  function getMimeType($imageType, $fileName) {
    switch ($imageType) {
      case 1 :
        return 'image/gif';
      case 2 :
        return 'image/jpeg';
      case 3 :
        return 'image/png';
      case 4 :
      case 13 :
        return 'application/x-shockwave-flash';
      default :
        $ext = strtolower(substr($fileName, strrpos($fileName,'.')+1));
        if (!(isset($this->mimeTypes) && is_object($this->mimeTypes))) {
          include_once(PAPAYA_INCLUDE_PATH.'system/base_mimetypes.php');
          $this->mimeTypes = new base_mimetypes();
        }
        if ($mime = $this->mimeTypes->loadMimeTypeByExt($ext)) {
          return $mime['mimetype_text'];
        } else {
          return NULL;
        }
    }
    return '';
  }

  /**
  *
  *
  * @param integer $imageType
  * @param string $fileName
  * @access public
  * @return mixed NULL or mimetype string
  */
  function getIcon($imageType, $fileName) {
    switch ($imageType) {
      case 1 : $ext = 'gif'; break;
      case 2 : $ext = 'jpeg'; break;
      case 3 : $ext = 'png'; break;
      case 4 : $ext = 'swf'; break;
      case 13 : $ext = 'swf'; break;
      default : $ext = strtolower(substr($fileName, strrpos($fileName,'.')+1));
    }
    if (isset($this->icons) && isset($this->icons[$ext])) {
      $icon = $this->icons[$ext];
      if (file_exists($this->getBasePath(TRUE).'pics/file/32/'.$icon)) {
        return $this->getLink(NULL, NULL, 'pics/file/32/'.$icon);
      }
    }
    return $this->getLink(NULL, NULL, 'pics/file/32/unknown.gif');
  }
  
  /**
  * Decode ID string
  *
  * @param string $str
  * @access public
  * @return array Id's or empty array
  */
  function decodeIdStr($str) {
    if (preg_match_all('#\d+#', $str, $regs, PREG_PATTERN_ORDER)) {
      return $regs[0];
    }
    return array();
  }


  /**
  * Load extension Icons
  *
  * @param string $extensions
  * @access public
  */
  function loadExtensionIcons($extensions) {
    if (!(isset($this->mimeTypes) && is_object($this->mimeTypes))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_mimetypes.php');
      $this->mimeTypes = new base_mimetypes();
    }
    if ($data = $this->mimeTypes->loadIconsByExtensions($extensions)) {
      if (isset($this->icons) && is_array($this->icons)) {
        $this->icons = array_merge($this->icons, $data);
      } else {
        $this->icons = $data;
      }
    }
  }

  /**
  * Size to string
  *
  * @param integer $size
  * @access public
  * @return integer size
  */
  function formatFileSizeToStr($size) {
    if ($size > 10000000000) {
      $size = round($size / 1073741824, 2).' GB';
    } elseif ($size > 10000000) {
      $size = round($size / 1048576, 2).' MB';
    } elseif ($size > 10000) {
      $size = round($size / 1024, 2).' kB';
    } else {
      $size = round($size).' Bytes';
    }
    return $size;
  }



  /**
  * Load file list from database
  *
  * @param integer $lngId
  * @param boolean $folderFilter optional, default value TRUE
  * @param boolean $imageOnly optional, default value FALSE
  * @param boolean $clipboard optional, default value FALSE
  * @access public
  * @return mixed boolean or files
  */
  function dataLoadFileList($lngId ,$folderFilter=TRUE,$imageOnly=FALSE, $clipboard=FALSE) {
    $emptyIds = NULL;
    $i = 0;
    unset($files);
    if ($clipboard) {
      $filter = "WHERE folder_id = -1";
    } elseif ($folderFilter) {
      $filter = "WHERE folder_id = '".(int)@$this->folderId."'";
      if ($imageOnly || (isset($this->params['imagesonly']) && $this->params['imagesonly'])) {
        $filter .= "AND file_type IN (1,2,3)";
      }
    } else {
      $filter = '';
    }
    $sql = "SELECT f.file_id, f.folder_id, f.file_name, f.file_size, f.file_type, f.file_date, t.file_title, t.file_comment, f.file_sort
            FROM %s AS f
            LEFT OUTER JOIN %s AS t ON (t.file_id = f.file_id AND t.lng_id = '%d')  
            $filter
           ORDER BY file_sort ASC, file_name ASC";    
    $params = array($this->tableFiles, $this->tableFilesTrans, $lngId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      $extensions = array();
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $files[$i] = $row;      
        if (($row['file_title'] == '') && (!defined('PAPAYA_ADMIN_PAGE')) && 
            defined('PAPAYA_CONTENT_LANGUAGE') && $lngId != PAPAYA_CONTENT_LANGUAGE && PAPAYA_CONTENT_LANGUAGE > 0) {
          // einzelne nachladen statt ganzer gruppe
          $emptyIds[$row['file_id']] = $i;
        }
        switch ($row['file_type']) {
          case 1 : $ext = 'gif'; break;
          case 2 : $ext = 'jpeg'; break;
          case 3 : $ext = 'png'; break;
          case 4 : $ext = 'swf'; break;
          default :
            $ext = strtolower(substr($row['file_name'], strrpos($row['file_name'],'.')+1));
            break;
        }
        $extensions[$ext] = TRUE;
        $i++;
      }      
      $this->getFileTransDetail($emptyIds, PAPAYA_CONTENT_LANGUAGE, $files);                
      $this->loadExtensionIcons(array_keys($extensions));
      if ($clipboard && isset($files)) {
        return $files;
      } elseif (isset($files)) {
        $this->files = $files;
        return TRUE;
      } else {
        return TRUE;
      }
    }
    return FALSE;
  }
  
  
  /**
  * Get file translation detail
  *
  * @param array $ids
  * @param integer $lngId
  * @param array &$fileArray
  * @access public
  */
  function getFileTransDetail($ids, $lngId, &$fileArray){
    if(isset($ids) && is_array($ids) && count($ids) > 0){
      $filter = '('.implode(',', array_keys($ids)).')';
      $sql = "SELECT file_id, file_title, file_comment, file_sort
                FROM %s   
               WHERE f.file_id IN $filter
                 AND lng_id = '%d'";
      $params = array($this->tableFilesTrans, $lngId);
      if($res = $this->databaseQueryFmt($sql, $params, 1)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {        
          $fileArray[$ids[$row['file_id']]]['file_sort'] = $row['file_sort'];
          $fileArray[$ids[$row['file_id']]]['file_comment'] = $row['file_comment'];
          $fileArray[$ids[$row['file_id']]]['file_title'] = $row['file_title'];          
        }
      }
    }      
  }

  /**
  * Load folder list from database
  *
  * @param $lngId
  * @access public
  * @return boolean
  */
  function dataLoadFolderList($lngId) {
    $emptyIds = NULL;
    unset($this->folders);
    unset($this->folderLinks);
    $sql = "SELECT f.folder_id, f.parent_id, t.folder_title, f.folder_path, f.surfer_useparent, f.surfer_permids
            FROM %s AS f
            LEFT OUTER JOIN %s AS t ON (t.folder_id = f.folder_id AND t.lng_id = '%d')  
           ORDER BY folder_title";
    $params = array($this->tableFolders,$this->tableFoldersTrans,$lngId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if (($row['folder_title'] == '') && (!defined('PAPAYA_ADMIN_PAGE')) && 
            defined('PAPAYA_CONTENT_LANGUAGE') && $lngId != PAPAYA_CONTENT_LANGUAGE && PAPAYA_CONTENT_LANGUAGE > 0) {
          // einzelne nachladen statt ganzer gruppe
          $emptyIds[$row['folder_id']] = TRUE;
        }
        $this->folders[(int)$row['folder_id']] = $row;
        $this->folderLinks[(int)$row['parent_id']]['CHILDS'][] = (int)$row['folder_id'];
      }
      $this->getFolderTransDetail($emptyIds, PAPAYA_CONTENT_LANGUAGE);
      $this->folders[0] = array('folder_id' => 0, 'folder_title' => $this->_gt('Desktop'));
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Get Folder Translation Details
  *
  * @param array $ids
  * @param integer $lngId
  * @access public
  */
  function getFolderTransDetail($ids, $lngId){
    if(isset($ids) && is_array($ids) && count($ids) > 0){
      $filter = '('.implode(',', array_keys($ids)).')';
      $sql = "SELECT folder_title, folder_id
                FROM %s   
               WHERE f.file_id IN $filter
                 AND lng_id = '%d'";
      $params = array($this->tableFolderTrans, $lngId);
      if($res = $this->databaseQueryFmt($sql, $params, 1)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {        
          $this->folders['folder_id']['folder_title'] = $row['folder_title']; 
        }
      }
    }      
  }

  
  /**
  * Load inherited permissions from database
  *
  * @param string $folderPath
  * @access public
  * @return array $perms
  */
  function dataLoadInheritedPerm($folderPath) {
    $ids = $this->decodeIdStr($folderPath);
    if (count($ids) > 1) {
      $filter = "IN ('".implode("','", $ids)."') ";
    } else {
      $filter = '= '.@(int)$ids[0];
    }
    $sql = "SELECT folder_id, surfer_useparent, surfer_permids
              FROM %s
              WHERE folder_id $filter AND surfer_useparent IN (1,3)";
    $tmp = array();
    $perms = array();
    if ($res = $this->databaseQueryFmt($sql, array($this->tableFolders))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $tmp[$row['folder_id']] = $row;
      }
      for($i = count($ids)-1; $i >= 0; $i--) {
        if(isset($tmp[$ids[$i]])) {
          $perm = $tmp[$ids[$i]];
          $perms = array_merge($perms, $this->decodeIdStr($perm['surfer_permids']));
          if ($perm['surfer_useparent'] == 1) {
            break;
          }
        }
      }
    }
    return $perms;
  }

  /**
  * Load folder
  *
  * @param integer $folderId
  * @access public
  * @return boolean
  */
  function dataLoadFolder($folderId, $lngId = 0) {
    unset($this->folder);
    if ($lngId > 0) {
      $sql = "SELECT f.folder_id, f.parent_id, t.folder_title, f.folder_path, f.surfer_useparent, f.surfer_permids
              FROM %s AS f
              LEFT OUTER JOIN %s AS t ON (t.folder_id = f.folder_id AND t.lng_id = '%d')
              WHERE f.folder_id = '%d'
             ORDER BY folder_title";
      $params = array($this->tableFolders, $this->tableFoldersTrans, $lngId, $folderId);
    } else {
      $sql = "SELECT f.folder_id, f.parent_id, f.folder_path, f.surfer_useparent, f.surfer_permids
              FROM %s AS f
              WHERE f.folder_id = '%d'";
      $params = array($this->tableFolders, $folderId);    
    }
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ((@$row['folder_title'] == '') && (!defined('PAPAYA_ADMIN_PAGE')) && 
            defined('PAPAYA_CONTENT_LANGUAGE') && $lngId != PAPAYA_CONTENT_LANGUAGE && PAPAYA_CONTENT_LANGUAGE > 0) {
          return $this->dataLoadFolder($folderId, PAPAYA_CONTENT_LANGUAGE);
        }
        if ($row['surfer_useparent'] == 1 || $row['surfer_useparent'] == 3) {
          $row['perms'] = array_flip($this->decodeIdStr($row['surfer_permids']));
        } else {
          $row['perms'] = array();
        }
        $row['inh_perms'] = array_flip($this->dataLoadInheritedPerm($row['folder_path']));
        if ($data = $this->checkFolderPath($row)) {
          $this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $folderId);
        }        
        $this->folder = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Get an random image from the current folder
  *
  * @param integer $lngId
  * @access public
  * @return mixed FALSE or data
  */
  function dataRandomImage($lngId) {
    $result = FALSE;
    $sql = "SELECT f.file_id, f.folder_id, f.file_name, f.file_size, f.file_type, f.file_date, t.file_title, t.file_comment
            FROM %s AS f
            LEFT OUTER JOIN %s AS t ON (t.file_id = f.file_id AND t.lng_id = '%d')  
           WHERE folder_id = %d AND file_type IN (1,2,3)
            ORDER BY RAND()";
    $params = array($this->tableFiles, $this->tableFilesTrans, $lngId, $this->folderId);
    if ($res = $this->databaseQueryFmt($sql,$params, 1)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result = $row;
      }
    }
    return $result;
  }
  
  /**
  * Check if file exists in database
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function dbFileExists($id) {
    unset($this->selectedFile);
    $sql = "SELECT count(*)
            FROM %s
            WHERE file_id LIKE '%s%%'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableFiles, $id))) {
      if ($row = $res->fetchRow()) {
        return ($row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Load file into database
  *
  * @param integer $id
  * @param integer $lngId optional, default value 0
  * @access public
  * @return boolean
  */
  function dataLoadFile($id, $lngId = 0) {
    unset($this->selectedFile);
    $id2 = substr($id, 0, strrpos($id, '.')-1);
    if ($lngId > 0) {
      $sql = "SELECT f.file_id, f.folder_id, f.file_name, f.file_size, f.file_type, f.file_date, 
                     t.file_title, t.file_comment, f.file_sort
                FROM %s AS f
                LEFT OUTER JOIN %s AS t ON (t.file_id = f.file_id AND t.lng_id = '%d')  
               WHERE f.file_id LIKE '%s%%'";
      $params = array($this->tableFiles, $this->tableFilesTrans, $lngId, $id2);
    } else {
      $sql = "SELECT f.file_id, f.folder_id, f.file_name, f.file_size, f.file_type, f.file_date
                FROM %s AS f
               WHERE f.file_id LIKE '%s%%'";
      $params = array($this->tableFiles, $id2);    
    }
    if ($res = $this->databaseQueryFmt($sql, $params, 1)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ((@$row['file_title'] == '') && (!defined('PAPAYA_ADMIN_PAGE')) && 
            defined('PAPAYA_CONTENT_LANGUAGE') && $lngId != PAPAYA_CONTENT_LANGUAGE && PAPAYA_CONTENT_LANGUAGE > 0) {
          return $this->dataLoadFile($id, PAPAYA_CONTENT_LANGUAGE);
        }
        $this->selectedFile = $row;
        return TRUE;
      }
    }
    return FALSE;
  }
  
  /**
  * Edit file properties in database
  *
  * @param integer $fileId
  * @param array $properties
  * @param integer $lngId
  * @access public
  * @return boolean
  */
  function dataEditFileProperties($fileId, $properties, $lngId) {
    $fields = array();
    if (isset($properties['file_name'])) $fields['file_name'] = $properties['file_name'];
    if (isset($properties['file_title'])) $fieldsTrans['file_title'] = $properties['file_title'];
    if (isset($properties['file_comment'])) $fieldsTrans['file_comment'] = $properties['file_comment'];
    if (isset($properties['file_sort'])) $fields['file_sort'] = $properties['file_sort'];
    
    
    $sql = "SELECT COUNT(*) FROM %s WHERE file_id = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableFilesTrans, $fileId))) {
      list($count) = $res->fetchRow();
      $data = array(
        'file_id' => $fileId,
        'lng_id' => $lngId,
        'file_title' => $fieldsTrans['file_title'],
        'file_comment' => $fieldsTrans['file_comment']
      );
      }
      $fileTrans = FALSE;
      if ($count > 0) {
        $fileTrans = $this->databaseUpdateRecord($this->tableFilesTrans, $fieldsTrans, array('file_id' => $fileId, 'lng_id' => $lngId));
      } else {
        $fileTrans = $this->databaseInsertRecord($this->tableFilesTrans, NULL, $data);
      }
    
    return (FALSE !== $this->databaseUpdateRecord($this->tableFiles, $fields, 'file_id', $fileId)
      && FALSE !== $fileTrans);
      
      
  }

  /**
  * Check folder path
  *
  * @param array &$folder
  * @access public
  * @return mixed array or FALSE
  */
  function checkFolderPath(&$folder) {
    if ($folder['parent_id'] == 0) {
      $path = ';0;';
    } elseif (preg_match_all('/\d+/',$folder['folder_path'], $regs, PREG_PATTERN_ORDER)) {
      $path = ';'.implode(';',$regs[0]).';';
    }
    if ($path != $folder['folder_path']) {
      return array('folder_path'=>$path);
    } else {
      return FALSE;
    }
  }

}

?>
