<?php
/**
* Language managment UI and Content
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_language_select.php,v 1.6 2006/01/12 16:27:07 weinert Exp $
*/

/**
* Base class for database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Language managment UI and Content
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_language_select extends base_db {

  var $paramName = 'lngsel';

  var $tableLanguage = PAPAYA_DB_TBL_LNG;

  var $currentLanguageId = 0;
  var $currentLanguage = NULL;

  var $languages = array();
  var $languageShort = array();

  function &getInstance() {
    $globalVariableName = 'PAPAYA_LNGSELECT';
    if (isset($GLOBALS[$globalVariableName]) && is_object($GLOBALS[$globalVariableName]) && is_a($GLOBALS[$globalVariableName], 'base_language_select')) {
      return $GLOBALS[$globalVariableName];
    } else {
      $GLOBALS[$globalVariableName] = &new base_language_select;
      return $GLOBALS[$globalVariableName];
    }
  }

  function __construct() {
    $this->initialize();
  }

  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_language_'.$this->paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    $this->loadLanguages();
    $this->initializeSessionParam('lng');

    if (isset($this->params['lng']) && (int)$this->params['lng'] > 0 && isset($this->languages[$this->params['lng']])) {
      $this->currentLanguageId = (int)$this->params['lng'];
    } elseif (isset($GLOBALS['PAPAYA_USER']) && isset($GLOBALS['PAPAYA_USER']->options['PAPAYA_CONTENT_LANGUAGE']) &&
      isset($this->languages[$GLOBALS['PAPAYA_USER']->options['PAPAYA_CONTENT_LANGUAGE']])) {
      $this->currentLanguageId = (int)$this->languages[$GLOBALS['PAPAYA_USER']->options['PAPAYA_CONTENT_LANGUAGE']]['lng_id'];
    } elseif (defined('PAPAYA_CONTENT_LANGUAGE') && isset($this->languages[PAPAYA_CONTENT_LANGUAGE])) {
      $this->currentLanguageId = (int)$this->languages[PAPAYA_CONTENT_LANGUAGE]['lng_id'];
    } else {
      $this->currentLanguageId = 0;
    }

    if (isset($this->languages[(int)$this->currentLanguageId])) {
      $this->currentLanguage = &$this->languages[(int)$this->currentLanguageId];
    } else {
      $this->currentLanguage = NULL;
    }

    $this->sessionParams['lng'] = (int)$this->currentLanguageId;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  function loadLanguages() {
    unset($this->languages);
    unset($this->languageShort);
    $sql = "SELECT lng_id, lng_ident, lng_short, lng_title, lng_glyph
              FROM %s
              WHERE is_content_lng = 1
              ORDER BY lng_title, lng_short, lng_ident, lng_id";
    if ($res = $this->databaseQueryFmt($sql, $this->tableLanguage)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->languages[$row['lng_id']] = $row;
        $this->languageShort[$row['lng_short']] = &$this->languages[$row['lng_id']];
      }
      return TRUE;
    }
    return FALSE;
  }

  function getComboFormXML($width = 250) {
    if (isset($this->languages) && is_array($this->languages)) {
      $result = sprintf('<dialog action="%s" title="%s" width="%d" name="languageselect">', $this->baseLink,
        papaya_strings::escapeHTMLChars($this->_gt('Content Language')), $width);
      $result .= '<lines>';
      $result .= '<line align="center">';
      $result .= sprintf('<select name="%s[lng]" class="dialogSelect" style="width: %dpx;" onchange="this.form.submit();">', $this->paramName, $width - 20);
      foreach($this->languages as $lng) {
        $selected = (@$lng['lng_id'] == $this->currentLanguageId) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s"%s>%s (%s)</option>', (int)$lng['lng_id'], $selected,
          papaya_strings::escapeHTMLChars($lng['lng_title']), papaya_strings::escapeHTMLChars($lng['lng_short']));
      }
      $result .= '</select>';
      $result .= '</line>';
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s" />', $this->_gt('Select'));
      $result .= '</dialog>';
      return $result;
    }
  }
  
  /**
  * Get language combo
  *
  * @param string $paramName
  * @param string $name
  * @param array $element
  * @param mixed $data
  * @access public
  * @return string XML
  */
  function getContentLanguageCombo($paramName, $name, $element, $data) {
    $result = '';
    if (isset($this->languages) && is_array($this->languages) && count($this->languages) > 0) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $paramName, $name);
      foreach ($this->languages as $language) {
        $selected = ($language['lng_id'] == $data) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%d"%s><![CDATA[%s (%s)]]></option>'.LF,
          papaya_strings::escapeHTMLChars($language['lng_id']), $selected, $language['lng_title'], $language['lng_short']);
      }
      $result .= '</select>'.LF;
    } else {
      $result = sprintf('<input type="text" disabled="disabled" value="%s" class="dialogInput dialogScale"/>', 'No language found');
    }
    return $result;
  }

}
?>