<?php
/**
* Provides basic functionality for log messages
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_log.php,v 1.14 2006/05/10 13:50:20 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Provides basic functionality for log messages
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_log extends base_db {
  var $table = PAPAYA_DB_TBL_LOG;

  var $tableTypes = '';
  var $messageList;
  var $images;
  var $layout;
  var $msgs;

  var $levels = array('All','Information','Warning','Error');

  var $paramName = 'plog';
  var $sessionParamName;
  var $baseLink;
  var $params;
  var $sessionParams;

  var $toolbar;

  var $selected = 0;
  var $selLevel = -1;
  var $selType = 0;

  var $steps = 20;

  function __construct() {
    $this->definePHPErrorHandler();
  }
  
  /**
  * Initialize parameters
  *
  * @access public
  */
  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_log_'.$this->paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('offset');
    $this->initializeSessionParam('level', array('offset'));
    $this->initializeSessionParam('type', array('offset'));
    $this->initializeSessionParam('id');
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    $this->selected = (int)$this->params['id'];
    $this->selLevel = (int)$this->params['level'];
    $this->selType = (int)$this->params['type'];

    foreach ($this->levels as $key=>$val) {
      $this->levels[$key] = $this->_gt($val);
    }
  }

  /**
  * Log messages
  *
  * @param integer $messageNumber
  * @param integer $msgType
  * @param string $msgShort
  * @param string $msgLong
  * @access public
  * @return boolean
  */
  function logMessage($messageNumber, $msgType, $msgShort, $msgLong = '') {
    $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
    $baseHref = $protocol."://".$_SERVER['HTTP_HOST'];
    $values = array(
      'log_time' => time(),
      'log_msgtype' => $msgType,
      'log_msgno' => $messageNumber,
      'log_msg_short' => $msgShort,
      'log_msg_long' => (@trim($msgLong) != '') ? $msgLong : $msgShort,
      'log_msg_uri' => $baseHref.$_SERVER['REQUEST_URI'],
      'log_msg_script' => $_SERVER['SCRIPT_FILENAME'],
      'log_msg_from_ip' => $_SERVER['REMOTE_ADDR']
    );
    if (isset($GLOBALS['PAPAYA_USER']) && is_object($GLOBALS['PAPAYA_USER']) &&
        is_object($GLOBALS['PAPAYA_USER']) && $GLOBALS['PAPAYA_USER']->isValid) {
      $values['user_id'] = $GLOBALS['PAPAYA_USER']->userId;
      $values['username'] = $GLOBALS['PAPAYA_USER']->user['fullname'];
    }
    return FALSE !== $this->databaseInsertRecord($this->table, 'log_id', $values);
  }

  function logMsg($level,$type,$short,$long = '') {
    echo (($long != '') ? $long : $short);
  }

  /**
  * Load list
  *
  * @access public
  * @return boolean
  */
  function loadList() {
    unset($this->messageList);
    $this->messageCount = 0;
    $this->messageAbsCount = 0;
    $sql = "SELECT log_id, log_time, log_msgtype, log_msgno, log_msg_short
            FROM %s
            WHERE 1=1 \n";
    if ($this->params['type'] > 0) $sql .= "AND log_msgtype = '".(int)$this->params['type']."'\n";
    if ($this->params['level'] > 0) $sql .= "AND log_msgno = '".(int)($this->params['level']-1)."'\n";
    $sql .= "  ORDER BY log_time DESC, log_id DESC";
    $offset = (isset($this->params['offset'])) ? (int)$this->params['offset'] : 0;
    if ($res = $this->databaseQueryFmt($sql, array($this->table),$this->steps, $offset)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->messageList[$row['log_id']] = $row;
      }
      $this->messageAbsCount = $res->absCount();
      return $this->loadMsgGroup();
    }
    return FALSE;
  }

  function loadMessage($id) {
    $sql = "SELECT log_id, log_time, log_msgtype, log_msgno,
                    log_msg_short, log_msg_long,
                    log_msg_uri, log_msg_script,
                    log_msg_from_ip, user_id, username
            FROM %s
            WHERE log_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->table, $id))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->messageList[$row['log_id']] = $row;
        return TRUE;
      }
    }
    return FALSE;

  }

  /**
  * Load message group
  *
  * @access public
  * @return boolean
  */
  function loadMsgGroup() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
    foreach(base_statictables::getTableLogGroups() as $key=>$val) {
      $this->messageTypeList[$key] = $this->_gt($val);
    }
    return TRUE;
  }

  /**
  * Delete old
  *
  * @param integer $timeFrame
  * @access public
  * @return mixed
  */
  function deleteOld($timeFrame) {
    $sql = "DELETE FROM %s WHERE log_time < '%d'";
    return $this->databaseQueryFmt($sql,  array($this->table, (int)$timeFrame));
  }

  /**
  * Delete all
  *
  * @access public
  * @return mixed
  */
  function deleteAll() {
    return ($this->databaseEmptyTable($this->table) !==  FALSE);
  }

  /**
  * Action clear
  *
  * @access public
  */
  function actionClear() {
    if (isset($this->messageList) && is_array($this->messageList)) {
      if (isset($this->params['confirm']) && $this->params['confirm']) {
        if ($this->deleteAll()) {
          $this->addMsg(MSG_INFO, $this->_gt('Protocol deleted.'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
        }
        $this->loadList();
      } else {
        $this->layout->add($this->getClearDlg());
      }
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('Protocol is empty.'));
    }
  }

  /**
  * Action delte old
  *
  * @access public
  */
  function actionDelOld() {
    if (isset($this->messageList) && is_array($this->messageList)) {
      $selected = $this->messageList[$this->selected];
      if (isset($selected) && is_array($selected)) {
        if (isset($this->params['confirm'])) {
          if ($this->deleteOld($selected['log_time'])) {
            $this->addMsg(MSG_INFO, $this->_gt('Old messages deleted.'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!'));
          }
          $this->loadList();
        } else {
          $this->layout->add($this->getDeleteOldDlg());
        }
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('No message selected.'));
      }
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('The protocol is empty.'));
    }
  }

  /**
  * Get XML
  *
  * @access public
  */
  function getXML() {
    switch (@$this->params['cmd']) {
      case 'clear' : $this->actionClear(); break;
      case 'del_old' : $this->actionDelOld(); break;
    }
    if (isset($this->layout) && is_object($this->layout)) {
      $this->getSelected();
      $this->layout->add($this->getList());
      $this->layout->add('<menu>'.$this->getButtonsXML().'</menu>', 'menus');
    }
  }

  /**
  * Get list
  *
  * @access public
  * @return string XML
  */
  function getList() {
    $result = '';
    if (isset($this->messageList) && is_array($this->messageList)) {
      $result .= sprintf('<listview title="%s" width="100%%">', $this->_gt('Event protocol'));
      $result .= sprintf('<cols><col>%s</col><col align="center">%s</col><col align="center">%s</col></cols>',
                         $this->_gt('Message'), $this->_gt('Group'), $this->_gt('Date'));
      $result .= '<items>';
      foreach ($this->messageList as $msgId=>$msg) {
        if (($this->selLevel < 1) || (($this->selLevel-1) == $msg['log_msgno'])) {
          if (($this->selType < 1) || ($this->selType == $msg['log_msgtype'])) {
            switch ($msg['log_msgno']) {
              case '0': $imageIndex = 63; break;
              case '1': $imageIndex = 64; break;
              case '2': $imageIndex = 65; break;
            }
            $href = $this->baseLink.'?'.$this->paramName.'[id]='.$msgId;

            $selected = ($msgId == $this->params['id']) ? ' selected="selected"' : '';

            $result .= sprintf('<listitem title="%s" image="%s" href="%s"%s>',
              papaya_strings::escapeHTMLChars($msg['log_msg_short']), $this->images[$imageIndex], $href, $selected);
            $result .= sprintf('<subitem align="center">%s</subitem>', papaya_strings::escapeHTMLChars($this->messageTypeList[$msg['log_msgtype']]));
            $result .= sprintf('<subitem align="center">%s</subitem>', date('Y-m-d H:i:s', $msg['log_time']));
            $result .= '</listitem>';
          }
        }
      }
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get selected
  *
  * @access public
  * @return string XML
  */
  function getSelected() {
    $selected = $this->selected;
    $result = '';
    if (isset($this->messageList[$selected]) && is_array($this->messageList[$selected])) {
      $this->loadMessage($selected);
      $result .= sprintf('<listview width="100%%" title="%s">', $this->_gt('Information'));
      $result .= '<items>';
      $result .= sprintf('<listitem title="%s">', $this->_gt('URL'));
      $result .= '<subitem>'.papaya_strings::escapeHTMLChars($this->messageList[$selected]['log_msg_uri']).'</subitem>';
      $result .= '</listitem>';
      $result .= sprintf('<listitem title="%s">', $this->_gt('Script'));
      $result .= '<subitem>'.papaya_strings::escapeHTMLChars($this->messageList[$selected]['log_msg_script']).'</subitem>';
      $result .= '</listitem>';
      if (trim($this->messageList[$selected]['log_msg_from_ip']) != '') {
        $host = gethostbyaddr($this->messageList[$selected]['log_msg_from_ip']);
        if ($host != $this->messageList[$selected]['log_msg_from_ip']) {
          $result .= sprintf('<listitem title="%s">', $this->_gt('Host'));
          $result .= '<subitem>'.papaya_strings::escapeHTMLChars($host).'</subitem>';
          $result .= '</listitem>';
        }
      }
      $result .= sprintf('<listitem title="%s">', $this->_gt('IP'));
      $result .= '<subitem>'.papaya_strings::escapeHTMLChars($this->messageList[$selected]['log_msg_from_ip']).'</subitem>';
      $result .= '</listitem>';
      $result .= sprintf('<listitem title="%s">', $this->_gt('Username'));
      $result .= '<subitem>'.papaya_strings::escapeHTMLChars($this->messageList[$selected]['username']).'</subitem>';
      $result .= '</listitem>';
      $result .= '</items>';
      $result .= '</listview>';
      $result .= '<sheet width="100%" align="center">';
      $result .= '<text><div style="padding: 10px;"><noescape><![CDATA[';
      $result .= wordwrap($this->messageList[$selected]['log_msg_long'],80," ", TRUE);
      $result .= ']]></noescape></div></text>';
      $result .= '</sheet>';
      $this->layout->add($result);
    }
  }

  /**
  * Get clear Dialog
  *
  * @access public
  * @return string XML
  */
  function getClearDlg() {
    $result = sprintf('<msgdialog title="%s" action="%s" type="question" width="100%%">',
        'Veffentlichung entfernen', $this->baseLink);
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
      $this->paramName, 'cmd', 'clear');
    $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
      $this->paramName, 'confirm', 1);
    $result .= '<message>';
    $result .= 'Soll das Ereignisprotokoll geleert werden?';
    $result .= '</message>';
    $result .= '<dlgbutton value="Leeren"/>';
    $result .= '</msgdialog>';
    return $result;
  }

  /**
  * Get delete old dialog
  *
  * @access public
  * @return string XML
  */
  function getDeleteOldDlg() {
    $selected = $this->messageList[$this->selected];
    if (isset($selected) && is_array($selected)) {
      $result = sprintf('<msgdialog action="%s" type="question" width="100%%">', $this->baseLink);
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'cmd', 'del_old');
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'id', $this->selected);
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'confirm', 1);
      $result .= '<message>';
      $result .= sprintf($this->_gt('Delete older (&lt;%s) events?'),
        date('Y-m-d H:i:s', $selected['log_time']));
      $result .= '</message>';
      $result .= sprintf('<dlgbutton value="%s"/>', $this->_gt('Delete'));
      $result .= '</msgdialog>';
    }
    return $result;
  }

  /**
  * Get pages navigation
  *
  * @param object &$toolbar
  * @param integer $offset
  * @param integer $step
  * @param integer $max
  * @access public
  */
  function getPagesNav(&$toolbar, $offset, $step, $max) {
    if (($max > $step) && is_object($toolbar)) {
      if ($offset > 0) {
        $i = (($offset - $step) > 0) ? ($offset - $step) : 0;
        $toolbar->addButton('Prior', $this->getLink(array('cmd'=>'show', 'offset'=>$i)), 8, '');
      }
      for($i=0,$x=1;$i<$max;$i+=$step,$x++) {
        $pages[$i] = $x;
      }
      $toolbar->addCombo('Pages', $this->getLink(array('cmd'=>'show')),
        $this->paramName.'[offset]', $offset, $pages);
      if ($offset < ($max-$step)) {
        $i = (($offset + $step) < $max) ? ($offset + $step) : ($max-$step);
        $toolbar->addButton('Next', $this->getLink(array('cmd'=>'show', 'offset'=>$i)), 9, '');
      }
    }
  }

  /**
  * Get XML buttons
  *
  * @access public
  * @return string XML
  */
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    $toolbar->addButton('Login Try', 'log_auth.php', 31, '');
    $toolbar->addSeperator();
    
    $toolbar->addCombo('Priority', $this->getLink(array('cmd'=>'show')),
      $this->paramName.'[level]', (int)$this->params['level'], $this->levels);
    $toolbar->addSeperator();
    $tempTypes = array_merge(array(0=>$this->_gt('All')), $this->messageTypeList);
    asort($tempTypes);
    $toolbar->addCombo('Type', $this->getLink(array('cmd'=>'show')),
      $this->paramName.'[type]', (int)$this->params['type'], $tempTypes);
    $toolbar->addSeperator();
    $toolbar->addButton('Refresh', $this->getLink(array('cmd'=>'show')), 38, '');
    $toolbar->addSeperator();
    if (isset($this->messageList[$this->selected])) {
      $toolbar->addButton('Delete old', $this->getLink(array('cmd'=>'del_old', 'id'=>$this->selected)), 10, 'Delete all events older like this.');
    }
    $toolbar->addButton('Empty', $this->getLink(array('cmd'=>'clear')), 10, 'Delete all events');
    $toolbar->addSeperator();
    $offset = (isset($this->params['offset'])) ? (int)$this->params['offset'] : 0;
    $this->getPagesNav($toolbar, (int)$offset,  $this->steps, $this->messageAbsCount);
    return $toolbar->getXML();
  }
  
  /**
  * Translate error, insert and attack parameter
  *
  * @param integer $id error typ
  * @param string $msg error text
  * @param array $vals parameters
  */
  function addMsgTranslatedFmt($id, $msg, $vals) {
    if (isset($vals) && is_array($vals)) {
      $this->add($id, vsprintf($this->_gt($msg), $vals));
    } else {
      $this->addMsgTranslated($id, $msg);
    }
  }
  
  function handlePHPError($errNo, $errStr, $errFile = '', $errLine = 0, $errContext = NULL) {
    $errorReporting = error_reporting();
    if ($errorReporting > 0) {   
      $stopScript = FALSE;
      switch ($errNo) {
      case E_ERROR:
      case E_USER_ERROR:
        $title = 'ERROR';
        $level = MSG_ERROR;
        $stopScript = TRUE;
        break;
      case E_WARNING:
      case E_USER_WARNING:
        $title = 'WARNING';
        $level = MSG_WARNING;
        break;
      case E_NOTICE:
      case E_USER_NOTICE:
        $title = 'NOTICE';
        $level = MSG_INFO;
        break;
      default:
        $title = 'UNKNOWN ERROR';
        $level = MSG_ERROR;
        break;
      }
      if ($errorReporting & $errNo) {
        if (defined('PAPAYA_DBG_PHP_SHOW_ERROR') && PAPAYA_DBG_PHP_SHOW_ERROR) {
          echo '</form>', '</table>';
          echo '<div class="errorApplication" style="position: relative; z-index: 10000;">';
          echo '<table border="1" width="800" cellpadding="2" cellspacing="0" align="center" bgcolor="#C0C0C0">';
          echo '<tr><th colspan="2">'.papaya_strings::escapeHTMLChars($title).'</th></tr>';
          echo '<tr><td colspan="2"><b>'.$errNo.':</b> '.papaya_strings::escapeHTMLChars($errStr).'</td></tr>';
          if (isset($errFile) && $errFile != '') {
            echo '<tr><td>'.papaya_strings::escapeHTMLChars($errFile).'</td><td>'.@(int)$errLine.'</td></tr>';
          }
          if (isset($errContext) && is_array($errContext) && count($errContext) > 0) {
            echo '<tr><td colspan="2"><table border="1" cellspacing="0" width="100%">';
            foreach ($errContext as $name=>$val) {
              echo '<tr style="vertical-align: top;">';
              switch(gettype($val)) {
              case 'object' :
                echo '<td>object</td><td><b>$'.papaya_strings::escapeHTMLChars($name).'</b></td><td><tt>'.get_class($val).'</tt></td>';
                break;
              case 'array' :
                echo '<td>array <b></td><td><b>$'.papaya_strings::escapeHTMLChars($name).'</b></td><td><tt>'.count($val).' entries</tt></td>';
                break;
              default :
                echo '<td>'.gettype($val).'</td><td><b>$'.papaya_strings::escapeHTMLChars($name).
                  '</b></td><td><tt>'.papaya_strings::escapeHTMLChars($val).'</tt></td>';
                break;              
              }
              echo '</tr>';
            }
            echo '</table></td></tr>';
          }
          if ($stopScript) {
            echo '<tr><th colspan="2">Aborting...</th></tr>';
          }
          echo '</table><br/>';
          echo '</div>';
          flush();
        }
        if (defined('PAPAYA_DBG_PHP_LOG_ERROR') && PAPAYA_DBG_PHP_LOG_ERROR) {
          if (!(isset($errContext) && isset($errContext['this']) && 
              get_class($errContext['this']) == get_class($this))) {
            $msg = $title.' #'.$errNo.' '.$errStr;
            if (isset($errFile) && $errFile != '') {
              $msgLong = $msg.' in '.$errFile.':'.@(int)$errLine;
            } else {
              $msgLong = $msg;
            }
            $this->logMessage($level, 9, $msg, $msgLong);
          }
        }
      }
      if ($stopScript) {
        exit();
      }
    }
  }
  
  function definePHPErrorHandler() {
    if (defined('PAPAYA_DBG_PHP_CAPTURE_ERROR') && PAPAYA_DBG_PHP_CAPTURE_ERROR) {
      set_error_handler(array(&$this, 'handlePHPError'));
    }
  }
}
?>