<?php
/**
* basic class translation engine
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_phraseeditor.php,v 1.7 2006/04/20 18:24:00 weinert Exp $
*/


/**
* basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* basic class translation engine
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_phraseeditor extends base_db {
  var $tablePhrase = PAPAYA_DB_TBL_PHRASE;
  var $tableTranslation = PAPAYA_DB_TBL_PHRASE_TRANS;
  var $tableModule = PAPAYA_DB_TBL_PHRASE_MODULE;
  var $tableModuleRelation = PAPAYA_DB_TBL_PHRASE_MODULE_REL;
  var $tableLanguage = PAPAYA_DB_TBL_LNG;
  var $tableLog = PAPAYA_DB_TBL_PHRASE_LOG;

  var $paramName = 'phr';
  var $sessionParamName = 'PAPAYA_SESS_phr';
  var $params = NULL;
  var $sessionParams = NULL;
  var $baseLink = '';

  var $messageSteps = 10;
  var $phraseSteps = 20;
  var $messageAbsCount = 0;
  var $phraseAbsCount = 0;

  /**
  * Initialize parameters
  *
  * @param string $paramName optional, default value 'phr'
  * @access public
  */
  function initialize($paramName = 'phr') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_phrases_'.$paramName;
    $this->initializeParams();

    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('msgoffset');
    $this->initializeSessionParam('phroffset');
    $this->initializeSessionParam('letter');
    //$this->initializeSessionParam('offset');
    $this->initializeSessionParam('module_id', array('msgoffset', 'phroffset', 'letter'));
    $this->initializeSessionParam('lng_id', array('msgoffset', 'phroffset', 'letter'));
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Baic function for handling parameters
  *
  * @access public
  */
  function execute() {
    $this->loadLanguages();
    switch (@$this->params['cmd']) {
      case 'emptymodules' :
        if ($this->emptyModules()) {
          $this->addMsg(MSG_INFO, $this->_gt('Optimisation flushed.'));
        }
        break;
      case 'emptylog' :
        if ($this->emptyLog()) {
          $this->addMsg(MSG_INFO, $this->_gt('Protocol of translation engine flushed.'));
        }
        break;
      case 'delmsg' :
        if ($this->deleteLogMsg($this->params['log_msg_id'])) {
          $this->addMsg(MSG_INFO, $this->_gt('Message deleted.'));
        }
      case 'add' :
        if ($new = $this->addPhrase(@$this->params['phrase'])) {
          $this->params['phrase_id'] = $new;
          unset($this->params['msgoffset']);
          unset($this->params['phroffset']);
          unset($this->params['letter']);
          $this->sessionParams['msgoffset'] = 0;
          $this->sessionParams['phroffset'] = 0;
          $this->sessionParams['letter'] = '';
          $this->addMsg(MSG_INFO, $this->_gt('Phrase added.'));
        }
        break;
      case 'del' :
        if (isset($this->params['phrase_id'])) {
          $this->loadPhrase($this->params['phrase_id'], FALSE);
          if ($this->deletePhrase()) {
            unset($this->params['phrase_id']);
            unset($this->params['phrase']);
            unset($this->params['cmd']);
            unset($this->phrase);
            $this->addMsg(MSG_INFO, $this->_gt('Phrase deleted.'));
          }
        }
        break;
      case 'save' :
        if (isset($this->params['phrase_id'])) {
          if ($this->loadPhrase($this->params['phrase_id'], FALSE)) {
            if ($this->saveTranslations()) {
                $this->addMsg(MSG_INFO, $this->_gt('Changes saved.'));
            }
          }
        }
        break;

    }
    
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    
    $this->loadPhrases($this->params['module_id']);
    if (isset($this->params['phrase_id'])) {
      $this->loadPhrase($this->params['phrase_id'], FALSE);
    }
    $this->loadModules();
    $this->loadLogMessages();
  }

  /**
  * Get XML
  *
  * @access public
  */
  function getXML() {
    if (is_object($this->layout)) {
      $this->layout->setParam('COLUMNWIDTH_LEFT', 400);
      $this->layout->setParam('COLUMNWIDTH_CENTER', '100%');
      $this->layout->setParam('COLUMNWIDTH_RIGHT', 0);
      $this->getXMLButtons();
      $this->getLogMsgList();
      $this->getPhrasesList();
      switch (@$this->params['cmd']) {
        case 'del' : $this->getDelForm(); break;
        case 'add' : $this->getAddForm();
        default : $this->getEditForm();
      }
    }
  }

  /**
  * load phrases
  *
  * @param integer $moduleId
  * @access public
  * @return boolean
  */
  function loadPhrases($moduleId) {
    unset($this->phrases);
    $res = FALSE;
    if (isset($this->params['letter']) && preg_match('~^[a-z]$~', $this->params['letter'])) {
      $letterFilter = " AND p.phrase_text_lower LIKE '".$this->params['letter']."%%'";
    } elseif (isset($this->params['letter']) && $this->params['letter'] == '#') {
      $subString = $this->databaseGetSQLSource('SUBSTRING', 'p.phrase_text_lower', FALSE, 1, TRUE, 1, TRUE);
      $letterFilter = " AND (NOT(".$subString." IN ('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z')))";
    } else {
      $letterFilter = '';
    }
    if (($moduleId > 0) && ($this->params['lng_id'] > 0)) {
      $sql = "SELECT DISTINCT p.phrase_id, p.phrase_text, p.phrase_text_lower
              FROM
                %s AS mr,
                %s AS p
                LEFT OUTER JOIN %s AS pt ON
                  (pt.lng_id = %d AND pt.phrase_id = p.phrase_id)
              WHERE mr.module_id = '%d'
                AND p.phrase_id = mr.phrase_id
                AND pt.lng_id IS NULL
                $letterFilter
              ORDER BY p.phrase_text_lower ASC";
      $params = array($this->tableModuleRelation,
                      $this->tablePhrase,
                      $this->tableTranslation,
                      (int)$this->params['lng_id'],
                      (int)$moduleId);
      $res = $this->databaseQueryFmt($sql, $params,$this->phraseSteps, @(int)$this->params['phroffset']);
    } elseif ($this->params['lng_id'] > 0) {
      $sql = "SELECT DISTINCT p.phrase_id, p.phrase_text, p.phrase_text_lower
              FROM %s AS p
                LEFT OUTER JOIN %s AS pt ON
                  (pt.lng_id = %d AND pt.phrase_id = p.phrase_id)
              WHERE pt.lng_id IS NULL
                $letterFilter
              ORDER BY p.phrase_text_lower ASC";
      $params = array($this->tablePhrase,
                      $this->tableTranslation,
                      (int)$this->params['lng_id']);
      $res = $this->databaseQueryFmt($sql, $params,$this->phraseSteps, @(int)$this->params['phroffset']);
    } elseif ($moduleId > 0) {
      $sql = "SELECT DISTINCT p.phrase_id, p.phrase_text, p.phrase_text_lower
              FROM %s AS mr
                LEFT JOIN %s AS p ON p.phrase_id = mr.phrase_id
              WHERE mr.module_id = '%d'
                $letterFilter
              ORDER BY p.phrase_text_lower ASC";
      $params = array($this->tableModuleRelation,$this->tablePhrase,(int)$moduleId);
      $res = $this->databaseQueryFmt($sql, $params,$this->phraseSteps, @(int)$this->params['phroffset']);
    } else {
      $sql = "SELECT p.phrase_id, p.phrase_text, p.phrase_text_lower
              FROM %s AS p
              WHERE 1 = 1
                $letterFilter
              ORDER BY p.phrase_text_lower ASC";
      $params = array($this->tablePhrase);
      $res = $this->databaseQueryFmt($sql, $params,$this->phraseSteps, @(int)$this->params['phroffset']);
    }
    if ($res) {
      $ids = NULL;
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->phrases[$row['phrase_id']] = $row;
        $ids[] = (int)$row['phrase_id'];
      }
      $this->phraseAbsCount = $res->absCount();
      if (@is_array($ids) && (count($ids)>0)) {
        $filter = (count($ids)>1) ? ' IN ('.implode(',', $ids).')' : ' = '.$ids[0];
        $sql = "SELECT pt.phrase_id, pt.translation, pt.lng_id
                FROM %s AS pt
                WHERE pt.phrase_id $filter
                ORDER BY pt.lng_id ASC";
        $params= array($this->tableTranslation);
        if ($res = $this->databaseQueryFmt($sql,$params)) {
          while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            if (trim($row['translation']) != '') {
              $this->phrases[$row['phrase_id']]['phrases'][$row['lng_id']] = $row['translation'];
            }
          }
        }
      }
      unset($ids);
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load phrase
  *
  * @param integer $id
  * @param boolean $useDatabase optional, default value FALSE
  * @access public
  * @return boolean
  */
  function loadPhrase($id, $useDatabase = FALSE) {
    unset($this->phrase);
    if ((!$useDatabase) && isset($this->phrases[(int)$id])) {
      $this->phrase = $this->phrases[(int)$id];
      return TRUE;
    } else {
      $sql = "SELECT p.phrase_id, p.phrase_text, pt.translation, pt.lng_id
              FROM %s AS p
                LEFT OUTER JOIN %s AS pt ON pt.phrase_id = p.phrase_id
              WHERE p.phrase_id = '%d'";
      $params = array($this->tablePhrase,$this->tableTranslation,(int)$id);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->phrase['phrase_id'] = $row['phrase_id'];
          $this->phrase['phrase_text'] = $row['phrase_text'];
          if (trim($row['translation']) != '') {
            $this->phrase['phrases'][$row['lng_id']]  = $row['translation'];
          }
        }
        return (isset($this->phrase) && is_array($this->phrase));
      }
    }
    return FALSE;
  }

  /**
  * Add phrase
  *
  * @param string $newPhrase
  * @access public
  * @return boolean
  */
  function addPhrase($newPhrase) {
    $phrase = trim($newPhrase);
    if ($phrase != '') {
      $data['phrase_text'] = $phrase;
      $data['phrase_text_lower'] = strToLower($phrase);
      $sql = "SELECT phrase_id
              FROM %s
              WHERE phrase_text_lower = '%s'";
      $params = array($this->tablePhrase,$data['phrase_text_lower']);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow()) {
          return FALSE;
        }
      }
      if ($new = $this->databaseInsertRecord($this->tablePhrase, 'phrase_id', $data)) {
        $this->cleanLog($phrase);
        return $new;
      }
    }
    return FALSE;
  }

  /**
  * Delete phrase
  *
  * @access public
  * @return boolean
  */
  function deletePhrase() {
    if (isset($this->params['confirm_delete']) && is_array($this->phrase)) {
      return ( FALSE !== $this->databaseDeleteRecord($this->tablePhrase, 'phrase_id', $this->phrase['phrase_id']));
    }
    return FALSE;
  }

  /**
  * Save translations
  *
  * @access public
  * @return boolean
  */
  function saveTranslations() {
    if (isset($this->languages) && is_array($this->languages) && isset($this->phrase)) {
      $result = TRUE;
      if (($this->phrase['phrase_text'] != $this->params['phrase']) && (trim($this->params['phrase']) != '')) {
        $values = array(  'phrase_text' => $this->params['phrase'],
                          'phrase_text_lower' => strtolower($this->params['phrase'])
                  );
        if (FALSE === $this->databaseUpdateRecord($this->tablePhrase, $values, 'phrase_id', (int)$this->params['phrase_id'])) return FALSE;
      }
      foreach($this->languages as $lngId=>$lng) {
        $old = trim(@$this->phrase['phrases'][$lngId]);
        $new = trim(@$this->params['trans'][$lngId]);
        $res=FALSE;
        if ($old != $new) {
          $sql = '';
          if (($old != '') && ($new != '')) {
            $sql = "UPDATE %s SET translation='%s' WHERE lng_id='%d' AND phrase_id='%d'";
            $params = array($this->tableTranslation,$new, (int)$lngId, (int)$this->phrase['phrase_id']);
            $res = $this->databaseQueryFmt($sql,$params);
          } elseif ($new != '') {
            $values = array('translation' => $new,
                            'lng_id' => (int)$lngId,
                            'phrase_id' => (int)$this->phrase['phrase_id']
                      );
            $res = $this->databaseInsertRecord($this->tableTranslation, NULL, $values);
          } elseif ($old != '') {
            $sql = "DELETE FROM %s WHERE lng_id='%d' AND phrase_id='%d'";
            $params = array($this->tableTranslation,(int)$lngId, (int)$this->phrase['phrase_id']);
            $res = $this->databaseQueryFmt($sql,$params);
          } else {
            continue;
          }
          if ($res) {
            $result = FALSE;
          } else {
            $this->cleanLog('', $this->phrase['phrase_id']);
          }
        }
      }
      return $result;
    }
    return FALSE;
  }

  /**
  * Clear module links
  *
  * @param mixed $module optional, default value NULL
  * @param mixed $phrase optional, default value NULL
  * @access public
  * @return mixed
  */
  function clearModuleLinks($module=NULL,$phrase=NULL) {
    $sql = "DELETE FROM %s WHERE 1 ";
    if (isset($module)) {
      $sql .= "AND module_id = ".(int)$module;
    }
    if (isset($phrase)) {
      $sql .= "AND phrase_id = ".(int)$phrase;
    }
    $params = array($this->tableModuleRelation);
    return $this->databaseQueryFmt($sql,$params);
  }

  /**
  * Load modules
  *
  * @access public
  * @return boolean
  */
  function loadModules() {
    unset($this->modules);
    $sql = "SELECT module_id, module_title
            FROM %s
            ORDER BY module_title_lower ASC";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableModule))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->modules[$row['module_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load languages
  *
  * @access public
  * @return boolean
  */
  function loadLanguages() {
    unset($this->languages);
    $sql = "SELECT lng_id, lng_ident, lng_short, lng_title, lng_glyph
            FROM %s
            WHERE is_interface_lng = 1
            ORDER BY lng_title ASC";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableLanguage))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->languages[$row['lng_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load log messages
  *
  * @access public
  * @return boolean
  */
  function loadLogMessages() {
    unset($this->logMessages);
    $sql = "SELECT log_id, log_msg, log_phrase_id, log_phrase, log_module, log_datetime
            FROM %s
            ORDER BY log_id DESC";
    $offset = (isset($this->params['msgoffset'])) ? (int)$this->params['msgoffset'] : 0;
    if ($res = $this->databaseQueryFmt($sql, array($this->tableLog) ,$this->messageSteps, (int)$offset)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->logMessages[$row['log_id']] = $row;
      }
      $this->messageAbsCount = $res->absCount();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Empty log
  *
  * @access public
  * @return boolean
  */
  function emptyLog() {
    return $this->databaseEmptyTable($this->tableLog);
  }

  /**
  * Empty modules
  *
  * @access public
  * @return boolean
  */
  function emptyModules() {
    return ($this->databaseEmptyTable($this->tableModuleRelation) && $this->databaseEmptyTable($this->tableModule));
  }

  /**
  * Clean log
  *
  * @param string $phrase optional, default value ''
  * @param mixed $phraseId optional, default value NULL
  * @access public
  * @return boolean
  */
  function cleanLog($phrase = '', $phraseId = NULL) {
    if (isset($phraseId) && $phraseId > 0) {
      return $this->databaseDeleteRecord($this->tableLog, 'log_phrase_id', (int)$phraseId);
    } elseif (trim($phrase) != '') {
      return $this->databaseDeleteRecord($this->tableLog, 'log_phrase', $phrase);
    } else {
      return FALSE;
    }
  }

  /**
  * Delete log message
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function deleteLogMsg($id) {
    $sql = "SELECT log_phrase, log_phrase_id FROM %s WHERE log_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableLog, $id))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if (trim($row['log_phrase']) != '') {
          return $this->cleanLog($row['log_phrase']);
        } else {
          return ( FALSE !== $this->databaseDeleteRecord($this->tableLog, 'log_id', (int)$id));
        }
      }
    }
    return FALSE;
  }

  /**
  * Get pages navigation
  *
  * @param object &$toolbar
  * @param integer $offset
  * @param integer $step
  * @param integer $max
  * @param string $paramName optional, default value 'offset'
  * @param string $caption optional, default value 'Pages'
  * @access public
  */
  function getPagesNav(&$toolbar, $offset, $step, $max, $paramName = 'offset', $caption = 'Pages') {
    if (($max > $step) && is_object($toolbar)) {
      if ($offset > 0) {
        $i = (($offset - $step) > 0) ? ($offset - $step) : 0;
        $toolbar->addButton('Prior', $this->buildLink(array('cmd'=>'show', $paramName =>$i)), 8, 'Prior page');
      }
      for($i=0,$x=1;$i<$max;$i+=$step,$x++) {
        $pages[$i] = $x;
      }
      $toolbar->addCombo($caption, $this->buildLink(array('cmd'=>'show')),
        $this->paramName.'['.$paramName.']', $offset, $pages);
      if ($offset < ($max-$step)) {
        $i = (($offset + $step) < $max) ? ($offset + $step) : ($max-$step);
        $toolbar->addButton('Next', $this->buildLink(array('cmd'=>'show', $paramName=>$i)), 9, 'Next page');
      }
    }
  }

  /**
  * build link
  *
  * @param mixed $params optional, default value NULL
  * @param mixed $baseParams optional, default value NULL
  * @param mixed $resetParams optional, default value NULL
  * @access public
  * @return string
  */
  function buildLink($params = NULL, $baseParams = NULL, $resetParams = NULL) {
    $paramString = '';
    if (isset($baseParams) && is_array($baseParams)) {
      foreach($baseParams as $key=>$val) {
        if (!isset($params[$key])) {
          $params[$key] = $val;
        }
      }
    }
    if (isset($resetParams) && is_array($resetParams)) {
      foreach($resetParams as $key) {
        unset($params[$key]);
      }
    }
    if (isset($params) && is_array($params)) {
      foreach($params as $key=>$val) {
        $paramString .= '&amp;'.$this->paramName.'['.$key.']='.urlencode($val);
      }
    }
    if (preg_match('/\?/',$this->baseLink)) {
      return $this->baseLink.$paramString;
    } else {
      return $this->baseLink.preg_replace('/^\&amp\;/','?',$paramString);
    }
  }

  /**
  * get XML buttons
  *
  * @access public
  */
  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    $toolbar->addButton('Phrases', $this->baseLink.'?'.$this->paramName.'[pagemode]=0&'.$this->paramName.'[lng_id]=0', 16, '', TRUE);
    $toolbar->addButton('Languages', $this->baseLink.'?'.$this->paramName.'[pagemode]=1', 16, '', FALSE);
    $toolbar->addSeperator();
    $toolbar->addButton('Clear log', $this->baseLink.'?'.$this->paramName.'[cmd]=emptylog', 10, 'Clear Log', FALSE);
    $toolbar->addButton('Clear modules', $this->baseLink.'?'.$this->paramName.'[cmd]=emptymodules', 10, 'Reset Optimisation', FALSE);
    $toolbar->addSeperator();
    $toolbar->addCombo(
      'Module', $this->baseLink,
      $this->paramName.'[module_id]', $this->params['module_id'],
      $this->getModuleComboItems());
    $toolbar->addSeperator();
    $toolbar->addCombo(
      'Language', $this->baseLink,
      $this->paramName.'[lng_id]', $this->params['lng_id'],
      $this->getLngComboItems());
    if ($result = $toolbar->getXML()) {
      $this->layout->add('<menu>'.$result.'</menu>', 'menus');
    }
    unset($toolbar);
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    
    $letterItems[] = $this->_gt('All');
    $letterItems['#'] = $this->_gt('#');
    for ($i = ord('a'); $i <= ord('z'); $i++) {
      $letterItems[chr($i)] = chr($i).'*';
    }
    $toolbar->addCombo(
      'Letter', $this->baseLink,
      $this->paramName.'[letter]', $this->params['letter'],
      $letterItems);
    $this->getPagesNav($toolbar, @(int)$this->params['msgoffset'],  $this->messageSteps, $this->messageAbsCount,
      'msgoffset', 'Messages');
    $toolbar->addSeperator();
    $this->getPagesNav($toolbar, @(int)$this->params['phroffset'],  $this->phraseSteps, $this->phraseAbsCount,
      'phroffset', 'Phrases');
    $toolbar->addSeperator();
    $toolbar->addButton('New phrase', $this->baseLink.'?'.$this->paramName.'[cmd]=add', 70, '', FALSE);
    if (isset($this->phrase) && is_array($this->phrase)) {
      $toolbar->addButton('Delete phrase',
        $this->baseLink.'?'.$this->paramName.'[cmd]=del&amp;'.$this->paramName.'[phrase_id]='.(int)$this->phrase['phrase_id'],
        10, '', FALSE);
    }
    if ($result = $toolbar->getXML()) {
      $this->layout->add('<menu>'.$result.'</menu>', 'menus');
    }
  }

  /**
  * Get language combo items
  *
  * @access public
  * @return array
  */
  function getLngComboItems() {
    $result[0] = $this->_gt('All');
    if (isset($this->languages) && is_array($this->languages)) {
      foreach($this->languages as $lngId=>$lng) {
        $result[$lngId] = $lng['lng_title'].' ('.$lng['lng_short'].')';
      }
    }
    return $result;
  }

  /**
  * Get module combo items
  *
  * @access public
  * @return array
  */
  function getModuleComboItems() {
    $result[0] = $this->_gt('All');
    if (isset($this->modules) && is_array($this->modules)) {
      foreach($this->modules as $modId=>$mod) {
        $result[$modId] = $mod['module_title'];
      }
    }
    return $result;
  }

  /**
  * Get phrases list
  *
  * @access public
  */
  function getPhrasesList() {
    if (isset($this->phrases) && is_array($this->phrases)) {
      if (isset($this->modules[$this->params['module_id']]['module_title'])) {
        $moduleTitle = $this->modules[$this->params['module_id']]['module_title'];
      } else {
        $moduleTitle = $this->_gt('All');
      }
      $result = sprintf('<listview title="%s [%s]" width="100%%">', $this->_gt('Phrases'),
        $moduleTitle);
      $result .= '<cols>';
      $result .= '<col>'.$this->_gt('Phrase').'</col>';
      $result .= '<col align="center">'.$this->_gt('Languages').'</col>';
      $result .= '</cols>';
      $result .= '<items>';
      $href = $this->baseLink.'?'.$this->paramName.'[phrase_id]=';
      foreach($this->phrases as $id=>$phrase) {
        if ($this->params['lng_id'] == 0 || (!isset($phrase['phrases'][$this->params['lng_id']]))) {
          $result .= sprintf('<listitem href="%s%d" title="%s">',
            $href, (int)$id, papaya_strings::escapeHTMLChars($phrase['phrase_text']));
          $result .= '<subitem align="center">';
          if (isset($phrase['phrases']) && is_array($phrase['phrases'])) {
            foreach($phrase['phrases'] as $lngId=>$translation) {
              if (isset($this->languages[$lngId]) && is_array($this->languages[$lngId])) {
                $lng = $this->languages[$lngId];
                if ($lng['lng_glyph'] != '' && file_exists($this->getBasePath(TRUE).'pics/language/'.$lng['lng_glyph'])) {
                  $result .= sprintf('<glyph src="./pics/language/%s" hint="%s"/>  ', papaya_strings::escapeHTMLChars($lng['lng_glyph']),
                      papaya_strings::escapeHTMLChars($translation));
                } elseif ($lng['lng_ident'] != '') {
                  $result .= sprintf('<span title="%s">%s</span> ', papaya_strings::escapeHTMLChars($translation),
                    papaya_strings::escapeHTMLChars($lng['lng_ident']));
                } else {
                  $result .= sprintf('<span title="%s">%s</span> ', papaya_strings::escapeHTMLChars($translation),
                    papaya_strings::escapeHTMLChars($lng['lng_short']));
                }
              } else {
                $result .= sprintf('<span title="%s">XX</span> ', papaya_strings::escapeHTMLChars($translation));
              }
            }
          }
          $result .= '</subitem>';
          $result .= '</listitem>';
        }
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get log message list
  *
  * @access public
  */
  function getLogMsgList() {
    if (isset($this->logMessages) && is_array($this->logMessages)) {
      $result = sprintf('<listview title="%s" width="100%%">', $this->_gt('Messages'));
      $result .= '<cols>';
      $result .= '<col>'.$this->_gt('Message').'</col>';
      $result .= '<col align="center">'.$this->_gt('Time').'</col>';
      $result .= '<col/>';
      $result .= '<col/>';
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->logMessages as $id=>$msg) {
        $result .= sprintf('<listitem title="%s">', papaya_strings::escapeHTMLChars($msg['log_msg']));
        $result .= sprintf('<subitem align="center">%s</subitem>',
          date('Y-m-d H:i:s', $msg['log_datetime']));
        $result .= '<subitem align="center">';
        if ($msg['log_phrase_id'] > 0) {
          $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
            $this->baseLink.'?'.$this->paramName.'[phrase_id]='.$msg['log_phrase_id'],
            $this->images[14], $this->_gt('Edit'));
        } elseif (trim($msg['log_phrase'] != '')) {
          $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
            $this->baseLink.'?'.$this->paramName.'[cmd]=add&amp;'.
              $this->paramName.'[phrase]='.urlencode($msg['log_phrase']),
            $this->images[70], $this->_gt('Add'));
        }
        $result .= '</subitem>';
        $result .= '<subitem align="center">';
        $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
          $this->baseLink.'?'.$this->paramName.'[cmd]=delmsg&amp;'.$this->paramName.'[log_msg_id]='.$id,
          $this->images[10], $this->_gt('Delete'));
        $result .= '</subitem>';
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get edit formular
  *
  * @access public
  */
  function getEditForm() {
    if (isset($this->phrase) && is_array($this->phrase)) {
      $result = sprintf('<dialog action="%s" method="POST" title="%s" width="100%%">',
        $this->baseLink, $this->_gt('Translations'));
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'cmd', 'save');
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'phrase_id', (int)$this->phrase['phrase_id']);
      $result .= '<lines class="dialogLarge">';
      $result .= sprintf('<line caption="%s">', $this->_gt('Phrase'));
      $result .= sprintf('<input type="text" name="%s[phrase]" value="%s" class="dialogInput dialogScale" />',
        $this->paramName, papaya_strings::escapeHTMLChars($this->phrase['phrase_text']));
      $result .= '</line>';
      if (isset($this->languages) && is_array($this->languages)) {
        foreach($this->languages as $lngId=>$lng) {
          $result .= sprintf('<line caption="%s">', papaya_strings::escapeHTMLChars($lng['lng_title']));
          $result .= sprintf('<input type="text" name="%s[trans][%d]" value="%s" class="dialogInput dialogScale" />',
            $this->paramName, $lngId, papaya_strings::escapeHTMLChars(@$this->phrase['phrases'][$lngId]));
          $result .= '</line>';
        }
      }
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s"/>', $this->_gt('Change'));
      $result .= '</dialog>';
      $this->layout->add($result);
    }
  }

  /**
  * Get del formular
  *
  * @access public
  */
  function getDelForm() {
    if (isset($this->phrase) && is_array($this->phrase) && @($this->params['cmd']=='del')) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del',
        'phrase_id' => (int)$this->phrase['phrase_id'],
        'confirm_delete'=>1,
      );
      $msg = sprintf($this->_gt('Delete phrase "%s"?'), $this->phrase['phrase_text']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * get add formular
  *
  * @access public
  */
  function getAddForm() {
    if ($this->params['cmd'] = 'add' && (!@is_array($this->phrase))) {
      $result = sprintf('<dialog action="%s" method="POST" title="%s" width="100%%">',
        $this->baseLink, $this->_gt('Add phrase'));
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s"/>',
        $this->paramName, 'cmd', 'add');
      $result .= '<lines class="dialogLarge">';
      $result .= sprintf('<line caption="%s">', $this->_gt('Phrase'));
      $result .= sprintf('<input type="text" name="%s[phrase]" value="%s" class="dialogInput dialogScale" />',
        $this->paramName, papaya_strings::escapeHTMLChars(@$this->phrase['phrase_text']));
      $result .= '</line>';
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s"/>', $this->_gt('Add'));
      $result .= '</dialog>';
      $this->layout->add($result);
    }
  }
}
?>