<?php
/**
* basic object for all plugin objects
*
* Content modules and action buttons must inherit this class
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_plugin.php,v 1.20 2006/04/26 10:29:00 weinert Exp $
*/

/**
* XML-Parser
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_simple_xmltree.php');
/**
* XSL-transformation
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_xsl.php');
/**
* Basic objekt
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');
/**
* Basic objekt for all plugin object
*
* Content modules and action buttons must inherit this class
*
* @package papaya_modules
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_plugin extends base_object {
  /**
  * Buffer for content/configurations data
  * @var array $data
  */
  var $data = NULL;

  /**
  * Parameter name for html formulars
  * @var string $paramName
  */
  var $paramName = '';

  /**
  * Parameter for HTML-formulars
  * @var array $params
  */
  var $params;

  /**
  * Default-link
  * @var string $baseLink
  */
  var $baseLink;

  /**
  * Referenz on message-objekt
  * @var int $msgs
  */
  var $msgs;

  /**
  * XML-tag-base
  * @var string $tagName
  */
  var $tagName = 'data';


  /**
  * Create XML for debug messages
  * @var $usesXML
  */
  var $usesXML = FALSE;

  /**
  * name => (
  *    inscription,
  *    check function,
  *    mandatory field (TRUE/FALSE),
  *    Type (input, combo, textarea)
  *    Type paramter(maximum length, value-hash, row)
  *    fast help)
  *
  * @var array $editFields
  */
  var $editFields;

  /**
  * Output of module cacheable?
  * @var boolean $cachable
  */
  var $cacheable = 1;

  /**
  * Is preview possible?
  * @var integer $preview
  */
  var $preview = FALSE;

  var $fieldError;
    var $innputError = 'The Input in this field contains errors.';
  var $emptyError = 'This field needs a value.';

  /**
  * dialog field size
  * @var string
  */
  var $inputFieldSize = 'medium';
  
  var $papayaTagPattern = '/<(papaya|ndim):([a-z]\w+)\s?([^>]*)\/?>(<\/(\1):(\2)>)?/ims';

  /**
  * Constructor PHP 5
  * @param object base_topic Topic object
  * @param string $paramName Parametername
  */
  function __construct(&$topic, $paramName = NULL) {
    $this->topicObj = &$topic;
    if (isset($paramName)) {
      $this->paramName = $this->getParamname($paramName);
    } elseif (isset($this->paramName) && trim($this->paramName) != '') {
      $this->paramName = $this->getParamname($this->paramName);
    } else {
      $this->paramName = $this->getParamname('bab');
    }
    $this->initializeParams();
    $this->baseLink = $this->getBaseLink();
  }

  /**
  * Constructor PHP 4
  * @param object base_topic Topic object
  * @param string $paramName Parametername
  */
  function base_plugin(&$topic, $paramName = NULL) {
    $this->__construct($topic, $paramName);
  }

  /**
  * Returns given parameter
  * @param string $paramName parameter name
  */
  function getParamName($paramName) {
    return $paramName;
  }

  /**
  * Load configuration data in object
  * @param string $xmlData XML-data string
  */
  function setData($xmlData) {
    $this->data = array();
    $this->unserializeArrayFromXML($this->data, $xmlData);
    $this->onLoad();
  }

  /**
  * Write data array to xml string
  * @return string $result XML-data string
  */
  function getData() {
    $result = "<".$this->tagName.">";
    if (isset($this->data) && is_array($this->data)) {
      $result .= $this->serializeArrayToXML($this->data);
    }
    $result .= "</".$this->tagName.">";
    return $result;
  }

  function serializeArrayToXML(&$dataArr) {
    $result = '';
    if (isset($dataArr) && is_array($dataArr) && count($dataArr) > 0) {
      foreach($dataArr as $name=>$value) {
        if (trim($name) != '') {
          if (isset($value) && is_array($value)) {
            $result .= sprintf('<%s-list name="%s">'.LF, $this->tagName, $name);
            $result .= $this->serializeArrayToXML($value);
            $result .= sprintf('</%s-list>'.LF, $this->tagName);
          } else {
            $result .= sprintf('<%s-element name="%s"><![CDATA[', $this->tagName, $name);
            $result .= simple_xmltree::strToXML($value);
            $result .= sprintf(']]></%s-element>'.LF, $this->tagName);
          }
        }
      }
    }
    return $result;
  }

  function unserializeArrayFromXML(&$dataArr, &$xml) {
    $xmlTree = simple_xmltree::create();
    $xmlTree->unserializeArrayFromXML($this->tagName, $dataArr, $xml);
    unset($xmlTree);
  }

  /**
  * data changed?
  * @return boolean
  */
  function modified() {
    if (isset($this->dialog) && is_object($this->dialog)) {
      return $this->dialog->modified();
    }
    return FALSE;
  }

  /**
  * Change input
  * @return boolean
  */
  function checkData() {
    return $this->checkDialogInput();
  }

  /**
  * XML for take-over in page
  * @return string
  */
  function getParsedData() {
    return $this->getData();
  }

  /**
  * Edit output
  * @return string XML-string with Formular
  */
  function getForm() {
    $result = $this->getDialog();
    return $result;
  }

  /**
  * Initialize dialog
  */
  function initializeDialog($hiddenValues = NULL) {
    if (!@is_object($this->dialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      if (isset($hiddenValues) && is_array($hiddenValues)) {
        $hidden = $hiddenValues;
      }
      $hidden['save'] = 1;
      $this->dialog = &new base_dialog($this, $this->paramName, $this->editFields, $this->data, $hidden);
      $this->dialog->msgs = &$this->msgs;
      $this->dialog->loadParams();
      $this->dialog->inputFieldSize = $this->inputFieldSize;
      $this->dialog->baseLink = $this->baseLink;
    }
  }

  /**
  * Gibt den Dialog zurueck
  * @return string $this->dialog->getDialogXML()
  */
  function getDialog() {
    if (isset($this->dialog) && is_object($this->dialog)) {
      $this->dialog->dialogTitle = $this->_gt('Edit content');
      $this->dialog->dialogDoubleButtons = TRUE;
      $this->dialog->expandPapayaTags = TRUE;
      if (isset($this->topicObj) && is_object($this->topicObj) && isset($this->topicObj->tableTopics)) {
        $this->dialog->tableTopics = $this->topicObj->tableTopics;
      }
      return $this->dialog->getDialogXML();
    } else {
      return '';
    }
  }

  /**
  * Check user dialog input
  * @return array $result
  */
  function checkDialogInput() {
    if ($result = $this->dialog->checkDialogInput()) {
      $this->data = $this->dialog->data;
    }
    $this->fieldErrors = $this->dialog->inputErrors;
    return $result;
  }

  /**
  * Callback-function - transform \r\n in &lt;br /&gt;.
  *
  * @param array $regs Trefferdaten
  * @return string $result
  */
  function htmlLinebreakToBr($regs) {
    $result = @($regs[2] == $regs[1]) ? $regs[1] : '<br />';
    return $result;
  }

  /**
  * Create XHTML-compliant string.
  *
  * @param string $str
  * @param boolean $nlToBr Convert line break in XHTML
  *
  * @return string
  */
  function getXHTMLString($str, $nlToBr = FALSE) {
    $iStr = papaya_strings::entityToXML($str);
    if ($nlToBr) {
      $iStr = preg_replace_callback("/((<[^>]*)|\r\n|\n|\r)/", array($this, 'htmlLinebreakToBr'), $iStr);
    }
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');
    if (trim($iStr) == '') {
      return '';
    } elseif ($this->createXMLTree("<dummy>".$iStr."</dummy>")) {
      return $iStr;
    } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_xsl.php');
      $lineOut = (papaya_xsl::getFormattedLines($iStr, $this->lastXMLError[1], FALSE));
      return '<noescape><![CDATA['.$lineOut.']]></noescape>';
    } else {
      return papaya_strings::escapeHTMLChars($iStr);
    }
  }
  
  function getPapayaImageTag($str, $width = 0, $height = 0, $alt = '', $resize = NULL) {
    if (preg_match($this->papayaTagPattern, $str, $regs)) {
      return $regs[0];
    } elseif (preg_match('~^([^.,]+\.\w+)(,(\d+)(,(\d+)(,(\w+))?)?)?$~i', $str, $regs)) {
      $result = '<papaya:media src="'.papaya_strings::escapeHTMLChars($regs[1]).'"';
      if ($width > 0) {
        $result .= ' width="'.(int)$width.'"';
      } elseif (isset($regs[3])) {
        $result .= ' width="'.(int)$regs[3].'"';
      }
      if ($height > 0) {
        $result .= ' height="'.(int)$height.'"';
      } elseif (isset($regs[5])) {
        $result .= ' height="'.(int)$regs[5].'"';      
      }
      if (isset($resize)) {
        $result .= ' resize="'.papaya_strings::escapeHTMLChars($resize).'"';
      } elseif (isset($regs[7])) {
        $result .= ' resize="'.papaya_strings::escapeHTMLChars($regs[7]).'"';
      }
      if (isset($alt) && trim($alt) != '') {
        $result .= ' alt="'.papaya_strings::escapeHTMLChars($alt).'"';
      }
      return $result.'/>';
    }
    return '';
  }

  /**
  * Create XML-tree.
  * @param string $xmlData XML-string
  * @return object simple_xmlnode $xmlTree XML-tree
  */
  function createXMLTree($xmlData) {
    if (trim($xmlData) != '') {
      $xmlTree = simple_xmltree::create();
      if (is_object($xmlTree)) {
        if (0 === $xmlTree->loadFromString($xmlData)) {
          if (is_object($xmlTree->root)) {
            return $xmlTree->root;
          }
        } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
          $this->lastXMLError = @$xmlTree->lastError;
          unset($xmlTree);
        }
      }
    }
    return FALSE;
  }

  /**
  * place holder - overload von ihertited classes
  */
  function onLoad() {
  }

  /**
    * Callback for Select-Box with rights
    *
    * @param string $name
    * @param array $field
    * @param mixed $data
    *
    * @return string
    */
  function getPermsCombo($name, $field, $data) {
    $result ="";
    $selectedTemp=FALSE;
    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
      $this->paramName, $name);
    if (isset($GLOBALS['PAPAYA_SURFER'])) {
      $surfer = &$GLOBALS['PAPAYA_SURFER'];
    } else {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
      $surfer = &new base_surfer($this->msgs);
    }
    $surfer->loadPermissionList();
    $selected = ($data == -1) ? ' selected="selected"' : '';
    $result .= sprintf('<option value="%d" %s>%s</option>'.LF,
      -1, $selected, papaya_strings::escapeHTMLChars($this->_gt('none')));
    if (isset($surfer->permissions) && is_array($surfer->permissions)) {
      foreach ($surfer->permissions as $id=>$perm) {
        $selected = ($data == $id) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%d" %s>%s</option>'.LF,
        $id,
        $selected,
        papaya_strings::escapeHTMLChars($perm['surferperm_title']));
      }
    }
    $result .= '</select>'.LF;
    return $result;
  }

  /**
  * Phrase translator - fetch translation
  * @param string $phrase Phrase
  * @param mixed  $module Modul
  * @return string $phrase
  */
  function _gt($phrase, $module = NULL) {
    if (@is_object($GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT])) {
      return $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]->getText($phrase, $module);
    }
    return $phrase;
  }

  /**
  * Check sufer permissions
  *
  * @param integer $permId
  * @access public
  * @return boolean
  */
  function checkSurferPerm($permId) {
    if ($permId <= 0) return TRUE;
    if (is_object($GLOBALS['PAPAYA_SURFER']) && $GLOBALS['PAPAYA_SURFER']->isValid) {
        return $GLOBALS['PAPAYA_SURFER']->hasPerm($permId);
    }
    return FALSE;
  }

  function parseImageFieldData($dateString) {
    $dataArr = array(
      'src' => '',
      'width' => 0,
      'height' => 0
    );
    if (FALSE !== strpos($dateString, '<')) {
      if (preg_match_all('~(src|width|height)=("|\')([^"]+)(\\2)~i', $dateString, $regs, PREG_SET_ORDER)) {
        foreach($regs as $reg) {
          $dataArr[strtolower($reg[1])] = $reg[3];
        }
      }
    } elseif (($arr = explode('/', $dateString)) && is_array($arr) && count($arr) > 0) {
      $dataArr['src'] = $arr[0];
      $dataArr['width'] = (int)@$arr[1];
      $dataArr['height'] = (int)@$arr[2];
    } else {
      $dataArr['src'] = trim($dateString);
    }
    return $dataArr;
  }

  function getMediaTag($data, $width = 0, $height = 0, $resize = 'max') {
    if (is_array($data) && isset($data['src'])) {
      $src = $data['src'];
      if ($width <= 0 && isset($data['width']) && $data['width'] > 0) {
        $width = (int)$data['width'];
      }
      if ($height <= 0 && isset($data['height']) && $data['height'] > 0) {
        $height = (int)$data['height'];
      }
    } else {
      $src = $data;
    }
    if (trim($src) != '') {
      $result = '<papaya:media src="'.papaya_strings::escapeHTMLChars($src).'"';
      if ($width > 0) {
        $result .= ' width="'.(int)$width.'"';
      }
      if ($height > 0) {
        $result .= ' height="'.(int)$height.'"';
      }
      if (in_array($resize, array('min', 'mincrop', 'abs'))) {
        $result .= ' resize="'.$resize.'"';
      }
      $result .= '/>';
      return $result;
    }
    return '';
  }
}
?>
