<?php
/**
* Object to display a alias tree (n-dimensional)
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_alias_tree.php,v 1.9 2006/01/12 16:27:07 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Object to display an alias tree (n-dimensional)
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_alias_tree extends base_db {
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;
  var $tableTopicsPublic = PAPAYA_DB_TBL_TOPICS_PUBLIC;
  var $tableTopicsPublicTRANS = PAPAYA_DB_TBL_TOPICS_PUBLIC_TRANS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  var $tableUrls = PAPAYA_DB_TBL_URLS;

  var $aliases;
  var $links;

  var $paramName = 'altr';

  function initialize() {
    $this->initializeParams();
  }

  /**
  * Execute - base function for handlig parameters
  *
  * @access public
  */
  function execute() {    
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();
    $this->layout->addLeft($this->lngSelect->getComboFormXML(450));
    
    if ($this->authUser->hasPerm(16)) {
      switch (@$this->params['cmd']) {
        case 'alias_delete' :
          if (isset($this->params['alias_id']) && $this->params['alias_id'] > 0 && 
              isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->deleteAlias((int)$this->params['alias_id'])) {
              unset($this->params['cmd']);
              $this->addMsg(MSG_INFO, $this->_gt('Alias deleted.'));
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Database error'));
            }
          }
          break;
        case 'alias_create' :$this->initializeAliasDialog();
            if ($this->dialogAlias->checkDialogInput() && $this->checkAliasInput()) {
              if ($newId = $this->createAlias()) {
                unset($this->dialogAlias);
                $this->params['alias_id'] = $newId;
                $this->addMsg(MSG_INFO,$this->_gt('Changes saved.'));
              } else {
                $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
              }
            }
          break;
        case 'alias_edit' :
          if (isset($this->params['alias_id']) && $this->params['alias_id'] > 0 &&
              $this->loadAlias($this->params['alias_id'])) {
            $this->initializeAliasDialog();
            if ($this->dialogAlias->checkDialogInput() && $this->checkAliasInput()) {
              if ($this->saveAlias()) {
                unset($this->dialogAlias);
                $this->load($this->params['alias_id']);
                $this->addMsg(MSG_INFO,$this->_gt('Changes saved.'));
              } else {
                $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
              }
            }
          }
          break;

      }
      if (isset($this->params['alias_id']) && $this->params['alias_id'] > 0) {
        $this->loadAlias($this->params['alias_id']);      
      }
    }
  }
  
  function getXML() {
    $this->load();
    $this->layout->addLeft($this->get());
    switch(@$this->params['cmd']) {
      case 'alias_delete' :
        $this->layout->add($this->getDeleteForm());      
        break; 
      default :
        $this->layout->add($this->getAliasDialog());
        break;      
    }
  }
  
  /**
  *
  * @access public
  * @return string
  */
  function get() {
    $result = '';
    if (isset($this->aliases) && is_array($this->aliases) && count($this->aliases) > 0) {
      $result .= sprintf('<listview width="450" title="%s">'.LF, $this->_gt('Alias'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Path'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Page'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $counter = '0';
      $result .= $this->getElements();
      $result .=  '</items></listview>'.LF;
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('No aliases defined.'));
    }
    return $result;
  }

  /**
  * Load
  *
  * @param integer $baseId
  * @access public
  */
  function load(){
    unset($this->aliases);
    unset($this->links);
    $sql = "SELECT a.id, a.path, a.topic_id as alias_topic_id,
                    t.topic_id, t.topic_modified, tt.topic_title, 
                    tp.topic_modified as topic_published,
                    v.view_id, v.view_title
                  FROM %s AS a
                    LEFT OUTER JOIN %s AS t ON t.topic_id = a.topic_id
                    LEFT OUTER JOIN %s AS tt ON (tt.topic_id = t.topic_id AND tt.lng_id = %d)
                    LEFT OUTER JOIN %s AS tp ON a.topic_id = tp.topic_id
                    LEFT OUTER JOIN %s AS v ON (v.view_id = tt.view_id)
                  ORDER BY a.path\r\n";
    $params = array(
      $this->tableUrls, 
      $this->tableTopics, $this->tableTopicsTrans, $this->lngSelect->currentLanguageId,
      $this->tableTopicsPublic, $this->tableViews);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while (0 === $res->fetchInto($row,DB_FETCHMODE_ASSOC)) {
        $this->aliases[$row['id']] = $row;
        $this->links[$row['alias_topic_id']][] = $row['id'];
      }
    }
  }
  
  function loadAlias($aliasId) {
    unset($this->alias);
    $sql = "SELECT id, topic_id, path, url_params, lng_id
              FROM %s 
             WHERE id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableUrls, $aliasId))) { 
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->alias = $row;
        return TRUE;
      }
    }    
    return FALSE;
  }

  /**
  * Get elemtents
  *
  * @access public
  * @return string
  */
  function getElements() {
    $result = '';
    if (isset($this->aliases) && is_array($this->aliases)) {
      $counter = 0;
      foreach($this->aliases as $id=>$val) {
        if (isset($val) && is_array($val)) {
          $counter++;
          $selected = (isset($this->params['id']) && $this->params['id'] == $id) ? ' selected="selected"' : '';
          $href = $this->getLink(array('alias_id'=>$id));
          if ($val['topic_published']) {
            if ($val['topic_published'] >= $val['topic_modified']) {
              $imageIdx = 16;
            } else {
              $imageIdx = 48;
            }
          } elseif (!$val['topic_id']) {
            $imageIdx = 0;
          } else {
            $imageIdx = 15;
          }
          $result .= sprintf('<listitem title="%s" href="%s" image="%s" %s>',
            papaya_strings::escapeHTMLChars($val['path']), $href,
            $this->images[12], $selected);
          if ($val['topic_id']) {
            $result .= sprintf('<subitem><a href="%s"><glyph src="%s"/>%s #%s</a></subitem>'.LF,
              $this->getLink(array('page_id'=>$val['topic_id']), 'tt', 'topic.php'), $this->images[$imageIdx],
              papaya_strings::escapeHTMLChars($val['topic_title']), $val['alias_topic_id']);
          } else {
            $result .= sprintf('<subitem><glyph src="%s"/><i>%s #%s</i></subitem>'.LF,
              $this->images[$imageIdx], papaya_strings::escapeHTMLChars($this->_gt('404 - invalid page')), $val['alias_topic_id']);
          }
          $result .= '</listitem>'."\r\n";
          if (isset($this->params['id']) && $this->params['id'] == $id) {
            $result .= $this->getSubElements($val['topic_id'], $val['id'], $counter);
          }
        }
      }
    }
    return $result;
  }

  /**
  * Get sub Elements
  *
  * @param integer $id
  * @param string $caller
  * @param integer &$counter
  * @access public
  * @return string
  */
  function getSubElements($id, $caller, &$counter) {
    $result = "";
    if(isset($this->links[$id]) && is_array($this->links[$id])){
      foreach($this->links[$id] as $val){
        if($val != $caller){
          $href = sprintf('%s?%s[%s]=%s&amp;%s[%s]=%s',
            $this->baseLink, $this->paramName, 'cmd', 'edit',
            $this->paramName, 'id', $val['id']);
          $result .= sprintf('<listitem title="%s" indent="1" image="%s" href="%s">',
            $this->aliases[$val]['path'], $this->images[12], $href);
          $result .= '<subitem />'."\r\n";
          $result .= '</listitem>';
        }
      }
    }
    return $result;
  }

  /**
  * Get delete formular
  *
  * @access public
  * @return string
  */
  function getDeleteForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'alias_delete',
      'alias_id' => $this->alias['id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Really delete Alias "%s"?'), $this->checkPathSlashes($this->alias['path']));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Check path slashes
  *
  * @param string $path
  * @access public
  * @return string
  */
  function checkPathSlashes($path) {
    $result = trim($path);
    if (substr($path, 0, 1) != '/') {
      $result = '/'.$path;
    }
    if (substr($result, -1) != '/') {
      $result .= '/';
    }
    return preg_replace('#//+#', '/', $result);
  }

  /**
  * Check input
  *
  * @access public
  * @return boolean
  */
  function checkAliasInput() {
    $result = TRUE;
    if (isset($this->params['topic_id']) && $this->params['topic_id'] > 0) {
      $topic = &new papaya_topic;
      if (!$topic->topicExists(@(int)$this->params['topic_id'])) {
        $this->addMsg(MSG_ERROR, $this->_gt('Specified page doesnt exists.'));
        $result = FALSE;
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No page specified.'));
      $result = FALSE;
    }
    return $result;
  }

  /**
  * Save alias
  *
  * @access public
  * @return boolean
  */
  function saveAlias() {
    $values = array(
      'path' => $this->checkPathSlashes($this->params['path']), 
      'topic_id' => $this->params['topic_id'], 
      'lng_id' => $this->params['lng_id'], 
      'url_params' => $this->params['url_params']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableUrls, $values, 'id', (int)$this->params['alias_id']));
  }
  
  /**
  * Create a new Alias
  *
  * @access public
  * @return mixed Id of new Alias (or NULL)
  */
  function createAlias() {
    $values = array(
      'path' => $this->checkPathSlashes($this->params['path']), 
      'topic_id' => $this->params['topic_id'], 
      'lng_id' => $this->params['lng_id'], 
      'url_params' => $this->params['url_params']
    );
    return (FALSE !== $this->databaseInsertRecord($this->tableUrls, 'id', $values));
  }

  /**
  * Delete
  *
  * @param integer $id
  * @access public
  * @return mixed
  */
  function deleteAlias($id) {
    return (FALSE !== $this->databaseDeleteRecord($this->tableUrls, 'id', $id));
  }


  /**
  * Get buttons
  *
  * @access public
  * @return mixed
  */
  function getButtons() {
    $result = NULL;
    $result[] = array(
      'New alias', 'Create a new alias', $this->images[13], 16,
      $this->baseLink.'?'.$this->paramName.'[alias_id]=0',
      '_self'
    );
    $result[] = '-';
    if (isset($this->alias)) {
      $result[] = array(
        'Delete', 'Delete alias', $this->images[10], 16,
        $this->baseLink.'?'.$this->paramName.'[cmd]=alias_delete&amp;'.$this->paramName.'[alias_id]='.(int)$this->params['alias_id'],
        '_self'
      );
    }
    return $result;
  }
  

  function initializeAliasDialog() {
    if (!(isset($this->dialogAlias) && is_object($this->dialogAlias))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      if (isset($this->alias)) {
        $data = $this->alias;
        $hidden = array(
          'cmd' => 'alias_edit',
          'alias_id' => (int)$this->alias['id']
        );
        $btnCaption = 'Edit';
      } else {
        $data = array();
        $hidden = array(
          'cmd' => 'alias_create'
        );
        $btnCaption = 'Add';
      }
      
      $topicSessionParams = $this->getSessionValue('PAPAYA_SESS_tt');
      $pageId = @(int)$topicSessionParams['page_id'];
      
      $fields = array(
        'topic_id' => array('Page Id', 'isNum', FALSE, 'input', 6, '', $pageId),
        'lng_id' => array ('Language', 'isNum', TRUE, 'function', 'getContentLanguageCombo', '',
           $this->lngSelect->currentLanguageId),
        'path' => array('Alias', 'internalSafeURL', FALSE, 'input', 40, '', '/path/'),
        'url_params' => array('Params', 'isNoHTML', FALSE, 'input', 100, '', '')
      );      

      $this->dialogAlias = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogAlias->msgs = &$this->msgs;
      $this->dialogAlias->loadParams();
      $this->dialogAlias->dialogTitle = $this->_gt('Alias');
      $this->dialogAlias->buttonTitle = $btnCaption;
      $this->dialogAlias->dialogDoubleButtons = FALSE;
      $this->dialogAlias->textYes = 'Yes';
      $this->dialogAlias->textNo = 'No';
    }
  }
   
  function getAliasDialog() {
    $this->initializeAliasDialog();
    return $this->dialogAlias->getDialogXML();
  }
  
  function getContentLanguageCombo($name, $element, $data) {
    return $this->lngSelect->getContentLanguageCombo($this->paramName, $name, $element, $data);
  }
}
?>
