<?php
/**
* Link Box with preview page
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_boxeslinks.php,v 1.5 2006/05/26 12:21:45 weinert Exp $
*/

/**
* Linked boxes base class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_boxeslinks.php');

/**
* Input check function
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* Link Box with page
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class papaya_boxeslinks extends base_boxeslinks {
  var $boxesList;
  var $linkList;
  var $used;

  function initialize() {
    $this->sessionParamName = 'PAPAYA_SESS_boxes';
    $this->initializeParams($this->sessionParamName);
    $this->initializeNodes();
  }

  /**
  * Initiation nodes
  *
  * @access public
  */
  function initializeNodes() {
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->opened = $this->sessionParams['opened'];
    switch (@$this->params['cmd']) {
      case 'open' :
        if ($this->params['gid'] > 0) {
          $this->opened[$this->params['gid']] = TRUE;
        }
        break;
      case 'close' :
        if (isset($this->opened[$this->params['gid']])) {
          unset($this->opened[$this->params['gid']]);
        }
        break;
    }
    $this->sessionParams['opened'] = $this->opened;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Load list
  *
  * @access public
  */
  function loadList() {
    unset($this->linkList);
    unset($this->used);
    $sql = "SELECT boxlink_id, box_id, box_sort, topic_id
            FROM %s
            WHERE topic_id = '%d'
            ORDER BY box_sort";
    $params = array ($this->tableLink, $this->topicId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->linkList[$row['boxlink_id']] = $row;
        $this->used[] = $row['box_id'];
      }
    }
  }

  /**
  * Load box list
  *
  * @access public
  */
  function loadBoxList() {
    $sql = "SELECT box_id, box_name, boxgroup_id
            FROM %s
            ORDER BY box_name";
    $params = array ( $this->tableBox );
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->boxesList[$row['box_id']] = $row;
      }
    }
  }

  /**
  * Load data list
  *
  * @access public
  * @return boolean
  */
  function loadDataList($lngId, $viewModeId) {
    if ($viewModeId > 0) {
      $sql = "SELECT bl.topic_id, bl.box_id, bl.box_sort, b.box_name, b.boxgroup_id,
                      bt.box_title, bt.box_data, bt.view_id,
                      m.module_guid, m.module_useoutputfilter, m.module_path, m.module_file, m.module_class
              FROM %s AS bl,
                    %s AS b,
                    %s AS bt,
                    %s AS v,
                    %s AS vl,
                    %s AS m
              WHERE bl.topic_id = '%d'
                AND b.box_id = bl.box_id
                AND bt.box_id = bl.box_id
                AND bt.lng_id = '%d'
                AND v.view_id = bt.view_id
                AND vl.view_id = bt.view_id
                AND vl.viewmode_id = %d
                AND m.module_guid = v.module_guid
                AND m.module_active = 1
                AND m.module_type = 'box'
              ORDER BY bl.topic_id, bl.box_sort, bl.box_id";
      $params = array($this->tableLink, $this->tableBox, $this->tableBoxTrans,
        $this->tableViews, $this->tableViewLinks, $this->tableModules, $this->topicId, $lngId, $viewModeId);
    } else {
      $sql = "SELECT bl.topic_id, bl.box_id, bl.box_sort, b.box_name, b.boxgroup_id,
                      bt.box_title, bt.box_data, bt.view_id,
                      m.module_guid, m.module_useoutputfilter, m.module_path, m.module_file, m.module_class
              FROM %s AS bl,
                    %s AS b,
                    %s AS bt,
                    %s AS v,
                    %s AS m
              WHERE bl.topic_id = '%d'
                AND b.box_id = bl.box_id
                AND bt.box_id = bl.box_id
                AND bt.lng_id = '%d'
                AND v.view_id = bt.view_id
                AND m.module_guid = v.module_guid
                AND m.module_active = 1
                AND m.module_type = 'box'
              ORDER BY bl.topic_id, bl.box_sort, bl.box_id";
      $params = array($this->tableLink, $this->tableBox, $this->tableBoxTrans,
        $this->tableViews, $this->tableModules, $this->topicId, $lngId);
    }
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['module_file'] = $row['module_path'].$row['module_file'];
        $this->data[$row['box_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Get list
  *
  * @param array &$images
  * @param boolean $editable
  * @param string $caption
  * @access public
  * @return string '' or XML
  */
  function getList(&$images, $editable, $caption) {
    $result = '';
    if (isset($this->linkList) && is_array($this->linkList)) {
      unset($groupLinks);
      foreach ($this->linkList as $key=>$val) {
        $box = $this->boxesList[$val['box_id']];
        $groupLinks[$box['boxgroup_id']][] = $val;
      }
      if ((isset($groupLinks) && is_array($groupLinks)) && (isset($this->boxGroupsList) && is_array($this->boxGroupsList))) {
        $result .= sprintf('<dialog title="%s" width="100%%">'.LF, $caption);
        $result .= '<listview width="100%"><items>'.LF;
        foreach($this->boxGroupsList as $groupId=>$group) {
          if (isset($groupLinks[$groupId]) && is_array($groupLinks[$groupId])) {
            $boxLinks = $groupLinks[$groupId];
            if (isset($this->opened[$groupId]) && $this->opened[$groupId]) {
              $node = 'open';
              $cmd = 'close';
            } else {
              $node = 'close';
              $cmd = 'open';
            }
            $nodeHref = sprintf('%s?%s[cmd]=%s&amp;%s[gid]=%s&amp;%s[page_id]=%s',
              $this->baseLink, $this->paramName, $cmd, $this->paramName, $groupId, $this->paramName, $this->topicId);
            $indent = 0;
            $result .= sprintf('<listitem title="%s" image="%s" indent="%s" node="%s" nhref="%s">',
              papaya_strings::escapeHTMLChars($group['boxgroup_title']), $images[51], $indent, $node, $nodeHref); 
            if ($this->authUser->hasPerm(13)) {
              $result .= '<subitem/>';
            }
            $result .= sprintf('<subitem>%s</subitem>', papaya_strings::escapeHTMLChars($group['boxgroup_name']));
            $result .= '<subitem/><subitem/><subitem/>';
            $result .= '</listitem>'.LF;
            if (isset($this->opened[$groupId]) && $this->opened[$groupId]) {
              $first = reset($boxLinks);
              $last = end($boxLinks);
              foreach($boxLinks as $boxLink) {
                $data = $this->boxesList[$boxLink['box_id']];
                if (isset($data)) {
                  $result .= sprintf('<listitem title="%s" image="%s" indent="2">'.LF,
                    papaya_strings::escapeHTMLChars($data['box_name']), $images[5]);
                  if ($this->authUser->hasPerm(13)) {
                    $result .= '<subitem align="center">';
                    $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
                      $this->getLink(array('cmd' => 'chg_show', 'bid' => $data['box_id'], 'p_mode' => 1), 'bb', 'boxes.php'),
                      papaya_strings::escapeHTMLChars($images[14]),
                      papaya_strings::escapeHTMLChars($this->_gt('Edit')));
                    $result .= '</subitem>';
                  }
                  $result .= '<subitem/>';
                  if ($editable) {
                    $result .= '<subitem align="right">';
                    if ($boxLink != $first) {
                      $result .= sprintf('<a href="%s?%s[cmd]=%s&amp;%s[boxlink_id]=%s&amp;%s[page_id]=%s"><glyph src="%s" /></a>'.LF,
                        $this->baseLink, $this->paramName, 'up', $this->paramName,
                        $boxLink['boxlink_id'],
                        $this->paramName, $this->topicId,
                        $images[6]);
                    }
                    $result .= '</subitem>';
                    $result .= '<subitem>';
                    if ($boxLink != $last) {
                      $result .= sprintf('<a href="%s?%s[cmd]=%s&amp;%s[boxlink_id]=%s&amp;%s[page_id]=%s"><glyph src="%s" /></a>'.LF,
                        $this->baseLink, $this->paramName, 'down', $this->paramName, $boxLink['boxlink_id'],
                        $this->paramName, $this->topicId,
                        $images[7]);
                    }
                    $result .= '</subitem>';
                    $result .= '<subitem align="right">';
                    $result .= sprintf('<a href="%s?%s[cmd]=%s&amp;%s[boxlink_id]=%s&amp;%s[page_id]=%s"><glyph src="%s" /></a>'.LF,
                      $this->baseLink, $this->paramName, 'del', $this->paramName, $boxLink['boxlink_id'],
                      $this->paramName, $this->topicId,
                      $images[10]);
                    $result .= '</subitem>';
                  } else {
                    $result .= '<subitem/><subitem/><subitem/>';
                  }
                  $result .= '</listitem>';
                }
              }
            }
          }
        }
        $result .= '</items></listview>'.LF;
        $result .= '</dialog>'.LF;
      }
    }
    return $result;
  }

  /**
  * Get Box list
  *
  * @param array &$images
  * @access public
  * @return string '' or XML
  */
  function getBoxList(&$images) {
    $result = '';
    if (isset($this->boxesList) && is_array($this->boxesList)) {
      unset($groupLinks);
      unset($linked);
      settype($linked, 'array');
      if (isset($this->linkList) && is_array($this->linkList)) {
        foreach ($this->linkList as $key=>$val) {
          $linked[] = $val['box_id'];
        }
      }
      foreach ($this->boxesList as $boxId=>$box) {
        if (!in_array($boxId, $linked)) {
          $groupLinks[$box['boxgroup_id']][] = $box['box_id'];
        }
      }
      if (isset($groupLinks) && is_array($groupLinks) && is_array($this->boxGroupsList)) {
        $result .= sprintf('<dialog title="%s" width="100%%">'.LF, 
          papaya_strings::escapeHTMLChars($this->_gt('Available boxes')));
        $result .= '<listview><items>'.LF;
        foreach($this->boxGroupsList as $groupId=>$group) {
          if (isset($groupLinks[$groupId]) && is_array($groupLinks[$groupId])) {
            $boxIds = $groupLinks[$groupId];
            if (isset($this->opened[$groupId]) && $this->opened[$groupId]) {
              $node = 'open';
              $cmd = 'close';
            } else {
              $node = 'close';
              $cmd = 'open';
            }
            $nodeHref = sprintf('%s?%s[cmd]=%s&amp;%s[gid]=%s',
              $this->baseLink, $this->paramName, $cmd, $this->paramName, $groupId);
            $indent = 0;
            $result .= sprintf('<listitem title="%s" image="%s" indent="%s" node="%s" nhref="%s">'.LF,
              papaya_strings::escapeHTMLChars($group['boxgroup_title']), $images[51], $indent, $node, $nodeHref);
            if ($this->authUser->hasPerm(13)) {
              $result .= '<subitem/>';
            }
            $result .= sprintf('<subitem>%s</subitem>', papaya_strings::escapeHTMLChars($group['boxgroup_name']));
            $result .= '<subitem/>';
            $result .= '</listitem>';
            if (isset($this->opened[$groupId]) && $this->opened[$groupId]) {
              foreach($boxIds as $boxId) {
                $data = $this->boxesList[$boxId];
                $result .= sprintf('<listitem title="%s" image="%s" indent="2">'.LF,
                  papaya_strings::escapeHTMLChars($data['box_name']), $images[5]);
                if ($this->authUser->hasPerm(13)) {
                  $result .= '<subitem align="center">';
                  $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
                    $this->getLink(array('cmd' => 'chg_show', 'bid' => $data['box_id'], 'p_mode' => 1), 'bb', 'boxes.php'),
                    papaya_strings::escapeHTMLChars($images[14]),
                    papaya_strings::escapeHTMLChars($this->_gt('Edit')));
                  $result .= '</subitem>';
                }
                $result .= '<subitem/>';
                $result .= '<subitem align="right">';
                $result .= sprintf('<a href="%s"><glyph src="%s" hint="%s"/></a>'.LF,
                  $this->getLink(array('cmd' => 'add', 'box_id' => $boxId)),
                  papaya_strings::escapeHTMLChars($images[11]),
                  papaya_strings::escapeHTMLChars($this->_gt('Connect')));
                $result .= '</subitem>';
                $result .= '</listitem>';
              }
            }
          }
        }
        $result .= '</items></listview>'.LF;
        $result .= '</dialog>'.LF;
      }
    }
    return $result;
  }

  /**
  * User input Processing
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'add' :
        $this->add();
        break;
      case 'del' :
        $this->delete();
        break;
      case 'up' :
        $this->move('up');
        break;
      case 'down' :
        $this->move('down');
        break;
    }
  }

  /**
  * Add a box into link table
  *
  * @access public
  */
  function add() {
    if ($this->params['box_id'] > 0 && $this->topicId > 0) {
        $sql = "SELECT MAX(box_sort)
                FROM %s
                WHERE topic_id = '%d'";
        $sort = 0;
        $params=array($this->tableLink,$this->topicId);
        if ($res = $this->databaseQueryFmt($sql,$params)) {
          if ($row = $res->fetchRow()) {
            $sort = $row[0];
          }
        }
        ++$sort;
        $values=array(
                      'box_id' => $this->params['box_id'],
                      'topic_id' => $this->topicId,
                      'box_sort' => $sort
                );
        if (FALSE !== $this->databaseInsertRecord($this->tableLink, 'boxlink_id', $values)) {
          $this->loadList();
        }
    }
  }

  /**
  * Delete box from link table
  *
  * @access public
  */
  function delete() {
    $link = @$this->linkList[$this->params['boxlink_id']];
    if (isset($link) && $this->topicId > 0) {
      if ($this->databaseDeleteRecord($this->tableLink, 'boxlink_id', $link['boxlink_id'])) {
        $sql = "UPDATE {$this->tableLink}
                  SET box_sort = box_sort-1
                WHERE topic_id = '{$link['topic_id']}'
                  AND (box_sort > {$link['box_sort']})";
        $this->databaseQuery($sql);
      }
      $this->loadList();
    }
  }

  /**
  * Get  grouped ids
  *
  * @param integer $groupId
  * @access public
  * @return mixed array or FALSE
  */
  function getGroupedIDs($groupId) {
    $result = FALSE;
    if (isset($this->linkList) && is_array($this->linkList)) {
      foreach($this->linkList as $id=>$link) {
        if ($groupId == $this->boxesList[$link['box_id']]['boxgroup_id']) {
          $result[$id] = $link;
        }
      }
    }
    if (count($result) < 2) {
      unset($result);
      $result = FALSE;
    }
    return $result;
  }

  /**
  * Move
  *
  * @param string $dir optional, default value 'up'
  * @access public
  * @return boolean
  */
  function move($dir='up') {
    $link = $this->linkList[$this->params['boxlink_id']];
    $groupId = $this->boxesList[$link['box_id']]['boxgroup_id'];
    $grouped = $this->getGroupedIDs($groupId);
    if (isset($grouped) && is_array($grouped)) {
      $prior = FALSE;
      $i = reset($grouped);
      while (isset($i) && ($i['boxlink_id'] != $link['boxlink_id'])) {
        $prior = $i;
        $i = next($grouped);
      }
      $next = next($grouped);
      switch ($dir) {
        case 'up' :
            $result = $this->exchange($link, $prior);
          break;
        case 'down' :
            $result = $this->exchange($link, $next);
          break;
      }
    }
    return $result;
  }

  /**
  * Exchange ( move box link from to )
  *
  * @param array &$src
  * @param array &$des
  * @access public
  * @return boolean
  */
  function exchange(&$src,&$des) {
    $result = FALSE;
    if ((isset($src) && is_array($src)) && (isset($des) && is_array($des))) {
      if (FALSE !== $this->databaseUpdateRecord($this->tableLink, array('box_sort' => $des['box_sort']), 'boxlink_id', $src['boxlink_id'])) {
        if (FALSE !== $this->databaseUpdateRecord($this->tableLink, array('box_sort' => $src['box_sort']), 'boxlink_id', $des['boxlink_id'])) {
          $result = TRUE;
        }
      }
      $this->loadList();
    }
    return $result;
  }
}
?>