<?php
/**
* Dynamic formulars consists of XML-data for admin section
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_dynform.php,v 1.4 2006/02/21 09:51:59 grassmann Exp $
*/


/**
* Dynamic formulars out of XML data
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_dynform.php');

/**
* Dynamic formulars consists of XML-data for admin section
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_dynamic_form extends base_dynamic_form {

  /**
  * Get dialog XML
  *
  * @access public
  * @return string
  */
  function getDialogXML() {
    $result = sprintf('<dialog action="%s" method="%s" title="%s" width="100%%" enctype="multipart/form-data">',
      $this->baseLink, 'POST', $this->dialogTitle);
    $result .= $this->getHiddenFieldsXML();
    if (isset($this->groups) && is_array($this->groups)) {
      $result .= '<lines class="dialogLarge">';
      foreach($this->groups as $group) {
        $result .= '<subtitle caption="'.papaya_strings::escapeHTMLChars($group['title']).'"/>';
        if (isset($group['fields']) && is_array($group['fields'])) {
          foreach($group['fields'] as $fieldId) {
            $field = $this->fields[$fieldId];
            $result .= sprintf('<line fid="%s" caption="%s" %s>',
              papaya_strings::escapeHTMLChars($field['name']), papaya_strings::escapeHTMLChars($field['title']),
              ($field['needed'] ? 'needed="'.papaya_strings::escapeHTMLChars($this->_gt('Needed field')).'"' : ''));
            $result .= $this->getFieldXML($field);
            $result .= '</line>';
          }
        }
      }
      $result .= '</lines>';
      $result .= '<dlgbutton value="'.papaya_strings::escapeHTMLChars($this->buttonTitle).'"/>';
    }
    $result .= '</dialog>';
    return $result;
  }

  /**
  * Get field name
  *
  * @param string $fieldName
  * @access public
  * @return string
  */
  function getFieldName($fieldName) {
    return (isset($this->paramName) && ($this->paramName != ''))
      ? $this->paramName.'['.$fieldName.']'
      : $fieldName;
  }

  /**
  * Get field CSS
  *
  * @param &$field
  * @access public
  * @return string
  */
  function getFieldCSS(&$field) {
    switch($field['type']) {
    case 'combobox' :
      return 'dialogSelect dialogScale';
      break;
    case 'checkbox' :
      return 'dialogCheckbox dialogScale';
    case 'checkboxes' :
      return 'dialogCheckboxes dialogScale';
    case 'file' :
      return 'dialogFileSelect dialogScale';
    case 'image' :
      return 'dialogImage dialogScale';
    case 'radio' :
      return 'dialogRadio dialogScale';
    case 'textarea' :
      return 'dialogTextarea dialogScale';
    case 'url' :
    case 'input' :
    default :
      return 'dialogInput dialogScale';
    }
  }

  /**
  * Load parameters
  *
  * @access public
  */
  function loadParams() {
    if (isset($this->paramName) && $this->paramName != '') {
      if (isset($_REQUEST[$this->paramName]) && is_array($_REQUEST[$this->paramName])) {
        $this->params = $_REQUEST[$this->paramName];
      }
    }
  }

  /**
  * Get file upload data
  *
  * @param string $fieldName
  * @access public
  * @return mixed array or FALSE
  */
  function getFileUploadData($fieldName) {
    if (isset($_FILES) && is_array($_FILES) && isset($_FILES[$this->paramName]) &&
        isset($_FILES[$this->paramName]['name'][$fieldName]) &&
        isset($_FILES[$this->paramName]['tmp_name'][$fieldName]) &&
        is_uploaded_file($_FILES[$this->paramName]['tmp_name'][$fieldName])) {
      return array(
        'name' => $_FILES[$this->paramName]['name'][$fieldName],
        'tmp_name' => $_FILES[$this->paramName]['tmp_name'][$fieldName]
      );
    } else {
      return FALSE;
    }
  }
}

?>