<?php
/**
* Media database administration area
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_fileman.php,v 1.34 2006/05/18 09:20:45 rekowski Exp $
*/

/**
* Media database basic functions
*/
require_once(dirname(__FILE__)."/base_fileman.php");

/**
* Media database administration area
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_filemanager extends base_filemanager {

  /**
  * Number of Upload fields in admin media database
  * @var number
  */
  var $numUploadFields = 3;

  /**
  * Folder list for rekursive upload
  * @var array
  */
  var $uploadFolderFolders = NULL;


  /**
  * Check string
  * @var string
  */
  var $validFileNamePattern = '~^[a-z\d][a-z\d_-]+\.([a-z]+)$~';

  /**
  * File dialog input fields
  * @var array
  */
  var $fileDialog = array(
    'file_title'=>array('Title', 'isNoHTML', FALSE, 'input', 255),
    'file_comment'=>array('Comment', 'isNoHTML', FALSE, 'textarea', 4),
    'Language independent',
    'file_name'=>array('Filename', 'isNoHTML', TRUE, 'input', 120),
    'file_sort'=>array('Sort', 'isAlphaNumChar', FALSE, 'input', 10),
  );

  /**
  * Initialize check parameters and perform action
  *
  * @access public
  */
  function initialize() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_language_select.php');
    $this->lngSelect = &base_language_select::getInstance();

    $this->initializeParams($this->sessionParamName);
    if (isset($this->params['file'])) {
      $this->params['file'] = papaya_strings::escapeHTMLChars($this->params['file']);
    } else {
      $this->params['file'] = '';
    }
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    switch (@$this->params['cmd']) {
      case 'open' :
        if (isset($this->params['folder_id']) && $this->params['folder_id'] > 0) {
          $this->sessionParams['opened'][$this->params['folder_id']] = TRUE;
        }
        break;
      case 'close' :
        if (isset($this->sessionParams['opened'][$this->params['folder_id']])) {
          unset($this->sessionParams['opened'][$this->params['folder_id']]);
        }
        break;
    }

    $this->initializeSessionParam('folder_id');
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    if (isset($this->params['folder_id'])) {
      $this->folderId = $this->params['folder_id'];
    } else {
      $this->folderId = 0;
    }
  }

  function executeBrowser() {
    $this->dataLoadFolderList($this->lngSelect->currentLanguageId);
    $this->dataLoadFileList($this->lngSelect->currentLanguageId);
  }

  /**
  * Function for handle parameters
  *
  * @access public
  */
  function execute() {
    $this->dataLoadFolderList($this->lngSelect->currentLanguageId);
    $this->dataLoadFileList($this->lngSelect->currentLanguageId);

    $result = FALSE;
    switch (@$this->params['cmd']) {
    case 'syncronize' :
      $this->syncronize();
      break;
    case 'rename' :
      $result .= $this->actRename();
      break;
    case 'del' :
      $result .= $this->actDelete();
      break;
    case 'cut' :
      $this->doFileMove(TRUE);
      break;
    case 'paste' :
      $this->doFileMove();
      break;
    case 'f_cut' :
      $this->doFolderMove(TRUE);
      break;
    case 'f_paste' :
      $this->doFolderMove();
      break;
    case 'f_delperm' :
    case 'f_addperm' :
    case 'f_prop' :
      $result .= $this->actFolderProperties();
      break;
    case 'f_create' :
      $result .= $this->actFolderCreate();
      break;
    case 'f_delete' :
      $result .= $this->actFolderDelete();
      break;
    case 'dir_upload' :
      $this->initializeDirectoryInputForm();
      if (isset($this->params['folder_id']) && $this->params['folder_id'] > 0){
        if ($this->dialogInput->modified()) {
          if ($this->dialogInput->checkDialogInput()) {
            if (is_dir($this->params['directory'])){
              if ($this->uploadDirectory($this->params['directory'], $this->params['directory'])){
              }
            }
          }
        }
      }
    default :
      $result .= $this->actUpload();
      if (isset($this->params['folder_id']) && isset($this->params['cmd']) && $this->params['cmd'] == 'folder') {
        $result .= $this->actFolderProperties();
      } else {
        $result .= $this->getSelected();
      }
      break;
    }

    $this->dataLoadFolderList($this->lngSelect->currentLanguageId);
    $this->dataLoadFileList($this->lngSelect->currentLanguageId);
    $this->getXMLUploadDirectory();
    if ($this->folderId > 0) {
      $this->dataLoadFolder($this->folderId, $this->lngSelect->currentLanguageId);
    }

    if ($result && trim($result) != '') {
      $this->layout->add($result);
    }

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Load directory
  *
  * @param boolean $imagesOnly optional, default value FALSE
  * @access public
  * @return array
  */
  function loadDirectory($imagesOnly = FALSE) {
    $result = FALSE;
    unset($this->files);
    if ($dh = @opendir($this->dataDirectory)) {
      while ($file = readdir($dh)) {
        if (!preg_match("/^\./",$file)) {
          list(,,$imageType) = @getimagesize($this->dataDirectory.$file);
          if ((($imageType >= 1 && $imageType <= 4) || $imageType == 13) || (!$imagesOnly)) {
            $result[] = $file;
          }
        }
      }
    }
    return $result;
  }

  /**
  * Synchronize
  *
  * @access public
  */
  function syncronize() {
    $hddFiles = $this->loadDirectory();
    if (isset($hddFiles) && is_array($hddFiles)) {
      foreach($hddFiles as $file) {
        if (!$this->dataLoadFile($file, $this->lngSelect->currentLanguageId)) {
          list(,,$imageType) = @getimagesize($this->dataDirectory.$file);
          $fileDate = filectime($this->dataDirectory.$file);
          $fileSize = filesize($this->dataDirectory.$file);
          if ($this->dataInsertFile($file, $file, $fileSize, $imageType, $fileDate)) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('File "%s" added.'), $file));
          }
        }
      }
    }
    $this->dataLoadFileList($this->lngSelect->currentLanguageId,FALSE);
    if (isset($this->files) && is_array($this->files)) {
      foreach($this->files as $file) {
        if (!file_exists($this->dataDirectory.$file['file_id'])) {
          if ($this->dbDeleteFile($file['file_id'])) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('File "%s (%s)" not found. Record deleted.'),
              $file['file_name'], $file['file_id']));
          }
        }
      }
      unset($this->files);
    }
    $this->repairPaths();
  }

  /**
  * Get XML for media library admin area
  * @access public
  */
  function getXML() {
    $this->layout->addLeft($this->lngSelect->getComboFormXML(435));
    $this->layout->addLeft($this->getFolders());
    $this->layout->addLeft($this->getList());
    $this->layout->addLeft($this->getClipboard());
  }

  /**
  * Get buttons
  *
  * @access public
  */
  function getButtons() {
    $result = FALSE;
    if (preg_match("/[^\/\\\]+$/",$this->params['file'],$regs)) {
      $iFile = trim($regs[0]);
    } else {
      $iFile = '';
    }
    $result[] =
      array('Syncronize', 'Syncronize database with directory', $this->images[58], 27,
        $this->getLink(array('cmd' => 'syncronize')), '_self'
      );
    $result[] = array('-');
    if (file_exists($this->dataDirectory.$iFile) && ($iFile != '')) {
      $result[] =
        array('Show file', 'Open file in a new window', $this->images[4], 15,
          $this->getWebMediaLink($iFile), '_blank'
        );
      $result[] =
        array('Properties', 'Edit file properties', $this->images[14], 9,
          $this->getLink(array('cmd' => 'rename', 'file' => $iFile)), '_self'
        );
      $result[] =
        array('Cut', 'Cut file', $this->images[61], 9,
          $this->getLink(array('cmd' => 'cut', 'file' => $iFile)), '_self'
        );
      $result[] =
        array('Delete', 'Delete file', $this->images[10], 10,
          $this->getLink(array('cmd' => 'del', 'file' => $iFile)), '_self'
        );
    } elseif (isset($this->folderId) && ($this->folderId > 0) && isset($this->folders[$this->folderId]) && is_array($this->folders[$this->folderId])) {
      $result[] =
        array('New folder', 'Create a new folder', $this->images[93], 27,
          $this->getLink(array('cmd' => 'f_create', 'folder_id' => (int)$this->folderId)), '_self');
      $result[] =
        array('Properties', 'Properties', $this->images[14], 27,
          $this->getLink(array('cmd' => 'f_prop', 'folder_id' => (int)$this->folderId)), '_self');
      $result[] =
        array('Cut', 'Cut folder', $this->images[61], 27,
          $this->getLink(array('cmd' => 'f_cut', 'folder_id' => (int)$this->folderId)), '_self');
      $result[] =
        array('Delete', 'Delete folder', $this->images[10], 27,
          $this->getLink(array('cmd' => 'f_delete', 'folder_id' => (int)$this->folderId)), '_self');
    } elseif ($this->folderId == 0) {
      $result[] =
        array('New folder', 'Create a new folder', $this->images[59], 27,
          $this->getLink(array('cmd' => 'f_create', 'folder_id' => (int)$this->folderId)), '_self');
    }
    return $result;
  }

  /**
  * Get thumbnail view
  *
  * @access public
  */
  function getThumbview() {
    $this->layout->setParam('PAGE_MODE', 'frame');
    switch (@$this->params['part']) {
      case 'thumbs' :
        $this->layout->add($this->getTVThumbs());
        break;
      case 'preview' :
        $this->layout->add($this->getTVPreview());
        break;
      case 'list' :
        $this->layout->add($this->getTVList());
        break;
      default :
        $this->layout->setParam('PAGE_MODE', 'frameset');
        $this->layout->add($this->getTVFrameset());
    }
  }

  /**
  * Get TV frameset
  *
  * @access public
  * @return string frameset
  */
  function getTVFrameset() {
    $result = '
<script type="text/javascript" src="script/imgbrowser.js"></script>
<script type="text/javascript">
<![CDATA[
var linkList = "'.sprintf('%s?%s[part]=list',$this->baseLink,$this->paramName).'";
var linkPreview = "'.sprintf('%s?%s[part]=preview',$this->baseLink,$this->paramName).'";
var linkThumbs = "'.sprintf('%s?%s[part]=thumbs',$this->baseLink,$this->paramName).'";

]]>
</script>
        ';
    $result .= '<frameset cols="300,*" frameborder="yes" framespacing="5" border="5">'."\r\n";
    $result .= '<frameset rows="*,200" frameborder="yes" framespacing="5" border="5">'."\r\n";
    $result .= sprintf('<frame src="%s?%s[%s]=%s" scrolling="auto" name="flist" />'."\r\n",
      $this->baseLink, $this->paramName, 'part', 'list');
    $result .= sprintf('<frame src="%s?%s[%s]=%s" scrolling="no" name="fpreview" />'."\r\n",
      $this->baseLink, $this->paramName, 'part', 'preview');
    $result .= '</frameset>'."\r\n";
    $result .= sprintf('<frame src="%s?%s[%s]=%s" scrolling="auto" name="fthumbs" />'."\r\n",
      $this->baseLink, $this->paramName, 'part', 'thumbs');
    $result .= '</frameset>'."\r\n";
    return $result;
  }

  /**
  * Get TV thumbnails
  *
  * @access public
  * @return string XMl
  */
  function getTVThumbs() {
    $max = 0;
    $end = 0;
    $start = 0;
    $result = '<thumbnails>'.LF;
    if (isset($this->files) && is_array($this->files)) {
      $result .= '<line>';

      $this->params['start'] = floor(($this->params['file']-1) / $this->maxPerPage) * $this->maxPerPage;

      if ($this->params['start'] > 0) {
        $start = $this->params['start'];
      } else {
        $start = 0;
      }
      $max = count($this->files)-1;
      $end = $start+$this->maxPerPage-1;
      $counter = 0;

      include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');

      $thumbnail = &new base_thumbnail;
      $thumbnail->baseDirectory = $this->dataDirectory;
      $thumbnail->thumbnailDirectory = $this->thumbnailDirectory;
      $thumbnail->width = $this->minithumbWidth;
      $thumbnail->height = $this->minithumbWidth;
      if (isset( $this->params['allnew']) &&  $this->params['allnew']) {
        $thumbnail->createAll = TRUE;
      } else {
        $thumbnail->createAll = FALSE;
      }

      foreach($this->files as $key=>$image) {
        if (($key >= $start) && ($key <= $end)) {
          list($widthString,$heightString,$imageType,$sizeString) = @getimagesize($this->dataDirectory.$image['file_id']);
          if (strlen($image['file_name'])>18) {
            $imageString = substr($image['file_name'],0,15).'...';
          } else {
            $imageString = $image['file_name'];
          }
          if ($this->params['file'] == ($key+1)) {
            $selected = ' selected="selected"';
          } else {
            $selected = '';
          }
          $href = sprintf("javascript:parent.changeFrames('&amp;%s[%s]=%s');",
            $this->paramName, 'file', $key+1);
          if (isset($widthString) && isset($heightString) && $widthString > 0 && $heightString > 0) {
            $sizeString = ' size="'.$widthString.'x'.$heightString.'"';
          } else {
            $sizeString = '';
          }
          if ($thumbFileName = $thumbnail->getThumb($image['file_id'])) {
            $result .= sprintf('<thumbnail src="%s" title="%s" hint="%s" href="%s" %s%s/>'.LF,
              $this->getWebMediaLink($thumbFileName, 'thumb'), $imageString,
              $this->getWebMediaLink($image['file_name']), $href, $sizeString, $selected);
          } elseif ($icon = $this->getIcon($image['file_type'], $image['file_name'])) {
            $result .= sprintf('<thumbnail src="%s" title="%s" hint="%s" href="%s" %s%s/>'.LF,
              $icon, $imageString,
              $this->getWebMediaLink($image['file_name']), $href, $sizeString, $selected);
          } else {
            $result .= sprintf('<thumbnail title="%s" hint="%s" href="%s" %s%s/>'.LF,
              $imageString, $this->getWebMediaLink($image['file_name']), $href, $sizeString, $selected);
          }
          $counter++;
          if (((($counter) % $this->cols) == 0) && ($key < $end)) {
            $result .= '</line><line>'.LF;
          }
        }
      }
      if (($spacerColumns = ($this->cols - (($counter) % $this->cols))) && ($counter > $this->cols)) {
        $result .= sprintf('<spacer colspan="%s" />'.LF, $spacerColumns);
      }
      $result .= '</line>';
    }
    $result .= '<buttons>';
    if ($start > 0) {
      $result .= sprintf("<button href=\"javascript:parent.changeFrames('&amp;%s[%s]=%s');\" title=\"Zurck\" hint=\"Vorherige Seite\" glyph=\"%s\" />\n",
        $this->paramName, 'file', $start, $this->images[8]);
    } else {
      $result .= "<button/>";
    }
    $result .= sprintf("<button href=\"javascript:parent.changeFrames('&amp;%s[%s]=%s&amp;%s[%s]=%s');\" title=\"Aktualisieren\" hint=\"Anzeige aktualisieren\" glyph=\"%s\" />\n",
      $this->paramName, 'file', (int)$this->params['file'],
      $this->paramName, 'allnew', 1, $this->images[38]);
    if ($end < $max) {
      $result .= sprintf("<button href=\"javascript:parent.changeFrames('&amp;%s[%s]=%s');\" title=\"Vor\" hint=\"Nchste Seite\" glyph=\"%s\" />\n",
        $this->paramName, 'file', $end+2, $this->images[9]);
    } else {
      $result .= "<button/>";
    }
    $result .= '</buttons>';
    $result .= '</thumbnails>';
    $result .= $this->getSelected('500');
    return $result;
  }

  /**
  * Get TV preview
  *
  * @access public
  * @return string
  */
  function getTVPreview() {
    $result = '';
    if (isset($this->params['file']) && $this->params['file'] > 0) {
      $file = $this->files[$this->params['file']-1];
    } elseif (isset($this->params['mid']) && $this->params['mid'] != '' && $this->dataLoadFile($this->params['mid'], $this->lngSelect->currentLanguageId)) {
      $file = $this->selectedFile;
    } else {
      $file = NULL;
    }
    if (isset($file) && is_array($file)) {
      list($widthString,$heightString,$imageType,$sizeString) = @getimagesize($this->dataDirectory.$file['file_id']);
      $css = sprintf('position:absolute; left:0px; top:0px; width:%spx; height:%spx; border: none;', $widthString, $heightString);
      if (($imageType >= 1 && $imageType <= 4) || $imageType == 13) {
        $previewTag = '';
        if ($imageType >= 1 && $imageType <= 3) {
          $previewTag = sprintf('<img src="%s" id="preview" onclick="resizeImage();" style="%s" %s/>',
            $this->getWebMediaLink($file['file_id']), $css, $sizeString);
        } elseif ($imageType == 4) {
          $previewTag = sprintf('<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"   codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=5,0,0,0" width="100%%" height="100%%"><param name="movie" value="%s" /><param name="play" value="FALSE" /><param name="loop" value="FALSE" /><param name="quality" value="high" /><embed src="%s" play="FALSE" loop="FALSE" quality="high" bgcolor="#FFFFFF"  width="100%%" height="100%%" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"></embed></object>',  $this->getWebMediaLink($file['file_id']), $this->getWebMediaLink($file['file_id']));
        }
        $result .= $previewTag;
        $result .= '
<script type="text/javascript" src="script/imgbrowser.js"></script>
<script type="text/javascript">
<![CDATA[
var actWidth = 0;
var actHeight = 0;
var actLeft = 0;
var actTop = 0;
var imgwidth = '.((int)$widthString).';
var imgheight = '.((int)$heightString).';
resizeImage();
]]>
</script>';
      } else {
        $result = $this->getFileInfos($file);
      }
    }
    return $result;
  }

  /**
  * Get file informations
  *
  * @param array &$file
  * @param mixed $widthString optional, default value NULL
  * @param mixed $heightString optional, default value NULL
  * @access public
  * @return string xml
  */
  function getFileInfos(&$file, $widthString = NULL, $heightString = NULL) {
    $result = '<listview width="100%">';
    $result .= '<items>';
    $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>',
      $this->_gt('Filename'), $file['file_name']);
    $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>'.LF, $this->_gt('Filesize'),
      $this->formatFileSizeToStr($file['file_size']));
    $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>'.LF, $this->_gt('File date'),
      date('Y-m-d H:i:s', $file['file_date']));
    $result .= sprintf('<listitem title="%s"><subitem>%s</subitem></listitem>'.LF, $this->_gt('File type'),
      $this->getMimeType($file['file_type'],$file['file_name']));
    if (isset($widthString) && $widthString > 0) {
      $result .= sprintf('<listitem title="%s"><subitem>%s Pixel</subitem></listitem>'.LF, $this->_gt('Image width'),
        number_format($widthString,0,',','.'));
    }
    if (isset($heightString) && $heightString > 0) {
      $result .= sprintf('<listitem title="%s"><subitem>%s Pixel</subitem></listitem>'.LF, $this->_gt('Image height'),
        number_format($heightString,0,',','.'));
    }
    $result .= '</items>';
    $result .= '</listview>';
    return $result;
  }

  /**
  * Get TV list
  *
  * @access public
  * @return string XML
  */
  function getTVList() {
    return $this->getFolders(TRUE, '100%');
  }

  /**
  * Get selected image
  *
  * @param integer $formWidth optional, default value 500
  * @access public
  * @return string Java script XML
  */
  function getSelectedImage($formWidth = 500) {
    $result = '';
    if (isset($this->selectedFile) && is_array($this->selectedFile)) {
      $file = &$this->selectedFile;
    } elseif (isset($this->files[(int)$this->params['file'] - 1])) {
      $file = $this->files[(int)$this->params['file'] - 1];
    } else {
      $file = NULL;
    }
    if (isset($file) && is_array($file)) {
      list($widthString,$heightString,$imageType,$sizeString) = @getimagesize($this->dataDirectory.$file['file_id']);
      $imageWidth = (int)$widthString;
      $imageHeight = (int)$heightString;

      //dialog
      $result =  <<<HEREDOC
<script type="text/javascript" src="script/imgbrowser.js"></script>
<script type="text/javascript">
<![CDATA[
var imgheight = $imageHeight;
var imgwidth = $imageWidth;
var imgurl = '{$file['file_id']}';
]]>
</script>
HEREDOC;

      $result .= sprintf('<dialog action="#" title="%s [%s]" width="%s" maxfirstcol="FALSE" name="imgform" align="center">', $this->_gt('Integration'), $file['file_name'], $formWidth);
      $result .= '<lines class="dialogMedium">';
      $result .= '<line align="center">';
      $result .= '<layout border="10">';
      $result .= '<row>';

      $result .= '<cell>';
      include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
      $thumbnail = &new base_thumbnail;
      $thumbnail->baseDirectory = $this->dataDirectory;
      $thumbnail->thumbnailDirectory = $this->thumbnailDirectory;
      $thumbnail->width = $this->thumbWidth;
      $thumbnail->height = $this->thumbHeight;

      if ($thumbFileName = $thumbnail->getThumb($file['file_id'])) {
        $result .= sprintf('<panel><a href="%s" target="_blank"><img src="%s" border="0" alt="%s" /></a></panel>',
          $this->getWebMediaLink($file['file_id']),
          $this->getWebMediaLink($thumbFileName, 'thumb'), papaya_strings::escapeHTMLChars($file['file_name']));
      } elseif ($icon = $this->getIcon($file['file_type'], $file['file_name'])) {
        $result .= sprintf('<a href="%s" target="_blank"><img src="%s" border="0" alt="%s" /></a>',
          $this->getWebMediaLink($file['file_id']), $icon, papaya_strings::escapeHTMLChars($file['file_name']));
      } else {
        $result .= sprintf('<a href="%s" target="_blank">'.$this->_gt('No thumbnail').'</a>',
          $this->getWebMediaLink($file['file_id']));
      }
      $result .= '</cell>';

      $result .= '<cell>';
      $result .= $this->getFileInfos($file, $widthString, $heightString);
      $result .= '</cell>';
      $result .= '</row>';
      $result .= '</layout>';
      $result .= '</line>';
      $result .= sprintf('<line align="center"><textarea class="dialogTextarea dialogScale" style="width:480px;" cols="30" rows="4" name="imgtag" wrap="virtual"></textarea></line>'."\r\n",
        $this->webDataDir.$file['file_id'], $widthString, $heightString);
      $result .= sprintf('<line caption="%s">
          <select name="imgdownload" onchange="createMediaImageTag();" class="dialogSelect dialogScale">
            <option value="yes">%s</option>
            <option value="no" selected="selected">%s</option>
          </select>
        </line>'."\r\n",
        $this->_gt('Download'), $this->_gt('Yes'), $this->_gt('No'));
      $result .= sprintf('<subtitle caption="%s" />', $this->_gt('Text'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgsubtitle" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" /></line>'."\r\n", $this->_gt('Subtitle'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgalt" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" /></line>'."\r\n", $this->_gt('Alternative text'));
      $result .= sprintf('<subtitle caption="%s" />', $this->_gt('Scaling'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgwidth" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Width'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgheight" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Height'));
      $result .= sprintf('<line caption="%s">
          <select name="imgresize" onchange="createMediaImageTag();" class="dialogInput dialogScale">
            <option value="abs">%s</option>
            <option value="max" selected="selected">%s</option>
            <option value="min">%s</option>
            <option value="mincrop">%s</option>
          </select>
        </line>'."\r\n",
        $this->_gt('Scaling mode'), $this->_gt('Absolute'), $this->_gt('Maximum'), $this->_gt('Minimum'), $this->_gt('Minimum crop'));

      $result .= sprintf('<subtitle caption="%s" />', $this->_gt('Linking'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imglink" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" /></line>'."\r\n", $this->_gt('Link'));
      $result .= sprintf('<line caption="%s">
          <select name="imgtarget" onchange="createMediaImageTag();" class="dialogSelect dialogScale">
            <option value="">%s</option>
            <option value="_self">%s</option>
            <option value="_parent">%s</option>
            <option value="_top">%s</option>
            <option value="_blank">%s</option>
          </select>
        </line>'."\r\n", $this->_gt('Link target'), $this->_gt('Default'), $this->_gt('Actual window and frame'),
        $this->_gt('Actual window and parent frame'), $this->_gt('Actual window and top frame'),
        $this->_gt('New window'));

      $result .= sprintf('<subtitle caption="%s" />', $this->_gt('Formatting'));
      $result .= sprintf('<line caption="%s">
          <select name="imgalign" onchange="createMediaImageTag();" class="dialogSelect dialogScale">
            <option value="">%s</option>
            <option value="left">%s</option>
            <option value="right">%s</option>
            <option value="center">%s</option>
            <option value="middle">%s</option>
          </select>
        </line>'."\r\n",
        $this->_gt('Alignment'), $this->_gt('None'), $this->_gt('Left'), $this->_gt('Right'),
        $this->_gt('Center (horizontal)'), $this->_gt('Middle (vertical)'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imglspace" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Left space'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgtspace" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Top space'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgrspace" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Right space'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imgbspace" onkeyup="createMediaImageTag();" class="dialogInput dialogScale" value="" /></line>'."\r\n", $this->_gt('Bottom space'));
      $result .= '</lines>';
      $result .= '</dialog>';
      $result .= '<script type="text/javascript">
        <![CDATA[
          createMediaImageTag();
        ]]>
        </script>';
    }
    return $result;
  }

  /**
  * Get selected file
  *
  * @param integer $formWidth optional, default value 500
  * @access public
  * @return string XML Javascript
  */
  function getSelectedFile($formWidth = 500) {
    $result = '';
    if (isset($this->selectedFile) && is_array($this->selectedFile)) {
      $file = &$this->selectedFile;
    } else {
      $file = $this->files[$this->params['file']-1];
    }
    if (isset($file) && is_array($file)) {
      $result =  <<<HEREDOC
<script type="text/javascript" src="script/imgbrowser.js"></script>
<script type="text/javascript">
<![CDATA[
var imgurl = '{$file['file_id']}';
]]>
</script>
HEREDOC;

      $result .= sprintf('<dialog action="#" title="Einbindung [%s]" width="%s" maxfirstcol="FALSE" name="imgform" align="center">', $file['file_name'], $formWidth);
      $result .= '<lines>';
      $result .= '<line align="center">';
      $result .= '<layout border="10">';
      $result .= '<row>';
      $result .= '<cell>';
      if ($icon = $this->getIcon($file['file_type'], $file['file_name'])) {
        $result .= sprintf('<a href="%s" target="_blank"><img src="%s" border="0" alt="%s" /></a>',
          $this->getWebMediaLink($file['file_id']), $icon, papaya_strings::escapeHTMLChars($file['file_name']));
      } else {
        $result .= sprintf('<a href="%s" target="_blank">'.$this->_gt('Icon').'</a>',
          $this->getWebMediaLink($file['file_id']));
      }
      $result .= '</cell>';
      $result .= '<cell>';
      $result .= $this->getFileInfos($file);
      $result .= '</cell>';
      $result .= '</row>';
      $result .= '</layout>';
      $result .= '</line>';
      $result .= '<line align="center"><textarea class="dialogInput dialogLarge" style="width:480px;" cols="30" rows="4" name="imgtag" wrap="virtual"></textarea></line>'."\r\n";
      $result .= sprintf('<line caption="%s"><input type="text" name="imgtext" onkeyup="createMediaLinkTag();" class="dialogInput dialogScale" /></line>'."\r\n", $this->_gt('Link text'));
      $result .= sprintf('<line caption="%s"><input type="text" name="imghint" onkeyup="createMediaLinkTag();" class="dialogInput dialogScale" /></line>'."\r\n", $this->_gt('Quickinfo'));
      $result .= sprintf('<line caption="%s">
          <select name="imgtarget" onchange="createMediaLinkTag();" class="dialogSelect dialogScale">
            <option value="">%s</option>
            <option value="_self">%s</option>
            <option value="_parent">%s</option>
            <option value="_top">%s</option>
            <option value="_blank">%s</option>
          </select>
        </line>'."\r\n", $this->_gt('Link target'), $this->_gt('Default'), $this->_gt('Actual window and frame'),
        $this->_gt('Actual window and parent frame'), $this->_gt('Actual window and top frame'),
        $this->_gt('New window'));
      $result .= '</lines>';
      $result .= '</dialog>';
      $result .= '<script type="text/javascript"><![CDATA[
createMediaLinkTag();
]]></script>';
    }
    return $result;
  }

  /**
  * Get folders
  *
  * @param boolean $javascript optional, default value FALSE
  * @param integer $width optional, default value 430
  * @access public
  * @return mixed string boolean
  */
  function getFolders($javascript=FALSE, $width = 430) {
    $result = '';
    if (isset($this->folders) && is_array($this->folders)) {
      $result .= sprintf('<listview title="%s" width="%s">', $this->_gt('Folder'), $width);
      $result .= '<items>';
      $result .= $this->getFolderItem(0,0,$javascript);
      $result .= '</items>';
      $result .= '</listview>';
    }
    return $result;
  }

  /**
  * Get folder item
  *
  * @param integer $id
  * @param integer $indent
  * @param string $javascript
  * @access public
  * @return string
  */
  function getFolderItem($id, $indent, $javascript) {
    $result = '';
    if (isset($this->folders[$id]) && is_array($this->folders[$id])) {
      $folder = $this->folders[$id];
      if (isset($this->folderLinks[$id]['CHILDS']) && is_array($this->folderLinks[$id]['CHILDS'])) {
        $subFolders = $this->folderLinks[$id]['CHILDS'];
      } else {
        $subFolders = NULL;
      }
      if ($id == 0) {
        $imageIndex = 55;
        $open = TRUE;
      } elseif ($id == $this->folderId) {
        $open = @$this->sessionParams['opened'][$id];
        $imageIndex = 57;
      } else {
        $open = @$this->sessionParams['opened'][$id];
        $imageIndex = 56;
      }
      if (isset($subFolders) && is_array($subFolders)) {
        if ($open) {
          $node = 'open';
          $cmd = 'close';
        } else {
          $node = 'close';
          $cmd = 'open';
        }
        $nodeHref = sprintf(' node="%s" nhref="%s?%s[cmd]=%s&amp;%s[folder_id]=%s&amp;%s[part]=list"',
          $node, $this->baseLink, $this->paramName, $cmd, $this->paramName, $id, $this->paramName);
        $iIndent = $indent;
      } else {
        $nodeHref = ' node="empty"';
        $iIndent = $indent;
      }
      if ($id == $this->folderId) {
        $selected = ' selected="selected"';
      } else {
        $selected = '';
      }
      if ($javascript) {
        $href = sprintf("javascript:parent.changeFrames('&amp;%s[cmd]=folder&amp;%s[folder_id]=%s');",
            $this->paramName, $this->paramName, $id);
      } else {
        $href = sprintf('%s?%s[cmd]=folder&amp;%s[folder_id]=%s',
            $this->baseLink, $this->paramName, $this->paramName, $id);
      }
      if (isset($folder['folder_title']) && $folder['folder_title'] != ''){
        $title = $folder['folder_title'];
      } else {
        $title = $this->_gt('No Title');
      }

      $result .= sprintf('<listitem title="%s" href="%s" indent="%s" image="%s" %s>',
        papaya_strings::escapeHTMLChars($title), $href, $iIndent, $this->images[$imageIndex], $nodeHref.$selected);
      switch (@$folder['surfer_useparent']) {
        case '3' :  $imageIndex = 17; break;
        case '1' :  $imageIndex = 1; break;
        default  :  $imageIndex = 0;
      }
      $result .= '<subitem align="right"><glyph src="'.$this->images[$imageIndex].'"/></subitem>';
      $result .= '</listitem>';
      if (isset($subFolders) && is_array($subFolders) && $open) {
        foreach($subFolders as $subId) {
          $result .= $this->getFolderItem($subId,$indent+1, $javascript);
        }
      }
    }
    return $result;
  }

  /**
  * Get list
  *
  * @access public
  * @return string
  */
  function getList() {
    $result = '';
    if (isset($this->files) && is_array($this->files)) {
      $result .= sprintf('<listview title="%s" width="430">'.LF, $this->_gt('Files'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Filename'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Filesize'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Width'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Height'));
      $result .= '<col/>'.LF;
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      foreach($this->files as $key=>$file) {
        list($fileWidth, $fileHeight, $fileType) = @getimagesize($this->dataDirectory.$file['file_id']);
        $imageIdx = $this->getGlyphIndex($fileType);
        $selected = ($this->params['file'] == $file['file_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem title="%s" href="%s?%s[%s]=%s" image="%s"%s>'.LF,
          papaya_strings::escapeHTMLChars($file['file_name']),
          $this->baseLink, $this->paramName, 'file', urlencode($file['file_id']),
          $this->images[$imageIdx], $selected);
        $result .= sprintf('<subitem align="right">%s</subitem>'.LF,
          $this->formatFileSizeToStr($file['file_size']));
        $result .= sprintf('<subitem align="right">%s</subitem>'.LF,
          number_format($fileWidth,0,',','.'));
        $result .= sprintf('<subitem align="right">%s</subitem>'.LF,
          number_format($fileHeight),0,',','.');
        $href = sprintf('%s?%s[%s]=%s&amp;%s[%s]=%s',
          $this->baseLink, $this->paramName, 'cmd', 'cut', $this->paramName, 'file', $file['file_id']);
        $result .= sprintf('<subitem align="right"><a href="%s"><glyph src="%s" hint="%s"/></a></subitem>',
          $href, $this->images[61], $this->_gt('Cut'));
        $result .= '</listitem>'.LF;
      }
      $result .= '</items></listview>'.LF;
    }
    return $result;
  }

  /**
  * Get clip board
  *
  * @access public
  * @return string XML or ''
  */
  function getClipBoard() {
    $result = '';
    $files = $this->dataLoadFileList($this->lngSelect->currentLanguageId, FALSE, FALSE, TRUE);
    if ((isset($this->folderLinks[-1]) && is_array($this->folderLinks[-1])) || isset($files)) {
      $folders = @$this->folderLinks[-1]['CHILDS'];
      if ((isset($folders) && is_array($folders)) || (isset($files) && is_array($files))) {
        $result .= sprintf('<listview title="%s" width="430">', $this->_gt('Clipboard'));
        $result .= '<items>';
        if (isset($folders) && is_array($folders)) {
          foreach($folders as $folder) {
            if (isset($this->folders[$folder]['folder_title']) && trim($this->folders[$folder]['folder_title']) != '') {
              $title = $this->folders[$folder]['folder_title'];
            } else {
              $title = $this->_gt('No Title');
            }
            $result .= sprintf('<listitem title="%s" image="%s">',
              papaya_strings::escapeHTMLChars($title), $this->images[56]);
            $href = $this->getLink(array('cmd' => 'f_paste', 'folder' => $folder, 'folder_id' => (int)$this->folderId));
            if ($folder != $this->folderId) {
              $result .= sprintf('<subitem align="right"><a href="%s"><glyph src="%s"/></a></subitem>',
                $href, $this->images[60], $this->_gt('Paste'));
            } else {
              $result .= '<subitem/>';
            }
            $result .= '</listitem>';
          }
        }
        if (isset($files) && is_array($files)) {
          foreach($files as $file) {
            $result .= sprintf('<listitem title="%s" image="%s">',
            papaya_strings::escapeHTMLChars($file['file_name']), $this->images[$this->getGlyphIndex($file['file_type'])]);
            $href = sprintf('%s?%s[%s]=%s&amp;%s[%s]=%s',
              $this->baseLink, $this->paramName, 'cmd', 'paste', $this->paramName, 'file', $file['file_id']);
            $result .= sprintf('<subitem align="right"><a href="%s"><glyph src="%s"/></a></subitem>',
              $href, $this->images[60], $this->_gt('Paste'));
            $result .= '</listitem>';
          }
        }
        $result .= '</items>';
        $result .= '</listview>';
      }
    }
    return $result;
  }

  /**
  * Initialize folder dialog
  *
  * @access public
  */
  function initializeFolderDialog() {
    if (!(isset($this->folderDialog) && is_object($this->folderDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'save'=> 1,
        'cmd'=> 'f_prop',
        'folder_id' => $this->folder['folder_id']
      );
      $data = array(
        'folder_title' => $this->folder['folder_title'],
        'surfer_useparent' => $this->folder['surfer_useparent'],
      );
      $permItems = array(
        1 => $this->_gt('Own'),
        2 => $this->_gt('Inherited'),
        3 => $this->_gt('Additional')
      );
      $fields = array(
        'folder_title' => array('Title', 'isNoHTML', TRUE, 'input', 100, '', ''),
        'Language independent',
        'surfer_useparent' => array('Permissions mode', 'isNum', TRUE, 'combo', $permItems, '', 2, 'center')

      );
      $this->folderDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->folderDialog->dialogTitle = $this->_gt('Folder properties');
      $this->folderDialog->tokenKeySuffix = 'folder_properties';
      $this->folderDialog->baseLink = $this->baseLink;
      $this->folderDialog->msgs = &$this->msgs;
      $this->folderDialog->loadParams();
    }
  }

  /**
  * Action rename file
  *
  * @access public
  * @return string
  */

  function actRename() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $result = '';
    if ($this->dataLoadFile($this->params['file'], $this->lngSelect->currentLanguageId)) {
      if ($this->authUser->hasPerm(9)) {
        //$result .= $this->doRename();
        $hidden = array('cmd'=>'rename', 'file'=>$this->selectedFile['file_id'], 'save'=>1);
        $dialog = &new base_dialog($this, $this->paramName, $this->fileDialog, $this->selectedFile, $hidden);
        $dialog->msgs = &$this->msgs;
        $dialog->dialogTitle = $this->_gt('Properties');
        $dialog->tokenKeySuffix = 'edit_file_properties';
        $dialog->loadParams();

        if ($dialog->modified('save')) {
          if ($dialog->checkDialogInput()) {
            if ($this->dataEditFileProperties($this->params['file'], $dialog->data, $this->lngSelect->currentLanguageId)) {
              $this->addMsg(MSG_INFO, $this->_gt('Changes saved.'));
            }
            $this->dataLoadFileList($this->lngSelect->currentLanguageId);
          }
        }
        $result = $dialog->getDialogXML();
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('You have no permission for this action.'));
      }
    }
    return $result;
  }

  /**
  * Action delete file
  *
  * @access public
  * @return string
  */
  function actDelete() {
    $result = '';
    if ($this->dataLoadFile($this->params['file'], $this->lngSelect->currentLanguageId)) {
      if ($this->authUser->hasPerm(10)) {
        $result .= $this->doDelete();
        if ($this->params['file'] != '') {
          include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
          $hidden = array(
            'cmd' => 'del',
            'file' => $this->selectedFile['file_id'],
            'delfile'=> $this->selectedFile['file_id'],
          );
          $msg = sprintf($this->_gt('Delete file "%s" (%s)?'), $this->selectedFile['file_name'],$this->selectedFile['file_id']);
          $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
          $dialog->msgs = &$this->msgs;
          $dialog->buttonTitle = 'Delete';
          $result .= $dialog->getMsgDialog();
        } else {
          $result .= $this->actUpload();
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('You have no permission for this action.'));
      }
    }
    return $result;
  }

  /**
  * Do file move
  *
  * @param boolean $clipboard optional, default value FALSE
  * @access public
  */
  function doFileMove($clipboard=FALSE) {
    if ($this->dataLoadFile($this->params['file'], $this->lngSelect->currentLanguageId)) {
      if ($clipboard) {
        $this->dbMoveFile($this->params['file'], -1);
      } else {
        if (isset($this->folders[$this->folderId]) && is_array($this->folders[$this->folderId])) {
          $folderId = (int)$this->folderId;
        } else {
          $folderId = 0;
        }
        $this->dbMoveFile($this->params['file'], $folderId);
      }
      $this->dataLoadFileList($this->lngSelect->currentLanguageId);
    }
  }

  /**
  * Do folder move
  *
  * @param boolean $clipboard optional, default value FALSE
  * @access public
  */
  function doFolderMove($clipboard = FALSE) {
    $folder = $this->folders[$this->folderId];
    if (isset($folder) && is_array($folder)) {
      if ($clipboard) {
        $this->dbMoveFolder($this->folderId, -1, '');
        $this->folderId = (int)$folder['parent_id'];
      } else {
        if (isset($this->folders[$this->params['folder']]) &&
            isset($this->folders[$this->folderId]) &&
            $this->folderId != $this->params['folder']) {
          if ($this->folderId > 0) {
            $path = $this->folders[$this->folderId]['folder_path'].$this->folderId.';';
          } else {
            $path = ';0;';
          }
          $this->dbMoveFolder((int)$this->params['folder'], $this->folderId, $path);
          $this->folderId = (int)$this->params['folder'];
        }
      }
    }
  }

  /**
  * Action create folder
  *
  * @access public
  * @return string '' or XML
  */
  function actFolderCreate() {
    $result = '';
    if ($this->authUser->hasPerm(27)) {
      if ($newId = $this->doFolderCreate()) {
        $this->sessionParams['opened'][$this->folderId] = TRUE;
        unset($this->folder);
        $this->setFolderID($newId);
        $this->dataLoadFolderList($this->lngSelect->currentLanguageId);
        $result = $this->actFolderProperties();
      }
    }
    return $result;
  }

  /**
  * Action delete folder
  *
  * @access public
  * @return string
  */
  function actFolderDelete() {
    $result = '';
    $folder = $this->folders[$this->folderId];
    if (isset($folder) && is_array($folder) && isset($this->folderId) && ($this->folderId > 0)) {
      if ($this->authUser->hasPerm(27)) {
        if (!is_array($this->files)) {
          if (!$this->doFolderDelete()) {
            include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
            $hidden = array(
              'cmd' => 'f_delete',
              'folder' => $this->folderId,
              'delfolder'=> $this->folderId,
            );
            $msg = sprintf($this->_gt('Delete folder "%s"?'), $folder['folder_title']);
            $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
            $dialog->msgs = &$this->msgs;
            $dialog->buttonTitle = 'Delete';
            $result .= $dialog->getMsgDialog();
          }
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('The folder is not empty.'));
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('You have no permission for this action.'));
      }
    }
    return $result;
  }

  /**
  * Do delete folder
  *
  * @access public
  * @return boolean
  */
  function doFolderDelete() {
    if (isset($this->params['delfolder']) && $this->params['delfolder'] > 0) {
      if ($this->dbDeleteFolder($this->params['delfolder'])) {
        $folderId = $this->folders[$this->params['folder']]['parent_id'];
        $this->params['cmd'] = 'folder';
        unset($this->params['folder']);
        unset($this->params['delfolder']);
        $this->setFolderId($folderId);
        $this->dataLoadFolderList($this->lngSelect->currentLanguageId);
        $this->layout->add($this->actFolderProperties());
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Action folder properties
  *
  * @access public
  * @return string XML
  */
  function actFolderProperties() {
    $result = '';
    if ($this->dataLoadFolder($this->folderId, $this->lngSelect->currentLanguageId) && $this->folder['parent_id'] >= 0) {
      if ($this->authUser->hasPerm(27)) {
        $this->initializeFolderDialog();
        switch ($this->params['cmd']) {
          case 'f_prop' :
            if ($this->folderDialog->modified()) {
              if ($this->folderDialog->checkDialogInput()) {
                if ($this->dbSaveFolder()) {
                  $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Folder'))) ;
                  $this->dataLoadFolder($this->folderId, $this->lngSelect->currentLanguageId);
                  $this->folders[$this->folderId] = $this->folder;
                }
              }
            }
            break;
          case 'f_addperm' :
            if (isset($this->params['perm_id'])) {
              if (isset($this->folder['inh_perms'][$this->params['perm_id']])) {
                $this->addMsg(MSG_INFO, $this->_gt('Inherited permission.'));
              } elseif (!isset($this->folder['perms'][$this->params['perm_id']])) {
                $this->folder['perms'][$this->params['perm_id']] = TRUE;
                if ($this->dbSaveFolderPerms()) {
                  $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Permissions')));
                }
              }
            }
            break;
          case 'f_delperm' :
            if (isset($this->params['perm_id']) && isset($this->folder['perms'][$this->params['perm_id']])) {
              unset($this->folder['perms'][$this->params['perm_id']]);
              if ($this->dbSaveFolderPerms()) {
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Permissions')));
              }
            }
            break;
        }
        $result .= $this->folderDialog->getDialogXML();

        include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
        $surfer = &new base_surfer(FALSE);
        if ($surfer->loadPermissionList() && isset($surfer->permissions)&& is_array($surfer->permissions)) {
          $result .= sprintf('<listview title="%s" width="100%%">'.LF, $this->_gt('Permissions'));
          $result .= '<cols>'.LF;
          $result .= '<col>'.$this->_gt('Title').'</col>'.LF;
          $result .= '<col align="center">'.$this->_gt('Active').'</col>'.LF;
          if ($this->folder['surfer_useparent'] > 1) {
            $result .= '<col align="center">'.$this->_gt('Inherited').'</col>'.LF;
            if ($this->folder['surfer_useparent'] > 2) {
              $result .= '<col align="center">'.$this->_gt('Linked').'</col>'.LF;
            }
          } else {
            $result .= '<col align="center">'.$this->_gt('Linked').'</col>'.LF;
          }
          $result .= '</cols>'.LF;
          $result .= '<items>'.LF;
          $href = $this->baseLink.'?'.$this->paramName.'[folder_id]='.$this->folder['folder_id'];
          foreach($surfer->permissions as $permId=>$perm) {
            $result .= sprintf('<listitem title="%s" image="%s">',
              papaya_strings::escapeHTMLChars($perm['surferperm_title']),  $this->images[26]);
            $result .= '<subitem align="center">';
            $result .= sprintf('<glyph src="%s" />',
              ($perm['surferperm_active']) ? $this->images[29] : $this->images[30]);
            $result .= '</subitem>';
            if ($this->folder['surfer_useparent'] > 1) {
              $result .= '<subitem align="center">';
              $result .= sprintf('<glyph src="%s" />',
                (isset($this->folder['inh_perms'][$permId])) ? $this->images[29] : $this->images[30]);
              $result .= '</subitem>';
              if ($this->folder['surfer_useparent'] > 2) {
                $result .= '<subitem align="center">';
                if (!isset($this->folder['inh_perms'][$permId])) {
                  if (isset($this->folder['perms'][$permId])) {
                    $aHref =
                      $href.'&amp;'.$this->paramName.'[cmd]=f_delperm&amp;'.$this->paramName.'[perm_id]='.$permId;
                    $imageIndex = 27;
                  } else {
                    $aHref =
                      $href.'&amp;'.$this->paramName.'[cmd]=f_addperm&amp;'.$this->paramName.'[perm_id]='.$permId;
                      $imageIndex = 28;
                  }
                  $result .= sprintf('<a href="%s"><glyph src="%s"/></a>', $aHref, $this->images[$imageIndex]);
                }
                $result .= '</subitem>';
              }
            } else {
              $result .= '<subitem align="center">';
              if (isset($this->folder['perms'][$permId])) {
                $aHref = $href.'&amp;'.$this->paramName.'[cmd]=f_delperm&amp;'.$this->paramName.'[perm_id]='.$permId;
                $imageIndex = 27;
              } else {
                $aHref = $href.'&amp;'.$this->paramName.'[cmd]=f_addperm&amp;'.$this->paramName.'[perm_id]='.$permId;
                $imageIndex = 28;
              }
              $result .= sprintf('<a href="%s"><glyph src="%s"/></a>', $aHref, $this->images[$imageIndex]);
              $result .= '</subitem>';
            }
            $result .= '</listitem>'.LF;
          }
          $result .= '</items>'.LF;
          $result .= '</listview>'.LF;
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('You have no permission for this action.'));
      }
    }
    return $result;
  }

  /**
  * Action upload file
  *
  * @access public
  * @return string
  */
  function actUpload() {
    if ($this->authUser->hasPerm(8)) {
      if (isset($this->params['file']) && $this->params['file'] != '') {
        $this->dataLoadFile($this->params['file'], $this->lngSelect->currentLanguageId);
      }
      $result = $this->doUpload();
      $result .= sprintf('<dialog action="%s" title="%s" width="100%%" type="file" enctype="multipart/form-data">'.LF,
        $this->baseLink, $this->_gt('Upload file'));
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'."\r\n",
        $this->paramName, 'cmd', 'upload');
      $result .= sprintf('<input type="hidden" name="%s[%s]" value="%s" />'."\r\n",
        $this->paramName, 'file',
        (isset($this->selectedFile)) ? papaya_strings::escapeHTMLChars($this->selectedFile['file_id']) : '');
      $result .= sprintf('<input type="hidden" name="MAX_FILE_SIZE" value="%s" />'."\r\n",
        $this->maximumSize);
      $result .= '<lines>';
      for($i = 0; $i < $this->numUploadFields; $i++) {
        $result .= sprintf('<line caption="%s %d">', $this->_gt('Upload'), $i);
        $result .= sprintf('<input type="file" size="40" class="file" name="%s[%s][%d]" />'.LF,
          $this->paramName, 'upload', $i);
        $result .= '</line>';
        if ($i == 0 && isset($this->selectedFile) && is_array($this->selectedFile)) {
          $result .= sprintf('<line caption="%s" align="center">', $this->_gt('Replace'));
          $result .= sprintf('<input type="checkbox" name="%s[%s]" value="1" />'.LF,
            $this->paramName, 'replace');
          $result .= '</line>';
        }
      }
      $result .= '</lines>';
      $result .= sprintf('<dlgbutton value="%s"/>'.LF, $this->_gt('Upload'));
      $result .= "</dialog>\r\n";
    }
    return $result;
  }

  /**
  * Do create folder
  *
  * @access public
  * @return mixed
  */
  function doFolderCreate() {
    return $this->dbCreateFolder();
  }

  /**
  * Create folder in database
  *
  * @access public
  * @return mixed id or FALSE
  */
  function dbCreateFolder() {
    $data = array(
      'parent_id'=>$this->folderId,
      'surfer_useparent'=> 2
    );
    if ($this->folderId > 0) {
      if ($this->dataLoadFolder($this->folderId, $this->lngSelect->currentLanguageId)) {
        $data['folder_path'] = $this->folder['folder_path'].$this->folderId.';';
      } else {
        return FALSE;
      }
    }
    if ($newId = $this->databaseInsertRecord($this->tableFolders, 'folder_id', $data)) {
      return $newId;
    }
    return FALSE;
  }

  /**
  * Do delete
  *
  * @access public
  * @return ''
  */
  function doDelete() {
    preg_match("/[^\/\\\]+$/",$this->params['file'],$regs);
    $iFile = trim($regs[0]);
    if (isset($this->params['delfile']) && $this->params['delfile']) {
      $this->deleteFile($this->params['file']);
    }
    return '';
  }

    /**
  * Do rename
  *
  * @access public
  * @return string ''
  */
  function doRename() {
    $newCaption = papaya_strings::escapeHTMLChars(strtolower(trim($this->params['newcap'])));
    if ($this->params['renamefile']) {
      if (preg_match($this->validFileNamePattern, $newCaption, $match)) {
        print_r($match);
        if (!in_array('.'.$match[1], $this->forbiddenExt)) {
          if ($this->dataRenameFile(addslashes($this->params['file']), addslashes($this->params['newcap']))) {
            $this->addMsg(MSG_INFO, $this->_gt('Changes saved!'));
            $this->dataLoadFileList($this->lngSelect->currentLanguageId);
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Invalid extension!'));
        }
      } else {
        $this->addMsg(MSG_ERROR, $this->_gt('Invalid filename!'));
      }
    }
    return '';
  }

    /**
  * Do Upload
  *
  * @access public
  */
  function doUpload() {
    if (isset($_FILES[$this->paramName])) {
      for($i = 0; $i < $this->numUploadFields; $i++) {
        if (isset($_FILES[$this->paramName]['tmp_name']['upload'][$i]) &&
            isset($_FILES[$this->paramName]['name']['upload'][$i])) {
          $uploadData = array(
            'tmp_name' => $_FILES[$this->paramName]['tmp_name']['upload'][$i],
            'name' => $_FILES[$this->paramName]['name']['upload'][$i],
            'error' => $_FILES[$this->paramName]['error']['upload'][$i]
          );
          if ($fileData = $this->checkUploadFile($uploadData, $uploadData['name'])) {
            $fileId = $fileData['id'];
            $folderId = (isset($this->folderId)) ? (int)$this->folderId : 0;
            if (isset($this->params['replace']) && $this->params['replace'] == 1 && $this->dataLoadFile(@$this->params['file'], $this->lngSelect->currentLanguageId)) {
              $ext = $this->getExtension($uploadData['name']);
              if ($ext == $this->getExtension($this->selectedFile['file_id'])) {
                $fileId = $this->selectedFile['file_id'];
              } else {
                $this->addMsg(MSG_WARNING, $this->_gt('Different file formats. Could not replace current file.'));
              }
            }
            if ($this->addFileToMediaDatabase($uploadData['tmp_name'], $fileId, $uploadData['name'], $fileData['size'], $fileData['format'], $folderId)) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('File "%s" added.'), $uploadData['name']));
              $this->dataLoadFileList($this->lngSelect->currentLanguageId);
              $this->params['file'] = $fileId;
            }
          }
        }
      }
    }
  }



  /**
  * Add file
  *
  * @param mixed $fileData upload data array or temporry filename
  * @param string $fileName
  * @param mixed $folderId optional, default value NULL
  * @access public
  * @return mixed FALSE or id
  */
  function addFile($fileData, $fileName, $folderId = NULL) {
    if (!isset($folderId) && isset($this->folder)) {
      $folderId = $this->folder['folder_id'];
    } elseif (!isset($folderId)) {
      $folderId = 0;
    }
    if ($file = $this->checkUploadFile($fileData, $fileName)) {
      if (is_array($fileData)) {
        $tempFileName = $fileData['tmp_name'];
      } else {
        $tempFileName = $fileData;
      }
      return $this->addFileToMediaDatabase($tempFileName, $file['id'], $fileName, $file['size'], $file['format'], $folderId);
    }
    return FALSE;
  }


  /**
  * Delete file
  *
  * @param integer $fileId
  * @access public
  * @return boolean
  */
  function deleteFile($fileId) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
    $thumbnail = &new base_thumbnail;
    $thumbnail->baseDirectory = $this->dataDirectory;
    $thumbnail->thumbnailDirectory = $this->thumbnailDirectory;
    $thumbnail->deleteThumbs($fileId);
    if ($this->dbDeleteFile($fileId)) {
      if (@unlink($this->dataDirectory.$fileId)) {
        $this->params['file'] = '';
        $this->addMsg(MSG_INFO, sprintf('File "%s" deleted.', $fileId));
        $this->dataLoadFileList($this->lngSelect->currentLanguageId);
        return TRUE;
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
    }
    return FALSE;
  }

  /**
  * Move file in database
  *
  * @param integer $fileId
  * @param integer $folderId
  * @access public
  * @return boolean
  */
  function dbMoveFile($fileId, $folderId) {
    return (FALSE !== $this->databaseUpdateRecord($this->tableFiles, array('folder_id' => $folderId), 'file_id', $fileId));
  }

  /**
  * Move folder in database
  *
  * @param integer $folderId
  * @param integer $parentId
  * @access public
  * @return boolean
  */
  function dbMoveFolder($folderId, $parentId, $parentPath) {
    $data = array('parent_id' => $parentId, 'folder_path' => $parentPath);
    if (FALSE !== $this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $folderId)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Rename file in database
  *
  * @param $fileId
  * @param $fileName
  * @access public
  * @return boolean
  */
  function dataRenameFile($fileId, $fileName) {
    return (FALSE !== $this->databaseUpdateRecord($this->tableFiles, array('file_name' => $fileName), 'file_id', $fileId));
  }

  /**
  * Delete file in databsae
  *
  * @param integer $fileId
  * @access public
  * @return boolean
  */
  function dbDeleteFile($fileId) {
    return (FALSE !== $this->databaseDeleteRecord($this->tableFiles, 'file_id', $fileId));
  }


  /**
  * Repair paths
  *
  * @access public
  */
  function repairPaths() {
    $count = 0;
    if (isset($this->folders) && is_array($this->folders)) {
      foreach($this->folders as $folderId=>$folder) {
        if ($folderId > 0) {
          $oldPath = $folder['folder_path'];
          $path = $this->calcPrevPath($folderId);
          if ($folder['parent_id'] == $folderId) {
            $data = array(
              'folder_path' => ';0;',
              'parent_id' => 0
            );
            $this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $folderId);
            $count++;
          } elseif ($oldPath != $path) {
            $data = array(
              'folder_path' => $path
            );
            $this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $folderId);
            $count++;
          }
        }
      }
      if ($count > 0) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paths changed.'), $count));
      }
    }
  }


  /**
  * Calculate preview path
  *
  * @param $folderId
  * @access public
  * @return string
  */
  function calcPrevPath($folderId) {
    if ($folderId > 0 && isset($this->folders[$folderId])) {
      $folder = $this->folders[$folderId];
      if (!isset($folder['newpath'])) {
        if ($folder['parent_id'] > 0 && $folder['parent_id'] != $folderId) {
          if (isset($this->folders[$folder['parent_id']]) && $folder['parent_id'] >= 0) {
            $newPath = $this->calcPrevPath($folder['parent_id']).$folder['parent_id'].';';
          } else {
            $newPath = ';0;';
          }
        } else {
          $newPath = ';0;';
        }
        $this->folders[$folderId]['newpath'] = $newPath;
        return $newPath;
      } else {
        return $folder['newpath'];
      }
    }
    return '';
  }



  /**
  * Set folder ID
  *
  * @param integer $folderId
  * @access public
  */
  function setFolderID($folderId) {
    $this->sessionParams['folder_id'] = $folderId;
    $this->folderId = $folderId;
    $this->params['folder_id'] = $folderId;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Save folder in database
  *
  * @access public
  * @return boolean
  */
  function dbSaveFolder() {
    $result = TRUE;
    if (isset($this->params['surfer_useparent'])) {
      $data = array(
        'surfer_useparent' => (int) $this->params['surfer_useparent']
      );
      if ($this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $this->folderId) === FALSE) {
        return FALSE;
      }
    }
    if ($result && isset($this->params['folder_title'])) {
      $sql = "SELECT folder_title FROM %s WHERE folder_id = %d AND lng_id = %d";
      $params = array($this->tableFoldersTrans, $this->folderId, $this->lngSelect->currentLanguageId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        if ($row = $res->fetchRow()) {
          if ($row[0] != $this->params['folder_title']) {
            $data = array(
              'folder_title' => $this->params['folder_title']
            );
            $filter = array(
              'folder_id' => $this->folderId,
              'lng_id' => $this->lngSelect->currentLanguageId
            );
            return (FALSE !== $this->databaseUpdateRecord($this->tableFoldersTrans, $data, $filter));
          }
        } else {
          $data = array(
            'folder_title' => $this->params['folder_title'],
            'folder_id' => $this->folderId,
            'lng_id' => $this->lngSelect->currentLanguageId
          );
          return (FALSE !== $this->databaseInsertRecord($this->tableFoldersTrans, NULL, $data));
        }
      } else {
        return FALSE;
      }
    }
    return $result;
  }

  /**
  * Save folder perissions in database
  *
  * @access public
  * @return boolean
  */
  function dbSaveFolderPerms() {
    $data = array(
      'surfer_permids' => $this->encodeIdArray($this->folder['perms'])
    );
    if ($this->databaseUpdateRecord($this->tableFolders, $data, 'folder_id', $this->folderId) !== FALSE) {
        return TRUE;
    }
    return FALSE;
  }



  /**
  * Delete folder in database
  *
  * @access public
  * @return boolean
  */
  function dbDeleteFolder($folderId) {
    return (FALSE !== $this->databaseDeleteRecord($this->tableFolders, 'folder_id', $folderId));
  }

  /**
  * Encode ID array
  *
  * @param array &$arr
  * @access public
  * @return string id string or ''
  */
  function encodeIdArray(&$arr) {
    if (isset($arr) && is_array($arr)) {
      return ';'.implode(';', array_keys($arr)).';';
    }
    return '';
  }

  /**
  * Get glyph index
  *
  * @param $fileType
  * @access public
  * @return integer
  */
  function getGlyphIndex($fileType) {
    switch ($fileType) {
      case 1 :
      case 2 :
      case 3 :
        return 31;
      case 4 :
      case 13 :
        return 32;
      default :
        return 33;
    }
  }

  /**
  * Upload directory field
  *
  * @access public
  */
  function getXMLUploadDirectory(){
    $this->initializeDirectoryInputForm();
    $this->layout->add($this->dialogInput->getDialogXML());
  }

  /**
  * Initialize directory input form
  *
  * @access public
  */
  function initializeDirectoryInputForm() {
    if (!(isset($this->dialogInput) && is_object($this->dialogInput))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');

      $data = array();
      $hidden = array(
         'cmd' => 'dir_upload',
         'save' => 1,
      );
      $btnCaption = 'Save';

      $fields = array(
        'directory' => array('Directory', 'isPath', TRUE, 'input', 250, 'Please enter the directory path with "/" not with "\\"')
      );
      $this->dialogInput = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogInput->msgs = &$this->msgs;
      $this->dialogInput->loadParams();
      $this->dialogInput->inputFieldSize = 'medium';
      $this->dialogInput->baseLink = $this->baseLink;
      $this->dialogInput->dialogTitle = papaya_strings::escapeHtmlChars($this->_gt('Directory Upload'));
      $this->dialogInput->buttonTitle = $btnCaption;
      $this->dialogInput->dialogDoubleButtons = FALSE;
    }
  }

  /**
  * Upload directory function
  *
  * @param string $directory
  * @param integer $parentFolder optional, default value 0
  * @param string $folderName optional, default value ''
  * @access public
  */
  function uploadDirectory($directory, $baseDirectory) {
    $this->dataLoadFolder($this->params['folder_id']);
    if (is_dir($directory)) {
      if ($dh = opendir($directory)) {
        $folderId = -1;
        while ($file = readdir($dh)) {
          if ($file != '.' && $file != '..') {
            if (is_dir($directory.$file)) {
              $this->uploadDirectory($directory.$file.'/', $baseDirectory);
            } elseif (is_file($directory.$file)) {
              //hier pfad prfen und anlegen
              if ($folderId < 0) {
                $folderId = $this->getFolderIdFromPath($directory, $baseDirectory);
              }
              if ($folderId != FALSE){
                $fileData['size'] = @filesize($directory.$file);
                $ext = $this->getExtension($file);
                $fileData['id'] = $this->getRandFilename($ext);
                $fileData['tmp_name'] = $directory.$file;
                $fileData['name'] = $file;
                list(,,$fileData['format']) = @getimagesize($directory.$file);
                $this->dataInsertFile($fileData['id'], $fileData['name'], $fileData['size'],
                  $fileData['format'], time(), $folderId);
                copy($directory.$file, $this->dataDirectory.$fileData['id']);
              }
            }
          }
        }
        closedir($dh);
      }
    }
  }


  /**
  * Get folder id from path
  *
  * @param string $directory
  * @param string &$baseDirectory
  * @access public
  * @return integer folderid
  */
  function getFolderIdFromPath($directory, &$baseDirectory){
    if ($directory == $baseDirectory){
      return $this->params['folder_id'];
    } else {
      $path = strtr($directory,array($baseDirectory => ''));
      if (!isset($this->uploadFolderFolders[$path])){
        // $folderId = anlegen des folders und rckgabe id
        $folderId = -1;
        $folderId = $this->createUploadFolder($path, $baseDirectory);
        // kreiert hier ablegen

        return $folderId;
      } else {
        return $this->uploadFolderFolders[$path];
      }
    }

  }


  /**
  * Create upload folder by directory
  *
  * @param string $path
  * @param string $baseDirectory
  * @access public
  * @return integer folderId
  */
  function createUploadFolder($path, $baseDirectory){
    $prevPath = '';
    $parent = 0;
    $foldersPath = '';
    $directories = explode('/', $path);
    foreach($directories as $directory){
      if ($directory != ''){
        $foldersPath = $foldersPath.$directory.'/';
        if (!isset($this->uploadFolderFolders[$foldersPath])){
          if ($parent == 0){
            // ein neues unterfolder anlegen und $this->params[folder_id] als ausgang nehmen
            $parent = $this->params['folder_id'];
            $prevPath = $this->folder['folder_path'].$this->params['folder_id'].';';
            $newId = $this->dbCreateUploadFolder($directory, $parent, $prevPath);
            $this->uploadFolderFolders[$foldersPath] = $newId;
          } else {
            // $parent als ausgangsfolder nehmen
            $prevPath = $prevPath.$parent.';';
            $newId = $this->dbCreateUploadFolder($directory, $parent, $prevPath);
            $this->uploadFolderFolders[$foldersPath] = $newId;
          }
        }
        $parent = $this->uploadFolderFolders[$foldersPath];
      }
    }
    return $this->uploadFolderFolders[$path];
  }

  /**
  * Database create upload folder
  *
  * @param string $directoryName
  * @param integer $parentFolderId
  * @param string $path
  * @access public
  * @return mixed False or $newId
  */
  function dbCreateUploadFolder($directoryName, $parentFolderId, $path){
    $data = array(
      'parent_id' => $parentFolderId,
      'folder_path' => $path,
      'surfer_useparent' => 2
      );
    if ($newId = $this->databaseInsertRecord($this->tableFolders, 'folder_id', $data)) {
      $data = array(
        'folder_title' => $directoryName,
        'folder_id' => $newId,
        'lng_id' => $this->lngSelect->currentLanguageId
      );
      $this->databaseInsertRecord($this->tableFoldersTrans, NULL, $data);
      return $newId;
    }
    return FALSE;
  }

  /**
  * Get selected
  *
  * @access public
  * @return string
  */
  function getSelected($dialogWidth = '100%') {
    $result = '';
    if (isset($this->selectedFile) && is_array($this->selectedFile)) {
      $file = &$this->selectedFile;
    } elseif (isset($this->files[(int)$this->params['file'] - 1])) {
      $file = $this->files[(int)$this->params['file'] - 1];
    } elseif ($this->params['file'] != '' && (!preg_match('/^\d+$/', $this->params['file'])) &&
      $this->dataLoadFile($this->params['file'], $this->lngSelect->currentLanguageId)) {
      $file = &$this->selectedFile;
    } else {
      $file = FALSE;
    }
    if ($file) {
      if ($this->syncronizeFileData($file['file_id'], $file['file_size'], $file['file_type'])) {
        $this->dataLoadFile($file['file_id'], $this->lngSelect->currentLanguageId);
      }
      list($imageWidth,$imageHeight,$imageType) = @getimagesize($this->dataDirectory.$file['file_id']);

      $imageWidth = (int)$imageWidth;
      $imageHeight = (int)$imageHeight;
      $imageType = (int)$imageType;

      if (isset($this->params['dialog']) && $this->params['dialog'] == 'no') {
        //push data to frame parent
        $fileName = papaya_strings::escapeHTMLChars($file['file_name']);
        $fileTitle = papaya_strings::escapeHTMLChars($file['file_title']);
        $fileSize = (int)$file['file_size'];
        $result =  <<<HEREDOC
<script type="text/javascript">
<![CDATA[
  if (parent.setMediaFileData) {
    parent.setMediaFileData('{$file['file_id']}', '$fileName', '$fileTitle', $fileSize,  $imageType, $imageWidth, $imageHeight);
  }
]]>
</script>
HEREDOC;
      } else {
        if ($imageType >= 1 && $imageType <= 3) {
          return $this->getSelectedImage($dialogWidth);
        } elseif ($imageType == 4 || $imageType == 13) {
          return $this->getSelectedImage($dialogWidth);
        } else {
          $ext = strtolower(strrchr($file['file_name'],'.'));
          switch ($ext) {
            case '.gif' :
            case '.jpg' :
            case '.jpeg' :
            case '.png' :
            case '.swf' :
            case '.swc' :
              return $this->getSelectedImage($dialogWidth);
              break;
            default :
              return $this->getSelectedFile($dialogWidth);
          }
        }
      }
    }
    return $result;
  }

}
?>