<?php
/**
* Implementation of the basic class for data base access
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_base_db.php,v 1.8 2006/04/20 18:24:00 weinert Exp $
*/

/**
* Aviable the basic class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');
/**
* Aviable the mistake catcher class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_error.php');


/**
* Object class for database access
*
* all other classes which have database access must be inherited from this one
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_db extends base_object {
  /**
  * Database URI
  * @var string $databaseURI
  */
  var $databaseURI = PAPAYA_DB_URI;

  /**
  * Prefix of the global variable
  * @var string $dbobjName
  */
  var $dbobjName = 'DATABASECONNECTION';

  /**
  * Reference to connection object
  * @var object $dbObj
  */
  var $dbObj;

  /**
  * Connection via Socket - data about Sockets, only MySQL
  * @var string $databaseSocket
  */
  var $databaseSocket = NULL;

  /**
  * Message for user
  * @var object base_errors $msgs
  */
  var $msgs;

  /**
  * Connect to data base object
  * @param integer $objNo Suffix(Zhler) fr globale Variable
  * @return boolean
  */
  function databaseConnect($objNo=0) {
    if (($objNo == 0) && isset($this->databaseObject)) {
      return $this->databaseObject->connect($this);
    } else {
      $objectName = $this->dbobjName.$objNo;
      if (isset($GLOBALS[$objectName]) && is_object($GLOBALS[$objectName])) {
        if ($this->databaseURI == $GLOBALS[$objectName]->databaseURI) {
          $this->databaseObject = &$GLOBALS[$objectName];
          return $this->databaseObject->connect($this);
        } else {
          return $this->databaseConnect($objNo+1);
        }
      } else {
        return $this->createDBObject($objNo);
      }
    }
  }

  /**
  * Create data base object(PEAR)
  *
  * @param integer $objNo Suffix fr globale Variable
  * @return boolean
  */
  function createDBObject($objNo) {
    if (isset($this->databaseSocket)) {
      ini_set('MYSQL_SOCKET', $this->databaseSocket);
      ini_set('mysql.default_socket', $this->databaseSocket);
    }
    if (trim($this->databaseURI) != '') {
      include_once(PAPAYA_INCLUDE_PATH.'system/sys_db_simple.php');
      $this->databaseObject = &new db_simple;
        $this->databaseObject->databaseURI = $this->databaseURI;
      $GLOBALS[$this->dbobjName.$objNo] = &$this->databaseObject;
      return $this->databaseObject->connect($this);
    }
  }

  /**
  * escape value for sql
  *
  * @param mixed $value Value to escape
  * @return string escaped value.
  */
  function escapeStr($value) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->escapeStr($this, $value);
    }
    return '';
  }

  /**
  * Set a counter to debug the next n queryies
  *
  * @param integer $count Queries to debug
  * @access public
  */
  function databaseDebugNextQuery($count = 1) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->debugNextQuery($count);
    }
    return FALSE;
  }

  /**
  * Interface function: submit database query
  *
  * @param string $sql
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function databaseQuery($sql, $max = NULL, $offset = NULL) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->query($sql, $this, $max, $offset);
    }
    return FALSE;
  }

  /**
  * Interface function: escape params and submit query
  *
  * @param string $sql SQL-String
  * @param array $vals values
  * @param integer $max maximum number returned data sets
  * @param integer $offset Offset
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function databaseQueryFmt($sql, $vals, $max = NULL, $offset = NULL) {
    if (isset($vals) && is_array($vals)) {
      foreach($vals as $sqlParam) {
        $sqlParams[] = $this->escapeStr($sqlParam);
      }
      $sql = vsprintf($sql, $sqlParams);
    } elseif (isset($vals)) {
      $sql = sprintf($sql, $this->escapeStr($vals));
    }
    return $this->databaseQuery($sql, $max, $offset);
  }

  /**
  * Interface function: create new record, return ID, set default values
  *
  * @param string $table table
  * @param mixed $idField field with ID (or NULL)
  * @param array $values default values
  * @return mixed FALSE or Id of new record
  */
  function databaseInsertRecord($table, $idField, $values = NULL) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->insertRecord($this, $table, $idField, $values);
    }
    return FALSE;
  }

  /**
  * Interface function: insert records
  *
  * @param string $table table
  * @param array $values default values
  * @return booolean
  */
  function databaseInsertRecords($table, $values) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->insertRecords($this, $table, $values);
    }
    return FALSE;
  }

  /**
  * Interface function: change data record
  *
  * @param string $table table
  * @param array $values default value
  * @param mixed $condition index field string or field value array or NULL (all records)
  * @param mixed $value index value (id $condition is a string)
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function databaseUpdateRecord($table, $values, $condition = NULL, $value = NULL) {
    if ($this->databaseConnect()) {
      if (!isset($condition)) {
        return $this->databaseObject->updateRecord($this, $table, $values, NULL);
      } elseif (isset($value)) {
        return $this->databaseObject->updateRecord($this, $table, $values, array((string)$condition => $value));
      } else {
        return $this->databaseObject->updateRecord($this, $table, $values, $condition);
      }
    }
    return FALSE;
  }

  /**
  * Interface function: delete database record
  *
  * @param string $table table
  * @param mixed $condition index field string or field value array or NULL (all records)
  * @param mixed $value index value (id $condition is a string)
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function databaseDeleteRecord($table, $condition, $value = NULL) {
    if ($this->databaseConnect()) {
      if (!isset($condition)) {
        return $this->databaseObject->deleteRecord($this, $table, NULL);
      } elseif (is_string($condition) && isset($value)) {
        return $this->databaseObject->deleteRecord($this, $table, array((string)$condition=>$value));
      } elseif (is_array($condition)) {
        return $this->databaseObject->deleteRecord($this, $table, $condition);
      }
    }
    return FALSE;
  }

  /**
  * Interface function: delete ALL data in this table
  *
  * @param string $table table
  * @return mixed deleted record count or FALSE
  */
  function databaseEmptyTable($table) {
    return $this->databaseDeleteRecord($table, NULL);
  }

  /**
  * Interface function: delete all table names
  *
  * @access public
  * @return array
  */
  function databaseQueryTableNames() {
    if ($this->databaseConnect()) {
      return $this->databaseObject->queryTableNames($this);
    }
    return array();
  }

  /**
  * Interface function: table structur returned as array
  *
  * @param string $tableName tablename
  * @param string $tablePrefix Prefix
  * @access public
  * @return array
  */
  function databaseQueryTableStructure($tableName, $tablePrefix = '') {
    if ($this->databaseConnect()) {
      return $this->databaseObject->queryTableStructure($this, $tableName, $tablePrefix);
    }
    return array();
  }

  /**
  * Interface function: create given table
  *
  * @param string &$tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function databaseCreateTable(&$tableData, $tablePrefix) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->createTable($this, $tableData, $tablePrefix);
    }
    return FALSE;
  }

  /**
  * Interface function: insert field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function databaseAddField($table, &$fieldData) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->addField($this, $table, $fieldData);
    }
    return FALSE;
  }

  /**
  * Interface function: change field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function databaseChangeField($table, &$fieldData) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->changeField($this, $table, $fieldData);
    }
    return FALSE;
  }

  /**
  * Interface function: delete field
  *
  * @param string $table
  * @param string $field
  * @access public
  * @return boolean
  */
  function databaseDropField($table, $field) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->dropField($this, $table, $field);
    }
    return FALSE;
  }

  /**
  * Interface function: insert index
  *
  * @param string $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function databaseAddIndex($table, &$index) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->addIndex($this, $table, $index);
    }
    return FALSE;
  }

  /**
  * Interface function: change index
  *
  * @param string $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function databaseChangeIndex($table, &$index) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->changeIndex($this, $table, $index);
    }
    return FALSE;
  }

  /**
  * Interface function: delete index
  *
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function databaseDropIndex($table, $name) {
    if ($this->databaseConnect()) {
      return $this->databaseObject->dropIndex($this, $table, $name);
    }
    return FALSE;
  }

  /**
  * DBMS spezific SQL source
  *
  * databaseGetSQLSource(function, param1, escaping, param2, escaping, ...)
  *
  * @param $function sql function
  * @access public
  * @return mixed sql string or FALSE
  */
  function databaseGetSQLSource($function) {
    $params = func_get_args();
    if (isset($params) && is_array($params) && count($params) >= 1) {
      if ($this->databaseConnect()) {
        array_shift($params);
        return $this->databaseObject->getSQLSource($function, $params);
      }
    }
    return FALSE;
  }
  
  /**
  * get the sql condition string
  *
  * @param mixed $filter fieldname or condition array
  * @param mixed $value optional value if $filter is a fieldname
  * @access public
  * @return string
  */
  function databaseGetSQLCondition($filter, $value = NULL) {
    if (isset($filter) && $this->databaseConnect()) {
      if (is_string($filter) && isset($value)) {
        return $this->databaseObject->getSQLCondition(array($filter=>$value));
      } else {
        return $this->databaseObject->getSQLCondition($filter);
      }
    }
    return FALSE;
  }

  /**
  * Compare new values with current values (from db)
  *
  * @param array &$newValues
  * @param array &$dbValues
  * @access public
  * @return boolean
  */
  function checkDataModified(&$newValues, &$dbValues) {
    if (isset($newValues) && is_array($newValues)) {
      foreach($newValues as $key=>$val) {
        if (isset($dbValues[$key]) && $dbValues[$key] != $newValues[$key]) {
          return TRUE;
        } elseif (!isset($dbValues[$key])) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }
}
?>
