<?php
/**
* Implementation super class
*
* Provides basic functionality for all other classes
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_base_object.php,v 1.52 2006/05/24 14:50:55 weinert Exp $
*/

/**
* log type for user messages (login/logout)
*/
define('PAPAYA_LOGTYPE_USER', 1);
/**
* log type for page messages (published)
*/
define('PAPAYA_LOGTYPE_PAGES', 2);
/**
* log type for database messages (errors)
*/
define('PAPAYA_LOGTYPE_DATABASE', 3);
/**
* log type for calendar messages
*/
define('PAPAYA_LOGTYPE_CALENDAR', 4);
/**
* log type for cronjob messages
*/
define('PAPAYA_LOGTYPE_CRONJOBS', 5);
/**
* log type for surfer/community messages
*/
define('PAPAYA_LOGTYPE_SURFER', 6);
/**
* log type for system messages
*/
define('PAPAYA_LOGTYPE_SYSTEM', 7);
/**
* log type for system messages
*/
define('PAPAYA_LOGTYPE_MODULES', 8);

/**
* line break string constant
*/
define('LF', "\n");

/**
* Include string functions class library
*/
require_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');

/**
* Implementation super class
*
* Provides basic functionality for all other classes
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_object {
  /**
  * Error message
  * @var object base_errors $msgs;
  */
  var $msgs = NULL;

  var $params = array();
  var $paramName = 'p';
  var $requestMethod = 'get';
  var $sessionParamName = NULL;

  function __construct() {
    //empty implementation for all child classes
  }

  /**
  * PHP 4 constuctor redirect
  *
  * @access public
  */
  function base_object() {
    $args = func_get_args();
    call_user_func_array(array(&$this, '__construct'), $args);
  }

  function getSessionValue($name) {
    if (isset($GLOBALS['PAPAYA_SESSION'])) {
      return $GLOBALS['PAPAYA_SESSION']->getValue($name);
    }
    return NULL;
  }

  function setSessionValue($name, $value) {
    if (isset($GLOBALS['PAPAYA_SESSION'])) {
      $GLOBALS['PAPAYA_SESSION']->setValue($name, $value);
    }
  }

  /**
  * Phrasetranslator - Fetch translation for all phrase
  *
  * @param string $phrase Phrase
  * @param mixed $module Modul
  * @return string
  */
  function _gt($phrase, $module = NULL) {
    if (@is_object($GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]) && trim($phrase) != '') {
      return $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]->getText($phrase, $module);
    }
    return $phrase;
  }

  /**
  * Phrasetranslator - Fetch translation of one phrase and insert variable
  *
  * @param string $phrase Phrase
  * @param array $vals Parameter
  * @param mixed $module Modul
  * @return string
  */
  function _gtf($phrase, $vals, $module = NULL) {
    if (@is_object($GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]) && trim($phrase) != '') {
      return $GLOBALS[PAPAYA_GLOBAL_PHRASEOBJECT]->getTextFmt($phrase, $vals, $module);
    }
    return $phrase;
  }

  /**
  * Phrases - Locate files
  *
  * @param string $fileName filename
  * @return string file content
  */
  function _gtfile($fileName) {
    if (isset($this->authUser)) {
      $lng = $this->authUser->options['PAPAYA_UI_LANGUAGE'];
    } else {
      $lng = PAPAYA_UI_LANGUAGE;
    }
    $fileName = $_SERVER['DOCUMENT_ROOT'].PAPAYA_PATH_WEB.PAPAYA_PATH_ADMIN.'/data/'.PAPAYA_UI_LANGUAGE.'/'.$fileName;
    $fileName = strtr($fileName, '//', '/');
    if ($fileName) {
      if ($fh = @fopen($fileName, 'r')) {
        $data = fread($fh, filesize($fileName));
        fclose($fh);
        return $data;
      }
    }
    return '';
  }


  /**
  * Log events
  *
  * @param integer $level message priority
  * @param integer $type message typ (Gruppe)
  * @param string $short message short (fr Liste)
  * @param string $long message detailed
  */
  function logMsg($level,$type,$short,$long = '') {
    if (function_exists('is_a') && is_a($this, 'base_log')) {
      return 0;
    }
      if (defined('PAPAYA_GLOBAL_LOGOBJECT') && (trim(PAPAYA_GLOBAL_LOGOBJECT) != '')) {
        if (isset($GLOBALS[PAPAYA_GLOBAL_LOGOBJECT]) && is_object($GLOBALS[PAPAYA_GLOBAL_LOGOBJECT])) {
          $GLOBALS[PAPAYA_GLOBAL_LOGOBJECT]->logMessage($level, $type, $short, $long);
      }
    }
  }

  /**
  * Adds a message in the message object.
  * All messages will be displayed in programm run
  *
  * @param integer $level message priority
  * @param string $text message
  */
  function addMsg($level,$text, $log = FALSE, $type = PAPAYA_LOGTYPE_SYSTEM) {
    if (is_object($this->msgs)) {
      $this->msgs->add($level,$text);
    }
    if ($log) {
      $this->logMsg($level, $type, $text);
    }
  }

  /**
  * Parameter initialisation
  *
  * All global parameters for the object will be read out of the global
  * namespace and put in $this->params. The current will script filename
  * is stored in the $this->baseLink.
  * If here is a parameter $sessionParamName or a property $this->sessionParamName,
  * the value will be registered in the session object.
  *
  * @param mixed $sessionParamName optional, default value NULL
  * @access public
  */
  function initializeParams($sessionParamName = NULL) {
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');
    if(isset($_REQUEST[$this->paramName])) {
      if (get_magic_quotes_gpc()) {
        $this->params = $this->stripSlashes($_REQUEST[$this->paramName]);
      } else {
        $this->params = $this->ensureParamsUTF8($_REQUEST[$this->paramName]);
      }
    } else {
      $this->params = array();
    }
    $this->baseLink = $this->getBaseLink();
    if ($_SERVER['REQUEST_METHOD'] == 'post') {
      $this->requestMethod = 'post';
    }
    if (isset($sessionParamName) && trim($sessionParamName) != '') {
      $this->sessionParamName = $sessionParamName;
    }
  }

  function ensureParamsUTF8($var) {
    if (isset($var)) {
      if (is_array($var)) {
        foreach($var as $key=>$value) {
          $result[$key] = $this->ensureParamsUTF8($value);
        }
        return $result;
      } else {
        return papaya_strings::decodeInputData($var);
      }
    }
    return NULL;
  }

  function stripSlashes($var) {
    if (isset($var)) {
      if (is_array($var)) {
        foreach($var as $key=>$value) {
          $result[$key] = $this->stripSlashes($value);
        }
        return $result;
      } else {
        return $this->ensureParamsUTF8(stripslashes($var));
      }
    }
    return NULL;
  }

  /**
  * Initialisation of session-parameter
  *
  * If the parameter this->params[name] is set all variables within $resetParams get reset.
  * Otherwise if $this->sessionParams[$name] is set the parameter will replace
  * this->params[$name]. If there is no parameter set with name $name, this->params set to NULL
  *
  * @param string $name name of the parameter
  * @param array $resetParams array parameter
  */
  function initializeSessionParam($name, $resetParams = NULL) {
    if (isset($this->params[$name])) {
      if (isset($resetParams) && is_array($resetParams) &&
          ((!isset($this->sessionParams[$name])) || ($this->params[$name] != $this->sessionParams[$name]))
          ) {
        foreach($resetParams as $paramName) {
          if (isset($this->params[$paramName])) unset($this->params[$paramName]);
          if (isset($this->sessionParams[$paramName])) unset($this->sessionParams[$paramName]);
        }
      }
      $this->sessionParams[$name] = $this->params[$name];
    } elseif (isset($this->sessionParams[$name])) {
      $this->params[$name] = $this->sessionParams[$name];
    }  else {
      $this->params[$name] = NULL;
    }
  }

  /**
  * Get base link
  *
  * @param integer $pageId
  * @param integer $categId
  * @return string $baseLink URL
  */
  function getBaseLink($pageId = 0, $categId = 0) {
    $data = $this->parseRequestURI();
    if (isset($data['output']) && ($data['output'] == 'media' || $data['output'] == 'image')) {
      return $data['filename'];
    } elseif ($pageId > 0) {
      $pId = (int)$pageId;
    } elseif ($data['page_id'] > 0) {
      $pId = (int)$data['page_id'];
    } elseif (isset($_GET['p_id']) && $_GET['p_id'] > 0) {
      $pId = (int)$_GET['p_id'];
    } else {
      $pId = 0;
    }
    if ($categId > 0) {
      $cId = $categId;
    } elseif (isset($data['categ_id']) && $data['categ_id'] > 0) {
      $cId = (int)$data['categ_id'];
    } else {
      $cId = 0;
    }

    $baseLink = $data['filename'];
    if ($pId > 0 && $cId > 0) {
      $baseLink .= '.'.$cId;
    }
    if ($pId > 0) {
      $baseLink .= '.'.$pId;
    }
    if (isset($data['language']) && $data['language'] != '') {
      $baseLink .= '.'.$data['language'];
    }
    $baseLink .= '.'.$data['ext'];

    if ($data['preview']) {
      if ($data['datetime']) {
        $baseLink .= '.'.(int)$data['datetime'];
      }
      $baseLink .= '.preview';
    }
    return $baseLink;
  }

  /**
  * get base path to current script
  *
  * @param boolean $withDocumentRoot optional, default value FALSE
  * @access public
  * @return string
  */
  function getBasePath($withDocumentRoot = FALSE) {
    $path = dirname($_SERVER['SCRIPT_FILENAME']);
    if ($withDocumentRoot) {
      $result = $path;
    } else {
      if (preg_match('~^\w:~', $_SERVER['DOCUMENT_ROOT']) &&
          (!preg_match('~^\w:~', $_SERVER['SCRIPT_FILENAME']))) {
        $result = substr($path, strlen($_SERVER['DOCUMENT_ROOT'])-2);
      } else {
        $result = substr($path, strlen($_SERVER['DOCUMENT_ROOT']));
      }
    }
    $result = str_replace( '\\', '/', $result);
    if (substr($result, 0, 1) != '/' && substr($result, 1, 1) != ':') {
      $result = '/'.$result;
    }
    if (substr($result, -1) != '/') {
      $result .= '/';
    }
    $result = strtr($result, array('////'=> '/', '///'=> '/', '//'=> '/'));

    return $result;
  }

  /**
  * Parse request URI to filter data out of it
  *
  * @return array
  */
  function parseRequestURI() {
    if (defined('PAPAYA_PATH_WEB')) {
      $webPathQuoted = preg_quote(PAPAYA_PATH_WEB);
    } else {
      $webPathQuoted = '/';
    }
    $pSid = '[a-zA-Z\d,-]{32}|[a-zA-Z\d,-]{40}';
    $pImage = '~/?(sid([a-z]*?('.$pSid.')))?('.$webPathQuoted.')([a-z\d_-]+)\.(image)(\.(jpg))~i';
    $pMedia = '~^/?(sid([a-z]*?('.$pSid.')))?
      ('.$webPathQuoted.')
      ([a-z\d_-]+\.(media|thumb|download|popup)(\.(preview))?
      (\.(([a-z\d_]+)(\.([a-z]+)))))~ix';
    $pPage = '~^/?(sid([a-z]*?('.$pSid.')))? #session id
      ('.$webPathQuoted.') # path
      (([a-z_-\d]+) #file title
        ((\.([\d]+))?\.([\d]+))? # category and page id
        ((\.([a-z]+))?\.(([a-oq-z]+|p[a-qs-z]|pr[a-df-z]|pre[a-uw-z]|prev[a-hj-z]|previ[a-df-z]|previe[a-vx-z])[a-z]*))
        ((\.([\d]+))?\.(preview))?)~ix';

    if (preg_match($pImage, $_SERVER['REQUEST_URI'], $regs)) {
      $result = array(
        'sid' => $regs[2],
        'path' => $regs[5],
        'filename' => '',
        'ident' => $regs[5],
        'language' => '',
        'ext' => 'image',
        'output' => 'image',
        'preview' => FALSE,
      );
    } elseif (preg_match($pMedia, $_SERVER['REQUEST_URI'], $regs)) {
      $mediaTypes = array('media', 'thumb', 'download', 'popup');
      $result = array(
        'sid' => $regs[2],
        'path' => $regs[4],
        'filename' => $regs[5],
        'media_id' => $regs[10],
        'categ_id' => 0,
        'page_id' => 0,
        'language' => '',
        'ext' => (in_array($regs[6], $mediaTypes) ? $regs[6] : 'media'),
        'preview' => ((@$regs[8] == 'preview') ? TRUE : FALSE),
        'datetime' => 0,
        'output' => 'media'
      );
    } elseif (preg_match($pPage, $_SERVER['REQUEST_URI'], $regs)) {
      $result = array(
        'sid' => $regs[2],
        'path' => $regs[4],
        'filename' => $regs[6],
        'categ_id' => (int)$regs[9],
        'page_id' => (int)$regs[10],
        'language' => ($regs[13] != '') ? $regs[13] : '',
        'ext' => $regs[14],
        'preview' => ((@$regs[19] == 'preview') ? TRUE : FALSE),
        'datetime' => ((@$regs[18] > 0) ? (int)$regs[18] : 0),
        'output' => 'page'
      );
      if ($pos = strpos($_SERVER['REQUEST_URI'], 'catalog=')) {
        $result['categ_id'] = (int)substr($_SERVER['REQUEST_URI'], $pos+strlen('catalog='));
      }
    } else {
      $fileName = 'index';
      $ext = 'php';
      if (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) {
        if (preg_match('~([a-z_\.-]+)\.([a-z_-]+)(\?|$)~', $_SERVER['REQUEST_URI'], $regs)) {
          $fileName = $regs[1];
          $ext = $regs[2];
        } elseif (preg_match('~([a-z_\.-]+)\.([a-z_-]+)(\?|$)~', $_SERVER['SCRIPT_FILENAME'], $regs)) {
          $fileName = $regs[1];
          $ext = $regs[2];
        }
      } elseif (defined('PAPAYA_URL_EXTENSION') && trim(PAPAYA_URL_EXTENSION) != '') {
        $ext = PAPAYA_URL_EXTENSION;
      }
      $result = array(
        'sid' => NULL,
        'path' => '',
        'filename' => $fileName,
        'page_id' => 0,
        'categ_id' => 0,
        'language' => '',
        'ext' => $ext,
        'preview' => FALSE,
        'datetime' => 0,
        'output' => 'page'
      );
    }
    return $result;
  }

  /**
  * Get web link
  *
  * @param mixed $pid optional, default value NULL
  * @param string $mode optional, default value 'page'
  * @param mixed $params optional, default value NULL
  * @param mixed $paramName optional, default value NULL
  * @param boolean $sid optional, default value TRUE
  * @access public
  * @return string Weblink
  */
  function getWebLink($pageId = NULL, $lng = NULL, $mode = NULL, $params = NULL, $paramName = NULL, $text = '', $categId = NULL) {
    if (!isset($pageId)) {
      if (isset($GLOBALS['PAPAYA_PAGE']) && isset($GLOBALS['PAPAYA_PAGE']->requestData['page_id'])) {
        $pageId = (int)$GLOBALS['PAPAYA_PAGE']->requestData['page_id'];
      } else {
        $pageId = 0;
      }
    }
    if (!isset($categId)) {
      if (isset($GLOBALS['PAPAYA_PAGE']) && isset($GLOBALS['PAPAYA_PAGE']->requestData['categ_id'])) {
        $categId = (int)$GLOBALS['PAPAYA_PAGE']->requestData['categ_id'];
      } else {
        $categId = 0;
      }
    }

    if ((isset($GLOBALS['PAPAYA_PAGE']) && $GLOBALS['PAPAYA_PAGE']->public == FALSE) || $mode == 'preview' || $mode == 'xmlpreview') {
      if (isset($GLOBALS['PAPAYA_PAGE']) && $GLOBALS['PAPAYA_PAGE']->versionDateTime > 0) {
        $preview = '.'.((int)$GLOBALS['PAPAYA_PAGE']->versionDateTime).'.preview';
      } else {
        $preview = '.preview';
      }
    } else {
      $preview = '';
    }

    $subCateg = ($categId > 0) ? '.'.(int)$categId : '';
    $subPage = ($pageId > 0) ? '.'.(int)$pageId : '';

    if (isset($lng) && trim($lng) != '') {
      $lngString = '.'.$lng;
    } elseif (isset($_REQUEST['language']) && preg_match('/^[a-z]{1,3}$/', $_REQUEST['language'])) {
      $lngString = '.'.$_REQUEST['language'];
    } else {
      $lngString = '';
    }


    $pageModes = array(
      'page' => '.'.PAPAYA_URL_EXTENSION,
      'preview' => '.'.PAPAYA_URL_EXTENSION,
      'xmlpreview' => '.xml'
    );

    if (isset($mode) && isset($pageModes[$mode])) {
      $ext = $pageModes[$mode];
    } elseif (isset($mode) && preg_match('~^[a-z]+$~', $mode)) {
      $ext = '.'.$mode;
    } elseif (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) {
      $ext = '.php';
    } elseif (defined('PAPAYA_URL_EXTENSION') && trim(PAPAYA_URL_EXTENSION) != '') {
      $ext = '.'.PAPAYA_URL_EXTENSION;
    } else {
      $ext = '.php';
    }
    $path = (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) ? '../' : '';
    $fileName = $this->escapeForFilename($text, 'index').$subCateg.$subPage.$lngString.$ext.$preview;
    return $fileName.$this->encodeQueryString($params, $paramName);
  }

  /**
  * Encode query string
  *
  * @param array &$params array of params
  * @param string $paramName optional param name
  * @param integer $maxDepth optional default 5
  * @access public
  * @return string
  */
  function encodeQueryString($params, $paramName = NULL, $maxDepth = 5) {
    if (isset($params) && is_array($params)) {
      $ignore = array('p_id', 'p_pagemode', 'preview');
      foreach($ignore as $key) {
        if (isset($params[$key])) unset($params[$key]);
      }
      $ignoreEmpty = array('preview_date');
      foreach($ignoreEmpty as $key) {
        if (isset($params[$key]) && $params[$key] == '') {
          unset($params[$key]);
        }
      }
      $queryString = $this->encodeQueryStringRec($params, $paramName, NULL, $maxDepth);
      if (strlen($queryString) > 0) {
        return '?'.substr($queryString, 1);
      }
    }
    return '';
  }

  /**
  * Encode/Create the QueryString params (rekursive function)
  *
  * @param array &$params array of params
  * @param mixed $paramName base param name
  * @param mixed $prefix paramname prefix
  * @param integer $maxDepth depth levels left
  * @access private
  * @return string
  */
  function encodeQueryStringRec(&$params, $paramName, $prefix, $maxDepth) {
    if ($maxDepth < 1) return '';
    if (isset($params) && is_array($params) && count($params) > 0) {
      $result = '';
      if (isset($paramName)) {
        if (isset($prefix)) {
          $namebase = $prefix.'['.urlencode($paramName).']';
        } else {
          $namebase = urlencode($paramName);
        }
        foreach ($params as $name=>$value) {
          if (is_array($value)) {
            $result .= $this->encodeQueryStringRec($value, $name, $namebase, $maxDepth - 1);
          } else {
            $result .= '&'.$namebase.'['.urlencode($name).']='.urlencode($value);
          }
        }
      } else {
        foreach ($params as $name=>$value) {
          if (is_array($value)) {
            $result .= $this->encodeQueryStringRec($value, $name, NULL, $maxDepth - 1);
          } else {
            $result .= '&'.urlencode($name).'='.urlencode($value);
          }
        }
      }
      return $result;
    }
    return '';
  }

  /**
  * Recode query string
  *
  * @param string $queryString
  * @access public
  * @return string
  */
  function recodeQueryString($queryString) {
    $queryParams = array();
    if (preg_match_all('~(^|[?&])(([^=&?]+)=([^&]+))|([^&]+)~', $queryString, $matches, PREG_SET_ORDER)) {
      foreach($matches as $match) {
        if (isset($match[3]) && $match[3] != '') {
          $queryParams[$match[3].'='] = $match[4];
        } elseif (isset($match[5])) {
          $queryParams[$match[5]] = TRUE;
        }
      }
    }
    $result = '';
    if (isset($queryParams) && is_array($queryParams) && count($queryParams) > 0) {
      foreach($queryParams as $paramName => $paramValue) {
        if(substr($paramName, -1) == '=') {
          $result .= '&'.urlencode(substr($paramName, 0, -1)).'='.urlencode($paramValue);
        } else {
          $result .= '&'.urlencode($paramName);
        }
      }
      return '?'.substr($result, 1);
    }
    return $result;
  }

  /**
  * Escape chars in a string to use it in a filename
  *
  * @param string $str
  * @param string $default returned if str is empty
  * @access public
  * @return string
  */
  function escapeForFilename($str, $default = 'index') {
    if (defined('PAPAYA_URL_NAMELENGTH') && PAPAYA_URL_NAMELENGTH > 0) {
      $str = papaya_strings::normalizeString($str, PAPAYA_URL_NAMELENGTH);
    } else {
      $str = papaya_strings::normalizeString($str, 50);    
    }
    if ($str != '') {
      return strtolower($str);
    } else {
      return $default;
    }
  }

  /**
  * Get web media link
  *
  * @param mixed $mid
  * @param string $mode optional, default value 'media'
  * @access public
  * @return mixed
  */
  function getWebMediaLink($mid, $mode = 'media', $text = '') {
    $pageModes = array(
      'media'    => '.media',
      'download' => '.download',
      'popup' => '.popup',
      'thumb'    => '.thumb'
    );
    $ext = (isset($pageModes[$mode])) ? $pageModes[$mode]: $pageModes['media'];
    $public = (isset($GLOBALS['PAPAYA_PAGE']) && $GLOBALS['PAPAYA_PAGE']->public) ? '' : '.preview';
    $text = preg_replace('#\.[a-z]{1,4}$#', '', $text);
    $path = (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) ? '../' : '';
    $result = $path.$this->escapeForFilename($text, 'index').$ext.$public.'.'.$mid;
    return $result;
  }

  /**
  * Return absolute URL
  *
  * @param string $url
  * @param string $text optional, default value ''
  * @param boolean $sid optional, default value TRUE
  * @access public
  * @return string URL
  */
  function getAbsoluteURL($url, $text = '', $sid = TRUE) {
    $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
    $baseHref = $protocol."://".$_SERVER['HTTP_HOST'];

    if (isset($GLOBALS['PAPAYA_SESSION']) && $GLOBALS['PAPAYA_SESSION']->checkSIDPath() && $sid) {
      $sidStr = '/'.$GLOBALS['PAPAYA_SESSION']->sessionName.$GLOBALS['PAPAYA_SESSION']->sessionId;
    } else {
      $sidStr = '';
    }

    $pageId = (int)$url;

    include_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');
    if ($url == '/') {
      $href = $baseHref.PAPAYA_PATH_WEB;
    } elseif (preg_match("#^(\d+)\.(\d+)$#", trim($url), $regs)) {
      $href = $baseHref.$sidStr.PAPAYA_PATH_WEB.$this->getWebLink((int)$regs[2], NULL, NULL, NULL, NULL, $text, (int)$regs[1]);
    } elseif ($pageId > 0 && preg_match("#^\d+$#", trim($url))) {
      $href = $baseHref.$sidStr.PAPAYA_PATH_WEB.$this->getWebLink($pageId, NULL, NULL, NULL, NULL, $text);
    } elseif (checkit::isHTTPX($url)) {
      $href = $url;
    } elseif (preg_match("#^/#", $url)) {
      $href = $baseHref.$sidStr.$url;
    } else {
      $iUrl = preg_replace('#[^/]+$#','',$_SERVER['REQUEST_URI']);
      $href = $baseHref.$iUrl.$url;
    }
    return $href;
  }

  /**
  * Get link
  *
  * @param mixed $params optional, default value NULL (no Query String)
  * @param mixed $paramName optional, default value NULL ($this->paramName)
  * @param string $fileName
  * @param integer $pageId
  * @access public
  * @return string
  */
  function getLink($params = NULL, $paramName = NULL, $fileName = '', $pageId = NULL) {
    if (isset($fileName) && trim($fileName) != '') {
      $link = $fileName;
    } else {
      $link = $this->getBaseLink(0);
    }
    if ((!isset($paramName)) && isset($this->paramName)) {
      $queryString = $this->encodeQueryString($params, $this->paramName);
    } else {
      $queryString = $this->encodeQueryString($params, $paramName);
    }
    if (isset($pageId) && $pageId > 0) {
      if (trim($queryString) != '')  {
        return $link.$queryString.'&p_id='.(int)$pageId;
      } else {
        return $link.'?p_id='.(int)$pageId;
      }
    } else {
      return $link.$queryString;
    }
  }

  /**
  * Create plugin object
  *
  * @param string $fileName plugin filename
  * @param string $class plugin classname
  * @param string $guid Unique Identifier of the object class
  * @access public
  * @return mixed object or NULL
  */
  function &createPluginObject($fileName, $className, $guid, &$parent, $data = NULL) {
    $obj = NULL;
    if ($this->includePluginFile($fileName, $className)) {
      if (is_object($obj = &new $className($parent))) {
        $obj->guid = $guid;
        if (isset($this->msgs)) {
          $obj->msgs = &$this->msgs;
        }
        if (isset($data)) {
          $obj->setData($data);
        }
        return $obj;
      } else {
        $obj = NULL;
      }
    }
    return $obj;
  }

  /**
  * Include plugin file
  *
  * @param string $fileName
  * @param string $className
  * @access public
  * @return boolean
  */
  function includePluginFile($fileName, $className) {
    if (trim($fileName) != '' && trim($className) != '' && include_once(PAPAYA_MODULES_PATH.$fileName)) {
      if (class_exists($className)) {
        return TRUE;
      } else {
        $this->addMsg(MSG_ERROR, sprintf($this->_gt('Can not initialize module class "%s"'), $className), TRUE);
      }
    } else {
      $this->addMsg(MSG_ERROR, sprintf($this->_gt('Can not initialize module file "%s"'), $fileName).' ('.$className.')', TRUE);
    }
    return FALSE;
  }

  /**
  * Show the debug of the variable &$var
  *
  * @param mixed $var auszugebende Variable
  */
  function debug() {
    static $lastMemoryUsage = 0;
    static $lastTime = 0;
    echo '<pre style="position: relative; z-index: 1000; color: red; background-color: white; border: 1px solid black; padding: 10px; font-size: 12px;">';
    $vars = func_get_args();
    if (isset($vars) && is_array($vars) && count($vars) > 0) {
      foreach($vars as $var) {
        var_dump($var);
        echo "\n";
      }
    }
    $bt = debug_backtrace();
    if (isset($bt) && is_array($bt) && count($bt) > 0) {
      echo '<span style="color: black;">';
      foreach($bt as $btItem) {
        echo '<b>';
        if (isset($btItem['class'])) {
          echo $btItem['class'].'::';
        }
        echo @$btItem['function'].'  ';
        echo '</b>';
        echo @$btItem['file'].':'.@$btItem['line'].LF;
      }
      echo '</span>';
    }
    if (function_exists('memory_get_usage')) {
      $memoryNeeded = memory_get_usage();
      echo LF.'<span style="color: black;">';
      echo '<b>Memory Usage:</b> '.number_format($memoryNeeded, 0, '.', ',').' Byte';
      if ($lastMemoryUsage > 0) {
        echo LF.'<b>Memory Diff:</b> ';
        if ($memoryNeeded >= $lastMemoryUsage) {
          echo '+', number_format($memoryNeeded - $lastMemoryUsage, 0, '.', ','), ' Byte';
        } else {
          echo '-', number_format($lastMemoryUsage - $memoryNeeded, 0, '.', ','), ' Byte';
        }
      }
      $lastMemoryUsage = $memoryNeeded;
      echo '</span>';
    }
    $timer = explode(' ',microtime());
    $timer = $timer[0]+(double)$timer[1];
    if ($lastTime > 0) {
      echo LF.'<span style="color: black;">';
      echo '<b>Time Diff:</b> '.number_format($timer - $lastTime, 3, '.', ',').' s';
      echo '</span>';
    }
    $lastTime = $timer;
    echo '</pre>';
  }
}
?>