<?php
/**
* Implementation phrasen management
*
* Provides functionality for translation
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_phrases.php,v 1.1.1.1 2005/09/30 15:51:08 weinert Exp $
*/

/**
* Include database object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');


/**
* Manage phrase class
*
* Provides functionality for translation
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_phrases extends base_db {
  /**
  * Phrase table
  * @var string $tablePhrase
  */
  var $tablePhrase = PAPAYA_DB_TBL_PHRASE;

  /**
  * Translation table
  * @var string $tableTranslation
  */
  var $tableTranslation = PAPAYA_DB_TBL_PHRASE_TRANS;

  /**
  * Module phrases
  * @var string $tableModule
  */
  var $tableModule = PAPAYA_DB_TBL_PHRASE_MODULE;

  /**
  * Relations between phrases - moduels
  * @var string $tableModuleRelation
  */
  var $tableModuleRelation = PAPAYA_DB_TBL_PHRASE_MODULE_REL;

  /**
  * Log messages - table
  * @var string $tableLog
  */
  var $tableLog = PAPAYA_DB_TBL_PHRASE_LOG;
  /**
  * Language - table
  * @var string $tableLanguage
  */
  var $tableLanguage = PAPAYA_DB_TBL_LNG;

  /**
  * Current language
  * @var string $lng
  */
  var $lng = NULL;

  /**
  * Modules
  * @var array $modules
  */
  var $modules = NULL;

  /**
  * Default module
  * @var string $defaultModule
  */
  var $defaultModule = NULL;

  /**
  * Load translation for module
  *
  * @param string $title title module
  * @param mixed $module module
  * @return mixed flase for no translation otherwise translation will be returned
  */
  function loadModule($title) {
    if (isset($this->modules[$title]['phrases']))
    {
      unset($this->modules[$title]['phrases']);
    }
    $sql = "SELECT p.phrase_text_lower, pt.translation
            FROM %s AS m
              LEFT JOIN %s AS m_rel ON m_rel.module_id = m.module_id
              LEFT JOIN %s AS p ON p.phrase_id = m_rel.phrase_id
              LEFT JOIN %s AS pt ON (pt.phrase_id = p.phrase_id AND pt.lng_id = '%d')
            WHERE m.module_title_lower = '%s'";
    $params = array($this->tableModule, $this->tableModuleRelation, $this->tablePhrase, $this->tableTranslation,
      $this->lng['lng_id'], $title);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow()) {
        $this->modules[$title]['phrases'][$row[0]] = $row[1];
      }
      return (isset($this->modules[$title]['phrases']) && is_array($this->modules[$title]['phrases']));
    }
    return FALSE;
  }

  /**
  * Load translation
  *
  * @param string $phrase Phrase
  * @return mixed
  */
  function loadTranslation($phrase) {
    $sql = "SELECT p.phrase_id, pt.translation
            FROM %s AS p
              LEFT OUTER JOIN %s AS pt ON
                (pt.phrase_id = p.phrase_id AND pt.lng_id = %d)
            WHERE p.phrase_text_lower = '%s'";
    $params = array($this->tablePhrase, $this->tableTranslation, $this->lng['lng_id'], strToLower($phrase));
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return array($row[0], $row[1]);
      }
    }
    return FALSE;
  }

  /**
  * Return PhraseID
  *
  * @param string $phrase Phrase
  * @return mixed PhraseID or FALSE
  */
  function getPhraseId($phrase) {
    $sql = "SELECT p.phrase_id
              FROM %s AS p
            WHERE p.phrase_text_lower = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tablePhrase, $phrase))) {
      if ($row = $res->fetchRow()) {
        return $row[0];
      }
    }
    return FALSE;
  }

  /**
  * Return ModulID
  *
  * @param mixed $module Modul
  * @return mixed ModuleID oder FALSE
  */
  function getModuleId($module) {
    $lModule = strtolower($module);
    if (!isset($this->modules[$lModule]['id'])) {
      $sql = "SELECT m.module_id
                FROM %s AS m
              WHERE m.module_title_lower = '%s'";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableModule, $module))) {
        if ($row = $res->fetchRow()) {
          $this->modules[$lModule]['id'] = $row[0];
          return $row[0];
        }
      }
    } else {
      return $this->modules[$lModule]['id'];
    }
    return FALSE;
  }

  /**
  * Get language id from database
  *
  * @param integer $lng language (de_DE, en_US, ...)
  * @return mixed ID or FALSE
  */
  function getLngId($lng) {
    $sql = "SELECT lng_id, lng_short, lng_title
            FROM %s
            WHERE lng_short = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableLanguage, $lng))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->lng = $row;
        return $row['lng_id'];
      }
    }
    return FALSE;
  }

  /**
  * Write a new Module in the language database
  *
  * @param mixed $module Modul
  * @return mixed record number or FALSE
  */
  function addModule($module) {
    $data = array(
      'module_title' => $module,
      'module_title_lower' => strToLower($module)
    );
    return $this->databaseInsertRecord($this->tableModule, 'module_id', $data);
  }

  /**
  * Create new connection between tables
  *
  * @param mixed $module Modul
  * @param integer $phraseId Phrase ID
  * @return mixed record number  or FALSE
  */
  function addRelation($module, $phraseId) {
    if ($moduleId = $this->getModuleId($module)) {
      $sql = "SELECT COUNT(*)
              FROM %s
              WHERE phrase_id = %d
                AND module_id = %d";
      $params = array($this->tableModuleRelation, $phraseId, $moduleId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        if ($row = $res->fetchRow()) {
          if ($row[0] === '0') {
            $data = array('phrase_id'=>(int)$phraseId, 'module_id'=>(int)$moduleId);
            return $this->databaseInsertRecord($this->tableModuleRelation, NULL, $data);
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Writes a message in the logger
  *
  * If debugging is enabled translation will be logged
  *
  * @param string $phrase Phrase
  * @param integer $phraseId Phrase
  * @param mixed $module Modul
  * @return mixed True without log function, otherwise return $this->insertRecord
  */
  function addPhraseLogMsg($msg, $phrase = NULL, $phraseId = NULL, $module = NULL) {
    if (defined('PAPAYA_DBG_PHRASES') && PAPAYA_DBG_PHRASES === '1') {
      $data['log_msg'] = $msg;
      $data['log_datetime'] = time();
      if (isset($phrase)) $data['log_phrase'] = $phrase;
      if (isset($phraseId)) $data['log_phrase_id'] = (int)$phraseId;
      if (isset($module)) $data['log_module'] = $module;
      return $this->databaseInsertRecord($this->tableLog, 'log_id', $data);
    } else {
      return TRUE;
    }
  }

  /**
  * Return translated phrase
  *
  * @param string $phrase Phrase
  * @param mixed $module Modul
  * @return mixed translation or FALSE
  */
  function getTranslation($phrase, $module) {
    $iPhrase = strToLower($phrase);
    $iModule = strToLower(((trim($module) != '') ? $module : 'default'));
    if (!isset($this->modules[$iModule]['phrases'])) {
      //modul noch nicht geladen - jetzt laden
      if (!$this->loadModule($iModule)) {
        //modul nicht vorhanden
        if (!$this->getModuleId($iModule)) {
          $this->addModule($module);
        }
        $this->modules[$iModule]['phrases'] = array();
      }
    }
    if (isset($this->modules[$iModule]['phrases'])) {
      if (isset($this->modules[$iModule]['phrases'][$iPhrase])) {
        //bersetzung aus modul
        return $this->modules[$iModule]['phrases'][$iPhrase];
      } elseif (isset($this->modules[$iModule]['error'][$iPhrase])) {
        //schon als fehler bemerkt...
        return FALSE;
      } else {
        //bersetzung nicht im geladenen Module
        if ($translation = $this->loadTranslation(@$phrase)) {
          if (isset($translation[1])) {
            //bersetzung gefunden - mit Modul verknpfen
            $this->addRelation($iModule, $translation[0]);
            $this->modules[$iModule]['phrases'][$iPhrase] = $translation[1];
            return $translation[1];
          } else {
            //Phrase im System - Protokoll
            $this->modules[$iModule]['error'][$iPhrase] = TRUE;
            $this->addPhraseLogMsg('Translation for phrase not found', $phrase, $translation[0], $module);
          }
        } else {
          //Phrase nicht im System - Protokoll
          $this->modules[$iModule]['error'][$iPhrase] = TRUE;
          $this->addPhraseLogMsg(sprintf('Phrase not found (%s).', $phrase), $phrase, NULL, $module);
        }
      }
    }
  }

  /**
  * Get translation via function getTranslation()
  *
  * @param string $phrase Phrase
  * @param mixed $module Modul
  * @return string if translation is aviable translation will be returned . otherwise phrase will
  * be returned
  */
  function getText($phrase, $module = NULL) {
    if ($result = $this->getTranslation($phrase, (isset($module) ? $module : $this->getDefaultModule()))) {
      return $result;
    } else {
      return $phrase;
    }
  }

  /**
  * Translate a string with format attributes
  *
  * @param string $phrase Phrase
  * @param mixed $module Modul
  * @param array $vals parameter
  * @return string
  */
  function getTextFmt($phrase, $vals, $module = NULL) {
    if (!is_array($vals)) {
      $vals = array($vals);
    }
    if (!($result = $this->getTranslation($phrase, (isset($module) ? $module : $this->getDefaultModule())))) {
      $result = $phrase;
    }
    if (isset($vals) && is_array($vals)) {
      return vsprintf($result, $vals);
    }
    return $result;
  }

  /**
  * Get default module from current url
  * @access private
  * @return string
  */
  function getDefaultModule() {
    if (isset($this->defaultModule)) {
      return $this->defaultModule;
    } else {
      if (preg_match('#^(([^\?]*)/)?([^?]+)(\.\d+)(\.(php|html))(\?.*)?#i', $_SERVER['REQUEST_URI'], $regs)) {
        $result = basename($regs[3].$regs[5]);
      } elseif (preg_match('#^(([^\?]*)/)?([^?]+)(\?.*)?#', $_SERVER['REQUEST_URI'], $regs)) {
        $result = basename($regs[3]);
      } else {
        $result = basename($_SERVER['SCRIPT_FILENAME']);
      }
      $this->defaultModule = $result;
      return $result;
    }
  }
}
?>