<?php
/**
* XML handling factory class
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_simple_xmltree.php,v 1.12 2006/04/25 11:19:13 weinert Exp $
*/


/**
* XML handling factory class
*
* @package papaya_base
*/
class simple_xmltree {

  /**
  * create an XML tree object
  *
  * @access public
  * @return object xmltree_common XML tree
  */
  function &create() {
    $path = dirname(__FILE__).'/xml/';
    if (extension_loaded('xml')) {
      include_once($path.'expat.php');
      $tree = &new xmltree_expat();
      return $tree;
    } else {
      return NULL;
    }
  }

  /**
  * Transform normal string into XML-string
  *
  * Transform all none alphanumeric characters into ASCII value of character
  *
  * @param string $str string
  * @return string Der transform string
  */
  function strToXML($str) {
    $result = $str;
    $max = strlen($result)-1;
    for($i=$max;$i>=0;$i--) {
      $c = $result[$i];
      if (preg_match("/[^a-zA-Z0-9 _,-]/",$c)) {
        $result = substr_replace($result,"&#".ord($c).";",$i,1);
      }
    }
    return $result;
  }

  /**
  * Transform XML string into normal string
  * All special chars will be transformed into white spaces
  *
  * @param string $str XML-string
  * @return string Der transformed String
  */
  function XMLtoStr($str) {
    $result = preg_replace("/\&\#([0-9]{1,7});/e","chr('\\1')",$str);
    return $result;
  }

}

/**
* xml node class
*
* @package papaya_base
*/
class simple_xmlnode {

  /**
  * parent element
  * @var object xmlnode_common $parent
  */
  var $parent;

  /**
  * Contains name of active element
  * @var string $name
  */
  var $name;

  /**
  * Element attributes
  * @var array $attributes
  */
  var $attributes;

  /**
  * Children of active Element
  * @var array $children
  */
  var $children;

  /**
  * Constuctor
  */
  function __construct(&$parent, $name, $attr = NULL) {
    $this->parent = &$parent;
    $this->name = $name;
    $this->children = array();
    $this->attributes = $attr;
    if (isset($this->parent) && is_object($this->parent)) {
      $this->parent->children[count($this->parent->children)] = &$this;
    }
  }

  function simple_xmlnode(&$parent, $name, $attr = NULL) {
    $this->__construct($parent, $name, $attr);
  }

  /**
  * Copy node until given level
  *
  * copy() does an infinitely deep copy, as does copy(-1); any non-negative
  * argument tells how deep the copy should be
  *
  * @param $deep Tiefe, die angibt, bis zu welchem Level der Knoten kopiert werde soll
  * @return array $newCopy
  */
  function &copy($deep="-1"){
    //create new node and set everything but parent here
    $nodeCopy = &new simple_xmlnode(NULL, $this->name, $this->attributes);

    if ($deep != 0) {
      // walk through children, copying nodes if necessary
      $childCount = count($this->children);
      for ($idx=0; $idx < $childCount; $idx++){
        $childNode = &$this->children[$idx];
        // copy this child node if depth still sufficient
        if (is_object($childNode)) {
          $nodeCopy->children[$idx] = $childNode->copy($deep - 1);
          $nodeCopy->children[$idx]->parent = &$this;
        }
      }
    }
    return $nodeCopy;
  }

  /**
  * Insert child node
  *
  * default for inserting a child node is appending; any non-negative
  * integer passed in will put child as close to that location as possible.
  *
  * @param ref &$newChild xml node reference
  * @param string $newPlace
  */
  function insertChildNode(&$newChild , $newPlace="-1"){
    // first tell this child who its parent is
    $newChild->parent = &$this;

    $lastPlace = count($this->children) - 1;
    // if $newPlace < 0, or $newPlace > max,
    // append new child to end of list
    if ($newPlace < 0 || $newPlace > $lastPlace) {
      $this->children[$lastPlace + 1] = &$newChild;
    } else {
        // otherwise have to insert and re-jigger all other children
        // first make an opening by starting from topmost and moving each up
        for ($idx = $lastPlace; $idx >= $newPlace; $idx--){
          $this->children[$idx + 1] = &$this->children[$idx];
        }
        // then stuff new one into opening
        $this->children[$newPlace] = &$newChild;
    }
  }


  /**
  * Insert child copy
  *
  * default for inserting a child node is appending; any non-negative
  * integer passed in will put child as close to that location as possible.*
  *
  * @param ref &$newChild xml node reference
  * @param string $newPlace
  */
  function insertChildCopy(&$newChild, $newPlace="-1"){
    // first tell this child who its parent is
    // //$newChild->parent = &$this;
    $lastPlace = count($this->children) - 1;
    // if $newPlace < 0, or $newPlace > max,
    // append new child to end of list
    if ($newPlace < 0 || $newPlace > $lastPlace){
      //$this->children[ $lastPlace + 1 ] = $newChild;
      $this->children[$lastPlace + 1] = &$newChild->copy();
      $this->children[$lastPlace + 1]->parent = &$this;
    } else {
      // otherwise have to insert and re-jigger all other children
      // first make an opening by starting from topmost and moving each up
      for ($idx = $lastPlace ; $idx >= $newPlace; $idx --) {
        $this->children[$idx + 1] = &$this->children[$idx];
      }
      //then stuff new one into opening
      //$this->children[ $newPlace] = $newChild;
      $this->children[$newPlace] = &$newChild->copy();
      $this->children[$newPlace]->parent = &$this;
    }
  }

  /**
  * Insert child text
  *
  * default for inserting a child node is appending; any non-negative
  * integer passed in will put child as close to that location as possible.
  * @param $newChild
  * @param string $newPlace optional, default value "-1"
  * @access public
  */
  function insertChildText($newChild , $newPlace="-1"){
    $lastPlace = count($this->children) - 1;
    // if $newPlace < 0, or $newPlace > max,
    // append new child to end of list
    if ($newPlace < 0 || $newPlace > $lastPlace) {
      $this->children[$lastPlace + 1] = $newChild;
    } else {
      // otherwise have to insert and re-jigger all other children
      // first make an opening by starting from topmost and moving each up
      for ( $idx = $lastPlace ; $idx >= $newPlace; $idx --){
        $this->children[$idx + 1] = $this->children[ $idx ];
      }
      // then stuff new one into opening
      $this->children[$newPlace] = $newChild;
    }
  }

  /**
  * Delete a child
  *
  * kills the child at the location $killPlace (or at the nearest
  * location to it), and fills in the leftover hole
  * @param integer $killPlace
  * @access public
  */
  function deleteChild($killPlace){
    $lastPlace = count($this->children) - 1;

    if ($killPlace < 0) $killPlace = 0;
    if ($killPlace > $lastPlace) $killPlace = $lastPlace;

    // move everybody down one, starting by overwriting the
    // to-be-killed child
    for ($idx = $killPlace; $idx <= $lastPlace; $idx++){
      $this->children[$idx] = &$this->children[$idx + 1];
    }
    // finally get rid of the final one (now redundant)
    unset($this->children[$lastPlace]);
  }

  /**
  * Unset given attribute
  *
  * @param $attKey $this->attributes key which will be unseted
  * @access public
  */
  function deleteAttribute($attKey){
    unset($this->attributes[$attKey]);
  }

  /**
  * Output function
  *
  * prints so that formatted XML is visible in Page Source
  * @access public
  */
  function printEcho() {
    $outString = "";
    $this->printOut($outString);
    echo $outString;
  }

  /**
  * Output function
  *
  * prints so that formatted XML is visible in browser display
  * @access public
  */
  function printHTML() {
    $outString = "";
    // to make indentation visible in HTML, you need spacers instead of
    // regular spaces.  So indent with some recognizable string, then replace
    // it with spacers (*after* converting angle brackets!).
    $this->printOut($outString, 'xxxY');
    // convert angle brackets and &'s
    $outString = papaya_strings::escapeHTMLChars($outString);
    // new-line to break conversion
    $outString = nl2br($outString);
    // create spacers from indentation placeholders
    $outString = str_replace("xxxY",'&nbsp;&nbsp;', $outString);
    echo $outString;
  }

  /**
  * Output function
  *
  * @param string &$xmlString Referenz auf einen XML-String
  * @param string $indent Einrueckung in Leerstellen; bsp.: " ".
  */
  function printOut(&$xmlString,  $indent = ' '){
    $indentInc = ($indent[0] != ' ') ? "xxxY" : "  ";

    $tagName = $this->name;
    $xmlString .=  "\n".$indent;
    $xmlString .=  sprintf('<%s', $tagName);

    // print attributes inside tag
    if (count($this->attributes) > 0) {
      reset($this->attributes);
      while (list($attrName, $attrValue) =  each($this->attributes) ) {
        $xmlString .=  sprintf(' %s="%s"', $attrName, addslashes(papaya_strings::escapeHTMLChars($attrValue)));
      }
    }

    if (isset($this->children) && is_array($this->children) && count($this->children) > 0) {
      $xmlString .= ">" ;
      $this->printOutChildren($xmlString, $indent.$indentInc);
      $xmlString .=  "\n$indent</$tagName>";
    } else {
      $xmlString .= "/>" ;
    }
  }
  
  /**
  * print child data including tags
  *
  * @param string &$xmlString
  * @param string $indent optional, default value ' '
  * @access public
  */
  function printOutChildren(&$xmlString,  $indent = ' ') {
    if (isset($this->children) && is_array($this->children)) {
      $childCount = count($this->children);
      for ($idx=0; $idx < $childCount; $idx++) {
        //print out this child node
        if (is_object($this->children[$idx])) {
          $this->children[$idx]->printOut($xmlString, $indent);
        }
      }
    }
  }

  /**
  * output function
  *
  * @param string &$xmlString Referenz auf einen XML-String
  */
  function printText(&$xmlString){
    // print content-text only, no tags or attributes
    $childCount = count($this->children);
    for ($idx=0; $idx < $childCount; $idx++){
      $childNode = &$this->children[$idx];
        // print out this child node
      if (is_object($childNode)) {
        $childNode->printText($xmlString);
      }
    }
  }

  /**
  * Extracted elements until given depth
  *
  * extracts references to any node with the same name and
  * attribute values as the template
  *
  * @param array &$resultArray ???
  * @param string $template ???
  * @param integer $depth Die Tiefe, bis in welche die Herausziehung vollzogen werden soll
  */
  function extract(&$resultArray, &$template, $depth=0){
    $nope=FALSE;
    // name in template should be matched
    if ($template->name && $template->name != $this->name){
      $nope="bad name";
    }

    // each attributes in template should be matched
    if ($template->attributes){
      reset($template->attributes);
      $myKey = key($template->attributes);
      while ($myKey){
        if ($template->attributes[$myKey] != $this->attributes[$myKey])
          $nope = "bad attributes";
          $myKey = next($template->attributes);
        }
      }
      if (!$nope) {
        $newIdx = count($resultArray["depth"]);
        $resultArray["nodes"][$newIdx] = &$this;
        $resultArray["depth"][$newIdx] = $depth;
      }

      // after checking this node, descend into children
      $childCount = count($this->children);
      for ($idx=0; $idx < $childCount ; $idx ++){
        $childNode = &$this->children[$idx];

      // print out this child node
      if (isset($childNode) && is_object($childNode)){
        if (function_exists('is_a') && is_a($childNode, 'simple_xmlnode')) {
          $childNode->extract($resultArray, $template, $depth + 1);
        }
      }
    }
  }

  /**
  * Extract element at depth
  *
  * extracts only those nodes at a specific depths, and returns
  *
  * a flat array of those nodes, rather than a 2xN array
  * i.e. ["nodes"][N] + ["depth"][N]
  *
  * @param array &$resultArray ???
  * @param string $template ???
  * @param integer $targetDepth Zieltieft, also in welcher Tiefe extrahiert werden soll
  */
  function extractAtDepth(&$resultArray, $template, $targetDepth ){
    // do an ordinary extract, and weed out everything not at the
    // target depth
    $this->extract($tempResults, $template);

    // to keep references intact, you need to explicitly address
    // each array by index; current() only gives a copy
    for ($idx = 0; $idx < count($tempResults["nodes"]); $idx++){
      $tempDepth = $tempResults["depth"][$idx];
      if ($targetDepth == $tempDepth){
        $resultArray[] = &$tempResults["nodes"][$idx];
      }
    }
  }
  
  
  function walk($path, &$callBack) {
    $result = 0;
    if (is_array($path)) {
      $tags = $path;
    } else {
      $tags = explode('/', $path);
    }
    if (count($tags) > 0 && is_array($this->children)) {
      $currentName = array_shift($tags);
      foreach($this->children as $childNode) {
        if (isset($childNode) &&  is_a($childNode, 'simple_xmlnode') && $currentName == $childNode->name) {
          if (count($tags) < 1) {
            if (is_array($callBack)) {
              $object = &$callBack[0];
              $method = $callBack[1];
              $object->$method($childNode, @$callBack[2]);
            } else {
              $callBack($childNode);            
            }
            $result++;
          } else {
            $result += $childNode->walk($tags, $callBack);
          }
        }
      }
    }
    return $result;
  }
  
  /**
  * returns the children count
  *
  * @access public
  * @return integer
  */
  function childCount() {
    if (isset($this->children) && is_array($this->children)) {
      return count($this->children);
    } else {
      return 0;
    }
  }
  
  /**
  * destroy nodes
  *
  * @access public
  */
  function free() {
    if (isset($this->children) && is_array($this->children)) {
      foreach(array_keys($this->children) as $id) {
        $this->children[$id]->free();
        unset($this->children[$id]);
      }
    }
  }
}

/**
* xml text node class
*
* @package papaya_base
*/
class simple_xmltextnode {

  /**
  * parent node
  * @var object simple_xmlnode
  */
  var $parent = NULL;
  /**
  * node content
  * @var string
  */
  var $text = '';

  /**
  * Constructor
  *
  * @param object simple_xmlnode &$parent
  * @param string $text content
  * @access public
  */
  function __construct(&$parent, $text) {
    $this->parent = &$parent;
    $this->text = $text;
    if (isset($this->parent) && is_object($this->parent)) {
      $this->parent->children[count($this->parent->children)] = &$this;
    }
  }

  /**
  * Constuctor PHP 4
  *
  * @param object simple_xmlnode &$parent
  * @param string $text content
  * @access public
  */
  function simple_xmltextnode(&$parent, $text) {
    $this->__construct($parent, $text);
  }

  /**
  * Copy node
  *
  * @access public
  * @return object simple_xmltextnode
  */
  function &copy() {
    //create new node and set everything but parent here
    $nodeCopy = &new simple_xmltextnode(NULL, $this->text);
    return $nodeCopy;
  }

  /**
  * Output node with markup
  *
  * @param string &$xmlString
  * @param string $indent optional, default value ' '
  * @access public
  */
  function printOut(&$xmlString, $indent =' ') {
    $xmlString .= $this->text;
  }
  
  function printOutChildren(&$xmlString,  $indent = ' ') {
    $xmlString .= $this->text;
  }

  /**
  * Output node content
  *
  * @param string &$xmlString
  * @access public
  */
  function printText(&$xmlString) {
    $xmlString .= $this->text;
  }
  
  /**
  * child count is 0 at any time
  *
  * @access public
  * @return integer
  */
  function childCount() {
    return 0;
  }
  
  /**
  * destroy node data
  *
  * @access public
  */
  function free() {
    unset($this->text);
  }
}

?>