<?php
/**
* Implementaion of XSL-processing
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_xsl.php,v 1.5 2006/02/13 11:31:28 weinert Exp $
*/

/**
* XML/XSL transformation Objekt for Sablotron
*
*@package papaya_base
*/
class sys_xsl {
  /**
  * XML data
  * @var string $xmlData
  */
  var $xmlData = "";

  /**
  * XSL stylesheet data
  * @var string $xslData
  */
  var $xslData = "";

  /**
  * XSL-stylesheet-file
  * @var string $xslFileName
  */
  var $xslFileName = "";

  /**
  * XML-data
  * @var string $xmlFileName
  */
  var $xmlFileName;

  /**
  * Parameter
  * @var string $param
  */
  var $params = "";

  /**
  * output string
  * @var string $result
  */
  var $result = "";

  /**
  * Letzte Fehlermeldung von Sablotron
  * @var array $lastError
  */
  var $lastError;

  /**
  * Available XSLT-extensions
  * @var array $extensions
  */
  var $extensions = array();

  /**
  * Preferred XSLT-Extension
  * @var array $preferred
  */
  var $preferred = 'xslt';

  /**
  * PHP 5.1 has an new error handling for libxml errors
  * @var boolean $canHandleLibxmlErrors
  */
  var $canHandleLibxmlErrors = FALSE;

  /**
  * Constructor
  * @param string $xslFileName XSL-Datei
  * @param string $xmlFileName XML-Datei
  */
  function __construct($xslFileName="",$xmlFileName="") {
    $this->xslFileName = $xslFileName;
    $this->xmlFileName = $xmlFileName;
    $this->xmlData = papaya_strings::entityToXML($this->loadFile($xmlFileName));
  }

  /**
  * PHP 4 constuctor redirect
  *
  * @access public
  */
  function sys_xsl() {
    $args = func_get_args();
    call_user_func_array(array(&$this, '__construct'), $args);
  }

  /**
  * Check extension
  *
  * @access public
  * @return boolean
  */
  function checkExtensions() {
    unset($this->extensions);
    if (extension_loaded('xslt')) $this->extensions['xslt'] = TRUE;
    if (extension_loaded('domxml') && function_exists('domxml_xslt_stylesheet_file')) $this->extensions['domxml'] = TRUE;
    if (extension_loaded('xsl')) $this->extensions['xsl'] = TRUE;

    //new php 5.1 with error handling
    if (function_exists('libxml_use_internal_errors')) {
      $this->canHandleLibxmlErrors = TRUE;
    } else {
      $this->canHandleLibxmlErrors = FALSE;
    }
  }

  /**
  * Parse data
  * @param bool $output = Direkt ausgeben
  */
  function parse($output=FALSE) {
    unset($this->lastError);
    
    if (defined('PAPAYA_DBG_RUNTIME_XSLT') && PAPAYA_DBG_RUNTIME_XSLT === '1') {
      $timeStart = microtime();
    }

    $this->checkExtensions();
    if (isset($this->extensions[$this->preferred])) {
      $ext = $this->preferred;
    } elseif (count($this->extensions) > 0) {
      $ext = key($this->extensions);
    } else {
      return 'No XSLT Extension avaliable';
    }

    if (defined('PAPAYA_VERSION_STRING')) {
      $this->setParam('PAPAYA_VERSION', PAPAYA_VERSION_STRING);
    }
    
    switch ($ext) {
      case 'domxml' : $result = $this->parsePHP4domxsl($output); break;
      case 'xsl' : $result = $this->parsePHP5xsl($output); break;
      case 'xslt' : $result = $this->parsePHP4xslt($output); break;
      default : return 'XSLT Extension not implemented.';
    }
    if (defined('PAPAYA_DBG_RUNTIME_XSLT') && PAPAYA_DBG_RUNTIME_XSLT === '1') {
      $timeEnd = microtime();
      $from = explode(' ',$timeStart);
      $to = explode(' ',$timeEnd);
      $to = $to[0]+(double)$to[1];
      $from = $from[1]+(double)$from[0];
      $msg['time'] = number_format(($to-$from)*1000, 4);
      $msg['parser'] = $ext;
      $this->showDebug($msg);
    }
    return $result;
  }

  /**
  * Output debug messages
  * @param string $msg output message
  * @return void
  */
  function showDebug($msg) {
    echo '</form>', '</table>';
    echo '<table style="position: relative; z-index: 10000;" border="1" width="800" cellpadding="2" cellspacing="0" align="center" bgcolor="#C0C0C0">';
    echo sprintf('<tr><th>Debugausgabe (%s)</th><th align="right">'.$msg['time'].' ms</th></tr>', $msg['parser']);
    echo '</table><br/>';
    flush();
    return 0;
  }

  /**
  * Parse XSLT with php 4 ext/dom_xml (libxslt)
  * @param boolean $output print result?
  * @access public
  * @return mixed FALSE or result string
  */
  function parsePHP4domxsl($output=FALSE) {
    $xsl = domxml_xslt_stylesheet_file($this->xslFileName);
    $xml = domxml_open_mem($this->xmlData);
    if (isset($xsl) && is_object($xsl)) {
      $params = $this->xslParams;
      $params['_baselink'] = basename($_SERVER['SCRIPT_FILENAME']);

      $result = $xsl->process($xml,$params,FALSE);
      if(!$result) {
        $this->result = FALSE;
      } else {
        $this->result = $result->html_dump_mem(TRUE);
        if ($output) {
          print $this->result;
        }
      }
      unset($xsl);
    }
    unset($xml);
    return $this->result;
  }

  /**
  * Parse XSLT with php 5 ext/xsl (libxslt)
  *
  * @param boolean $output print result?
  * @access public
  * @return mixed FALSE or result string
  */
  function parsePHP5xsl($output=FALSE) {
    $this->result = FALSE;

    if ($this->canHandleLibxmlErrors) {
      //new php 5.1 with error handling
      libxml_use_internal_errors(TRUE);
    }

    $xmlDom = &new domDocument();
    $ret = $xmlDom->load($this->xslFileName);
    if (!$ret) {
      $this->handlerLibxmlErrors('file');
    } else {
      $xsl = &new xsltprocessor;
      $xsl->importStylesheet($xmlDom);
      unset($xmlDom);

      $xslDom = &new domDocument();
      $ret = $xslDom->loadXML($this->xmlData);
      if (!$ret) {
        $this->handlerLibxmlErrors('xml');
      } else {
        if (isset($xsl) && is_object($xsl)) {
          if (isset($this->xslParams) && is_array($this->xslParams)) {
            foreach($this->xslParams as $paramName=>$value) {
              $xsl->setParameter('', $paramName, $value);
            }
          }
          $xsl->setParameter('', '_baselink', basename($_SERVER['SCRIPT_FILENAME']));

          if ($this->canHandleLibxmlErrors) {
            $res = @$xsl->transformToXml($xslDom);
          } else {
            $res = $xsl->transformToXml($xslDom);
          }
          unset($xslDom);
          unset($xsl);
          if(!$res) {
            $this->handlerLibxmlErrors('file');
          } else {
            $this->result = $res;
            if ($output) {
              print $this->result;
            }
          }
        }
      }
    }
    return $this->result;
  }

  function handlerLibxmlErrors($errorType = '') {
    if ($this->canHandleLibxmlErrors) {
      $errors = libxml_get_errors();
      foreach ($errors as $error) {
        $this->lastError[] = array(
          'msgtype'=>$error->level,
          'code'=>$error->code,
          'module' => '',
          'file' => $error->file,
          'line' => $error->line,
          'msg' => $error->message,
          'errortype' => $errorType
        );
      }
      libxml_clear_errors();
    }
  }


  /**
  * Parse XSLT with php 4 ext/xslt (Sablotron)
  * @param boolean $output print result?
  * @access public
  * @return mixed FALSE or result string
  */
  function parsePHP4xslt($output=FALSE) {
    $this->result = '';
    if ($parser = @xslt_create()) {
      xslt_set_error_handler($parser, array(&$this, 'handlerXsltError'));

      if (isset($this->xslParams) && is_array($this->xslParams)) {
        $params = $this->xslParams;
      } else {
        $params = array();
      }
      $params['_baselink'] = basename($_SERVER['SCRIPT_FILENAME']);

      if (file_exists($this->xslFileName)) {
        $args = array(
          '_xml' => $this->xmlData
        );
        $xslFileName = strtr($this->xslFileName, array('\\'=>'/'));
        $this->result = xslt_process($parser, 'arg:/_xml', 'file://'.$xslFileName, NULL, $args, $params);
      } else {
        $args = array(
          '_xml' => $this->xmlData,
          '_xsl' => $this->xslData
        );
        $this->result = xslt_process($parser, 'arg:/_xml', 'arg:/_xsl', NULL, $args, $params);
      }
      if(!$this->result) {
        $this->result = FALSE;
      } else if ($output) {
        print $this->result;
      }
      xslt_free($parser);
    }
    return $this->result;
  }

  /**
  * Error handler for new extensions
  *
  * @access public
  */
  function handlerXsltError() {
    unset($this->lastError);
    $data = func_get_arg(3);
    $caps = array('msgtype', 'code', 'module', 'URI', 'line', 'node', 'msg');
    $error = array();
    $error['msgtype'] = (isset($data['msgtype'])) ? $data['msgtype'] : 0;
    $error['code']    = (isset($data['code'])) ? $data['code'] : 0;
    $error['line']    = (isset($data['line'])) ? $data['line'] : 0;
    if (isset($data['node']) && $data['node'] != '') {
      $error['msg'] = $data['node'].' '.$data['msg'];
    } else {
      $error['msg'] = $data['msg'];
    }
    if (($data["URI"] == 'arg:/_xsl') || ($this->xslFileName == $data["URI"])) {
      $error["errortype"] = "xsl";
      $error['file'] = $this->xslFileName;
    } elseif (strpos($data["URI"], 'file:') === 0) {
      $error["errortype"] = "file";
      $error['file'] = substr($data["URI"], 5);
    } elseif ($data["URI"] == 'arg:/_xml') {
      $error["errortype"] = "xml";
      $error['file'] = '';
    } else {
      $error["errortype"] = "unknown";
      $error['file'] = '';
    }
    $this->lastError[] = $error;
  }

  /**
  * load data out of file
  * @param string $fileName file name
  * @return mixed result or FALSE
  */
  function loadFile($fileName) {
    if (@file_exists($fileName) && @is_file($fileName)) {
      $fp = fopen($fileName, "r");
      if ($fp) {
        $result = fread($fp,filesize($fileName));
        fclose($fp);
        return $result;
      } else {
        return FALSE;
      }
    } else {
      return FALSE;
    }
  }

  /**
  * Sow last error
  * @param boolean $linkIt create hyperlink ?
  * @return string $result
  */
  function getLastErrors($fileName) {
    if (isset($this->lastError) && is_array($this->lastError) && count($this->lastError) > 0) {
      $result = '<ul id="msg" style="list-style: none; margin-left: 0px; padding: 0px;">';
      foreach($this->lastError as $error) {
        $result .= '<li>';
        $result .= sprintf('<b>%d: %s</b>', $error['code'], papaya_strings::escapeHTMLChars($error['msg']));
        if ($error['file'] != '') {
          $result .= sprintf('<br/>%s', papaya_strings::escapeHTMLChars($error['file']));
          if ($fileName == $error['file']) {
            $result .= sprintf(':<a href="#line%d">%d</a>', $error['line'], $error['line']);
          } else {
            $result .= ':'.(int)$error['line'];
          }
        }
        $result .= '</li>';
      }
      $result .= '</ul>';
    }
    return $result;
  }


  /**
  * HTML error message
  * @return string $result;
  */
  function getLastErrorHTML() {
    $result = '';
    if (isset($this->lastError) && is_array($this->lastError) && count($this->lastError) > 0) {
      $firstError = $this->lastError[0];
      $result = $this->getLastErrors($firstError['file']);
      $markLines = NULL;
      foreach ($this->lastError as $error) {
        if ($error['file'] == $firstError['file'] && $error['line'] > 0) {
          $markLines[] = (int)$error['line'];
        }
      }
      switch ($firstError["errortype"]) {
        case "file" :
          if ($firstError['file'] != '') {
            $result .= $this->getFormattedLines(@file_get_contents($firstError['file']),$firstError['line'],$markLines);
          }
          break;
        case "xsl" :
          $result .= $this->getFormattedLines($this->xslData, $firstError["line"],$markLines);
          break;
        case "xml" :
          $result .= $this->getFormattedLines($this->xmlData, $firstError["line"],$markLines);
          break;
      }
    }
    return $result;
  }

  /**
  * Output formatting of sourcecode
  *
  * @param string $lineString Code line as String
  * @param integer $errorLine error line
  * @param bool $linked create hyperlink
  * @return string $result
  */
  function getFormattedLines($lineString, $errorLine, $markLines = NULL) {
    $result = '';
    $lines = split("\r\n|\n", $lineString);
    if (isset($lines) && is_array($lines)) {
      $lineNumber = 0;
      $result .= '<ul type="1" style="font-family: monospace">';
      foreach($lines as $lineNumber=>$line) {
        $result .= sprintf('<li id="line%d">', ++$lineNumber);
        if (($lineNumber) == $errorLine) {
          $result .= '<span style="color: white; background-color:red;">'.papaya_strings::escapeHTMLChars($line).'</span>';
        } elseif (isset($markLines) && is_array($markLines) && in_array($lineNumber, $markLines)) {
          $result .= '<span style="background-color: silver;">'.papaya_strings::escapeHTMLChars($line).'</span>';
        } else {
          $result .= htmlentities($line);
        }
        $result .= '</li>';
      }
      $result .= "</ul>";
    }
    return $result;
  }

  /**
  * Set parameters and escape them if necessary
  * @param string $name parameter name
  * @param string $value value of parameter
  */
  function setParam($name, $value) {
    if (defined('PAPAYA_LATIN1_COMPATIBILITY') && PAPAYA_LATIN1_COMPATIBILITY) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');
      $this->xslParams[strtoupper($name)] = papaya_strings::ensureUTF8($value);
    } else {
      $this->xslParams[strtoupper($name)] = $value;
    }
  }
}
?>