<?php
/**
* Installer
* @module papaya-admin
* @author Thomas Weinert <weinert@papaya-cms.de>
* @version $Id: install.php,v 1.3 2006/03/21 14:31:17 weinert Exp $
*/

/**
* Configuration file
*/
require_once("./inc.conf.php");
require_once("./inc.version.php");

define('PAPAYA_ADMIN_PAGE', TRUE);

/**
* check include path - try to include installer
*/
if (defined('PAPAYA_DBG_DEVMODE') && PAPAYA_DBG_DEVMODE) {
  $found = include_once(PAPAYA_INCLUDE_PATH.'system/papaya_installer.php');
} else {
  $found = @include_once(PAPAYA_INCLUDE_PATH.'system/papaya_installer.php');
}

if (!$found) {
  //no installer found - output error message and exit
?>
<html>
  <head>
    <title>Papaya-CMS Installer</title>
    <style type="text/css">
      body {
        font-family: Verdana, Arial, Helvetic, sans serif;
        font-size: 14px;
        background-color: threedface;
      }
      h1 {
        width: 400px;
        margin: auto;
        float: center;
        margin-top: 40px;
        margin-bottom: 40px;
        padding: 4px;
        text-align: center;
      }
      div.error {
        width: 400px;
        float: center;
        margin: auto;
        background-color: window;
        color: windowtext;
        border: 1px inset threedface;
        margin-top: 20px;
        margin-bottom: 20px;
        padding: 4px;
      }
      .optname {
        font-family: monospace;
        font-weight: bold;
      }
      .filename {
        font-family: monospace;
      }
      div.options {
        width: 600px;
        float: center;
        margin: auto;
        background-color: window;
        color: windowtext;
        border: 1px inset threedface;
        margin-top: 20px;
        margin-bottom: 20px;
        padding: 0px;
      }
      div.options table {
        width: 100%;
        background-color: window;
        color: windowtext;
        border-collapse: collapse;
      }
      div.options table th {
        border: 1px solid threedshadow;
        font-size: 13px;
      }
      div.options table td {
        border: 1px solid threedshadow;
        font-weight: bold;
        font-family: monospace;
        font-size: 12px;
      }
      div.options table td + td {
        font-weight: normal;
        font-size: 12px;
      }
    </style>
  </head>
  <body>
    <h1>Error</h1>
    <div class="error">
      The system can not find the class framework. Please check the <span class="optname">PAPAYA_INCLUDE_PATH</span> in the <span class="filename">conf.inc.php</span> in your install path. The <span class="optname">PAPAYA_INCLUDE_PATH</span> can be an absolute path or a subdirectory of your <span class="optname">include_path</span> in the php configuration.
    </div>
    <div class="options">
      <table summary="Current option values">
        <thead>
          <th>Name</th>
          <th>Value</th>
        </thead>
        <tbody>
          <tr>
            <td>DOCUMENT_ROOT</td>
            <td><?php echo $_SERVER['DOCUMENT_ROOT'] ?></td>
          </tr>
          <tr>
            <td>include_path</td>
            <td><?php echo get_include_path(); ?></td>
          </tr>
          <tr>
            <td>PAPAYA_INCLUDE_PATH</td>
            <td>
              <?php
                if (defined('PAPAYA_INCLUDE_PATH')) {
                  echo PAPAYA_INCLUDE_PATH;
                }
              ?>
            </td>
          </tr>
        </tbody>
      </table>
    </div>
  </body>
</html>

<?php
} else {
  //installer found - continue installation

  /**
  * button glyphs
  */
  include_once('./inc.glyphs.php');

  /**
  * Error handling
  */
  include_once(PAPAYA_INCLUDE_PATH.'system/papaya_errors.php');
  $PAPAYA_MSG = &new papaya_errors();

  //do not show database errors
  define('PAPAYA_DBG_DATABASE_ERROR', '0');

  /**
  * options object
  */
  include_once(PAPAYA_INCLUDE_PATH.'system/papaya_options.php');
  $PAPAYA_OPTIONS = &new papaya_options;

  if (defined('PAPAYA_DBG_PHP_ERROR')) {
    error_reporting(PAPAYA_DBG_PHP_ERROR);
  }
  $PAPAYA_OPTIONS->loadAndDefine();

  /**
  * layout object
  */
  include_once(PAPAYA_INCLUDE_PATH.'system/papaya_xsl.php');
  $PAPAYA_LAYOUT = &new papaya_xsl(dirname(__FILE__)."/xsl/style.xsl");
  $PAPAYA_LAYOUT->setParam('PAPAYA_BGCOLOR', defined('PAPAYA_BGCOLOR') ? PAPAYA_BGCOLOR : 'threedface');
  $PAPAYA_LAYOUT->setParam('PAPAYA_BGCOLOR_LIGHT', defined('PAPAYA_BGCOLOR_LIGHT') ? PAPAYA_BGCOLOR_LIGHT : 'threedlightshadow');
  $PAPAYA_LAYOUT->setParam('PAPAYA_BGCOLOR_DARKER', defined('PAPAYA_BGCOLOR_DARKER') ? PAPAYA_BGCOLOR_DARKER : 'threedshadow');
  $PAPAYA_LAYOUT->setParam('PAPAYA_BGCOLOR_DARK', defined('PAPAYA_BGCOLOR_DARK') ? PAPAYA_BGCOLOR_DARK : 'appworkspace');
  $PAPAYA_LAYOUT->setParam('PAPAYA_FGCOLOR', defined('PAPAYA_FGCOLOR') ? PAPAYA_FGCOLOR : 'windowtext');
  $PAPAYA_LAYOUT->setParam('PAPAYA_WINDOWCOLOR', defined('PAPAYA_WINDOWCOLOR') ? PAPAYA_WINDOWCOLOR : 'window');
  $PAPAYA_LAYOUT->setParam('PAPAYA_HIGHLIGHT_COLOR', defined('PAPAYA_HIGHLIGHT_COLOR') ? PAPAYA_HIGHLIGHT_COLOR : 'highlight');
  $PAPAYA_LAYOUT->setParam('PAPAYA_HIGHLIGHT_TEXTCOLOR', defined('PAPAYA_HIGHLIGHT_TEXTCOLOR') ? PAPAYA_HIGHLIGHT_TEXTCOLOR : 'highlighttext');
  $PAPAYA_LAYOUT->setParam('PAPAYA_SYSTEMPICS_PATH', defined('PAPAYA_SYSTEMPICS_PATH') ? PAPAYA_SYSTEMPICS_PATH : './themes/system/');
  $PAPAYA_LAYOUT->setParam('PAGE_PROJECT', '');
  $PAPAYA_LAYOUT->setParam('PAGE_TITLE', 'INSTALLER');
  $PAPAYA_LAYOUT->setParam('PAGE_USER', 'NO USER');

  //create session
  include_once(PAPAYA_INCLUDE_PATH.'system/sys_session.php');
  $PAPAYA_SESSION = &new rewrite_session();

  //create installer object
  $installer = &new papaya_installer();
  $installer->msgs = &$PAPAYA_MSG;
  $installer->options = &$PAPAYA_OPTIONS;
  $installer->images = &$PAPAYA_IMAGES;
  $installer->layout = &$PAPAYA_LAYOUT;

  $installer->initialize();
  $installer->execute();

  if (!$installer->javascriptRPC) {
    if (isset($installer->authUser) && isset($installer->authUser->options['PAPAYA_UI_THEME'])) {
      $PAPAYA_OPTIONS->activateTheme(dirname(__FILE__), $installer->authUser->options['PAPAYA_UI_THEME']);
    } else {
      $PAPAYA_OPTIONS->activateTheme(dirname(__FILE__), PAPAYA_UI_THEME);
    }
    $PAPAYA_LAYOUT->add($PAPAYA_MSG->get(), 'messages');
    print $PAPAYA_LAYOUT->xhtml();
  }
}

?>